@@ -1,9 +1,12 @@
 defmodule EEx.SyntaxError do
-  defexception [:message, :file, :line, :column]
+  defexception [:file, :line, :column, :snippet, message: "syntax error"]
 
   @impl true
   def message(exception) do
-    "#{exception.file}:#{exception.line}:#{exception.column}: #{exception.message}"
+    %{file: file, line: line, column: column, message: message, snippet: snippet} = exception
+
+    Exception.format_file_line_column(file && Path.relative_to_cwd(file), line, column, " ") <>
+      message <> (snippet || "")
   end
 end
 
@@ -50,7 +53,7 @@ defmodule EEx do
     * `:indentation` - (since v1.11.0) an integer added to the column after every
       new line. Defaults to `0`.
 
-    * `:engine` - the EEx engine to be used for compilation.
+    * `:engine` - the EEx engine to be used for compilation. Defaults to `EEx.SmartEngine`.
 
     * `:trim` - if `true`, trims whitespace left and right of quotation as
       long as at least one newline is present. All subsequent newlines and
@@ -103,7 +106,7 @@ defmodule EEx do
 
   @type line :: non_neg_integer
   @type column :: non_neg_integer
-  @type marker :: '=' | '/' | '|' | ''
+  @type marker :: [?=] | [?/] | [?|] | []
   @type metadata :: %{column: column, line: line}
   @type token ::
           {:comment, charlist, metadata}
@@ -217,7 +220,7 @@ defmodule EEx do
   def compile_string(source, options \\ []) when is_binary(source) and is_list(options) do
     case tokenize(source, options) do
       {:ok, tokens} ->
-        EEx.Compiler.compile(tokens, options)
+        EEx.Compiler.compile(tokens, source, options)
 
       {:error, message, %{column: column, line: line}} ->
         file = options[:file] || "nofile"
@@ -314,8 +317,8 @@ defmodule EEx do
 
   ## Examples
 
-      iex> EEx.tokenize('foo', line: 1, column: 1)
-      {:ok, [{:text, 'foo', %{column: 1, line: 1}}, {:eof, %{column: 4, line: 1}}]}
+      iex> EEx.tokenize(~c"foo", line: 1, column: 1)
+      {:ok, [{:text, ~c"foo", %{column: 1, line: 1}}, {:eof, %{column: 4, line: 1}}]}
 
   ## Result
 
@@ -332,7 +335,7 @@ defmodule EEx do
   Note new tokens may be added in the future.
   """
   @doc since: "1.14.0"
-  @spec tokenize(IO.chardata(), opts :: keyword) ::
+  @spec tokenize([char()] | String.t(), opts :: keyword) ::
           {:ok, [token()]} | {:error, String.t(), metadata()}
   def tokenize(contents, opts \\ []) do
     EEx.Compiler.tokenize(contents, opts)
