@@ -10,17 +10,21 @@ defmodule EEx.Compiler do
   Tokenize EEx contents.
   """
   def tokenize(contents, opts) when is_binary(contents) do
-    tokenize(String.to_charlist(contents), opts)
+    tokenize(String.to_charlist(contents), contents, opts)
   end
 
   def tokenize(contents, opts) when is_list(contents) do
+    tokenize(contents, List.to_string(contents), opts)
+  end
+
+  def tokenize(contents, source, opts) when is_list(contents) do
     file = opts[:file] || "nofile"
     line = opts[:line] || 1
     trim = opts[:trim] || false
     indentation = opts[:indentation] || 0
     column = indentation + (opts[:column] || 1)
 
-    state = %{trim: trim, indentation: indentation, file: file}
+    state = %{trim: trim, indentation: indentation, file: file, source: source}
 
     {contents, line, column} =
       (trim && trim_init(contents, line, column, state)) || {contents, line, column}
@@ -28,14 +32,15 @@ defmodule EEx.Compiler do
     tokenize(contents, line, column, state, [{line, column}], [])
   end
 
-  defp tokenize('<%%' ++ t, line, column, state, buffer, acc) do
+  defp tokenize(~c"<%%" ++ t, line, column, state, buffer, acc) do
     tokenize(t, line, column + 3, state, [?%, ?< | buffer], acc)
   end
 
-  defp tokenize('<%!--' ++ t, line, column, state, buffer, acc) do
+  defp tokenize(~c"<%!--" ++ t, line, column, state, buffer, acc) do
     case comment(t, line, column + 5, state, []) do
-      {:error, line, column, message} ->
-        {:error, message, %{line: line, column: column}}
+      {:error, message} ->
+        meta = %{line: line, column: column}
+        {:error, message <> code_snippet(state.source, state.indentation, meta), meta}
 
       {:ok, new_line, new_column, rest, comments} ->
         token = {:comment, Enum.reverse(comments), %{line: line, column: column}}
@@ -43,10 +48,16 @@ defmodule EEx.Compiler do
     end
   end
 
-  # TODO: Deprecate this on Elixir v1.18
-  defp tokenize('<%#' ++ t, line, column, state, buffer, acc) do
+  # TODO: Remove me on Elixir v2.0
+  defp tokenize(~c"<%#" ++ t, line, column, state, buffer, acc) do
+    IO.warn("<%# is deprecated, use <%!-- or add a space between <% and # instead",
+      line: line,
+      column: column,
+      file: state.file
+    )
+
     case expr(t, line, column + 3, state, []) do
-      {:error, line, column, message} ->
+      {:error, message} ->
         {:error, message, %{line: line, column: column}}
 
       {:ok, _, new_line, new_column, rest} ->
@@ -54,35 +65,35 @@ defmodule EEx.Compiler do
     end
   end
 
-  defp tokenize('<%' ++ t, line, column, state, buffer, acc) do
+  defp tokenize(~c"<%" ++ t, line, column, state, buffer, acc) do
     {marker, t} = retrieve_marker(t)
+    marker_length = length(marker)
 
-    case expr(t, line, column + 2 + length(marker), state, []) do
-      {:error, line, column, message} ->
-        {:error, message, %{line: line, column: column}}
+    case expr(t, line, column + 2 + marker_length, state, []) do
+      {:error, message} ->
+        meta = %{line: line, column: column}
+        {:error, message <> code_snippet(state.source, state.indentation, meta), meta}
 
       {:ok, expr, new_line, new_column, rest} ->
         {key, expr} =
           case :elixir_tokenizer.tokenize(expr, 1, file: "eex", check_terminators: false) do
-            {:ok, _line, _column, warnings, tokens} ->
-              Enum.each(Enum.reverse(warnings), fn {location, file, msg} ->
-                :elixir_errors.erl_warn(location, file, msg)
-              end)
-
-              token_key(tokens, expr)
+            {:ok, _line, _column, _warnings, rev_tokens, []} ->
+              # We ignore warnings because the code will be tokenized
+              # again later with the right line+column info
+              token_key(rev_tokens, expr)
 
             {:error, _, _, _, _} ->
               {:expr, expr}
           end
 
         marker =
-          if key in [:middle_expr, :end_expr] and marker != '' do
+          if key in [:middle_expr, :end_expr] and marker != ~c"" do
             message =
               "unexpected beginning of EEx tag \"<%#{marker}\" on \"<%#{marker}#{expr}%>\", " <>
                 "please remove \"#{marker}\""
 
             :elixir_errors.erl_warn({line, column}, state.file, message)
-            ''
+            ~c""
           else
             marker
           end
@@ -92,7 +103,7 @@ defmodule EEx.Compiler do
     end
   end
 
-  defp tokenize('\n' ++ t, line, _column, state, buffer, acc) do
+  defp tokenize([?\n | t], line, _column, state, buffer, acc) do
     tokenize(t, line + 1, state.indentation + 1, state, [?\n | buffer], acc)
   end
 
@@ -119,7 +130,7 @@ defmodule EEx.Compiler do
   end
 
   defp retrieve_marker(t) do
-    {'', t}
+    {~c"", t}
   end
 
   # Tokenize a multi-line comment until we find --%>
@@ -128,16 +139,16 @@ defmodule EEx.Compiler do
     {:ok, line, column + 4, t, buffer}
   end
 
-  defp comment('\n' ++ t, line, _column, state, buffer) do
-    comment(t, line + 1, state.indentation + 1, state, '\n' ++ buffer)
+  defp comment([?\n | t], line, _column, state, buffer) do
+    comment(t, line + 1, state.indentation + 1, state, [?\n | buffer])
   end
 
   defp comment([head | t], line, column, state, buffer) do
     comment(t, line, column + 1, state, [head | buffer])
   end
 
-  defp comment([], line, column, _state, _buffer) do
-    {:error, line, column, "missing token '--%>'"}
+  defp comment([], _line, _column, _state, _buffer) do
+    {:error, "expected closing '--%>' for EEx expression"}
   end
 
   # Tokenize an expression until we find %>
@@ -146,7 +157,7 @@ defmodule EEx.Compiler do
     {:ok, Enum.reverse(buffer), line, column + 2, t}
   end
 
-  defp expr('\n' ++ t, line, _column, state, buffer) do
+  defp expr([?\n | t], line, _column, state, buffer) do
     expr(t, line + 1, state.indentation + 1, state, [?\n | buffer])
   end
 
@@ -154,13 +165,13 @@ defmodule EEx.Compiler do
     expr(t, line, column + 1, state, [h | buffer])
   end
 
-  defp expr([], line, column, _state, _buffer) do
-    {:error, line, column, "missing token '%>'"}
+  defp expr([], _line, _column, _state, _buffer) do
+    {:error, "expected closing '%>' for EEx expression"}
   end
 
   # Receives tokens and check if it is a start, middle or an end token.
-  defp token_key(tokens, expr) do
-    case {tokens, tokens |> Enum.reverse() |> drop_eol()} do
+  defp token_key(rev_tokens, expr) do
+    case {Enum.reverse(rev_tokens), drop_eol(rev_tokens)} do
       {[{:end, _} | _], [{:do, _} | _]} ->
         {:middle_expr, expr}
 
@@ -275,24 +286,33 @@ defmodule EEx.Compiler do
   and the engine together by handling the tokens and invoking
   the engine every time a full expression or text is received.
   """
-  @spec compile([EEx.token()], keyword) :: Macro.t()
-  def compile(tokens, opts) do
+  @spec compile([EEx.token()], String.t(), keyword) :: Macro.t()
+  def compile(tokens, source, opts) do
     file = opts[:file] || "nofile"
     line = opts[:line] || 1
+    indentation = opts[:indentation] || 0
     parser_options = opts[:parser_options] || Code.get_compiler_option(:parser_options)
     engine = opts[:engine] || @default_engine
 
     state = %{
       engine: engine,
       file: file,
+      source: source,
       line: line,
       quoted: [],
-      start_line: nil,
-      start_column: nil,
-      parser_options: parser_options
+      parser_options: parser_options,
+      indentation: indentation
     }
 
     init = state.engine.init(opts)
+
+    if function_exported?(state.engine, :handle_text, 2) and
+         not function_exported?(state.engine, :handle_text, 3) do
+      IO.warn(
+        "#{inspect(state.engine)}.handle_text/2 is deprecated, implement handle_text/3 instead"
+      )
+    end
+
     generate_buffer(tokens, init, [], state)
   end
 
@@ -310,8 +330,7 @@ defmodule EEx.Compiler do
         meta = [line: meta.line, column: meta.column]
         state.engine.handle_text(buffer, meta, IO.chardata_to_string(chars))
       else
-        # TODO: Deprecate this branch on Elixir v1.18.
-        # We should most likely move this check to init to emit the deprecation once.
+        # TODO: Remove this on Elixir v2.0. The deprecation is on init.
         state.engine.handle_text(buffer, IO.chardata_to_string(chars))
       end
 
@@ -334,92 +353,94 @@ defmodule EEx.Compiler do
          scope,
          state
        ) do
-    if mark == '' do
+    {rest, line, contents} = look_ahead_middle(rest, meta.line, chars) || {rest, meta.line, chars}
+    start_line = meta.line
+    start_column = column(meta.column, mark)
+
+    {contents, rest} =
+      generate_buffer(
+        rest,
+        state.engine.handle_begin(buffer),
+        [{contents, start_line, start_column} | scope],
+        %{state | quoted: [], line: line}
+      )
+
+    if mark == ~c"" and not match?({:=, _, [_, _]}, contents) do
       message =
         "the contents of this expression won't be output unless the EEx block starts with \"<%=\""
 
       :elixir_errors.erl_warn({meta.line, meta.column}, state.file, message)
     end
 
-    {rest, line, contents} = look_ahead_middle(rest, meta.line, chars) || {rest, meta.line, chars}
-
-    {contents, rest} =
-      generate_buffer(
-        rest,
-        state.engine.handle_begin(buffer),
-        [contents | scope],
-        %{
-          state
-          | quoted: [],
-            line: line,
-            start_line: meta.line,
-            start_column: column(meta.column, mark)
-        }
-      )
-
     buffer = state.engine.handle_expr(buffer, IO.chardata_to_string(mark), contents)
     generate_buffer(rest, buffer, scope, state)
   end
 
   defp generate_buffer(
-         [{:middle_expr, '', chars, meta} | rest],
+         [{:middle_expr, ~c"", chars, meta} | rest],
          buffer,
-         [current | scope],
+         [{current, current_line, current_column} | scope],
          state
        ) do
     {wrapped, state} = wrap_expr(current, meta.line, buffer, chars, state)
     state = %{state | line: meta.line}
-    generate_buffer(rest, state.engine.handle_begin(buffer), [wrapped | scope], state)
+
+    generate_buffer(
+      rest,
+      state.engine.handle_begin(buffer),
+      [{wrapped, current_line, current_column} | scope],
+      state
+    )
   end
 
-  defp generate_buffer([{:middle_expr, _, chars, meta} | _], _buffer, [], state) do
-    raise EEx.SyntaxError,
-      message: "unexpected middle of expression <%#{chars}%>",
-      file: state.file,
-      line: meta.line,
-      column: meta.column
+  defp generate_buffer([{:middle_expr, _, chars, meta} | _tokens], _buffer, [], state) do
+    message = "unexpected middle of expression <%#{chars}%>"
+    syntax_error!(message, meta, state)
   end
 
   defp generate_buffer(
-         [{:end_expr, '', chars, meta} | rest],
+         [{:end_expr, ~c"", chars, meta} | rest],
          buffer,
-         [current | _],
+         [{current, line, column} | _],
          state
        ) do
     {wrapped, state} = wrap_expr(current, meta.line, buffer, chars, state)
-    column = state.start_column
-    options = [file: state.file, line: state.start_line, column: column] ++ state.parser_options
+    options = [file: state.file, line: line, column: column] ++ state.parser_options
     tuples = Code.string_to_quoted!(wrapped, options)
     buffer = insert_quoted(tuples, state.quoted)
     {buffer, rest}
   end
 
   defp generate_buffer([{:end_expr, _, chars, meta} | _], _buffer, [], state) do
-    raise EEx.SyntaxError,
-      message: "unexpected end of expression <%#{chars}%>",
-      file: state.file,
-      line: meta.line,
-      column: meta.column
+    message = "unexpected end of expression <%#{chars}%>"
+    syntax_error!(message, meta, state)
   end
 
   defp generate_buffer([{:eof, _meta}], buffer, [], state) do
     state.engine.handle_body(buffer)
   end
 
-  defp generate_buffer([{:eof, meta}], _buffer, _scope, state) do
-    raise EEx.SyntaxError,
-      message: "unexpected end of string, expected a closing '<% end %>'",
-      file: state.file,
-      line: meta.line,
-      column: meta.column
+  defp generate_buffer([{:eof, _meta}], _buffer, [{content, line, column} | _scope], state) do
+    message = "expected a closing '<% end %>' for block expression in EEx"
+    expr_meta = non_whitespace_meta(content, line, column, state)
+    syntax_error!(message, expr_meta, state)
   end
 
+  defp non_whitespace_meta([space | rest], line, column, state) when space in @h_spaces,
+    do: non_whitespace_meta(rest, line, column + 1, state)
+
+  defp non_whitespace_meta([?\n | rest], line, _column, state),
+    do: non_whitespace_meta(rest, line + 1, state.indentation + 1, state)
+
+  defp non_whitespace_meta(_, line, column, _),
+    do: %{line: line, column: column}
+
   # Creates a placeholder and wrap it inside the expression block
 
   defp wrap_expr(current, line, buffer, chars, state) do
     new_lines = List.duplicate(?\n, line - state.line)
     key = length(state.quoted)
-    placeholder = '__EEX__(' ++ Integer.to_charlist(key) ++ ');'
+    placeholder = ~c"__EEX__(" ++ Integer.to_charlist(key) ++ ~c");"
     count = current ++ placeholder ++ new_lines ++ chars
     new_state = %{state | quoted: [{key, state.engine.handle_end(buffer)} | state.quoted]}
 
@@ -475,7 +496,41 @@ defmodule EEx.Compiler do
   end
 
   defp column(column, mark) do
-    # length('<%') == 2
+    # length(~c"<%") == 2
     column + 2 + length(mark)
   end
+
+  defp syntax_error!(message, meta, state) do
+    raise EEx.SyntaxError,
+      message: message,
+      snippet: code_snippet(state.source, state.indentation, meta),
+      file: state.file,
+      line: meta.line,
+      column: meta.column
+  end
+
+  defp code_snippet(source, indentation, meta) do
+    line_start = max(meta.line - 3, 1)
+    line_end = meta.line
+    digits = line_end |> Integer.to_string() |> byte_size()
+    number_padding = String.duplicate(" ", digits)
+    indentation = String.duplicate(" ", indentation)
+
+    source
+    |> String.split(["\r\n", "\n"])
+    |> Enum.slice((line_start - 1)..(line_end - 1))
+    |> Enum.map_reduce(line_start, fn
+      expr, line_number when line_number == line_end ->
+        arrow = String.duplicate(" ", meta.column - 1) <> "^"
+        {"#{line_number} | #{indentation}#{expr}\n #{number_padding}| #{arrow}", line_number + 1}
+
+      expr, line_number ->
+        line_number_padding = String.pad_leading("#{line_number}", digits)
+        {"#{line_number_padding} | #{indentation}#{expr}", line_number + 1}
+    end)
+    |> case do
+      {[], _} -> ""
+      {snippet, _} -> Enum.join(["\n #{number_padding}|" | snippet], "\n")
+    end
+  end
 end
