@@ -6,6 +6,9 @@ defmodule Code.Formatter do
   @double_heredoc "\"\"\""
   @single_quote "'"
   @single_heredoc "'''"
+  @sigil_c_double "~c\""
+  @sigil_c_single "~c'"
+  @sigil_c_heredoc "~c\"\"\""
   @newlines 2
   @min_line 0
   @max_line 9_999_999
@@ -13,7 +16,7 @@ defmodule Code.Formatter do
   @ampersand_prec Code.Identifier.unary_op(:&) |> elem(1)
 
   # Operators that are composed of multiple binary operators
-  @multi_binary_operators [:"..//"]
+  @multi_binary_operators [:..//]
 
   # Operators that do not have space between operands
   @no_space_binary_operators [:.., :"//"]
@@ -28,30 +31,37 @@ defmodule Code.Formatter do
   @right_new_line_before_binary_operators [:|, :when]
 
   # Operators that are logical cannot be mixed without parens
-  @required_parens_logical_binary_operands [:||, :|||, :or, :&&, :&&&, :and]
+  @required_parens_logical_binary_operands [:|||, :||, :or, :&&&, :&&, :and]
 
-  # Operators with next break fits. = and :: do not consider new lines though
+  # Operators with next break fits
   @next_break_fits_operators [:<-, :==, :!=, :=~, :===, :!==, :<, :>, :<=, :>=, :=, :"::"]
 
-  # Operators that always require parens on operands when they are the parent
+  # Operators that always require parens even
+  # when they are their own parents as they are not semantically associative
+  @required_parens_even_when_parent [:--, :---]
+
+  # Operators that always require parens on operands
+  # when they are the parent of another operator with a difference precedence
+  # Most operators are listed, except comparison, arithmetic, and low precedence
   @required_parens_on_binary_operands [
-    :|>,
     :<<<,
     :>>>,
+    :|>,
     :<~,
     :~>,
     :<<~,
     :~>>,
     :<~>,
     :"<|>",
-    :"^^^",
-    :+++,
-    :---,
     :in,
+    :"^^^",
+    :"//",
     :++,
     :--,
-    :..,
-    :<>
+    :+++,
+    :---,
+    :<>,
+    :..
   ]
 
   @locals_without_parens [
@@ -141,24 +151,6 @@ defmodule Code.Formatter do
 
   @do_end_keywords [:rescue, :catch, :else, :after]
 
-  @bitstring_modifiers [
-    :integer,
-    :float,
-    :bits,
-    :bitstring,
-    :binary,
-    :bytes,
-    :utf8,
-    :utf16,
-    :utf32,
-    :signed,
-    :unsigned,
-    :little,
-    :big,
-    :native,
-    :_
-  ]
-
   @doc """
   Converts the quoted expression into an algebra document.
   """
@@ -197,18 +189,22 @@ defmodule Code.Formatter do
     locals_without_parens = Keyword.get(opts, :locals_without_parens, [])
     file = Keyword.get(opts, :file, nil)
     sigils = Keyword.get(opts, :sigils, [])
-    normalize_bitstring_modifiers = Keyword.get(opts, :normalize_bitstring_modifiers, true)
+    migrate = Keyword.get(opts, :migrate, false)
+    migrate_bitstring_modifiers = Keyword.get(opts, :migrate_bitstring_modifiers, migrate)
+    migrate_charlists_as_sigils = Keyword.get(opts, :migrate_charlists_as_sigils, migrate)
+    migrate_unless = Keyword.get(opts, :migrate_unless, migrate)
+    syntax_colors = Keyword.get(opts, :syntax_colors, [])
 
     sigils =
       Map.new(sigils, fn {key, value} ->
         with true <- is_atom(key) and is_function(value, 2),
-             [char] <- Atom.to_charlist(key),
-             true <- char in ?A..?Z do
-          {char, value}
+             name = Atom.to_charlist(key),
+             true <- Enum.all?(name, &(&1 in ?A..?Z)) do
+          {name, value}
         else
           _ ->
             raise ArgumentError,
-                  ":sigils must be a keyword list with a single uppercased letter as key and an " <>
+                  ":sigils must be a keyword list with uppercased atoms as keys and an " <>
                     "anonymous function expecting two arguments as value, got: #{inspect(sigils)}"
         end
       end)
@@ -221,7 +217,10 @@ defmodule Code.Formatter do
       comments: comments,
       sigils: sigils,
       file: file,
-      normalize_bitstring_modifiers: normalize_bitstring_modifiers
+      migrate_bitstring_modifiers: migrate_bitstring_modifiers,
+      migrate_charlists_as_sigils: migrate_charlists_as_sigils,
+      migrate_unless: migrate_unless,
+      inspect_opts: %Inspect.Opts{syntax_colors: syntax_colors}
     }
   end
 
@@ -277,7 +276,7 @@ defmodule Code.Formatter do
   end
 
   defp quoted_to_algebra({var, _meta, var_context}, _context, state) when is_atom(var_context) do
-    {var |> Atom.to_string() |> string(), state}
+    {var |> Atom.to_string() |> string() |> color_doc(:variable, state.inspect_opts), state}
   end
 
   defp quoted_to_algebra({:<<>>, meta, entries}, _context, state) do
@@ -301,6 +300,7 @@ defmodule Code.Formatter do
     end
   end
 
+  # TODO: Remove this clause on Elixir v2.0 once single-quoted charlists are removed
   defp quoted_to_algebra(
          {{:., _, [List, :to_charlist]}, meta, [entries]} = quoted,
          context,
@@ -311,15 +311,18 @@ defmodule Code.Formatter do
         remote_to_algebra(quoted, context, state)
 
       meta[:delimiter] == ~s['''] ->
+        {opener, quotes} = get_charlist_quotes(:heredoc, state)
+
         {doc, state} =
           entries
           |> prepend_heredoc_line()
-          |> list_interpolation_to_algebra(~s['''], state, @single_heredoc, @single_heredoc)
+          |> list_interpolation_to_algebra(quotes, state, opener, quotes)
 
         {force_unfit(doc), state}
 
       true ->
-        list_interpolation_to_algebra(entries, @single_quote, state, @single_quote, @single_quote)
+        {opener, quotes} = get_charlist_quotes({:regular, entries}, state)
+        list_interpolation_to_algebra(entries, quotes, state, opener, quotes)
     end
   end
 
@@ -374,15 +377,23 @@ defmodule Code.Formatter do
     tuple_to_algebra(meta, [left, right], :flex_break, state)
   end
 
+  # (left -> right)
+  defp quoted_to_algebra({:__block__, _, [[{:->, _, _} | _] = clauses]}, _context, state) do
+    paren_fun_to_algebra(clauses, @max_line, @min_line, state)
+  end
+
   defp quoted_to_algebra({:__block__, meta, [list]}, _context, state) when is_list(list) do
     case meta[:delimiter] do
       ~s['''] ->
-        string = list |> List.to_string() |> escape_heredoc(~s['''])
-        {@single_heredoc |> concat(string) |> concat(@single_heredoc) |> force_unfit(), state}
+        {opener, quotes} = get_charlist_quotes(:heredoc, state)
+        string = list |> List.to_string() |> escape_heredoc(quotes)
+        {opener |> concat(string) |> concat(quotes) |> force_unfit(), state}
 
       ~s['] ->
-        string = list |> List.to_string() |> escape_string(@single_quote)
-        {@single_quote |> concat(string) |> concat(@single_quote), state}
+        string = list |> List.to_string()
+        {opener, quotes} = get_charlist_quotes({:regular, [string]}, state)
+        string = escape_string(string, quotes)
+        {opener |> concat(string) |> concat(quotes), state}
 
       _other ->
         list_to_algebra(meta, list, state)
@@ -392,26 +403,36 @@ defmodule Code.Formatter do
   defp quoted_to_algebra({:__block__, meta, [string]}, _context, state) when is_binary(string) do
     if meta[:delimiter] == ~s["""] do
       string = escape_heredoc(string, ~s["""])
-      {@double_heredoc |> concat(string) |> concat(@double_heredoc) |> force_unfit(), state}
+
+      {@double_heredoc
+       |> concat(string)
+       |> concat(@double_heredoc)
+       |> color_doc(:string, state.inspect_opts)
+       |> force_unfit(), state}
     else
       string = escape_string(string, @double_quote)
-      {@double_quote |> concat(string) |> concat(@double_quote), state}
+
+      {@double_quote
+       |> concat(string)
+       |> concat(@double_quote)
+       |> color_doc(:string, state.inspect_opts), state}
     end
   end
 
   defp quoted_to_algebra({:__block__, meta, [atom]}, _context, state) when is_atom(atom) do
-    {atom_to_algebra(atom, meta), state}
+    {atom_to_algebra(atom, meta, state.inspect_opts), state}
   end
 
   defp quoted_to_algebra({:__block__, meta, [integer]}, _context, state)
        when is_integer(integer) do
-    {integer_to_algebra(Keyword.fetch!(meta, :token)), state}
+    {Keyword.fetch!(meta, :token) |> integer_to_algebra(state.inspect_opts), state}
   end
 
   defp quoted_to_algebra({:__block__, meta, [float]}, _context, state) when is_float(float) do
-    {float_to_algebra(Keyword.fetch!(meta, :token)), state}
+    {Keyword.fetch!(meta, :token) |> float_to_algebra(state.inspect_opts), state}
   end
 
+  # (unquote_splicing(...))
   defp quoted_to_algebra(
          {:__block__, _meta, [{:unquote_splicing, meta, [_] = args}]},
          context,
@@ -426,10 +447,10 @@ defmodule Code.Formatter do
   end
 
   defp quoted_to_algebra({:__block__, _meta, []}, _context, state) do
-    {"nil", state}
+    {color_doc("nil", nil, state.inspect_opts), state}
   end
 
-  defp quoted_to_algebra({:__block__, meta, _} = block, _context, state) do
+  defp quoted_to_algebra({:__block__, meta, args} = block, _context, state) when is_list(args) do
     {block, state} = block_to_algebra(block, line(meta), closing_line(meta), state)
     {surround("(", block, ")"), state}
   end
@@ -442,7 +463,8 @@ defmodule Code.Formatter do
         quoted_to_algebra_with_parens_if_operator(head, context, state)
       end
 
-    {Enum.reduce(tail, doc, &concat(&2, "." <> Atom.to_string(&1))), state}
+    {Enum.reduce(tail, doc, &concat(&2, "." <> Atom.to_string(&1)))
+     |> color_doc(:atom, state.inspect_opts), state}
   end
 
   # &1
@@ -465,6 +487,53 @@ defmodule Code.Formatter do
     binary_op_to_algebra(:in, "not in", meta, left, right, context, state)
   end
 
+  # disable migrate_unless within defmacro
+  defp quoted_to_algebra(
+         {atom, _, [{:unless, _, _}, _]} = ast,
+         context,
+         %{migrate_unless: true} = state
+       )
+       when atom in [:defmacro, :defmacrop] do
+    quoted_to_algebra(ast, context, %{state | migrate_unless: false})
+  end
+
+  # rewrite unless as if!
+  defp quoted_to_algebra(
+         {:unless, meta, [condition, block]},
+         context,
+         %{migrate_unless: true} = state
+       ) do
+    quoted_to_algebra({:if, meta, [negate_condition(condition), block]}, context, state)
+  end
+
+  # a |> b() |> unless(...) => a |> b() |> Kernel.!() |> unless(...)
+  defp quoted_to_algebra(
+         {:|>, meta1, [{:|>, _, _} = condition, {:unless, meta2, [block]}]},
+         context,
+         %{migrate_unless: true} = state
+       ) do
+    negated_condition = {:|>, [], [condition, {{:., [], [Kernel, :!]}, [closing: []], []}]}
+
+    quoted_to_algebra(
+      {:|>, meta1, [negated_condition, {:if, meta2, [block]}]},
+      context,
+      state
+    )
+  end
+
+  # condition |> unless(...) => negated(condition) |> unless(...)
+  defp quoted_to_algebra(
+         {:|>, meta1, [condition, {:unless, meta2, [block]}]},
+         context,
+         %{migrate_unless: true} = state
+       ) do
+    quoted_to_algebra(
+      {:|>, meta1, [negate_condition(condition), {:if, meta2, [block]}]},
+      context,
+      state
+    )
+  end
+
   # ..
   defp quoted_to_algebra({:.., _meta, []}, context, state) do
     if context in [:no_parens_arg, :no_parens_one_arg] do
@@ -474,8 +543,13 @@ defmodule Code.Formatter do
     end
   end
 
+  # ...
+  defp quoted_to_algebra({:..., _meta, []}, _context, state) do
+    {"...", state}
+  end
+
   # 1..2//3
-  defp quoted_to_algebra({:"..//", meta, [left, middle, right]}, context, state) do
+  defp quoted_to_algebra({:..//, meta, [left, middle, right]}, context, state) do
     quoted_to_algebra({:"//", meta, [{:.., meta, [left, middle]}, right]}, context, state)
   end
 
@@ -494,11 +568,6 @@ defmodule Code.Formatter do
     remote_to_algebra(quoted, context, state)
   end
 
-  # (left -> right)
-  defp quoted_to_algebra([{:->, _, _} | _] = clauses, _context, state) do
-    type_fun_to_algebra(clauses, @max_line, @min_line, state)
-  end
-
   # [keyword: :list] (inner part)
   # %{:foo => :bar} (inner part)
   defp quoted_to_algebra(list, context, state) when is_list(list) do
@@ -512,19 +581,12 @@ defmodule Code.Formatter do
       if keyword_key?(left_arg) do
         {left, state} =
           case left_arg do
-            # TODO: Remove this clause in v1.16 when we no longer quote operator :..//
-            {:__block__, _, [:"..//"]} ->
-              {string(~S{"..//":}), state}
-
             {:__block__, _, [atom]} when is_atom(atom) ->
-              key =
-                if Macro.classify_atom(atom) in [:identifier, :unquoted] do
-                  IO.iodata_to_binary([Atom.to_string(atom), ?:])
-                else
-                  IO.iodata_to_binary([?", Atom.to_string(atom), ?", ?:])
-                end
+              formatted = Macro.inspect_atom(:key, atom, escape: &escape_atom/2)
 
-              {string(key), state}
+              {formatted
+               |> string()
+               |> color_doc(:atom, state.inspect_opts), state}
 
             {{:., _, [:erlang, :binary_to_atom]}, _, [{:<<>>, _, entries}, :utf8]} ->
               interpolation_to_algebra(entries, @double_quote, state, "\"", "\":")
@@ -548,14 +610,17 @@ defmodule Code.Formatter do
   end
 
   # #PID's and #Ref's may appear on regular AST
+  # Other foreign structures, such as maps and structs,
+  # may appear from Macro.to_string, so we stick a limit,
+  # although they won't be formatted accordingly.
   defp quoted_to_algebra(unknown, _context, state) do
-    {inspect(unknown), state}
+    {inspect(unknown, printable_limit: :infinity), state}
   end
 
   ## Blocks
 
-  defp block_to_algebra([{:->, _, _} | _] = type_fun, min_line, max_line, state) do
-    type_fun_to_algebra(type_fun, min_line, max_line, state)
+  defp block_to_algebra([{:->, _, _} | _] = paren_fun, min_line, max_line, state) do
+    paren_fun_to_algebra(paren_fun, min_line, max_line, state)
   end
 
   defp block_to_algebra({:__block__, _, []}, min_line, max_line, state) do
@@ -619,12 +684,12 @@ defmodule Code.Formatter do
         Atom.to_string(op)
       end
 
-    {concat(op_string, doc), state}
+    {color_doc(op_string, :operator, state.inspect_opts) |> concat(doc), state}
   end
 
   defp maybe_binary_op_to_algebra(fun, meta, args, context, state) do
     with [left, right] <- args,
-         {_, _} <- Code.Identifier.binary_op(fun) do
+         {_, _} <- augmented_binary_op(fun) do
       binary_op_to_algebra(fun, Atom.to_string(fun), meta, left, right, context, state)
     else
       _ -> :error
@@ -651,7 +716,7 @@ defmodule Code.Formatter do
 
   defp binary_op_to_algebra(op, op_string, meta, left_arg, right_arg, context, state, _nesting)
        when op in @right_new_line_before_binary_operators do
-    op_info = Code.Identifier.binary_op(op)
+    op_info = augmented_binary_op(op)
     op_string = op_string <> " "
     left_context = left_op_context(context)
     right_context = right_op_context(context)
@@ -688,7 +753,7 @@ defmodule Code.Formatter do
 
   defp binary_op_to_algebra(op, _, meta, left_arg, right_arg, context, state, _nesting)
        when op in @pipeline_operators do
-    op_info = Code.Identifier.binary_op(op)
+    op_info = augmented_binary_op(op)
     left_context = left_op_context(context)
     right_context = right_op_context(context)
     max_line = line(meta)
@@ -702,7 +767,7 @@ defmodule Code.Formatter do
         {{doc, @empty, 1}, state}
 
       {{op, context}, arg}, _args, state ->
-        op_info = Code.Identifier.binary_op(op)
+        op_info = augmented_binary_op(op)
         op_string = Atom.to_string(op) <> " "
         {doc, state} = binary_operand_to_algebra(arg, context, state, op, op_info, :right, 0)
         {{concat(op_string, doc), @empty, 1}, state}
@@ -712,7 +777,7 @@ defmodule Code.Formatter do
   end
 
   defp binary_op_to_algebra(op, op_string, meta, left_arg, right_arg, context, state, nesting) do
-    op_info = Code.Identifier.binary_op(op)
+    op_info = augmented_binary_op(op)
     left_context = left_op_context(context)
     right_context = right_op_context(context)
 
@@ -725,11 +790,12 @@ defmodule Code.Formatter do
     doc =
       cond do
         op in @no_space_binary_operators ->
-          concat(concat(group(left), op_string), group(right))
+          op_doc = color_doc(op_string, :operator, state.inspect_opts)
+          concat(concat(group(left), op_doc), group(right))
 
         op in @no_newline_binary_operators ->
-          op_string = " " <> op_string <> " "
-          concat(concat(group(left), op_string), group(right))
+          op_doc = color_doc(" " <> op_string <> " ", :operator, state.inspect_opts)
+          concat(concat(group(left), op_doc), group(right))
 
         true ->
           eol? = eol?(meta, state)
@@ -738,8 +804,8 @@ defmodule Code.Formatter do
             op in @next_break_fits_operators and next_break_fits?(right_arg, state) and not eol?
 
           with_next_break_fits(next_break_fits?, right, fn right ->
-            op_string = " " <> op_string
-            right = nest(glue(op_string, group(right)), nesting, :break)
+            op_doc = color_doc(" " <> op_string, :operator, state.inspect_opts)
+            right = nest(glue(op_doc, group(right)), nesting, :break)
             right = if eol?, do: force_unfit(right), else: right
             concat(group(left), group(right))
           end)
@@ -770,19 +836,18 @@ defmodule Code.Formatter do
     {parent_assoc, parent_prec} = parent_info
 
     with {op, meta, [left, right]} <- operand,
-         op_info = Code.Identifier.binary_op(op),
+         op_info = augmented_binary_op(op),
          {_assoc, prec} <- op_info do
       op_string = Atom.to_string(op)
 
       cond do
-        # If the operator has the same precedence as the parent and is on
-        # the correct side, we respect the nesting rule to avoid multiple
-        # nestings. This only applies for left associativity or same operator.
-        parent_prec == prec and parent_assoc == side and (side == :left or op == parent_op) ->
+        # If we have the same operator and it is in the correct side,
+        # we don't add parens unless it is explicitly required.
+        parent_assoc == side and op == parent_op and op not in @required_parens_even_when_parent ->
           binary_op_to_algebra(op, op_string, meta, left, right, context, state, nesting)
 
-        # If the parent requires parens or the precedence is inverted or
-        # it is in the wrong side, then we *need* parenthesis.
+        # If the operator requires parens (most of them do) or we are mixing logical operators
+        # or the precedence is inverted or it is in the wrong side, then we *need* parenthesis.
         (parent_op in @required_parens_on_binary_operands and op not in @no_space_binary_operators) or
           (op in @required_parens_logical_binary_operands and
              parent_op in @required_parens_logical_binary_operands) or parent_prec > prec or
@@ -925,10 +990,10 @@ defmodule Code.Formatter do
   defp capture_to_algebra(arg, context, state) do
     {doc, state} = capture_target_to_algebra(arg, context, state)
 
-    if doc |> format_to_string() |> String.starts_with?("&") do
-      {concat("& ", doc), state}
-    else
-      {concat("&", doc), state}
+    case format_to_string(doc) do
+      <<"&", _::binary>> -> {concat("& ", doc), state}
+      <<int, _::binary>> when int in ?0..?9 -> {concat("& ", doc), state}
+      _ -> {concat("&", doc), state}
     end
   end
 
@@ -939,7 +1004,7 @@ defmodule Code.Formatter do
        )
        when is_atom(fun) and is_integer(arity) do
     {target_doc, state} = remote_target_to_algebra(target, state)
-    fun = Macro.inspect_atom(:remote_call, fun)
+    fun = Macro.inspect_atom(:remote_call, fun, escape: &escape_atom/2)
     {target_doc |> nest(1) |> concat(string(".#{fun}/#{arity}")), state}
   end
 
@@ -984,8 +1049,13 @@ defmodule Code.Formatter do
   defp remote_to_algebra({{:., _, [target, fun]}, meta, args}, context, state)
        when is_atom(fun) do
     {target_doc, state} = remote_target_to_algebra(target, state)
-    fun = Macro.inspect_atom(:remote_call, fun)
-    remote_doc = target_doc |> concat(".") |> concat(string(fun))
+
+    fun_doc =
+      Macro.inspect_atom(:remote_call, fun, escape: &escape_atom/2)
+      |> string()
+      |> color_doc(:call, state.inspect_opts)
+
+    remote_doc = target_doc |> concat(".") |> concat(fun_doc)
 
     if args == [] and not remote_target_is_a_module?(target) and not meta?(meta, :closing) do
       {remote_doc, state}
@@ -1051,6 +1121,7 @@ defmodule Code.Formatter do
       fun
       |> Atom.to_string()
       |> string()
+      |> color_doc(:call, state.inspect_opts)
       |> concat(call_doc)
 
     doc = if wrap_in_parens?, do: wrap_in_parens(doc), else: doc
@@ -1151,6 +1222,7 @@ defmodule Code.Formatter do
               right_doc
               |> nest(2, :break)
               |> concat(break(""))
+              |> concat(")")
               |> group(:inherit)
               |> next_break_fits(:enabled)
 
@@ -1182,7 +1254,7 @@ defmodule Code.Formatter do
           if skip_parens? do
             nest(args_doc, :cursor, :break)
           else
-            nest(args_doc, 2, :break) |> concat(break(""))
+            nest(args_doc, 2, :break) |> concat(break("")) |> concat(")")
           end
 
         {args_doc, next_break_fits?, state}
@@ -1208,7 +1280,6 @@ defmodule Code.Formatter do
           |> concat(break(""))
           |> nest(2, :break)
           |> concat(args_doc)
-          |> concat(")")
           |> concat(extra)
           |> group()
       end
@@ -1329,18 +1400,18 @@ defmodule Code.Formatter do
   ## Sigils
 
   defp maybe_sigil_to_algebra(fun, meta, args, state) do
-    with <<"sigil_", name>> <- Atom.to_string(fun),
+    with <<"sigil_", name::binary>> <- Atom.to_string(fun),
          [{:<<>>, _, entries}, modifiers] when is_list(modifiers) <- args,
          opening_delimiter when not is_nil(opening_delimiter) <- meta[:delimiter] do
-      doc = <<?~, name, opening_delimiter::binary>>
+      doc = <<?~, name::binary, opening_delimiter::binary>>
 
       entries =
-        case state.sigils do
-          %{^name => callback} ->
+        case Map.fetch(state.sigils, String.to_charlist(name)) do
+          {:ok, callback} ->
             metadata = [
               file: state.file,
               line: meta[:line],
-              sigil: List.to_atom([name]),
+              sigil: String.to_atom(name),
               modifiers: modifiers,
               opening_delimiter: opening_delimiter
             ]
@@ -1354,7 +1425,7 @@ defmodule Code.Formatter do
                       "expected sigil callback to return iodata, got: #{inspect(other)}"
             end
 
-          %{} ->
+          :error ->
             entries
         end
 
@@ -1411,7 +1482,9 @@ defmodule Code.Formatter do
 
   defp bitstring_segment_to_algebra({{:"::", _, [segment, spec]}, i}, state, last) do
     {doc, state} = quoted_to_algebra(segment, :parens_arg, state)
-    {spec, state} = bitstring_spec_to_algebra(spec, state)
+
+    {spec, state} =
+      bitstring_spec_to_algebra(spec, state, state.migrate_bitstring_modifiers, :"::")
 
     spec = wrap_in_parens_if_inspected_atom(spec)
     spec = if i == last, do: bitstring_wrap_parens(spec, i, last), else: spec
@@ -1430,31 +1503,42 @@ defmodule Code.Formatter do
     {bitstring_wrap_parens(doc, i, last), state}
   end
 
-  defp bitstring_spec_to_algebra({op, _, [left, right]}, state) when op in [:-, :*] do
-    {left, state} = bitstring_spec_to_algebra(left, state)
-    {right, state} = bitstring_spec_element_to_algebra(right, state)
-    {concat(concat(left, Atom.to_string(op)), right), state}
+  defp bitstring_spec_to_algebra({op, _, [left, right]}, state, normalize_modifiers, paren_op)
+       when op in [:-, :*] do
+    normalize_modifiers = normalize_modifiers && op != :*
+    {left, state} = bitstring_spec_to_algebra(left, state, normalize_modifiers, op)
+    {right, state} = bitstring_spec_element_to_algebra(right, state, normalize_modifiers)
+    doc = concat(concat(left, Atom.to_string(op)), right)
+    doc = if paren_op == :*, do: wrap_in_parens(doc), else: doc
+    {doc, state}
   end
 
-  defp bitstring_spec_to_algebra(spec, state) do
-    bitstring_spec_element_to_algebra(spec, state)
+  defp bitstring_spec_to_algebra(spec, state, normalize_modifiers, _paren_op) do
+    bitstring_spec_element_to_algebra(spec, state, normalize_modifiers)
   end
 
   defp bitstring_spec_element_to_algebra(
          {atom, meta, empty_args},
-         state = %{normalize_bitstring_modifiers: true}
+         state,
+         _normalize_modifiers = true
        )
        when is_atom(atom) and empty_args in [nil, []] do
     empty_args = bitstring_spec_normalize_empty_args(atom)
     quoted_to_algebra_with_parens_if_operator({atom, meta, empty_args}, :parens_arg, state)
   end
 
-  defp bitstring_spec_element_to_algebra(spec_element, state) do
+  defp bitstring_spec_element_to_algebra(spec_element, state, _normalize_modifiers) do
     quoted_to_algebra_with_parens_if_operator(spec_element, :parens_arg, state)
   end
 
-  defp bitstring_spec_normalize_empty_args(atom) when atom in @bitstring_modifiers, do: nil
-  defp bitstring_spec_normalize_empty_args(_atom), do: []
+  defp bitstring_spec_normalize_empty_args(:_), do: nil
+
+  defp bitstring_spec_normalize_empty_args(atom) do
+    case :elixir_bitstring.validate_spec(atom, nil) do
+      :none -> []
+      _ -> nil
+    end
+  end
 
   defp bitstring_wrap_parens(doc, i, last) when i == 0 or i == last do
     string = format_to_string(doc)
@@ -1478,7 +1562,10 @@ defmodule Code.Formatter do
     {args_doc, _join, state} =
       args_to_algebra_with_comments(args, meta, false, :none, join, state, fun)
 
-    {surround("[", args_doc, "]"), state}
+    left_bracket = color_doc("[", :list, state.inspect_opts)
+    right_bracket = color_doc("]", :list, state.inspect_opts)
+
+    {surround(left_bracket, args_doc, right_bracket), state}
   end
 
   defp map_to_algebra(meta, name_doc, [{:|, _, [left, right]}], state) do
@@ -1494,8 +1581,7 @@ defmodule Code.Formatter do
       |> wrap_in_parens_if_binary_operator(left)
       |> glue(concat("| ", nest(right_doc, 2)))
 
-    name_doc = "%" |> concat(name_doc) |> concat("{")
-    {surround(name_doc, args_doc, "}"), state}
+    do_map_to_algebra(name_doc, args_doc, state)
   end
 
   defp map_to_algebra(meta, name_doc, args, state) do
@@ -1505,8 +1591,12 @@ defmodule Code.Formatter do
     {args_doc, _join, state} =
       args_to_algebra_with_comments(args, meta, false, :none, join, state, fun)
 
-    name_doc = "%" |> concat(name_doc) |> concat("{")
-    {surround(name_doc, args_doc, "}"), state}
+    do_map_to_algebra(name_doc, args_doc, state)
+  end
+
+  defp do_map_to_algebra(name_doc, args_doc, state) do
+    name_doc = "%" |> concat(name_doc) |> concat("{") |> color_doc(:map, state.inspect_opts)
+    {surround(name_doc, args_doc, color_doc("}", :map, state.inspect_opts)), state}
   end
 
   defp tuple_to_algebra(meta, args, join, state) do
@@ -1516,23 +1606,25 @@ defmodule Code.Formatter do
     {args_doc, join, state} =
       args_to_algebra_with_comments(args, meta, false, :none, join, state, fun)
 
+    left_bracket = color_doc("{", :tuple, state.inspect_opts)
+    right_bracket = color_doc("}", :tuple, state.inspect_opts)
+
     if join == :flex_break do
-      {"{" |> concat(args_doc) |> nest(1) |> concat("}") |> group(), state}
+      {left_bracket |> concat(args_doc) |> nest(1) |> concat(right_bracket) |> group(), state}
     else
-      {surround("{", args_doc, "}"), state}
+      {surround(left_bracket, args_doc, right_bracket), state}
     end
   end
 
-  defp atom_to_algebra(atom, _) when atom in [nil, true, false] do
-    Atom.to_string(atom)
+  defp atom_to_algebra(atom, _, inspect_opts) when atom in [true, false] do
+    Atom.to_string(atom) |> color_doc(:boolean, inspect_opts)
   end
 
-  # TODO: Remove this clause in v1.16 when we no longer quote operator :..//
-  defp atom_to_algebra(:"..//", _) do
-    string(":\"..//\"")
+  defp atom_to_algebra(nil, _, inspect_opts) do
+    Atom.to_string(nil) |> color_doc(nil, inspect_opts)
   end
 
-  defp atom_to_algebra(:\\, meta) do
+  defp atom_to_algebra(:\\, meta, inspect_opts) do
     # Since we parse strings without unescaping, the atoms
     # :\\ and :"\\" have the same representation, so we need
     # to check the delimiter and handle them accordingly.
@@ -1542,10 +1634,10 @@ defmodule Code.Formatter do
         _ -> ":\\\\"
       end
 
-    string(string)
+    string(string) |> color_doc(:atom, inspect_opts)
   end
 
-  defp atom_to_algebra(atom, _) do
+  defp atom_to_algebra(atom, _, inspect_opts) do
     string = Atom.to_string(atom)
 
     iodata =
@@ -1555,10 +1647,10 @@ defmodule Code.Formatter do
         [?:, ?", String.replace(string, "\"", "\\\""), ?"]
       end
 
-    iodata |> IO.iodata_to_binary() |> string()
+    iodata |> IO.iodata_to_binary() |> string() |> color_doc(:atom, inspect_opts)
   end
 
-  defp integer_to_algebra(text) do
+  defp integer_to_algebra(text, inspect_otps) do
     case text do
       <<?0, ?x, rest::binary>> ->
         "0x" <> String.upcase(rest)
@@ -1572,34 +1664,46 @@ defmodule Code.Formatter do
       decimal ->
         insert_underscores(decimal)
     end
+    |> color_doc(:number, inspect_otps)
   end
 
-  defp float_to_algebra(text) do
+  defp float_to_algebra(text, inspect_otps) do
     [int_part, decimal_part] = :binary.split(text, ".")
     decimal_part = String.downcase(decimal_part)
-    insert_underscores(int_part) <> "." <> decimal_part
+
+    string = insert_underscores(int_part) <> "." <> decimal_part
+    color_doc(string, :number, inspect_otps)
+  end
+
+  defp insert_underscores("-" <> digits) do
+    "-" <> insert_underscores(digits)
   end
 
   defp insert_underscores(digits) do
+    byte_size = byte_size(digits)
+
     cond do
       digits =~ "_" ->
         digits
 
-      byte_size(digits) >= 6 ->
-        digits
-        |> String.to_charlist()
-        |> Enum.reverse()
-        |> Enum.chunk_every(3)
-        |> Enum.intersperse('_')
-        |> List.flatten()
-        |> Enum.reverse()
-        |> List.to_string()
+      byte_size >= 6 ->
+        offset = rem(byte_size, 3)
+        {prefix, rest} = String.split_at(digits, offset)
+        do_insert_underscores(prefix, rest)
 
       true ->
         digits
     end
   end
 
+  defp do_insert_underscores(acc, ""), do: acc
+
+  defp do_insert_underscores("", <<next::binary-3, rest::binary>>),
+    do: do_insert_underscores(next, rest)
+
+  defp do_insert_underscores(acc, <<next::binary-3, rest::binary>>),
+    do: do_insert_underscores(<<acc::binary, "_", next::binary>>, rest)
+
   defp escape_heredoc(string, escape) do
     string = String.replace(string, escape, "\\" <> escape)
     heredoc_to_algebra(["" | String.split(string, "\n")])
@@ -1637,6 +1741,7 @@ defmodule Code.Formatter do
   end
 
   defp heredoc_line(["", _ | _]), do: nest(line(), :reset)
+  defp heredoc_line(["\r", _ | _]), do: nest(line(), :reset)
   defp heredoc_line(_), do: line()
 
   defp args_to_algebra_with_comments(args, meta, skip_parens?, last_arg_mode, join, state, fun) do
@@ -1765,13 +1870,13 @@ defmodule Code.Formatter do
 
   ## Type functions
 
-  # (() -> block)
-  defp type_fun_to_algebra([{:->, meta, [[], body]}] = clauses, _min_line, max_line, state) do
+  # (-> block)
+  defp paren_fun_to_algebra([{:->, meta, [[], body]}] = clauses, _min_line, max_line, state) do
     min_line = line(meta)
     {body_doc, state} = block_to_algebra(body, min_line, max_line, state)
 
     doc =
-      "(() -> "
+      "(-> "
       |> concat(nest(body_doc, :cursor))
       |> concat(")")
       |> maybe_force_clauses(clauses, state)
@@ -1783,7 +1888,7 @@ defmodule Code.Formatter do
   # (x -> y)
   # (x ->
   #    y)
-  defp type_fun_to_algebra([{:->, meta, [args, body]}] = clauses, _min_line, max_line, state) do
+  defp paren_fun_to_algebra([{:->, meta, [args, body]}] = clauses, _min_line, max_line, state) do
     min_line = line(meta)
     {args_doc, state} = clause_args_to_algebra(args, min_line, state)
     {body_doc, state} = block_to_algebra(body, min_line, max_line, state)
@@ -1808,7 +1913,7 @@ defmodule Code.Formatter do
   #   args2 ->
   #     block2
   # )
-  defp type_fun_to_algebra(clauses, min_line, max_line, state) do
+  defp paren_fun_to_algebra(clauses, min_line, max_line, state) do
     {clauses_doc, state} = clauses_to_algebra(clauses, min_line, max_line, state)
     {"(" |> line(clauses_doc) |> nest(2) |> line(")") |> force_unfit(), state}
   end
@@ -1909,6 +2014,14 @@ defmodule Code.Formatter do
   # fn a, b, c when d -> e end
   defp clause_args_to_algebra([{:when, meta, args}], state) do
     {args, right} = split_last(args)
+
+    # If there are any keywords, wrap them in lists
+    args =
+      Enum.map(args, fn
+        [_ | _] = keyword -> {:__block__, [], [keyword]}
+        other -> other
+      end)
+
     left = {{:special, :clause_args}, meta, [args]}
     binary_op_to_algebra(:when, "when", meta, left, right, :no_parens_arg, state)
   end
@@ -1931,16 +2044,30 @@ defmodule Code.Formatter do
         Enum.split_while(comments, fn %{line: line} -> line <= min_line end)
       end)
 
-    {docs, comments?, state} =
-      each_quoted_to_algebra_with_comments(args, acc, max_line, state, false, fun)
+    {reverse_docs, comments?, state} =
+      if state.comments == [] do
+        each_quoted_to_algebra_without_comments(args, acc, state, fun)
+      else
+        each_quoted_to_algebra_with_comments(args, acc, max_line, state, false, fun)
+      end
 
+    docs = merge_algebra_with_comments(Enum.reverse(reverse_docs), @empty)
     {docs, comments?, update_in(state.comments, &(pre_comments ++ &1))}
   end
 
+  defp each_quoted_to_algebra_without_comments([], acc, state, _fun) do
+    {acc, false, state}
+  end
+
+  defp each_quoted_to_algebra_without_comments([arg | args], acc, state, fun) do
+    {doc_triplet, state} = fun.(arg, args, state)
+    acc = [doc_triplet | acc]
+    each_quoted_to_algebra_without_comments(args, acc, state, fun)
+  end
+
   defp each_quoted_to_algebra_with_comments([], acc, max_line, state, comments?, _fun) do
     {acc, comments, comments?} = extract_comments_before(max_line, acc, state.comments, comments?)
-    args_docs = merge_algebra_with_comments(Enum.reverse(acc), @empty)
-    {args_docs, comments?, %{state | comments: comments}}
+    {acc, comments?, %{state | comments: comments}}
   end
 
   defp each_quoted_to_algebra_with_comments([arg | args], acc, max_line, state, comments?, fun) do
@@ -2001,10 +2128,11 @@ defmodule Code.Formatter do
   defp adjust_trailing_newlines(doc_triplet, _, _), do: doc_triplet
 
   defp traverse_line({expr, meta, args}, {min, max}) do
+    # This is a hot path, so use :lists.keyfind/3 instead Keyword.fetch!/2
     acc =
-      case Keyword.fetch(meta, :line) do
-        {:ok, line} -> {min(line, min), max(line, max)}
-        :error -> {min, max}
+      case :lists.keyfind(:line, 1, meta) do
+        {:line, line} -> {min(line, min), max(line, max)}
+        false -> {min, max}
       end
 
     traverse_line(args, traverse_line(expr, acc))
@@ -2123,10 +2251,15 @@ defmodule Code.Formatter do
     unary_operator?(quoted) or binary_operator?(quoted)
   end
 
+  # We convert ..// into two operators for simplicity,
+  # so we need to augment the binary table.
+  defp augmented_binary_op(:"//"), do: {:right, 190}
+  defp augmented_binary_op(op), do: Code.Identifier.binary_op(op)
+
   defp binary_operator?(quoted) do
     case quoted do
       {op, _, [_, _, _]} when op in @multi_binary_operators -> true
-      {op, _, [_, _]} when is_atom(op) -> Code.Identifier.binary_op(op) != :error
+      {op, _, [_, _]} when is_atom(op) -> augmented_binary_op(op) != :error
       _ -> false
     end
   end
@@ -2162,6 +2295,7 @@ defmodule Code.Formatter do
       (not interpolated?(entries) and eol_or_comments?(meta, state))
   end
 
+  # TODO: Remove this clause on Elixir v2.0 once single-quoted charlists are removed
   defp next_break_fits?({{:., _, [List, :to_charlist]}, meta, [[_ | _]]}, _state) do
     meta[:delimiter] == ~s[''']
   end
@@ -2303,6 +2437,10 @@ defmodule Code.Formatter do
     meta[:closing][:line] || @min_line
   end
 
+  defp escape_atom(string, char) do
+    String.replace(string, <<char>>, <<?\\, char>>)
+  end
+
   ## Algebra helpers
 
   # Relying on the inner document is brittle and error prone.
@@ -2346,4 +2484,66 @@ defmodule Code.Formatter do
     {left, [right]} = Enum.split(list, -1)
     {left, right}
   end
+
+  defp get_charlist_quotes(:heredoc, state) do
+    if state.migrate_charlists_as_sigils do
+      {@sigil_c_heredoc, @double_heredoc}
+    else
+      {@single_heredoc, @single_heredoc}
+    end
+  end
+
+  defp get_charlist_quotes({:regular, chunks}, state) do
+    cond do
+      !state.migrate_charlists_as_sigils -> {@single_quote, @single_quote}
+      Enum.any?(chunks, &has_double_quote?/1) -> {@sigil_c_single, @single_quote}
+      true -> {@sigil_c_double, @double_quote}
+    end
+  end
+
+  defp has_double_quote?(chunk) do
+    is_binary(chunk) and chunk =~ @double_quote
+  end
+
+  # Migration rewrites
+
+  @bool_operators [
+    :>,
+    :>=,
+    :<,
+    :<=,
+    :in
+  ]
+  @guards [
+    :is_atom,
+    :is_boolean,
+    :is_nil,
+    :is_number,
+    :is_integer,
+    :is_float,
+    :is_binary,
+    :is_map,
+    :is_struct,
+    :is_non_struct_map,
+    :is_exception,
+    :is_list,
+    :is_tuple,
+    :is_function,
+    :is_reference,
+    :is_pid,
+    :is_port
+  ]
+
+  defp negate_condition(condition) do
+    case condition do
+      {neg, _, [condition]} when neg in [:!, :not] -> condition
+      {op, _, [_, _]} when op in @bool_operators -> {:not, [], [condition]}
+      {guard, _, [_ | _]} when guard in @guards -> {:not, [], [condition]}
+      {:==, meta, [left, right]} -> {:!=, meta, [left, right]}
+      {:===, meta, [left, right]} -> {:!==, meta, [left, right]}
+      {:!=, meta, [left, right]} -> {:==, meta, [left, right]}
+      {:!==, meta, [left, right]} -> {:===, meta, [left, right]}
+      _ -> {:!, [], [condition]}
+    end
+  end
 end
