@@ -15,24 +15,46 @@ defmodule DynamicSupervisor do
   A dynamic supervisor is started with no children and often a name:
 
       children = [
-        {DynamicSupervisor, name: MyApp.DynamicSupervisor}
+        {DynamicSupervisor, name: MyApp.DynamicSupervisor, strategy: :one_for_one}
       ]
 
       Supervisor.start_link(children, strategy: :one_for_one)
 
   The options given in the child specification are documented in `start_link/1`.
 
-  Once the dynamic supervisor is running, we can start children
-  with `start_child/2`, which receives a child specification:
+  Once the dynamic supervisor is running, we can use it to start children
+  on demand. Given this sample `GenServer`:
 
-      {:ok, agent1} = DynamicSupervisor.start_child(MyApp.DynamicSupervisor, {Agent, fn -> %{} end})
-      Agent.update(agent1, &Map.put(&1, :key, "value"))
-      Agent.get(agent1, & &1)
-      #=> %{key: "value"}
+      defmodule Counter do
+        use GenServer
 
-      {:ok, agent2} = DynamicSupervisor.start_child(MyApp.DynamicSupervisor, {Agent, fn -> %{} end})
-      Agent.get(agent2, & &1)
-      #=> %{}
+        def start_link(initial) do
+          GenServer.start_link(__MODULE__, initial)
+        end
+
+        def inc(pid) do
+          GenServer.call(pid, :inc)
+        end
+
+        def init(initial) do
+          {:ok, initial}
+        end
+
+        def handle_call(:inc, _, count) do
+          {:reply, count, count + 1}
+        end
+      end
+
+  We can use `start_child/2` with a child specification to start a `Counter`
+  server:
+
+      {:ok, counter1} = DynamicSupervisor.start_child(MyApp.DynamicSupervisor, {Counter, 0})
+      Counter.inc(counter1)
+      #=> 0
+
+      {:ok, counter2} = DynamicSupervisor.start_child(MyApp.DynamicSupervisor, {Counter, 10})
+      Counter.inc(counter2)
+      #=> 10
 
       DynamicSupervisor.count_children(MyApp.DynamicSupervisor)
       #=> %{active: 2, specs: 2, supervisors: 0, workers: 2}
@@ -53,7 +75,7 @@ defmodule DynamicSupervisor do
 
   and:
 
-      DynamicSupervisor.start_child(MyApp.DynamicSupervisor, {Agent, fn -> %{} end})
+      DynamicSupervisor.start_child(MyApp.DynamicSupervisor, {Counter, 0})
 
   You can do this:
 
@@ -67,7 +89,7 @@ defmodule DynamicSupervisor do
 
       DynamicSupervisor.start_child(
         {:via, PartitionSupervisor, {MyApp.DynamicSupervisors, self()}},
-        {Agent, fn -> %{} end}
+        {Counter, 0}
       )
 
   In the code above, we start a partition supervisor that will by default
@@ -101,6 +123,12 @@ defmodule DynamicSupervisor do
   `use DynamicSupervisor` will be attached to the generated `child_spec/1`
   function.
 
+  > #### `use DynamicSupervisor` {: .info}
+  >
+  > When you `use DynamicSupervisor`, the `DynamicSupervisor` module will
+  > set `@behaviour DynamicSupervisor` and define a `child_spec/1`
+  > function, so your module can be used as a child in a supervision tree.
+
   ## Name registration
 
   A supervisor is bound to the same name registration rules as a `GenServer`.
@@ -187,10 +215,7 @@ defmodule DynamicSupervisor do
           extra_arguments: [term()]
         }
 
-  @typedoc "Options given to `start_link` functions"
-  @type option :: GenServer.option()
-
-  @typedoc "Options given to `start_link` and `init/1` functions"
+  @typedoc "Options given to `start_link/1` and `init/1` functions"
   @type init_option ::
           {:strategy, strategy()}
           | {:max_restarts, non_neg_integer()}
@@ -225,12 +250,14 @@ defmodule DynamicSupervisor do
   @doc """
   Returns a specification to start a dynamic supervisor under a supervisor.
 
-  See `Supervisor`.
+  It accepts the same options as `start_link/1`.
+
+  See `Supervisor` for more information about child specifications.
   """
   @doc since: "1.6.1"
-  def child_spec(opts) when is_list(opts) do
+  def child_spec(options) when is_list(options) do
     id =
-      case Keyword.get(opts, :name, DynamicSupervisor) do
+      case Keyword.get(options, :name, DynamicSupervisor) do
         name when is_atom(name) -> name
         {:global, name} -> name
         {:via, _module, name} -> name
@@ -238,7 +265,7 @@ defmodule DynamicSupervisor do
 
     %{
       id: id,
-      start: {DynamicSupervisor, :start_link, [opts]},
+      start: {DynamicSupervisor, :start_link, [options]},
       type: :supervisor
     }
   end
@@ -247,7 +274,7 @@ defmodule DynamicSupervisor do
   defmacro __using__(opts) do
     quote location: :keep, bind_quoted: [opts: opts] do
       @behaviour DynamicSupervisor
-      unless Module.has_attribute?(__MODULE__, :doc) do
+      if not Module.has_attribute?(__MODULE__, :doc) do
         @doc """
         Returns a specification to start this module under a supervisor.
 
@@ -315,9 +342,11 @@ defmodule DynamicSupervisor do
       specified in the child spec given to `start_child/2`. Defaults to
       an empty list.
 
+    * Any of the standard [GenServer options](`t:GenServer.option/0`)
+
   """
   @doc since: "1.6.0"
-  @spec start_link([option | init_option]) :: Supervisor.on_start()
+  @spec start_link([init_option | GenServer.option()]) :: Supervisor.on_start()
   def start_link(options) when is_list(options) do
     keys = [:extra_arguments, :max_children, :max_seconds, :max_restarts, :strategy]
     {sup_opts, start_opts} = Keyword.split(options, keys)
@@ -351,9 +380,15 @@ defmodule DynamicSupervisor do
   Note that a supervisor started with this function is linked to the parent
   process and exits not only on crashes but also if the parent process exits
   with `:normal` reason.
+
+  ## Options
+
+  This function accepts any regular [`GenServer` options](`t:GenServer.option/0`).
+  Options specific to `DynamicSupervisor` must be returned from the `c:init/1`
+  callback.
   """
   @doc since: "1.6.0"
-  @spec start_link(module, term, [option]) :: Supervisor.on_start()
+  @spec start_link(module, term, [GenServer.option()]) :: Supervisor.on_start()
   def start_link(module, init_arg, opts \\ []) do
     GenServer.start_link(__MODULE__, {module, init_arg, opts[:name]}, opts)
   end
@@ -411,6 +446,7 @@ defmodule DynamicSupervisor do
     restart = Map.get(child, :restart, :permanent)
     type = Map.get(child, :type, :worker)
     modules = Map.get(child, :modules, [mod])
+    significant = Map.get(child, :significant, false)
 
     shutdown =
       case type do
@@ -418,23 +454,24 @@ defmodule DynamicSupervisor do
         :supervisor -> Map.get(child, :shutdown, :infinity)
       end
 
-    validate_child(start, restart, shutdown, type, modules)
+    validate_child(start, restart, shutdown, type, modules, significant)
   end
 
   defp validate_child({_, start, restart, shutdown, type, modules}) do
-    validate_child(start, restart, shutdown, type, modules)
+    validate_child(start, restart, shutdown, type, modules, false)
   end
 
   defp validate_child(other) do
     {:invalid_child_spec, other}
   end
 
-  defp validate_child(start, restart, shutdown, type, modules) do
+  defp validate_child(start, restart, shutdown, type, modules, significant) do
     with :ok <- validate_start(start),
          :ok <- validate_restart(restart),
          :ok <- validate_shutdown(shutdown),
          :ok <- validate_type(type),
-         :ok <- validate_modules(modules) do
+         :ok <- validate_modules(modules),
+         :ok <- validate_significant(significant) do
       {:ok, {start, restart, shutdown, type, modules}}
     end
   end
@@ -448,10 +485,13 @@ defmodule DynamicSupervisor do
   defp validate_restart(restart) when restart in [:permanent, :temporary, :transient], do: :ok
   defp validate_restart(restart), do: {:invalid_restart_type, restart}
 
-  defp validate_shutdown(shutdown) when is_integer(shutdown) and shutdown > 0, do: :ok
+  defp validate_shutdown(shutdown) when is_integer(shutdown) and shutdown >= 0, do: :ok
   defp validate_shutdown(shutdown) when shutdown in [:infinity, :brutal_kill], do: :ok
   defp validate_shutdown(shutdown), do: {:invalid_shutdown, shutdown}
 
+  defp validate_significant(false), do: :ok
+  defp validate_significant(significant), do: {:invalid_significant, significant}
+
   defp validate_modules(:dynamic), do: :ok
 
   defp validate_modules(mods) do
@@ -621,12 +661,14 @@ defmodule DynamicSupervisor do
     max_restarts = Map.get(flags, :intensity, 1)
     max_seconds = Map.get(flags, :period, 5)
     strategy = Map.get(flags, :strategy, :one_for_one)
+    auto_shutdown = Map.get(flags, :auto_shutdown, :never)
 
     with :ok <- validate_strategy(strategy),
          :ok <- validate_restarts(max_restarts),
          :ok <- validate_seconds(max_seconds),
          :ok <- validate_dynamic(max_children),
-         :ok <- validate_extra_arguments(extra_arguments) do
+         :ok <- validate_extra_arguments(extra_arguments),
+         :ok <- validate_auto_shutdown(auto_shutdown) do
       {:ok,
        %{
          state
@@ -655,6 +697,11 @@ defmodule DynamicSupervisor do
   defp validate_extra_arguments(list) when is_list(list), do: :ok
   defp validate_extra_arguments(extra), do: {:error, {:invalid_extra_arguments, extra}}
 
+  defp validate_auto_shutdown(auto_shutdown) when auto_shutdown in [:never], do: :ok
+
+  defp validate_auto_shutdown(auto_shutdown),
+    do: {:error, {:invalid_auto_shutdown, auto_shutdown}}
+
   @impl true
   def handle_call(:which_children, _from, state) do
     %{children: children} = state
@@ -709,10 +756,18 @@ defmodule DynamicSupervisor do
 
   def handle_call({:start_task, args, restart, shutdown}, from, state) do
     {init_restart, init_shutdown} = Process.get(Task.Supervisor)
-    restart = restart || init_restart
-    shutdown = shutdown || init_shutdown
-    child = {{Task.Supervised, :start_link, args}, restart, shutdown, :worker, [Task.Supervised]}
-    handle_call({:start_child, child}, from, state)
+
+    if restart == nil and init_restart != :temporary do
+      {:reply, {:restart, init_restart}, state}
+    else
+      restart = restart || init_restart
+      shutdown = shutdown || init_shutdown
+
+      child =
+        {{Task.Supervised, :start_link, args}, restart, shutdown, :worker, [Task.Supervised]}
+
+      handle_call({:start_child, child}, from, state)
+    end
   end
 
   def handle_call({:start_child, child}, _from, state) do
@@ -1086,15 +1141,6 @@ defmodule DynamicSupervisor do
     ]
   end
 
-  @impl true
-  def format_status(:terminate, [_pdict, state]) do
-    state
-  end
-
-  def format_status(_, [_pdict, %{mod: mod} = state]) do
-    [data: [{~c"State", state}], supervisor: [{~c"Callback", mod}]]
-  end
-
   ## Helpers
 
   @compile {:inline, call: 2}
@@ -1108,6 +1154,6 @@ defmodule DynamicSupervisor do
         label: {__MODULE__, :unexpected_msg},
         report: %{msg: msg}
       }) do
-    {'DynamicSupervisor received unexpected message: ~p~n', [msg]}
+    {~c"DynamicSupervisor received unexpected message: ~p~n", [msg]}
   end
 end
