@@ -4,9 +4,8 @@ defmodule Inspect.Opts do
 
   The following fields are available:
 
-    * `:base` - prints integers as `:binary`, `:octal`, `:decimal`, or `:hex`,
-      defaults to `:decimal`. When inspecting binaries any `:base` other than
-      `:decimal` implies `binaries: :as_binaries`.
+    * `:base` - prints integers and binaries as `:binary`, `:octal`, `:decimal`,
+      or `:hex`. Defaults to `:decimal`.
 
     * `:binaries` - when `:as_binaries` all binaries will be printed in bit
       syntax.
@@ -14,9 +13,9 @@ defmodule Inspect.Opts do
       When `:as_strings` all binaries will be printed as strings, non-printable
       bytes will be escaped.
 
-      When the default `:infer`, the binary will be printed as a string if it
-      is printable, otherwise in bit syntax. See `String.printable?/1` to learn
-      when a string is printable.
+      When the default `:infer`, the binary will be printed as a string if `:base`
+      is `:decimal` and if it is printable, otherwise in bit syntax. See
+      `String.printable?/1` to learn when a string is printable.
 
     * `:charlists` - when `:as_charlists` all lists will be printed as charlists,
       non-printable elements will be escaped.
@@ -31,6 +30,12 @@ defmodule Inspect.Opts do
       options. Useful when implementing the `Inspect` protocol for nested structs
       to pass the custom options through.
 
+      It supports some pre-defined keys:
+
+      - `:sort_maps` (since v1.14.4) - if set to `true`, sorts key-value pairs
+        in maps. This can be helpful to make map inspection deterministic for
+        testing, given maps key order is random.
+
     * `:inspect_fun` (since v1.9.0) - a function to build algebra documents.
       Defaults to `Inspect.Opts.default_inspect_fun/0`.
 
@@ -62,7 +67,9 @@ defmodule Inspect.Opts do
       colorized. The keys are types and the values are the colors to use for
       each type (for example, `[number: :red, atom: :blue]`). Types can include
       `:atom`, `:binary`, `:boolean`, `:list`, `:map`, `:number`, `:regex`,
-      `:string`, and `:tuple`. Custom data types may provide their own options.
+      `:string`, `:tuple`, or some types to represent AST like `:variable`,
+      `:call`, and `:operator`.
+      Custom data types may provide their own options.
       Colors can be any `t:IO.ANSI.ansidata/0` as accepted by `IO.ANSI.format/1`.
       A default list of colors can be retrieved from `IO.ANSI.syntax_colors/0`.
 
@@ -136,9 +143,8 @@ defmodule Inspect.Opts do
   function as this must be controlled by applications. Libraries
   should instead define their own structs with custom inspect
   implementations. If a library must change the default inspect
-  function, then it is best to define to ask users of your library
-  to explicitly call `default_inspect_fun/1` with your function of
-  choice.
+  function, then it is best to ask users of your library to explicitly
+  call `default_inspect_fun/1` with your function of choice.
 
   The default is `Inspect.inspect/2`.
 
@@ -224,13 +230,13 @@ defmodule Inspect.Algebra do
 
   This implementation provides two types of breaks: `:strict` and `:flex`.
   When a group does not fit, all strict breaks are treated as newlines.
-  Flex breaks however are re-evaluated on every occurrence and may still
+  Flex breaks, however, are re-evaluated on every occurrence and may still
   be rendered flat. See `break/1` and `flex_break/1` for more information.
 
   This implementation also adds `force_unfit/1` and `next_break_fits/2` which
   give more control over the document fitting.
 
-    [0]: http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.34.2200
+    [0]: https://lindig.github.io/papers/strictly-pretty-2000.pdf
 
   """
 
@@ -327,6 +333,7 @@ defmodule Inspect.Algebra do
   defguardp is_width(limit) when limit == :infinity or (is_integer(limit) and limit >= 0)
 
   # Elixir + Inspect.Opts conveniences
+  # These have the _doc suffix.
 
   @doc """
   Converts an Elixir term to an algebra document
@@ -432,7 +439,7 @@ defmodule Inspect.Algebra do
 
   """
   @doc since: "1.6.0"
-  @spec container_doc(t, [any], t, Inspect.Opts.t(), (term, Inspect.Opts.t() -> t), keyword()) ::
+  @spec container_doc(t, [term], t, Inspect.Opts.t(), (term, Inspect.Opts.t() -> t), keyword()) ::
           t
   def container_doc(left, collection, right, inspect_opts, fun, opts \\ [])
       when is_doc(left) and is_list(collection) and is_doc(right) and is_function(fun, 2) and
@@ -449,7 +456,7 @@ defmodule Inspect.Algebra do
           container_each(collection, inspect_opts.limit, inspect_opts, fun, [], break == :maybe)
 
         flex? = simple? or break == :flex
-        docs = fold_doc(docs, &join(&1, &2, flex?, separator))
+        docs = fold(docs, &join(&1, &2, flex?, separator))
 
         case flex? do
           true -> group(concat(concat(left, nest(docs, 1)), right))
@@ -519,6 +526,32 @@ defmodule Inspect.Algebra do
     container_doc(left, docs, right, inspect, fun, separator: separator)
   end
 
+  # TODO: Deprecate me on Elixir v1.23
+  @doc deprecated: "Use color_doc/3 instead"
+  def color(doc, key, opts) do
+    color_doc(doc, key, opts)
+  end
+
+  @doc ~S"""
+  Colors a document if the `color_key` has a color in the options.
+  """
+  @doc since: "1.18.0"
+  @spec color_doc(t, Inspect.Opts.color_key(), Inspect.Opts.t()) :: t
+  def color_doc(doc, color_key, %Inspect.Opts{syntax_colors: syntax_colors}) when is_doc(doc) do
+    if precolor = Keyword.get(syntax_colors, color_key) do
+      postcolor = Keyword.get(syntax_colors, :reset, :reset)
+      concat(doc_color(doc, ansi(precolor)), doc_color(empty(), ansi(postcolor)))
+    else
+      doc
+    end
+  end
+
+  defp ansi(color) do
+    color
+    |> IO.ANSI.format_fragment(true)
+    |> IO.iodata_to_binary()
+  end
+
   # Algebra API
 
   @doc """
@@ -582,6 +615,21 @@ defmodule Inspect.Algebra do
     doc_cons(doc1, doc2)
   end
 
+  @doc ~S"""
+  Disable any rendering limit while rendering the given document.
+
+  ## Examples
+
+      iex> doc = Inspect.Algebra.glue("hello", "world") |> Inspect.Algebra.group()
+      iex> Inspect.Algebra.format(doc, 10)
+      ["hello", "\n", "world"]
+      iex> doc = Inspect.Algebra.no_limit(doc)
+      iex> Inspect.Algebra.format(doc, 10)
+      ["hello", " ", "world"]
+
+  """
+  @doc since: "1.14.0"
+  @spec no_limit(t) :: t
   def no_limit(doc) do
     doc_limit(doc, :infinity)
   end
@@ -598,21 +646,16 @@ defmodule Inspect.Algebra do
   """
   @spec concat([t]) :: t
   def concat(docs) when is_list(docs) do
-    fold_doc(docs, &concat(&1, &2))
+    fold(docs, &concat(&1, &2))
   end
 
   @doc ~S"""
-  Colors a document if the `color_key` has a color in the options.
+  Colors a document with the given color (preceding the document itself).
   """
-  @doc since: "1.4.0"
-  @spec color(t, Inspect.Opts.color_key(), Inspect.Opts.t()) :: t
-  def color(doc, color_key, %Inspect.Opts{syntax_colors: syntax_colors}) when is_doc(doc) do
-    if precolor = Keyword.get(syntax_colors, color_key) do
-      postcolor = Keyword.get(syntax_colors, :reset, :reset)
-      concat(doc_color(doc, precolor), doc_color(empty(), postcolor))
-    else
-      doc
-    end
+  @doc since: "1.18.0"
+  @spec color(t, binary) :: t
+  def color(doc, color) when is_doc(doc) and is_binary(color) do
+    doc_color(doc, color)
   end
 
   @doc ~S"""
@@ -912,6 +955,10 @@ defmodule Inspect.Algebra do
   @spec line(t, t) :: t
   def line(doc1, doc2), do: concat(doc1, concat(line(), doc2))
 
+  # TODO: Deprecate me on Elixir v1.23
+  @doc deprecated: "Use fold/2 instead"
+  def fold_doc(docs, folder_fun), do: fold(docs, folder_fun)
+
   @doc ~S"""
   Folds a list of documents into a document using the given folder function.
 
@@ -923,21 +970,22 @@ defmodule Inspect.Algebra do
 
       iex> docs = ["A", "B", "C"]
       iex> docs =
-      ...>   Inspect.Algebra.fold_doc(docs, fn doc, acc ->
+      ...>   Inspect.Algebra.fold(docs, fn doc, acc ->
       ...>     Inspect.Algebra.concat([doc, "!", acc])
       ...>   end)
       iex> Inspect.Algebra.format(docs, 80)
       ["A", "!", "B", "!", "C"]
 
   """
-  @spec fold_doc([t], (t, t -> t)) :: t
-  def fold_doc(docs, folder_fun)
+  @doc since: "1.18.0"
+  @spec fold([t], (t, t -> t)) :: t
+  def fold(docs, folder_fun)
 
-  def fold_doc([], _folder_fun), do: empty()
-  def fold_doc([doc], _folder_fun), do: doc
+  def fold([], _folder_fun), do: empty()
+  def fold([doc], _folder_fun), do: doc
 
-  def fold_doc([doc | docs], folder_fun) when is_function(folder_fun, 2),
-    do: folder_fun.(doc, fold_doc(docs, folder_fun))
+  def fold([doc | docs], folder_fun) when is_function(folder_fun, 2),
+    do: folder_fun.(doc, fold(docs, folder_fun))
 
   @doc ~S"""
   Formats a given document for a given width.
@@ -967,20 +1015,17 @@ defmodule Inspect.Algebra do
   #
   #   * flat - represents a document with breaks as flats (a break may fit, as it may break)
   #   * break - represents a document with breaks as breaks (a break always fits, since it breaks)
-  #
-  # The following modes are exclusive to fitting:
-  #
   #   * flat_no_break - represents a document with breaks as flat not allowed to enter in break mode
   #   * break_no_flat - represents a document with breaks as breaks not allowed to enter in flat mode
   #
   @typep mode :: :flat | :flat_no_break | :break | :break_no_flat
 
-  @spec fits?(
+  @spec fits(
           width :: non_neg_integer() | :infinity,
           column :: non_neg_integer(),
           break? :: boolean(),
           entries
-        ) :: boolean()
+        ) :: :fit | :no_fit | :break_next
         when entries:
                maybe_improper_list({integer(), mode(), t()}, {:tail, boolean(), entries} | [])
 
@@ -989,63 +1034,68 @@ defmodule Inspect.Algebra do
   #
   # In case we have groups and the group fits, we need to consider the group
   # parent without the child breaks, hence {:tail, b?, t} below.
-  defp fits?(w, k, b?, _) when k > w and b?, do: false
-  defp fits?(_, _, _, []), do: true
-  defp fits?(w, k, _, {:tail, b?, t}), do: fits?(w, k, b?, t)
+  defp fits(w, k, b?, _) when k > w and b?, do: :no_fit
+  defp fits(_, _, _, []), do: :fit
+  defp fits(w, k, _, {:tail, b?, t}), do: fits(w, k, b?, t)
 
   ## Flat no break
 
-  defp fits?(w, k, b?, [{i, _, doc_fits(x, :disabled)} | t]),
-    do: fits?(w, k, b?, [{i, :flat_no_break, x} | t])
+  defp fits(w, k, b?, [{i, _, doc_fits(x, :disabled)} | t]),
+    do: fits(w, k, b?, [{i, :flat_no_break, x} | t])
 
-  defp fits?(w, k, b?, [{i, :flat_no_break, doc_fits(x, _)} | t]),
-    do: fits?(w, k, b?, [{i, :flat_no_break, x} | t])
+  defp fits(w, k, b?, [{i, :flat_no_break, doc_fits(x, _)} | t]),
+    do: fits(w, k, b?, [{i, :flat_no_break, x} | t])
 
   ## Breaks no flat
 
-  defp fits?(w, k, b?, [{i, _, doc_fits(x, :enabled)} | t]),
-    do: fits?(w, k, b?, [{i, :break_no_flat, x} | t])
+  defp fits(w, k, b?, [{i, _, doc_fits(x, :enabled)} | t]),
+    do: fits(w, k, b?, [{i, :break_no_flat, x} | t])
 
-  defp fits?(w, k, b?, [{i, :break_no_flat, doc_force(x)} | t]),
-    do: fits?(w, k, b?, [{i, :break_no_flat, x} | t])
+  defp fits(w, k, b?, [{i, :break_no_flat, doc_force(x)} | t]),
+    do: fits(w, k, b?, [{i, :break_no_flat, x} | t])
 
-  defp fits?(_, _, _, [{_, :break_no_flat, doc_break(_, _)} | _]), do: true
-  defp fits?(_, _, _, [{_, :break_no_flat, :doc_line} | _]), do: true
+  defp fits(w, k, b?, [{i, :break_no_flat, x} | t])
+       when x == :doc_line or (is_tuple(x) and elem(x, 0) == :doc_break) do
+    case fits(w, k, b?, [{i, :flat, x} | t]) do
+      :no_fit -> :break_next
+      fits -> fits
+    end
+  end
 
   ## Breaks
 
-  defp fits?(_, _, _, [{_, :break, doc_break(_, _)} | _]), do: true
-  defp fits?(_, _, _, [{_, :break, :doc_line} | _]), do: true
+  defp fits(_, _, _, [{_, :break, doc_break(_, _)} | _]), do: :fit
+  defp fits(_, _, _, [{_, :break, :doc_line} | _]), do: :fit
 
-  defp fits?(w, k, b?, [{i, :break, doc_group(x, _)} | t]),
-    do: fits?(w, k, b?, [{i, :flat, x} | {:tail, b?, t}])
+  defp fits(w, k, b?, [{i, :break, doc_group(x, _)} | t]),
+    do: fits(w, k, b?, [{i, :flat, x} | {:tail, b?, t}])
 
   ## Catch all
 
-  defp fits?(w, _, _, [{i, _, :doc_line} | t]), do: fits?(w, i, false, t)
-  defp fits?(w, k, b?, [{_, _, :doc_nil} | t]), do: fits?(w, k, b?, t)
-  defp fits?(w, _, b?, [{i, _, doc_collapse(_)} | t]), do: fits?(w, i, b?, t)
-  defp fits?(w, k, b?, [{i, m, doc_color(x, _)} | t]), do: fits?(w, k, b?, [{i, m, x} | t])
-  defp fits?(w, k, b?, [{_, _, doc_string(_, l)} | t]), do: fits?(w, k + l, b?, t)
-  defp fits?(w, k, b?, [{_, _, s} | t]) when is_binary(s), do: fits?(w, k + byte_size(s), b?, t)
-  defp fits?(_, _, _, [{_, _, doc_force(_)} | _]), do: false
-  defp fits?(w, k, _, [{_, _, doc_break(s, _)} | t]), do: fits?(w, k + byte_size(s), true, t)
-  defp fits?(w, k, b?, [{i, m, doc_nest(x, _, :break)} | t]), do: fits?(w, k, b?, [{i, m, x} | t])
+  defp fits(w, _, _, [{i, _, :doc_line} | t]), do: fits(w, i, false, t)
+  defp fits(w, k, b?, [{_, _, :doc_nil} | t]), do: fits(w, k, b?, t)
+  defp fits(w, _, b?, [{i, _, doc_collapse(_)} | t]), do: fits(w, i, b?, t)
+  defp fits(w, k, b?, [{i, m, doc_color(x, _)} | t]), do: fits(w, k, b?, [{i, m, x} | t])
+  defp fits(w, k, b?, [{_, _, doc_string(_, l)} | t]), do: fits(w, k + l, b?, t)
+  defp fits(w, k, b?, [{_, _, s} | t]) when is_binary(s), do: fits(w, k + byte_size(s), b?, t)
+  defp fits(_, _, _, [{_, _, doc_force(_)} | _]), do: :no_fit
+  defp fits(w, k, _, [{_, _, doc_break(s, _)} | t]), do: fits(w, k + byte_size(s), true, t)
+  defp fits(w, k, b?, [{i, m, doc_nest(x, _, :break)} | t]), do: fits(w, k, b?, [{i, m, x} | t])
 
-  defp fits?(w, k, b?, [{i, m, doc_nest(x, j, _)} | t]),
-    do: fits?(w, k, b?, [{apply_nesting(i, k, j), m, x} | t])
+  defp fits(w, k, b?, [{i, m, doc_nest(x, j, _)} | t]),
+    do: fits(w, k, b?, [{apply_nesting(i, k, j), m, x} | t])
 
-  defp fits?(w, k, b?, [{i, m, doc_cons(x, y)} | t]),
-    do: fits?(w, k, b?, [{i, m, x}, {i, m, y} | t])
+  defp fits(w, k, b?, [{i, m, doc_cons(x, y)} | t]),
+    do: fits(w, k, b?, [{i, m, x}, {i, m, y} | t])
 
-  defp fits?(w, k, b?, [{i, m, doc_group(x, _)} | t]),
-    do: fits?(w, k, b?, [{i, m, x} | {:tail, b?, t}])
+  defp fits(w, k, b?, [{i, m, doc_group(x, _)} | t]),
+    do: fits(w, k, b?, [{i, m, x} | {:tail, b?, t}])
 
-  defp fits?(w, k, b?, [{i, m, doc_limit(x, :infinity)} | t]) when w != :infinity,
-    do: fits?(:infinity, k, b?, [{i, :flat, x}, {i, m, doc_limit(empty(), w)} | t])
+  defp fits(w, k, b?, [{i, m, doc_limit(x, :infinity)} | t]) when w != :infinity,
+    do: fits(:infinity, k, b?, [{i, :flat, x}, {i, m, doc_limit(empty(), w)} | t])
 
-  defp fits?(_w, k, b?, [{i, m, doc_limit(x, w)} | t]),
-    do: fits?(w, k, b?, [{i, m, x} | t])
+  defp fits(_w, k, b?, [{i, m, doc_limit(x, w)} | t]),
+    do: fits(w, k, b?, [{i, m, x} | t])
 
   @spec format(
           width :: non_neg_integer() | :infinity,
@@ -1056,10 +1106,14 @@ defmodule Inspect.Algebra do
   defp format(w, k, [{_, _, :doc_nil} | t]), do: format(w, k, t)
   defp format(w, _, [{i, _, :doc_line} | t]), do: [indent(i) | format(w, i, t)]
   defp format(w, k, [{i, m, doc_cons(x, y)} | t]), do: format(w, k, [{i, m, x}, {i, m, y} | t])
-  defp format(w, k, [{i, m, doc_color(x, c)} | t]), do: [ansi(c) | format(w, k, [{i, m, x} | t])]
+  defp format(w, k, [{i, m, doc_color(x, c)} | t]), do: [c | format(w, k, [{i, m, x} | t])]
   defp format(w, k, [{_, _, doc_string(s, l)} | t]), do: [s | format(w, k + l, t)]
   defp format(w, k, [{_, _, s} | t]) when is_binary(s), do: [s | format(w, k + byte_size(s), t)]
   defp format(w, k, [{i, m, doc_force(x)} | t]), do: format(w, k, [{i, m, x} | t])
+
+  defp format(w, k, [{i, :flat_no_break, doc_fits(x, :enabled)} | t]),
+    do: format(w, k, [{i, :break_no_flat, x} | t])
+
   defp format(w, k, [{i, m, doc_fits(x, _)} | t]), do: format(w, k, [{i, m, x} | t])
   defp format(w, _, [{i, _, doc_collapse(max)} | t]), do: collapse(format(w, i, t), max, 0, i)
 
@@ -1067,7 +1121,7 @@ defmodule Inspect.Algebra do
   defp format(w, k, [{i, m, doc_break(s, :flex)} | t]) do
     k = k + byte_size(s)
 
-    if w == :infinity or m == :flat or fits?(w, k, true, t) do
+    if w == :infinity or m == :flat or fits(w, k, true, t) != :no_fit do
       [s | format(w, k, t)]
     else
       [indent(i) | format(w, i, t)]
@@ -1097,11 +1151,17 @@ defmodule Inspect.Algebra do
     format(w, k, [{i, :break, x} | t])
   end
 
+  defp format(w, k, [{i, :break_no_flat, doc_group(x, _)} | t]) do
+    format(w, k, [{i, :break, x} | t])
+  end
+
   defp format(w, k, [{i, _, doc_group(x, _)} | t]) do
-    if w == :infinity or fits?(w, k, false, [{i, :flat, x}]) do
-      format(w, k, [{i, :flat, x} | t])
-    else
-      format(w, k, [{i, :break, x} | t])
+    fits = if w == :infinity, do: :fit, else: fits(w, k, false, [{i, :flat, x}])
+
+    case fits do
+      :fit -> format(w, k, [{i, :flat, x} | t])
+      :no_fit -> format(w, k, [{i, :break, x} | t])
+      :break_next -> format(w, k, [{i, :flat_no_break, x} | t])
     end
   end
 
@@ -1130,10 +1190,6 @@ defmodule Inspect.Algebra do
   defp apply_nesting(_, _, :reset), do: 0
   defp apply_nesting(i, _, j), do: i + j
 
-  defp ansi(color) do
-    IO.ANSI.format_fragment(color, true)
-  end
-
   defp indent(0), do: @newline
   defp indent(i), do: @newline <> :binary.copy(" ", i)
 end
