@@ -1,11 +1,9 @@
 defmodule IO.StreamError do
-  defexception [:reason, :message]
+  defexception [:reason]
 
   @impl true
-  def exception(opts) do
-    reason = opts[:reason]
-    formatted = IO.iodata_to_binary(:file.format_error(reason))
-    %IO.StreamError{message: "error during streaming: #{formatted}", reason: reason}
+  def message(%{reason: reason}) do
+    "error during streaming: #{inspect(reason)}"
   end
 end
 
