@@ -8,6 +8,7 @@ defmodule Kernel.LexicalTracker do
   @moduledoc false
   @timeout :infinity
   @behaviour :gen_server
+  @warn_key 0
 
   @doc """
   Returns all references in this lexical scope.
@@ -16,6 +17,24 @@ defmodule Kernel.LexicalTracker do
     :gen_server.call(pid, :references, @timeout)
   end
 
+  @doc """
+  Invoked during module expansion to annotate an alias
+  must be warned if unused.
+  """
+  def warn_alias(pid, meta, alias, module) do
+    :gen_server.cast(pid, {:warn_alias, alias, meta})
+    module
+  end
+
+  @doc """
+  Invoked during module expansion to annotate an import
+  must be warned if unused.
+  """
+  def warn_import(pid, module) do
+    :gen_server.cast(pid, {:warn_import, module})
+    module
+  end
+
   # Internal API
 
   # Starts the tracker and returns its PID.
@@ -35,13 +54,8 @@ defmodule Kernel.LexicalTracker do
   end
 
   @doc false
-  def add_import(pid, module, fas, line, warn) when is_atom(module) do
-    :gen_server.cast(pid, {:add_import, module, fas, line, warn})
-  end
-
-  @doc false
-  def add_alias(pid, module, line, warn) when is_atom(module) do
-    :gen_server.cast(pid, {:add_alias, module, line, warn})
+  def add_import(pid, module, fas, meta, warn) when is_atom(module) do
+    :gen_server.cast(pid, {:add_import, module, fas, meta, warn})
   end
 
   @doc false
@@ -93,16 +107,12 @@ defmodule Kernel.LexicalTracker do
 
   @doc false
   def collect_unused_imports(pid) do
-    unused(pid, :unused_imports)
+    :gen_server.call(pid, :unused_imports, @timeout)
   end
 
   @doc false
   def collect_unused_aliases(pid) do
-    unused(pid, :unused_aliases)
-  end
-
-  defp unused(pid, tag) do
-    :gen_server.call(pid, tag, @timeout)
+    :gen_server.call(pid, :unused_aliases, @timeout)
   end
 
   # Callbacks
@@ -123,11 +133,17 @@ defmodule Kernel.LexicalTracker do
 
   @doc false
   def handle_call(:unused_aliases, _from, state) do
-    {:reply, Enum.sort(state.aliases), state}
+    aliases = for {alias, meta} when is_list(meta) <- state.aliases, do: {alias, meta}
+    {:reply, Enum.sort(aliases), state}
   end
 
   def handle_call(:unused_imports, _from, state) do
-    {:reply, Enum.sort(state.imports), state}
+    imports =
+      for {module, %{@warn_key => _} = map} <- state.imports do
+        {module, Map.delete(map, @warn_key)}
+      end
+
+    {:reply, Enum.sort(imports), state}
   end
 
   def handle_call(:references, _from, state) do
@@ -171,8 +187,8 @@ defmodule Kernel.LexicalTracker do
     {:noreply, %{state | imports: imports, references: references}}
   end
 
-  def handle_cast({:alias_dispatch, module}, %{aliases: aliases} = state) do
-    {:noreply, %{state | aliases: Map.delete(aliases, module)}}
+  def handle_cast({:alias_dispatch, module}, state) do
+    {:noreply, put_in(state.aliases[module], :used)}
   end
 
   def handle_cast({:import_quoted, module, function, arities}, state) do
@@ -209,21 +225,20 @@ defmodule Kernel.LexicalTracker do
     {:noreply, put_in(state.exports[module], true)}
   end
 
-  def handle_cast({:add_import, module, fas, line, warn}, state) do
-    if warn do
-      imports = for module_or_fa <- [module | fas], do: {module_or_fa, line}, into: %{}
-      {:noreply, put_in(state.imports[module], imports)}
-    else
-      {:noreply, state}
+  def handle_cast({:add_import, module, fas, meta, warn}, state) do
+    keys = if warn, do: [@warn_key, module | fas], else: [module | fas]
+    {:noreply, put_in(state.imports[module], Map.from_keys(keys, meta))}
+  end
+
+  def handle_cast({:warn_alias, alias, meta}, %{aliases: aliases} = state) do
+    case aliases do
+      %{^alias => :used} -> {:noreply, state}
+      %{} -> {:noreply, %{state | aliases: Map.put(aliases, alias, meta)}}
     end
   end
 
-  def handle_cast({:add_alias, module, line, warn}, state) do
-    if warn do
-      {:noreply, put_in(state.aliases[module], line)}
-    else
-      {:noreply, state}
-    end
+  def handle_cast({:warn_import, module}, state) do
+    {:noreply, put_in(state.imports[module][@warn_key], true)}
   end
 
   @doc false
