@@ -112,26 +112,24 @@ defmodule Kernel.SpecialForms do
 
       %User{age: age} = user
 
-  An update operation specific for structs is also available:
-
-      %User{user | age: 28}
-
   The advantage of structs is that they validate that the given
   keys are part of the defined struct. The example below will fail
   because there is no key `:full_name` in the `User` struct:
 
       %User{full_name: "john doe"}
 
-  The syntax above will guarantee the given keys are valid at
-  compilation time and it will guarantee at runtime the given
-  argument is a struct, failing with `BadStructError` otherwise.
+  An update operation specific for structs is also available:
 
-  Although structs are maps, by default structs do not implement
-  any of the protocols implemented for maps. Check
-  `Kernel.defprotocol/2` for more information on how structs
-  can be used with protocols for polymorphic dispatch. Also
-  see `Kernel.struct/2` and `Kernel.struct!/2` for examples on
-  how to create and update structs dynamically.
+      %User{user | age: 28}
+
+  Once again, the syntax above will guarantee the given keys
+  are valid at compilation time and it will guarantee at runtime
+  the given argument is a struct, failing with `BadStructError`
+  otherwise. The map update syntax can also be used for updating
+  structs, and it is useful when you want to update any struct,
+  regardless of their name, as long as they have matching fields:
+
+      %{user | age: 28}
 
   ## Pattern matching on struct names
 
@@ -187,20 +185,23 @@ defmodule Kernel.SpecialForms do
       iex> <<0, "foo">>
       <<0, 102, 111, 111>>
 
-  Binaries need to be explicitly tagged as `binary`:
+  You can use one of `utf8` (the default), `utf16`, and `utf32` to
+  control how the string is encoded:
+
+      iex> <<"foo"::utf16>>
+      <<0, 102, 0, 111, 0, 111>>
+
+  Which is equivalent to writing:
+
+      iex> <<?f::utf16, ?o::utf16, ?o::utf16>>
+      <<0, 102, 0, 111, 0, 111>>
+
+  At runtime, binaries need to be explicitly tagged as `binary`:
 
       iex> rest = "oo"
       iex> <<102, rest::binary>>
       "foo"
 
-  The `utf8`, `utf16`, and `utf32` types are for Unicode code points. They
-  can also be applied to literal strings and charlists:
-
-      iex> <<"foo"::utf16>>
-      <<0, 102, 0, 111, 0, 111>>
-      iex> <<"foo"::utf32>>
-      <<0, 0, 0, 102, 0, 0, 0, 111, 0, 0, 0, 111>>
-
   Otherwise we get an `ArgumentError` when constructing the binary:
 
       rest = "oo"
@@ -249,7 +250,7 @@ defmodule Kernel.SpecialForms do
   The size can be a variable or any valid guard expression:
 
       iex> name_size = 5
-      iex> <<name::binary-size(name_size), " the ", species::binary>> = <<"Frank the Walrus">>
+      iex> <<name::binary-size(^name_size), " the ", species::binary>> = <<"Frank the Walrus">>
       iex> {name, species}
       {"Frank", "Walrus"}
 
@@ -294,7 +295,7 @@ defmodule Kernel.SpecialForms do
   `unsigned` (default) | `integer`
   `little`             | `integer`, `float`, `utf16`, `utf32`
   `big` (default)      | `integer`, `float`, `utf16`, `utf32`
-  `native`             | `integer`, `utf16`, `utf32`
+  `native`             | `integer`, `float`, `utf16`, `utf32`
 
   ### Sign
 
@@ -346,7 +347,7 @@ defmodule Kernel.SpecialForms do
 
   Or as a part of function definitions to pattern match:
 
-      defmodule ImageTyper do
+      defmodule ImageType do
         @png_signature <<137::size(8), 80::size(8), 78::size(8), 71::size(8),
                          13::size(8), 10::size(8), 26::size(8), 10::size(8)>>
         @jpg_signature <<255::size(8), 216::size(8)>>
@@ -547,7 +548,7 @@ defmodule Kernel.SpecialForms do
   Elixir won't emit any warnings though, since the alias
   was not explicitly defined.
 
-  Both warning behaviours could be changed by explicitly
+  Both warning behaviors could be changed by explicitly
   setting the `:warn` option to `true` or `false`.
 
   """
@@ -671,7 +672,7 @@ defmodule Kernel.SpecialForms do
   Elixir won't emit any warnings though, since the import
   was not explicitly defined.
 
-  Both warning behaviours could be changed by explicitly
+  Both warning behaviors could be changed by explicitly
   setting the `:warn` option to `true` or `false`.
 
   ## Ambiguous function/macro names
@@ -837,8 +838,8 @@ defmodule Kernel.SpecialForms do
     * `:context` - sets the resolution context.
 
     * `:generated` - marks the given chunk as generated so it does not emit warnings.
-      Currently it only works on special forms (for example, you can annotate a `case`
-      but not an `if`).
+      It is also useful to avoid dialyzer reporting errors when macros generate
+      unused clauses.
 
     * `:file` - sets the quoted expressions to have the given file.
 
@@ -885,7 +886,7 @@ defmodule Kernel.SpecialForms do
   At first, there is nothing in this example that actually reveals it is a
   macro. But what is happening is that, at compilation time, `squared(5)`
   becomes `5 * 5`. The argument `5` is duplicated in the produced code, we
-  can see this behaviour in practice though because our macro actually has
+  can see this behavior in practice though because our macro actually has
   a bug:
 
       import Math
@@ -912,7 +913,7 @@ defmodule Kernel.SpecialForms do
       my_number.() * my_number.()
 
   Which invokes the function twice, explaining why we get the printed value
-  twice! In the majority of the cases, this is actually unexpected behaviour,
+  twice! In the majority of the cases, this is actually unexpected behavior,
   and that's why one of the first things you need to keep in mind when it
   comes to macros is to **not unquote the same value more than once**.
 
@@ -960,7 +961,7 @@ defmodule Kernel.SpecialForms do
       import Math
       squared(5)
       x
-      ** (CompileError) undefined variable x or undefined function x/0
+      ** (CompileError) undefined variable "x"
 
   We can see that `x` did not leak to the user context. This happens
   because Elixir macros are hygienic, a topic we will discuss at length
@@ -1023,9 +1024,10 @@ defmodule Kernel.SpecialForms do
         end
       end
 
+      require Hygiene
       Hygiene.write()
       Hygiene.read()
-      ** (RuntimeError) undefined variable a or undefined function a/0
+      ** (CompileError) undefined variable "a" (context Hygiene)
 
   For such, you can explicitly pass the current module scope as
   argument:
@@ -1044,10 +1046,22 @@ defmodule Kernel.SpecialForms do
         end
       end
 
+      require ContextHygiene
       ContextHygiene.write()
       ContextHygiene.read()
       #=> 1
 
+  The contexts of a variable is identified by the third element of the tuple.
+  The default context is `nil` and `quote` assigns another context to all
+  variables within:
+
+      quote(do: var)
+      #=> {:var, [], Elixir}
+
+  In case of variables returned by macros, there may also be a `:counter` key
+  in the metadata, which is used to further refine its contexts and guarantee
+  isolation between macro invocations as seen in the previous example.
+
   ## Hygiene in aliases
 
   Aliases inside quote are hygienic by default.
@@ -1212,7 +1226,9 @@ defmodule Kernel.SpecialForms do
   reported to where `defadd` was invoked. `location: :keep` affects
   only definitions inside the quote.
 
-  > **Important:** do not use location: :keep if the function definition
+  > #### `location: :keep` and unquote {: .warning}
+  >
+  > Do not use `location: :keep` if the function definition
   > also `unquote`s some of the macro arguments. If you do so, Elixir
   > will store the file definition of the current location but the
   > unquoted arguments may contain line information of the macro caller,
@@ -1372,8 +1388,10 @@ defmodule Kernel.SpecialForms do
       iex> for n <- [1, 2, 3, 4], do: n * 2
       [2, 4, 6, 8]
 
-  A comprehension accepts many generators and filters. Enumerable
-  generators are defined using `<-`:
+  A comprehension accepts many generators and filters. `for` uses
+  the `<-` operator to extract values from the enumerable on its
+  right side and match them against the pattern on the left.
+  We call them generators:
 
       # A list generator:
       iex> for n <- [1, 2, 3, 4], do: n * 2
@@ -1424,7 +1442,7 @@ defmodule Kernel.SpecialForms do
       [elixir: :prolog]
 
   Given the grandparents of Erlang and Prolog were nil, those values were
-  filtered out. If you don't want this behaviour, a simple option is to
+  filtered out. If you don't want this behavior, a simple option is to
   move the filter inside the do-block:
 
       iex> languages = [elixir: :erlang, erlang: :prolog, prolog: nil]
@@ -1474,7 +1492,9 @@ defmodule Kernel.SpecialForms do
 
   ## The `:reduce` option
 
-  While the `:into` option allows us to customize the comprehension behaviour
+  *Available since Elixir v1.8*.
+
+  While the `:into` option allows us to customize the comprehension behavior
   to a given data type, such as putting all of the values inside a map or inside
   a binary, it is not always enough.
 
@@ -1517,9 +1537,48 @@ defmodule Kernel.SpecialForms do
   defmacro for(args), do: error!([args])
 
   @doc """
-  Used to combine matching clauses.
+  Combine matching clauses.
 
-  Let's start with an example:
+  One of the ways to understand `with` is to show which code
+  patterns it improves. Imagine you have a map where the fields
+  `width` and `height` are optional and you want to compute its
+  area, as `{:ok, area}` or return `:error`. We could implement
+  this function as:
+
+      def area(opts) do
+        case Map.fetch(opts, :width) do
+          {:ok, width} ->
+            case Map.fetch(opts, :height) do
+              {:ok, height} -> {:ok, width * height}
+              :error -> :error
+            end
+
+          :error ->
+            :error
+        end
+      end
+
+  when called as `area(%{width: 10, height: 15})`, it should return
+  `{:ok, 150}`. If any of the fields are missing, it returns `:error`.
+
+  While the code above works, it is quite verbose. Using `with`,
+  we could rewrite it as:
+
+      def area(opts) do
+        with {:ok, width} <- Map.fetch(opts, :width),
+             {:ok, height} <- Map.fetch(opts, :height) do
+          {:ok, width * height}
+        end
+      end
+
+  Instead of defining nested `case`s with clauses, we use `with`
+  alongside the `PATTERN <- EXPRESSION` operator to match
+  expressions on its right side against the pattern on the left.
+  Consider `<-` as a sibling to `=`, except that, while `=` raises
+  in case of not matches, `<-` will simply abort the `with` chain
+  and return the non-matched value.
+
+  Let's give it a try on IEx:
 
       iex> opts = %{width: 10, height: 15}
       iex> with {:ok, width} <- Map.fetch(opts, :width),
@@ -1546,7 +1605,9 @@ defmodule Kernel.SpecialForms do
       ...> end
       {:ok, "admin"}
 
-  As in `for/1`, variables bound inside `with/1` won't leak.
+  As in `for/1`, variables bound inside `with/1` won't be accessible
+  outside of `with/1`.
+
   Expressions without `<-` may also be used in clauses. For instance,
   you can perform regular matches with the `=` operator:
 
@@ -1561,7 +1622,7 @@ defmodule Kernel.SpecialForms do
       iex> width
       nil
 
-  The behaviour of any expression in a clause is the same as if it was
+  The behavior of any expression in a clause is the same as if it was
   written outside of `with`. For example, `=` will raise a `MatchError`
   instead of returning the non-matched value:
 
@@ -1600,7 +1661,7 @@ defmodule Kernel.SpecialForms do
       ...> end
       {:error, :wrong_data}
 
-  The `else` block works like a `case` clause: it can have multiple clauses,
+  The `else` block works like a `case`: it can have multiple clauses,
   and the first match will be used. Variables bound inside `with` (such as
   `width` in this example) are not available in the `else` block.
 
@@ -1629,7 +1690,8 @@ defmodule Kernel.SpecialForms do
 
   Note how we are having to reconstruct the result types of `Path.extname/1`
   and `File.exists?/1` to build error messages. In this case, it is better
-  to change the with clauses to already return the desired format, like this:
+  to refactor the code so each `<-` already return the desired format in case
+  of errors, like this:
 
       with :ok <- validate_extension(path),
            :ok <- validate_exists(path) do
@@ -1647,7 +1709,7 @@ defmodule Kernel.SpecialForms do
       end
 
   Note how the code above is better organized and clearer once we
-  make sure each clause in `with` returns a normalized format.
+  make sure each `<-` in `with` returns a normalized format.
   """
   defmacro with(args), do: error!([args])
 
@@ -1692,6 +1754,15 @@ defmodule Kernel.SpecialForms do
   """
   defmacro unquote(:__block__)(args), do: error!([args])
 
+  @doc """
+  Internal special form for cursor position.
+
+  This is the special form used whenever we need to represent
+  the cursor position in Elixir's AST. See `Code.Fragment` for
+  more information.
+  """
+  defmacro unquote(:__cursor__)(args), do: error!([args])
+
   @doc """
   Capture operator. Captures or creates an anonymous function.
 
@@ -1715,6 +1786,15 @@ defmodule Kernel.SpecialForms do
 
       &local_function/1
 
+  Note that `&local_function/1` creates a local capture, but
+  `&__MODULE__.local_function/1` or `&imported_function/1` create a remote
+  capture. For more information, refer to the ["Functions" section in the Erlang Reference Manual](https://www.erlang.org/doc/system/eff_guide_functions.html#function-calls).
+
+  Whether a capture is local or remote has implications when using hot code
+  reloading: local captures dispatch to the version of the module that existed
+  at the time they were created, while remote captures dispatch to the current
+  version of the module.
+
   See also `Function.capture/3`.
 
   ## Anonymous functions
@@ -1814,6 +1894,11 @@ defmodule Kernel.SpecialForms do
   @doc ~S"""
   Matches the given expression against the given clauses.
 
+  `case/2` relies on pattern matching and guards to choose
+  which clause to execute. If your logic cannot be expressed
+  within patterns and guards, consider using `if/2` or `cond/1`
+  instead.
+
   ## Examples
 
       case File.read(file) do
@@ -1844,6 +1929,9 @@ defmodule Kernel.SpecialForms do
       end
       #=> "This clause would match any value (x = 10)"
 
+  If you find yourself nesting `case` expressions inside
+  `case` expressions, consider using `with/1`.
+
   ## Variable handling
 
   Note that variables bound in a clause do not leak to the outer context:
@@ -1903,17 +1991,20 @@ defmodule Kernel.SpecialForms do
   Evaluates the expression corresponding to the first clause that
   evaluates to a truthy value.
 
+  ## Examples
+
+  The following example has a single clause that always evaluates
+  to true:
+
       cond do
         hd([1, 2, 3]) ->
           "1 is considered as true"
       end
       #=> "1 is considered as true"
 
-  Raises an error if all conditions evaluate to `nil` or `false`.
+  If all clauses evaluate to `nil` or `false`, `cond` raises an error.
   For this reason, it may be necessary to add a final always-truthy condition
-  (anything non-`false` and non-`nil`), which will always match.
-
-  ## Examples
+  (anything non-`false` and non-`nil`), which will always match:
 
       cond do
         1 + 1 == 1 ->
@@ -1925,6 +2016,9 @@ defmodule Kernel.SpecialForms do
       end
       #=> "This will"
 
+
+  If your `cond` has two clauses, and the last one falls back to
+  `true`, you may consider using `if/2` instead.
   """
   defmacro cond(clauses), do: error!([clauses])
 
@@ -2238,11 +2332,13 @@ defmodule Kernel.SpecialForms do
   defmacro try(args), do: error!([args])
 
   @doc """
-  Checks if there is a message matching the given clauses
-  in the current process mailbox.
+  Checks if there is a message matching any of the given clauses in the current
+  process mailbox.
 
-  In case there is no such message, the current process hangs
-  until a message arrives or waits until a given timeout value.
+  If there is no matching message, the current process waits until a matching
+  message arrives or until after a given timeout value.
+
+  Any new and existing messages that do not match will remain in the mailbox.
 
   ## Examples
 
@@ -2255,8 +2351,8 @@ defmodule Kernel.SpecialForms do
           IO.puts(:stderr, "Unexpected message received")
       end
 
-  An optional `after` clause can be given in case the message was not
-  received after the given timeout period, specified in milliseconds:
+  An optional `after` clause can be given in case no matching message is
+  received during the given timeout period, specified in milliseconds:
 
       receive do
         {:selector, number, name} when is_integer(number) ->
