@@ -57,8 +57,8 @@ defmodule List do
       iex> list ++ [4] # slow
       [1, 2, 3, 4]
 
-  Most of the functions in this module work in linear time. This means that,
-  that the time it takes to perform an operation grows at the same rate as the
+  Most of the functions in this module work in linear time. This means that
+  the time it takes to perform an operation grows at the same rate as the
   length of the list. For example `length/1` and `last/1` will run in linear
   time because they need to iterate through every element of the list, but
   `first/1` will run in constant time because it only needs the first element.
@@ -84,30 +84,30 @@ defmodule List do
     * and be out of the range `0xD800..0xDFFF` (`55_296..57_343`), which is
       reserved in Unicode for UTF-16 surrogate pairs.
 
-  Elixir uses single quotes to define charlists:
+  Elixir uses the [`~c` sigil](`sigil_c/2`) to define charlists:
 
-      iex> 'héllo'
+      iex> ~c"héllo"
       [104, 233, 108, 108, 111]
 
-  In particular, charlists will be printed back by default in single
-  quotes if they contain only printable ASCII characters:
+  In particular, charlists will be printed back by default with the `~c`
+  sigil if they contain only printable ASCII characters:
 
-      iex> 'abc'
-      'abc'
+      iex> ~c"abc"
+      ~c"abc"
 
   Even though the representation changed, the raw data does remain a list of
-  numbers, which can be handled as such:
+  integers, which can be handled as such:
 
-      iex> inspect('abc', charlists: :as_list)
+      iex> inspect(~c"abc", charlists: :as_list)
       "[97, 98, 99]"
-      iex> Enum.map('abc', fn num -> 1000 + num end)
+      iex> Enum.map(~c"abc", fn num -> 1000 + num end)
       [1097, 1098, 1099]
 
   You can use the `IEx.Helpers.i/1` helper to get a condensed rundown on
   charlists in IEx when you encounter them, which shows you the type, description
   and also the raw representation in one single summary.
 
-  The rationale behind this behaviour is to better support
+  The rationale behind this behavior is to better support
   Erlang libraries which may return text as charlists
   instead of Elixir strings. In Erlang, charlists are the default
   way of handling strings, while in Elixir it's binaries. One
@@ -115,11 +115,11 @@ defmodule List do
 
       Application.loaded_applications()
       #=>  [
-      #=>    {:stdlib, 'ERTS  CXC 138 10', '2.6'},
-      #=>    {:compiler, 'ERTS  CXC 138 10', '6.0.1'},
-      #=>    {:elixir, 'elixir', '1.0.0'},
-      #=>    {:kernel, 'ERTS  CXC 138 10', '4.1'},
-      #=>    {:logger, 'logger', '1.0.0'}
+      #=>    {:stdlib, ~c"ERTS  CXC 138 10", ~c"2.6"},
+      #=>    {:compiler, ~c"ERTS  CXC 138 10", ~c"6.0.1"},
+      #=>    {:elixir, ~c"elixir", ~c"1.0.0"},
+      #=>    {:kernel, ~c"ERTS  CXC 138 10", ~c"4.1"},
+      #=>    {:logger, ~c"logger", ~c"1.0.0"}
       #=>  ]
 
   A list can be checked if it is made of only printable ASCII
@@ -633,27 +633,39 @@ defmodule List do
     [other]
   end
 
-  @doc """
-  Zips corresponding elements from each list in `list_of_lists`.
-
-  The zipping finishes as soon as any list terminates.
-
-  ## Examples
-
-      iex> List.zip([[1, 2], [3, 4], [5, 6]])
-      [{1, 3, 5}, {2, 4, 6}]
-
-      iex> List.zip([[1, 2], [3], [5, 6]])
-      [{1, 3, 5}]
-
-  """
-  @spec zip([list]) :: [tuple]
+  @deprecated "Use Enum.zip/1 instead"
+  # We keep the old implementation because it also supported lists
+  # of tuples, even though this was not included in its @spec.
   def zip([]), do: []
+  def zip(list_of_lists) when is_list(list_of_lists), do: do_zip(list_of_lists, [])
 
-  def zip(list_of_lists) when is_list(list_of_lists) do
-    do_zip(list_of_lists, [])
+  defp do_zip(list, acc) do
+    converter = fn x, acc -> do_zip_each(to_list(x), acc) end
+
+    case :lists.mapfoldl(converter, [], list) do
+      {_, nil} ->
+        :lists.reverse(acc)
+
+      {mlist, heads} ->
+        do_zip(mlist, [to_tuple(:lists.reverse(heads)) | acc])
+    end
   end
 
+  defp do_zip_each(_, nil) do
+    {nil, nil}
+  end
+
+  defp do_zip_each([head | tail], acc) do
+    {tail, [head | acc]}
+  end
+
+  defp do_zip_each([], _) do
+    {nil, nil}
+  end
+
+  defp to_list(tuple) when is_tuple(tuple), do: Tuple.to_list(tuple)
+  defp to_list(list) when is_list(list), do: list
+
   @doc ~S"""
   Checks if `list` is a charlist made only of printable ASCII characters.
 
@@ -678,18 +690,18 @@ defmodule List do
 
   ## Examples
 
-      iex> List.ascii_printable?('abc')
+      iex> List.ascii_printable?(~c"abc")
       true
 
-      iex> List.ascii_printable?('abc' ++ [0])
+      iex> List.ascii_printable?(~c"abc" ++ [0])
       false
 
-      iex> List.ascii_printable?('abc' ++ [0], 2)
+      iex> List.ascii_printable?(~c"abc" ++ [0], 2)
       true
 
   Improper lists are not printable, even if made only of ASCII characters:
 
-      iex> List.ascii_printable?('abc' ++ ?d)
+      iex> List.ascii_printable?(~c"abc" ++ ?d)
       false
 
   """
@@ -895,7 +907,7 @@ defmodule List do
   end
 
   @doc """
-  Returns `true` if `list` starts with the given `prefix` list; otherwise returns `false`.
+  Returns `true` if `list` starts with the given `prefix` list, otherwise returns `false`.
 
   If `prefix` is an empty list, it returns `true`.
 
@@ -924,20 +936,48 @@ defmodule List do
   def starts_with?(list, []) when is_list(list), do: true
   def starts_with?(list, [_ | _]) when is_list(list), do: false
 
+  @doc """
+  Returns `true` if `list` ends with the given `suffix` list, otherwise returns `false`.
+
+  If `suffix` is an empty list, it returns `true`.
+
+  ### Examples
+
+      iex> List.ends_with?([1, 2, 3], [2, 3])
+      true
+
+      iex> List.ends_with?([1, 2], [1, 2, 3])
+      false
+
+      iex> List.ends_with?([:alpha], [])
+      true
+
+      iex> List.ends_with?([], [:alpha])
+      false
+
+  """
+  @doc since: "1.18.0"
+  @spec ends_with?(nonempty_list, nonempty_list) :: boolean
+  @spec ends_with?(list, []) :: true
+  @spec ends_with?([], nonempty_list) :: false
+  def ends_with?(list, suffix) do
+    :lists.suffix(suffix, list)
+  end
+
   @doc """
   Converts a charlist to an atom.
 
-  Elixir supports conversions from charlists which contains any Unicode
+  Elixir supports conversions from charlists which contain any Unicode
   code point.
 
   Inlined by the compiler.
 
   ## Examples
 
-      iex> List.to_atom('Elixir')
+      iex> List.to_atom(~c"Elixir")
       :Elixir
 
-      iex> List.to_atom('🌢 Elixir')
+      iex> List.to_atom(~c"🌢 Elixir")
       :"🌢 Elixir"
 
   """
@@ -949,7 +989,7 @@ defmodule List do
   @doc """
   Converts a charlist to an existing atom.
 
-  Elixir supports conversions from charlists which contains any Unicode
+  Elixir supports conversions from charlists which contain any Unicode
   code point. Raises an `ArgumentError` if the atom does not exist.
 
   Inlined by the compiler.
@@ -966,11 +1006,11 @@ defmodule List do
   ## Examples
 
       iex> _ = :my_atom
-      iex> List.to_existing_atom('my_atom')
+      iex> List.to_existing_atom(~c"my_atom")
       :my_atom
 
       iex> _ = :"🌢 Elixir"
-      iex> List.to_existing_atom('🌢 Elixir')
+      iex> List.to_existing_atom(~c"🌢 Elixir")
       :"🌢 Elixir"
 
   """
@@ -986,7 +1026,7 @@ defmodule List do
 
   ## Examples
 
-      iex> List.to_float('2.2017764e+0')
+      iex> List.to_float(~c"2.2017764e+0")
       2.2017764
 
   """
@@ -1002,7 +1042,7 @@ defmodule List do
 
   ## Examples
 
-      iex> List.to_integer('123')
+      iex> List.to_integer(~c"123")
       123
 
   """
@@ -1020,7 +1060,7 @@ defmodule List do
 
   ## Examples
 
-      iex> List.to_integer('3FF', 16)
+      iex> List.to_integer(~c"3FF", 16)
       1023
 
   """
@@ -1068,7 +1108,7 @@ defmodule List do
       iex> List.to_string([0x0061, "bc"])
       "abc"
 
-      iex> List.to_string([0x0064, "ee", ['p']])
+      iex> List.to_string([0x0064, "ee", [~c"p"]])
       "deep"
 
       iex> List.to_string([])
@@ -1117,14 +1157,14 @@ defmodule List do
 
   ## Examples
 
-      iex> List.to_charlist([0x00E6, 0x00DF])
-      'æß'
+      iex> ~c"æß" = List.to_charlist([0x00E6, 0x00DF])
+      [230, 223]
 
       iex> List.to_charlist([0x0061, "bc"])
-      'abc'
+      ~c"abc"
 
-      iex> List.to_charlist([0x0064, "ee", ['p']])
-      'deep'
+      iex> List.to_charlist([0x0064, "ee", [~c"p"]])
+      ~c"deep"
 
   """
   @doc since: "1.8.0"
@@ -1168,7 +1208,7 @@ defmodule List do
   An *edit script* is a keyword list. Each key describes the "editing action" to
   take in order to bring `list1` closer to being equal to `list2`; a key can be
   `:eq`, `:ins`, or `:del`. Each value is a sublist of either `list1` or `list2`
-  that should be inserted (if the corresponding key `:ins`), deleted (if the
+  that should be inserted (if the corresponding key is `:ins`), deleted (if the
   corresponding key is `:del`), or left alone (if the corresponding key is
   `:eq`) in `list1` in order to be closer to `list2`.
 
@@ -1372,33 +1412,4 @@ defmodule List do
   defp do_pop_at([head | tail], index, default, acc) do
     do_pop_at(tail, index - 1, default, [head | acc])
   end
-
-  # zip
-
-  defp do_zip(list, acc) do
-    converter = fn x, acc -> do_zip_each(to_list(x), acc) end
-
-    case :lists.mapfoldl(converter, [], list) do
-      {_, nil} ->
-        :lists.reverse(acc)
-
-      {mlist, heads} ->
-        do_zip(mlist, [to_tuple(:lists.reverse(heads)) | acc])
-    end
-  end
-
-  defp do_zip_each(_, nil) do
-    {nil, nil}
-  end
-
-  defp do_zip_each([head | tail], acc) do
-    {tail, [head | acc]}
-  end
-
-  defp do_zip_each([], _) do
-    {nil, nil}
-  end
-
-  defp to_list(tuple) when is_tuple(tuple), do: Tuple.to_list(tuple)
-  defp to_list(list) when is_list(list), do: list
 end
