@@ -24,7 +24,7 @@ defprotocol List.Chars do
 end
 
 defimpl List.Chars, for: Atom do
-  def to_charlist(nil), do: ''
+  def to_charlist(nil), do: ~c""
 
   def to_charlist(atom), do: Atom.to_charlist(atom)
 end
@@ -58,6 +58,6 @@ end
 
 defimpl List.Chars, for: Float do
   def to_charlist(term) do
-    :io_lib_format.fwrite_g(term)
+    Float.to_charlist(term)
   end
 end
