@@ -44,17 +44,17 @@ defmodule Path do
   """
   @spec absname(t) :: binary
   def absname(path) do
-    absname(path, File.cwd!())
+    absname(path, &File.cwd!/0)
   end
 
   @doc """
   Builds a path from `relative_to` to `path`.
 
   If `path` is already an absolute path, `relative_to` is ignored. See also
-  `relative_to/2`.
+  `relative_to/3`. `relative_to` is either a path or an anonymous function,
+  which is invoked only when necessary, that returns a path.
 
-  Unlike `expand/2`, no attempt is made to
-  resolve `..`, `.` or `~`.
+  Unlike `expand/2`, no attempt is made to resolve `..`, `.` or `~`.
 
   ## Examples
 
@@ -64,20 +64,37 @@ defmodule Path do
       iex> Path.absname("../x", "bar")
       "bar/../x"
 
+      iex> Path.absname("foo", fn -> "lazy" end)
+      "lazy/foo"
+
   """
-  @spec absname(t, t) :: binary
+  @spec absname(t, t | (-> t)) :: binary
   def absname(path, relative_to) do
     path = IO.chardata_to_string(path)
 
     case type(path) do
       :relative ->
+        relative_to =
+          if is_function(relative_to, 0) do
+            relative_to.()
+          else
+            relative_to
+          end
+
         absname_join([relative_to, path])
 
       :absolute ->
         absname_join([path])
 
       :volumerelative ->
-        relative_to = IO.chardata_to_string(relative_to)
+        relative_to =
+          if is_function(relative_to, 0) do
+            relative_to.()
+          else
+            relative_to
+          end
+          |> IO.chardata_to_string()
+
         absname_vr(split(path), split(relative_to), relative_to)
     end
   end
@@ -120,7 +137,7 @@ defmodule Path do
 
   defp do_absname_join(<<c1, c2, rest::binary>>, relativename, [], :win32)
        when c1 in @slash and c2 in @slash,
-       do: do_absname_join(rest, relativename, '//', :win32)
+       do: do_absname_join(rest, relativename, ~c"//", :win32)
 
   defp do_absname_join(<<?\\, rest::binary>>, relativename, result, :win32),
     do: do_absname_join(<<?/, rest::binary>>, relativename, result, :win32)
@@ -155,15 +172,21 @@ defmodule Path do
   Converts the path to an absolute one, expanding
   any `.` and `..` components and a leading `~`.
 
+  If a relative path is provided it is expanded relatively to
+  the current working directory.
+
   ## Examples
 
       Path.expand("/foo/bar/../baz")
       #=> "/foo/baz"
 
+      Path.expand("foo/bar/../baz")
+      #=> "$PWD/foo/baz"
+
   """
   @spec expand(t) :: binary
   def expand(path) do
-    expand_dot(absname(expand_home(path), File.cwd!()))
+    expand_dot(absname(expand_home(path), &File.cwd!/0))
   end
 
   @doc """
@@ -192,7 +215,7 @@ defmodule Path do
   """
   @spec expand(t, t) :: binary
   def expand(path, relative_to) do
-    expand_dot(absname(absname(expand_home(path), expand_home(relative_to)), File.cwd!()))
+    expand_dot(absname(absname(expand_home(path), expand_home(relative_to)), &File.cwd!/0))
   end
 
   @doc """
@@ -226,6 +249,8 @@ defmodule Path do
   @doc """
   Forces the path to be a relative path.
 
+  If an absolute path is given, it is stripped from its root component.
+
   ## Examples
 
   ### Unix-like operating systems
@@ -242,6 +267,12 @@ defmodule Path do
       Path.relative("/bar/foo.ex")      #=> "bar/foo.ex"
 
   """
+  # Note this function does not expand paths because the behavior
+  # is ambiguous. If we expand it before converting to relative, then
+  # "/usr/../../foo" means "/foo". If we expand it after, it means "../foo".
+  # We could expand only relative paths but it is best to say it never
+  # expands and then provide a `Path.expand_relative` function (or an
+  # option) if desired.
   @spec relative(t) :: binary
   def relative(name) do
     relative(name, major_os_type())
@@ -260,7 +291,7 @@ defmodule Path do
     end
   end
 
-  defp unix_pathtype(path) when path in ["/", '/'], do: {:absolute, "."}
+  defp unix_pathtype(path) when path in ["/", ~c"/"], do: {:absolute, "."}
   defp unix_pathtype(<<?/, relative::binary>>), do: {:absolute, relative}
   defp unix_pathtype([?/ | relative]), do: {:absolute, relative}
   defp unix_pathtype([list | rest]) when is_list(list), do: unix_pathtype(list ++ rest)
@@ -297,52 +328,143 @@ defmodule Path do
   defp win32_pathtype(relative), do: {:relative, relative}
 
   @doc """
-  Returns the given `path` relative to the given `from` path.
+  Returns the direct relative path from `path` in relation to `cwd`.
 
-  In other words, this function tries to strip the `from` prefix from `path`.
+  In other words, this function attempts to return a path such that
+  `Path.expand(result, cwd)` points to `path`. This function aims
+  to return a relative path whenever possible, but that's not guaranteed:
 
-  This function does not query the file system, so it assumes
-  no symlinks between the paths.
+    * If both paths are relative, a relative path is always returned
 
-  In case a direct relative path cannot be found, it returns
-  the original path.
+    * If both paths are absolute, a relative path may be returned if
+      they share a common prefix. You can pass the `:force` option to
+      force this function to traverse up, but even then a relative
+      path is not guaranteed (for example, if the absolute paths
+      belong to different drives on Windows)
+
+    * If a mixture of paths are given, the result will always match
+      the given `path` (the first argument)
+
+  This function expands `.` and `..` entries without traversing the
+  file system, so it assumes no symlinks between the paths. See
+  `safe_relative_to/2` for a safer alternative.
+
+  ## Options
+
+    * `:force` - (boolean since v1.16.0) if `true` forces a relative
+    path to be returned by traversing the path up. Except if the paths
+    are in different volumes on Windows. Defaults to `false`.
 
   ## Examples
 
-      iex> Path.relative_to("/usr/local/foo", "/usr/local")
-      "foo"
+  ### With relative `cwd`
 
-      iex> Path.relative_to("/usr/local/foo", "/")
-      "usr/local/foo"
+  If both paths are relative, a minimum path is computed:
 
-      iex> Path.relative_to("/usr/local/foo", "/etc")
-      "/usr/local/foo"
+      Path.relative_to("tmp/foo/bar", "tmp")      #=> "foo/bar"
+      Path.relative_to("tmp/foo/bar", "tmp/foo")  #=> "bar"
+      Path.relative_to("tmp/foo/bar", "tmp/bat")  #=> "../foo/bar"
 
-      iex> Path.relative_to("/usr/local/foo", "/usr/local/foo")
-      "."
+  If an absolute path is given with relative `cwd`, it is returned as:
+
+      Path.relative_to("/usr/foo/bar", "tmp/bat")  #=> "/usr/foo/bar"
+
+  ### With absolute `cwd`
+
+  If both paths are absolute, a relative is computed if possible,
+  without traversing up:
+
+      Path.relative_to("/usr/local/foo", "/usr/local")      #=> "foo"
+      Path.relative_to("/usr/local/foo", "/")               #=> "usr/local/foo"
+      Path.relative_to("/usr/local/foo", "/etc")            #=> "/usr/local/foo"
+      Path.relative_to("/usr/local/foo", "/usr/local/foo")  #=> "."
+      Path.relative_to("/usr/local/../foo", "/usr/foo")     #=> "."
+      Path.relative_to("/usr/local/../foo/bar", "/usr/foo") #=> "bar"
+
+  If `:force` is set to `true` paths are traversed up:
+
+      Path.relative_to("/usr", "/usr/local", force: true)          #=> ".."
+      Path.relative_to("/usr/foo", "/usr/local", force: true)      #=> "../foo"
+      Path.relative_to("/usr/../foo/bar", "/etc/foo", force: true) #=> "../../foo/bar"
+
+  If a relative path is given, it is assumed to be relative to the
+  given path, so the path is returned with "." and ".." expanded:
+
+      Path.relative_to(".", "/usr/local")          #=> "."
+      Path.relative_to("foo", "/usr/local")        #=> "foo"
+      Path.relative_to("foo/../bar", "/usr/local") #=> "bar"
+      Path.relative_to("foo/..", "/usr/local")     #=> "."
+      Path.relative_to("../foo", "/usr/local")     #=> "../foo"
 
   """
-  @spec relative_to(t, t) :: binary
-  def relative_to(path, from) do
-    path = IO.chardata_to_string(path)
-    relative_to(split(path), split(from), path)
+  @spec relative_to(t, t, keyword) :: binary
+  def relative_to(path, cwd, opts \\ []) when is_list(opts) do
+    os_type = major_os_type()
+    split_path = split(path)
+    split_cwd = split(cwd)
+    force = Keyword.get(opts, :force, false)
+
+    case {split_absolute?(split_path, os_type), split_absolute?(split_cwd, os_type)} do
+      {true, true} ->
+        split_path = expand_split(split_path)
+        split_cwd = expand_split(split_cwd)
+
+        case force do
+          true -> relative_to_forced(split_path, split_cwd, split_path)
+          false -> relative_to_unforced(split_path, split_cwd, split_path)
+        end
+
+      {false, false} ->
+        split_path = expand_relative(split_path, [], [])
+        split_cwd = expand_relative(split_cwd, [], [])
+        relative_to_forced(split_path, split_cwd, [])
+
+      {_, _} ->
+        join(expand_relative(split_path, [], []))
+    end
   end
 
-  defp relative_to(path, path, _original) do
-    "."
+  defp relative_to_unforced(path, path, _original), do: "."
+
+  defp relative_to_unforced([h | t1], [h | t2], original),
+    do: relative_to_unforced(t1, t2, original)
+
+  defp relative_to_unforced([_ | _] = l1, [], _original), do: join(l1)
+  defp relative_to_unforced(_, _, original), do: join(original)
+
+  defp relative_to_forced(path, path, _original), do: "."
+  defp relative_to_forced(["."], _path, _original), do: "."
+  defp relative_to_forced(path, ["."], _original), do: join(path)
+  defp relative_to_forced([h | t1], [h | t2], original), do: relative_to_forced(t1, t2, original)
+
+  # this should only happen if we have two paths on different drives on windows
+  defp relative_to_forced(original, _, original), do: join(original)
+
+  defp relative_to_forced(l1, l2, _original) do
+    base = List.duplicate("..", length(l2))
+    join(base ++ l1)
   end
 
-  defp relative_to([h | t1], [h | t2], original) do
-    relative_to(t1, t2, original)
-  end
+  defp expand_relative([".." | t], [_ | acc], up), do: expand_relative(t, acc, up)
+  defp expand_relative([".." | t], acc, up), do: expand_relative(t, acc, [".." | up])
+  defp expand_relative(["." | t], acc, up), do: expand_relative(t, acc, up)
+  defp expand_relative([h | t], acc, up), do: expand_relative(t, [h | acc], up)
+  defp expand_relative([], [], []), do: ["."]
+  defp expand_relative([], acc, up), do: up ++ :lists.reverse(acc)
 
-  defp relative_to([_ | _] = l1, [], _original) do
-    join(l1)
-  end
+  defp expand_split([head | tail]), do: expand_split(tail, [head])
+  defp expand_split([".." | t], [_, last | acc]), do: expand_split(t, [last | acc])
+  defp expand_split([".." | t], acc), do: expand_split(t, acc)
+  defp expand_split(["." | t], acc), do: expand_split(t, acc)
+  defp expand_split([h | t], acc), do: expand_split(t, [h | acc])
+  defp expand_split([], acc), do: :lists.reverse(acc)
 
-  defp relative_to(_, _, original) do
-    original
-  end
+  defp split_absolute?(split, :win32), do: win32_split_absolute?(split)
+  defp split_absolute?(split, _), do: match?(["/" | _], split)
+
+  defp win32_split_absolute?(["//" | _]), do: true
+  defp win32_split_absolute?([<<_, ":/">> | _]), do: true
+  defp win32_split_absolute?(_), do: false
 
   @doc """
   Convenience to get the path relative to the current working
@@ -350,11 +472,13 @@ defmodule Path do
 
   If, for some reason, the current working directory
   cannot be retrieved, this function returns the given `path`.
+
+  Check `relative_to/3` for the supported options.
   """
-  @spec relative_to_cwd(t) :: binary
-  def relative_to_cwd(path) do
+  @spec relative_to_cwd(t, keyword) :: binary
+  def relative_to_cwd(path, opts \\ []) when is_list(opts) do
     case :file.get_cwd() do
-      {:ok, base} -> relative_to(path, IO.chardata_to_string(base))
+      {:ok, base} -> relative_to(path, IO.chardata_to_string(base), opts)
       _ -> path
     end
   end
@@ -433,12 +557,8 @@ defmodule Path do
   @doc """
   Returns the extension of the last component of `path`.
 
-  The behaviour of this function changed in Erlang/OTP 24 for filenames
-  starting with a dot and without an extension. For example, for a file
-  named `.gitignore`, `extname/1` now returns an empty string, while it
-  would return `".gitignore"` in previous Erlang/OTP versions. This was
-  done to match the behaviour of `rootname/1`, which would return
-  `".gitignore"` as its name (and therefore it cannot also be an extension).
+  For filenames starting with a dot and without an extension, it returns
+  an empty string.
 
   See `basename/1` and `rootname/1` for related functions to extract
   information from paths.
@@ -451,6 +571,9 @@ defmodule Path do
       iex> Path.extname("~/foo/bar")
       ""
 
+      iex> Path.extname(".gitignore")
+      ""
+
   """
   @spec extname(t) :: binary
   def extname(path) do
@@ -603,6 +726,10 @@ defmodule Path do
       :file.read_link_info(file)
     end
 
+    def read_file_info(file) do
+      :file.read_file_info(file)
+    end
+
     def list_dir(dir) do
       case :file.list_dir(dir) do
         {:ok, files} -> {:ok, for(file <- files, hd(file) != ?., do: file)}
@@ -611,7 +738,7 @@ defmodule Path do
     end
   end
 
-  @doc """
+  @doc ~S"""
   Traverses paths according to the given `glob` expression and returns a
   list of matches.
 
@@ -643,6 +770,10 @@ defmodule Path do
   You may call `Path.expand/1` to normalize the path before invoking
   this function.
 
+  A character preceded by `\\` loses its special meaning.
+  Note that `\\` must be written as `\\\\` in a string literal.
+  For example, `"\\\\?*"` will match any filename starting with `?.`.
+
   By default, the patterns `*` and `?` do not match files starting
   with a dot `.`. See the `:match_dot` option in the "Options" section
   below.
@@ -711,21 +842,18 @@ defmodule Path do
     end
   end
 
-  # expand_dot the given path by expanding "..", "." and "~".
-  defp expand_dot(<<"/", rest::binary>>), do: "/" <> do_expand_dot(rest)
+  # expands dots in an absolute path represented as a string
+  defp expand_dot(path) do
+    [head | tail] = :binary.split(path, "/", [:global])
+    IO.iodata_to_binary(expand_dot(tail, [head <> "/"]))
+  end
 
-  defp expand_dot(<<letter, ":/", rest::binary>>) when letter in ?a..?z,
-    do: <<letter, ":/">> <> do_expand_dot(rest)
-
-  defp expand_dot(path), do: do_expand_dot(path)
-
-  defp do_expand_dot(path), do: do_expand_dot(:binary.split(path, "/", [:global]), [])
-  defp do_expand_dot([".." | t], [_, _ | acc]), do: do_expand_dot(t, acc)
-  defp do_expand_dot([".." | t], []), do: do_expand_dot(t, [])
-  defp do_expand_dot(["." | t], acc), do: do_expand_dot(t, acc)
-  defp do_expand_dot([h | t], acc), do: do_expand_dot(t, ["/", h | acc])
-  defp do_expand_dot([], []), do: ""
-  defp do_expand_dot([], ["/" | acc]), do: IO.iodata_to_binary(:lists.reverse(acc))
+  defp expand_dot([".." | t], [_, _ | acc]), do: expand_dot(t, acc)
+  defp expand_dot([".." | t], acc), do: expand_dot(t, acc)
+  defp expand_dot(["." | t], acc), do: expand_dot(t, acc)
+  defp expand_dot([h | t], acc), do: expand_dot(t, ["/", h | acc])
+  defp expand_dot([], ["/", head | acc]), do: :lists.reverse([head | acc])
+  defp expand_dot([], acc), do: :lists.reverse(acc)
 
   defp major_os_type do
     :os.type() |> elem(0)
@@ -734,6 +862,18 @@ defmodule Path do
   @doc """
   Returns a relative path that is protected from directory-traversal attacks.
 
+  See `safe_relative/2` for a non-deprecated version of this API.
+  """
+  # TODO: Deprecate me on Elixir v1.19
+  @doc since: "1.14.0", deprecated: "Use safe_relative/2 instead"
+  @spec safe_relative_to(t, t) :: {:ok, binary} | :error
+  def safe_relative_to(path, cwd) do
+    safe_relative(path, cwd)
+  end
+
+  @doc """
+  Returns a relative path that is protected from directory-traversal attacks.
+
   The given relative path is sanitized by eliminating `..` and `.` components.
 
   This function checks that, after expanding those components, the path is still "safe".
@@ -741,29 +881,32 @@ defmodule Path do
 
     * The path is not relative, such as `"/foo/bar"`.
 
-    * A `..` component would make it so that the path would travers up above
+    * A `..` component would make it so that the path would traverse up above
       the root of `relative_to`.
 
     * A symbolic link in the path points to something above the root of `relative_to`.
 
   ## Examples
 
-      iex> Path.safe_relative_to("deps/my_dep/app.beam", "deps")
+      iex> Path.safe_relative("foo")
+      {:ok, "foo"}
+
+      iex> Path.safe_relative("deps/my_dep/app.beam")
       {:ok, "deps/my_dep/app.beam"}
 
-      iex> Path.safe_relative_to("deps/my_dep/./build/../app.beam", "deps")
+      iex> Path.safe_relative("deps/my_dep/./build/../app.beam", File.cwd!())
       {:ok, "deps/my_dep/app.beam"}
 
-      iex> Path.safe_relative_to("my_dep/../..", "deps")
+      iex> Path.safe_relative("my_dep/../..")
       :error
 
-      iex> Path.safe_relative_to("/usr/local", ".")
+      iex> Path.safe_relative("/usr/local", File.cwd!())
       :error
 
   """
   @doc since: "1.14.0"
-  @spec safe_relative_to(t, t) :: {:ok, binary} | :error
-  def safe_relative_to(path, relative_to) do
+  @spec safe_relative(t, t) :: {:ok, binary} | :error
+  def safe_relative(path, relative_to \\ File.cwd!()) do
     path = IO.chardata_to_string(path)
 
     case :filelib.safe_relative_path(path, relative_to) do
@@ -771,33 +914,4 @@ defmodule Path do
       relative_path -> {:ok, IO.chardata_to_string(relative_path)}
     end
   end
-
-  @doc """
-  Returns a path relative to the current working directory that is
-  protected from directory-traversal attacks.
-
-  Same as `safe_relative_to/2` with the current working directory as
-  the second argument. If there is an issue retrieving the current working
-  directory, this function raises an error.
-
-  ## Examples
-
-      iex> Path.safe_relative("foo")
-      {:ok, "foo"}
-
-      iex> Path.safe_relative("foo/../bar")
-      {:ok, "bar"}
-
-      iex> Path.safe_relative("foo/../..")
-      :error
-
-      iex> Path.safe_relative("/usr/local")
-      :error
-
-  """
-  @doc since: "1.14.0"
-  @spec safe_relative(t) :: {:ok, binary} | :error
-  def safe_relative(path) do
-    safe_relative_to(path, File.cwd!())
-  end
 end
