@@ -24,9 +24,9 @@ defmodule Range do
 
   When defining a range without a step, the step will be
   defined based on the first and last position of the
-  range, If `first >= last`, it will be an increasing range
+  range, If `last >= first`, it will be an increasing range
   with a step of 1. Otherwise, it is a decreasing range.
-  Note however implicit decreasing ranges are deprecated.
+  Note, however, implicit decreasing ranges are deprecated.
   Therefore, if you need a decreasing range from `3` to `1`,
   prefer to write `3..1//-1` instead.
 
@@ -172,7 +172,7 @@ defmodule Range do
   one element, which is the number itself.
 
   If `first` is greater than `last`, the range will be decreasing from `first`
-  to `last`, albeit this behaviour is deprecated. Therefore, it is advised to
+  to `last`, albeit this behavior is deprecated. Therefore, it is advised to
   explicitly list the step with `new/3`.
 
   ## Examples
@@ -184,8 +184,22 @@ defmodule Range do
 
   @spec new(limit, limit) :: t
   def new(first, last) when is_integer(first) and is_integer(last) do
-    # TODO: Deprecate inferring a range with a step of -1 on Elixir v1.17
-    step = if first <= last, do: 1, else: -1
+    step =
+      if first <= last do
+        1
+      else
+        # TODO: Remove me on v2.0
+        IO.warn_once(
+          {__MODULE__, :new},
+          fn ->
+            "Range.new/2 and first..last default to a step of -1 when last < first. Use Range.new(first, last, -1) or first..last//-1, or pass 1 if that was your intention"
+          end,
+          3
+        )
+
+        -1
+      end
+
     %Range{first: first, last: last, step: step}
   end
 
@@ -231,8 +245,6 @@ defmodule Range do
       iex> Range.size(1..10//-1)
       0
 
-      iex> Range.size(10..1)
-      10
       iex> Range.size(10..1//-1)
       10
       iex> Range.size(10..1//-2)
@@ -244,6 +256,7 @@ defmodule Range do
 
   """
   @doc since: "1.12.0"
+  @spec size(t) :: non_neg_integer
   def size(range)
   def size(first..last//step) when step > 0 and first > last, do: 0
   def size(first..last//step) when step < 0 and first < last, do: 0
@@ -267,14 +280,159 @@ defmodule Range do
 
       iex> Range.shift(0..10//2, 2)
       4..14//2
+      iex> Range.shift(10..0//-2, 2)
+      6..-4//-2
+
   """
   @doc since: "1.14.0"
+  @spec shift(t, integer) :: t
   def shift(first..last//step, steps_to_shift)
-      when is_integer(first) and is_integer(last) and is_integer(step) and
-             is_integer(steps_to_shift) do
+      when is_integer(steps_to_shift) do
     new(first + steps_to_shift * step, last + steps_to_shift * step, step)
   end
 
+  @doc """
+  Splits a range in two.
+
+  It returns a tuple of two elements.
+
+  If `split` is less than the number of elements in the range, the first
+  element in the range will have `split` entries and the second will have
+  all remaining entries.
+
+  If `split` is more than the number of elements in the range, the second
+  range in the tuple will emit zero elements.
+
+  ## Examples
+
+  Increasing ranges:
+
+      iex> Range.split(1..5, 2)
+      {1..2, 3..5}
+
+      iex> Range.split(1..5//2, 2)
+      {1..3//2, 5..5//2}
+
+      iex> Range.split(1..5//2, 0)
+      {1..-1//2, 1..5//2}
+
+      iex> Range.split(1..5//2, 10)
+      {1..5//2, 7..5//2}
+
+  Decreasing ranges can also be split:
+
+      iex> Range.split(5..1//-1, 2)
+      {5..4//-1, 3..1//-1}
+
+      iex> Range.split(5..1//-2, 2)
+      {5..3//-2, 1..1//-2}
+
+      iex> Range.split(5..1//-2, 0)
+      {5..7//-2, 5..1//-2}
+
+      iex> Range.split(5..1//-2, 10)
+      {5..1//-2, -1..1//-2}
+
+  Empty ranges preserve their property but still return empty ranges:
+
+      iex> Range.split(2..5//-1, 2)
+      {2..3//-1, 4..5//-1}
+
+      iex> Range.split(2..5//-1, 10)
+      {2..3//-1, 4..5//-1}
+
+      iex> Range.split(5..2//1, 2)
+      {5..4//1, 3..2//1}
+
+      iex> Range.split(5..2//1, 10)
+      {5..4//1, 3..2//1}
+
+  If the number to split is negative, it splits from the back:
+
+      iex> Range.split(1..5, -2)
+      {1..3, 4..5}
+
+      iex> Range.split(5..1//-1, -2)
+      {5..3//-1, 2..1//-1}
+
+  If it is negative and greater than the elements in the range,
+  the first element of the tuple will be an empty range:
+
+      iex> Range.split(1..5, -10)
+      {1..0//1, 1..5}
+
+      iex> Range.split(5..1//-1, -10)
+      {5..6//-1, 5..1//-1}
+
+  ## Properties
+
+  When a range is split, the following properties are observed.
+  Given `split(input)` returns `{left, right}`, we have:
+
+      assert input.first == left.first
+      assert input.last == right.last
+      assert input.step == left.step
+      assert input.step == right.step
+      assert Range.size(input) == Range.size(left) + Range.size(right)
+
+  """
+  @doc since: "1.15.0"
+  @spec split(t, integer) :: {t, t}
+  def split(first..last//step = range, split) when is_integer(split) do
+    if split >= 0 do
+      split(first, last, step, split)
+    else
+      split(first, last, step, size(range) + split)
+    end
+  end
+
+  defp split(first, last, step, split) when first < last or (first == last and step > 0) do
+    if step > 0 do
+      mid = max(min(first + step * (split - 1), last), first - step)
+      {first..mid//step, (mid + step)..last//step}
+    else
+      {first..(first - step)//step, (last + step)..last//step}
+    end
+  end
+
+  defp split(last, first, step, split) do
+    if step < 0 do
+      mid = min(max(last + step * (split - 1), first), last - step)
+      {last..mid//step, (mid + step)..first//step}
+    else
+      {last..(last - step)//step, (first + step)..first//step}
+    end
+  end
+
+  @doc """
+  Converts a range to a list.
+
+  ## Examples
+
+      iex> Range.to_list(0..5)
+      [0, 1, 2, 3, 4, 5]
+      iex> Range.to_list(-3..0)
+      [-3, -2, -1, 0]
+
+  """
+  @doc since: "1.15.0"
+  @spec to_list(t) :: list(integer)
+  def to_list(first..last//step)
+      when step > 0 and first <= last
+      when step < 0 and first >= last do
+    :lists.seq(first, last, step)
+  end
+
+  def to_list(_first.._last//_step) do
+    []
+  end
+
+  # TODO: Remove me on v2.0
+  def to_list(%{__struct__: Range, first: first, last: last}) do
+    step = if first <= last, do: 1, else: -1
+    :lists.seq(first, last, step)
+  end
+
   @doc """
   Checks if two ranges are disjoint.
 
@@ -282,7 +440,7 @@ defmodule Range do
 
       iex> Range.disjoint?(1..5, 6..9)
       true
-      iex> Range.disjoint?(5..1, 6..9)
+      iex> Range.disjoint?(5..1//-1, 6..9)
       true
       iex> Range.disjoint?(1..5, 5..9)
       false
@@ -294,22 +452,24 @@ defmodule Range do
       iex> Range.disjoint?(1..10//2, 2..10//2)
       true
 
-      # First element in common in all below is 29
-      iex> Range.disjoint?(2..100//3, 9..100//5)
-      false
-      iex> Range.disjoint?(101..2//-3, 99..9//-5)
-      false
+      # First element in common is 29
       iex> Range.disjoint?(1..100//14, 8..100//21)
       false
       iex> Range.disjoint?(57..-1//-14, 8..100//21)
       false
-      iex> Range.disjoint?(1..100//14, 51..8//-21)
+      iex> Range.disjoint?(1..100//14, 50..8//-21)
       false
-
-      # If 29 is out of range
       iex> Range.disjoint?(1..28//14, 8..28//21)
       true
+
+      # First element in common is 14
       iex> Range.disjoint?(2..28//3, 9..28//5)
+      false
+      iex> Range.disjoint?(26..2//-3, 29..9//-5)
+      false
+
+      # Starting from the back without alignment
+      iex> Range.disjoint?(27..11//-3, 30..0//-7)
       true
 
   """
@@ -334,27 +494,36 @@ defmodule Range do
           # progressions and see if they belong within the ranges
           # https://math.stackexchange.com/questions/1656120/formula-to-find-the-first-intersection-of-two-arithmetic-progressions
           {gcd, u, v} = Integer.extended_gcd(-step1, step2)
-          c = first1 - first2 + step2 - step1
-          t1 = -c / step1 * u
-          t2 = -c / step2 * v
-          t = max(floor(t1) + 1, floor(t2) + 1)
-          x = div(c * u + t * step2, gcd) - 1
-          y = div(c * v + t * step1, gcd) - 1
 
-          x < 0 or first1 + x * step1 > last1 or
-            y < 0 or first2 + y * step2 > last2
+          if rem(first2 - first1, gcd) == 0 do
+            c = first1 - first2 + step2 - step1
+            t1 = -c / step2 * u
+            t2 = -c / step1 * v
+            t = max(floor(t1) + 1, floor(t2) + 1)
+            x = div(c * u + t * step2, gcd) - 1
+            y = div(c * v + t * step1, gcd) - 1
+
+            x < 0 or first1 + x * step1 > last1 or
+              y < 0 or first2 + y * step2 > last2
+          else
+            true
+          end
       end
     end
   end
 
   @compile inline: [normalize: 3]
-  defp normalize(first, last, step) when first > last, do: {last, first, -step}
+  defp normalize(first, last, step) when first > last,
+    do: {first - abs(div(first - last, step) * step), first, -step}
+
   defp normalize(first, last, step), do: {first, last, step}
 
   @doc false
   @deprecated "Pattern match on first..last//step instead"
-  def range?(term)
-  def range?(first..last) when is_integer(first) and is_integer(last), do: true
+  def range?(%{__struct__: Range, first: first, last: last})
+      when is_integer(first) and is_integer(last),
+      do: true
+
   def range?(_), do: false
 end
 
@@ -387,16 +556,11 @@ defimpl Enumerable, for: Range do
     {:done, acc}
   end
 
-  def member?(first..last//step = range, value) when is_integer(value) do
-    cond do
-      Range.size(range) == 0 ->
-        {:ok, false}
-
-      first <= last ->
-        {:ok, first <= value and value <= last and rem(value - first, step) == 0}
-
-      true ->
-        {:ok, last <= value and value <= first and rem(value - first, step) == 0}
+  def member?(first..last//step, value) when is_integer(value) do
+    if step > 0 do
+      {:ok, first <= value and value <= last and rem(value - first, step) == 0}
+    else
+      {:ok, last <= value and value <= first and rem(value - first, step) == 0}
     end
   end
 
