@@ -20,7 +20,9 @@ defmodule Supervisor do
   a generic server, that keeps a counter. Other processes can then send
   messages to this process to read the counter and bump its value.
 
-  > Note: in practice you would not define a counter as a GenServer. Instead,
+  > #### Disclaimer {: .neutral}
+  >
+  > In practice you would not define a counter as a GenServer. Instead,
   > if you need a counter, you would pass it around as inputs and outputs to
   > the functions that need it. The reason we picked a counter in this example
   > is due to its simplicity, as it allows us to focus on how supervisors work.
@@ -82,7 +84,7 @@ defmodule Supervisor do
       GenServer.call(Counter, :get)
       #=> 0
 
-      GenServer.cast(Counter, {:bump, 3})
+      GenServer.call(Counter, {:bump, 3})
       #=> 0
 
       GenServer.call(Counter, :get)
@@ -145,6 +147,12 @@ defmodule Supervisor do
       and such. It is set automatically based on the `:start` value and it is rarely
       changed in practice.
 
+    * `:significant` - a boolean indicating if the child process should be
+      considered significant with regard to automatic shutdown.  Only `:transient`
+      and `:temporary` child processes can be marked as significant. This key is
+      optional and defaults to `false`. See section "Automatic shutdown" below
+      for more details.
+
   Let's understand what the `:shutdown` and `:restart` options control.
 
   ### Shutdown values (:shutdown)
@@ -228,6 +236,27 @@ defmodule Supervisor do
         }
       end
 
+  Then the supervisor will call `Counter.start_link(arg)` to start the child
+  process. This flow is summarized in the diagram below. Caller is a process
+  which spawns the Supervisor process. The Supervisor then proceeds to call
+  your code (Module) to spawn its child process:
+
+  ```mermaid
+  sequenceDiagram
+      participant C as Caller (Process)
+      participant S as Supervisor (Process)
+      participant M as Module (Code)
+
+      note right of C: child is a {module, arg} specification
+      C->>+S: Supervisor.start_link([child])
+      S-->>+M: module.child_spec(arg)
+      M-->>-S: %{id: term, start: {module, :start_link, [arg]}}
+      S-->>+M: module.start_link(arg)
+      M->>M: Spawns child process (child_pid)
+      M-->>-S: {:ok, child_pid} | :ignore | {:error, reason}
+      S->>-C: {:ok, supervisor_pid} | {:error, reason}
+  ```
+
   Luckily for us, `use GenServer` already defines a `Counter.child_spec/1`
   exactly like above, so you don't need to write the definition above yourself.
   If you want to customize the automatically generated `child_spec/1` function,
@@ -302,6 +331,10 @@ defmodule Supervisor do
     * `:max_seconds` - the time frame in which `:max_restarts` applies.
       Defaults to `5`.
 
+    * `:auto_shutdown` - the automatic shutdown option. It can be
+      `:never`, `:any_significant`, or `:all_significant`. Optional.
+      See the "Automatic shutdown" section.
+
     * `:name` - a name to register the supervisor process. Supported values are
       explained in the "Name registration" section in the documentation for
       `GenServer`. Optional.
@@ -327,6 +360,27 @@ defmodule Supervisor do
 
   To efficiently supervise children started dynamically, see `DynamicSupervisor`.
 
+  ### Automatic shutdown
+
+  Supervisors have the ability to automatically shut themselves down when child
+  processes marked as `:significant` exit.
+
+  Supervisors support different automatic shutdown options (through
+  the `:auto_shutdown` option, as seen above):
+
+    * `:never` - this is the default, automatic shutdown is disabled.
+
+    * `:any_significant` - if any significant child process exits, the supervisor
+    will automatically shut down its children, then itself.
+
+    * `:all_significant` - when all significant child processes have exited,
+    the supervisor will automatically shut down its children, then itself.
+
+  Only `:transient` and `:temporary` child processes can be marked as significant,
+  and this configuration affects the behavior. Significant `:transient` child
+  processes must exit normally for automatic shutdown to be considered, where
+  `:temporary` child processes may exit for any reason.
+
   ### Name registration
 
   A supervisor is bound to the same name registration rules as a `GenServer`.
@@ -359,10 +413,17 @@ defmodule Supervisor do
   The difference between the two approaches is that a module-based
   supervisor gives you more direct control over how the supervisor
   is initialized. Instead of calling `Supervisor.start_link/2` with
-  a list of child specifications that are automatically initialized, we manually
-  initialize the children by calling `Supervisor.init/2` inside its
-  `c:init/1` callback. `Supervisor.init/2` accepts the same `:strategy`,
-  `:max_restarts`, and `:max_seconds` options as `start_link/2`.
+  a list of child specifications that are implicitly initialized for us,
+  we must explicitly initialize the children by calling `Supervisor.init/2`
+  inside its `c:init/1` callback. `Supervisor.init/2` accepts the same
+  `:strategy`, `:max_restarts`, and `:max_seconds` options as `start_link/2`.
+
+  > #### `use Supervisor` {: .info}
+  >
+  > When you `use Supervisor`, the `Supervisor` module will
+  > set `@behaviour Supervisor` and define a `child_spec/1`
+  > function, so your module can be used as a child
+  > in a supervision tree.
 
   `use Supervisor` also defines a `child_spec/1` function which allows
   us to run `MyApp.Supervisor` as a child of another supervisor or
@@ -432,8 +493,11 @@ defmodule Supervisor do
   restart the child in case it exits with reason `:normal`, `:shutdown` or
   `{:shutdown, term}`.
 
-  So one may ask: which exit reason should I choose when exiting? There are
-  three options:
+  Those exits also impact logging. By default, behaviours such as GenServers
+  do not emit error logs when the exit reason is `:normal`, `:shutdown` or
+  `{:shutdown, term}`.
+
+  So one may ask: which exit reason should I choose? There are three options:
 
     * `:normal` - in such cases, the exit won't be logged, there is no restart
       in transient mode, and linked processes do not exit
@@ -446,6 +510,9 @@ defmodule Supervisor do
       restarts in transient mode, and linked processes exit with the same
       reason unless they're trapping exits
 
+  Generally speaking, if you are exiting for expected reasons, you want to use
+  `:shutdown` or `{:shutdown, term}`.
+
   Note that the supervisor that reaches maximum restart intensity will exit with
   `:shutdown` reason. In this case the supervisor will only be restarted if its
   child specification was defined with the `:restart` option set to `:permanent`
@@ -458,7 +525,7 @@ defmodule Supervisor do
       import Supervisor.Spec
       @behaviour Supervisor
 
-      unless Module.has_attribute?(__MODULE__, :doc) do
+      if not Module.has_attribute?(__MODULE__, :doc) do
         @doc """
         Returns a specification to start this module under a supervisor.
 
@@ -491,13 +558,13 @@ defmodule Supervisor do
                {sup_flags(), [child_spec() | (old_erlang_child_spec :: :supervisor.child_spec())]}}
               | :ignore
 
-  @typedoc "Return values of `start_link` functions"
+  @typedoc "Return values of `start_link/2` and `start_link/3`."
   @type on_start ::
           {:ok, pid}
           | :ignore
           | {:error, {:already_started, pid} | {:shutdown, term} | term}
 
-  @typedoc "Return values of `start_child` functions"
+  @typedoc "Return values of `start_child/2`."
   @type on_start_child ::
           {:ok, child}
           | {:ok, child, info :: term}
@@ -511,43 +578,46 @@ defmodule Supervisor do
   """
   @type child :: pid | :undefined
 
-  @typedoc "The supervisor name"
+  @typedoc "The supervisor name."
   @type name :: atom | {:global, term} | {:via, module, term}
 
-  @typedoc "Option values used by the `start*` functions"
+  @typedoc "Option values used by the `start_link/2` and `start_link/3` functions."
   @type option :: {:name, name}
 
-  @typedoc "The supervisor flags returned on init"
+  @typedoc "The supervisor flags returned on init."
   @type sup_flags() :: %{
           strategy: strategy(),
           intensity: non_neg_integer(),
-          period: pos_integer()
+          period: pos_integer(),
+          auto_shutdown: auto_shutdown()
         }
 
-  @typedoc "The supervisor reference"
+  @typedoc "The supervisor reference."
   @type supervisor :: pid | name | {atom, node}
 
-  @typedoc "Options given to `start_link/2` and `init/2`"
+  @typedoc "Options given to `start_link/2` and `init/2`."
   @type init_option ::
           {:strategy, strategy}
           | {:max_restarts, non_neg_integer}
           | {:max_seconds, pos_integer}
+          | {:auto_shutdown, auto_shutdown}
 
-  @typedoc "Supported restart options"
+  @typedoc "Supported restart options."
   @type restart :: :permanent | :transient | :temporary
 
-  # TODO: Update :shutdown to "timeout() | :brutal_kill" when we require Erlang/OTP 24.
-  #       Additionally apply https://github.com/elixir-lang/elixir/pull/11836
-  @typedoc "Supported shutdown options"
-  @type shutdown :: pos_integer() | :infinity | :brutal_kill
+  @typedoc "Supported shutdown options."
+  @type shutdown :: timeout() | :brutal_kill
 
-  @typedoc "Supported strategies"
+  @typedoc "Supported strategies."
   @type strategy :: :one_for_one | :one_for_all | :rest_for_one
 
-  @typedoc """
-  Supervisor type.
+  @typedoc "Supported automatic shutdown options."
+  @type auto_shutdown :: :never | :any_significant | :all_significant
 
-  Whether the supervisor is a worker or a supervisor.
+  @typedoc """
+  Type of a supervised child.
+
+  Whether the supervised child is a worker or a supervisor.
   """
   @type type :: :worker | :supervisor
 
@@ -563,21 +633,43 @@ defmodule Supervisor do
           optional(:restart) => restart(),
           optional(:shutdown) => shutdown(),
           optional(:type) => type(),
-          optional(:modules) => [module()] | :dynamic
+          optional(:modules) => [module()] | :dynamic,
+          optional(:significant) => boolean()
         }
 
+  @typedoc """
+  A module-based child spec.
+
+  This is a form of child spec that you can pass to functions such as `child_spec/2`,
+  `start_child/2`, and `start_link/2`, in addition to the normalized `t:child_spec/0`.
+
+  A module-based child spec can be:
+
+    * a **module** — the supervisor calls `module.child_spec([])` to retrieve the
+      child specification
+
+    * a **two-element tuple** in the shape of `{module, arg}` — the supervisor
+      calls `module.child_spec(arg)` to retrieve the child specification
+
+  """
+  @typedoc since: "1.16.0"
+  @type module_spec :: {module(), args :: term()} | module()
+
   @doc """
   Starts a supervisor with the given children.
 
   `children` is a list of the following forms:
 
-    * a [child specification](`t:child_spec/0`)
+    * a child specification (see `t:child_spec/0`)
 
-    * a module, where `module.child_spec([])` will be invoked to retrieve
-      its child specification
+    * a module, where the supervisor calls `module.child_spec([])`
+      to retrieve the child specification (see `t:module_spec/0`)
 
-    * a two-element tuple in the shape of `{module, arg}`, where `module.child_spec(arg)`
-      will be invoked to retrieve its child specification
+    * a `{module, arg}` tuple, where the supervisor calls `module.child_spec(arg)`
+      to retrieve the child specification (see `t:module_spec/0`)
+
+    * a (old) Erlang-style child specification (see
+      [`:supervisor.child_spec()`](`t::supervisor.child_spec/0`))
 
   A strategy is required to be provided through the `:strategy` option. See
   "Supervisor strategies and options" for examples and other options.
@@ -604,16 +696,14 @@ defmodule Supervisor do
   with `:normal` reason.
   """
   @spec start_link(
-          [
-            child_spec()
-            | {module, term}
-            | module
-            | (old_erlang_child_spec :: :supervisor.child_spec())
-          ],
+          [child_spec | module_spec | (old_erlang_child_spec :: :supervisor.child_spec())],
           [option | init_option]
-        ) :: {:ok, pid} | {:error, {:already_started, pid} | {:shutdown, term} | term}
+        ) ::
+          {:ok, pid} | {:error, {:already_started, pid} | {:shutdown, term} | term}
   def start_link(children, options) when is_list(children) do
-    {sup_opts, start_opts} = Keyword.split(options, [:strategy, :max_seconds, :max_restarts])
+    {sup_opts, start_opts} =
+      Keyword.split(options, [:strategy, :max_seconds, :max_restarts, :auto_shutdown])
+
     start_link(Supervisor.Default, init(children, sup_opts), start_opts)
   end
 
@@ -648,18 +738,16 @@ defmodule Supervisor do
     * `:max_seconds` - the time frame in seconds in which `:max_restarts`
       applies. Defaults to `5`.
 
+    * `:auto_shutdown` - the automatic shutdown option. It can be either
+      `:never`, `:any_significant`, or `:all_significant`
+
   The `:strategy` option is required and by default a maximum of 3 restarts
   is allowed within 5 seconds. Check the `Supervisor` module for a detailed
   description of the available strategies.
   """
   @doc since: "1.5.0"
   @spec init(
-          [
-            child_spec()
-            | {module, term}
-            | module
-            | (old_erlang_child_spec :: :supervisor.child_spec())
-          ],
+          [child_spec | module_spec | (old_erlang_child_spec :: :supervisor.child_spec())],
           [init_option]
         ) ::
           {:ok,
@@ -683,7 +771,15 @@ defmodule Supervisor do
 
     intensity = Keyword.get(options, :max_restarts, 3)
     period = Keyword.get(options, :max_seconds, 5)
-    flags = %{strategy: strategy, intensity: intensity, period: period}
+    auto_shutdown = Keyword.get(options, :auto_shutdown, :never)
+
+    flags = %{
+      strategy: strategy,
+      intensity: intensity,
+      period: period,
+      auto_shutdown: auto_shutdown
+    }
+
     {:ok, {flags, Enum.map(children, &init_child/1)}}
   end
 
@@ -759,10 +855,10 @@ defmodule Supervisor do
             start: {#{inspect(module)}, :start_link, [arg1, arg2]}
           }
 
-      See the Supervisor documentation for more information.
+      See the Supervisor documentation for more information
       """
     else
-      "The module #{inspect(module)} was given as a child to a supervisor but it does not exist."
+      "The module #{inspect(module)} was given as a child to a supervisor but it does not exist"
     end
   end
 
@@ -779,7 +875,7 @@ defmodule Supervisor do
   `module.child_spec([])`.
 
   After the child specification is retrieved, the fields on `overrides`
-  are directly applied on the child spec. If `overrides` has keys that
+  are directly applied to the child spec. If `overrides` has keys that
   do not map to any child specification field, an error is raised.
 
   See the "Child specification" section in the module documentation
@@ -796,7 +892,7 @@ defmodule Supervisor do
       #=>   start: {Agent, :start_link, [fn -> :ok end]}}
 
   """
-  @spec child_spec(child_spec() | {module, arg :: term} | module, keyword) :: child_spec()
+  @spec child_spec(child_spec() | module_spec(), keyword()) :: child_spec()
   def child_spec(module_or_map, overrides)
 
   def child_spec({_, _, _, _, _, _} = tuple, _overrides) do
@@ -807,7 +903,8 @@ defmodule Supervisor do
 
   def child_spec(module_or_map, overrides) do
     Enum.reduce(overrides, init_child(module_or_map), fn
-      {key, value}, acc when key in [:id, :start, :restart, :shutdown, :type, :modules] ->
+      {key, value}, acc
+      when key in [:id, :start, :restart, :shutdown, :type, :modules, :significant] ->
         Map.put(acc, key, value)
 
       {key, _value}, _acc ->
@@ -892,12 +989,8 @@ defmodule Supervisor do
   """
   @spec start_child(
           supervisor,
-          child_spec()
-          | {module, term}
-          | module
-          | (old_erlang_child_spec :: :supervisor.child_spec())
-        ) ::
-          on_start_child
+          child_spec | module_spec | (old_erlang_child_spec :: :supervisor.child_spec())
+        ) :: on_start_child
   def start_child(supervisor, {_, _, _, _, _, _} = child_spec) do
     call(supervisor, {:start_child, child_spec})
   end
@@ -905,7 +998,9 @@ defmodule Supervisor do
   def start_child(supervisor, args) when is_list(args) do
     IO.warn_once(
       {__MODULE__, :start_child},
-      "Supervisor.start_child/2 with a list of args is deprecated, please use DynamicSupervisor instead",
+      fn ->
+        "Supervisor.start_child/2 with a list of args is deprecated, please use DynamicSupervisor instead"
+      end,
       _stacktrace_drop_levels = 2
     )
 
