@@ -17,6 +17,9 @@ defmodule Task do
   They are implemented by spawning a process that sends a message
   to the caller once the given computation is performed.
 
+  Compared to plain processes, started with `spawn/1`, tasks
+  include monitoring metadata and logging in case of errors.
+
   Besides `async/1` and `await/2`, tasks can also be
   started as part of a supervision tree and dynamically spawned
   on remote nodes. We will explore these scenarios next.
@@ -37,14 +40,42 @@ defmodule Task do
        as they are *always* sent. If you are not expecting a reply,
        consider using `Task.start_link/1` as detailed below.
 
-    2. async tasks link the caller and the spawned process. This
+    2. Async tasks link the caller and the spawned process. This
        means that, if the caller crashes, the task will crash
        too and vice-versa. This is on purpose: if the process
        meant to receive the result no longer exists, there is
-       no purpose in completing the computation.
+       no purpose in completing the computation. If this is not
+       desired, you will want to use supervised tasks, described
+       in a subsequent section.
 
-       If this is not desired, you will want to use supervised
-       tasks, described next.
+  ## Tasks are processes
+
+  Tasks are processes and so data will need to be completely copied
+  to them. Take the following code as an example:
+
+      large_data = fetch_large_data()
+      task = Task.async(fn -> do_some_work(large_data) end)
+      res = do_some_other_work()
+      res + Task.await(task)
+
+  The code above copies over all of `large_data`, which can be
+  resource intensive depending on the size of the data.
+  There are two ways to address this.
+
+  First, if you need to access only part of `large_data`,
+  consider extracting it before the task:
+
+      large_data = fetch_large_data()
+      subset_data = large_data.some_field
+      task = Task.async(fn -> do_some_work(subset_data) end)
+
+  Alternatively, if you can move the data loading altogether
+  to the task, it may be even better:
+
+      task = Task.async(fn ->
+        large_data = fetch_large_data()
+        do_some_work(large_data)
+      end)
 
   ## Dynamically supervised tasks
 
@@ -78,13 +109,13 @@ defmodule Task do
       |> Task.await()
 
   We encourage developers to rely on supervised tasks as much as possible.
-  Supervised tasks improves the visibility of how many tasks are running
-  at a given moment and enable a huge variety of patterns that gives you
+  Supervised tasks improve the visibility of how many tasks are running
+  at a given moment and enable a variety of patterns that give you
   explicit control on how to handle the results, errors, and timeouts.
   Here is a summary:
 
     * Using `Task.Supervisor.start_child/2` allows you to start a fire-and-forget
-      task that you don't care about its results or if it completes successfully or not.
+      task when you don't care about its results or if it completes successfully or not.
 
     * Using `Task.Supervisor.async/2` + `Task.await/2` allows you to execute
       tasks concurrently and retrieve its result. If the task fails,
@@ -96,22 +127,22 @@ defmodule Task do
       the caller won't fail. You will receive the error reason either on
       `yield` or `shutdown`.
 
-  Furthermore, the supervisor guarantee all tasks first terminate, within a
-  configurable shutdown period, when your application shuts down. See the
+  Furthermore, the supervisor guarantees all tasks terminate within a
+  configurable shutdown period when your application shuts down. See the
   `Task.Supervisor` module for details on the supported operations.
 
   ### Distributed tasks
 
   With `Task.Supervisor`, it is easy to dynamically start tasks across nodes:
 
-      # On the remote node named :remote@local
+      # First on the remote node named :remote@local
       Task.Supervisor.start_link(name: MyApp.DistSupervisor)
 
-      # On the client
+      # Then on the local client node
       supervisor = {MyApp.DistSupervisor, :remote@local}
       Task.Supervisor.async(supervisor, MyMod, :my_fun, [arg1, arg2, arg3])
 
-  Note that, when working with distributed tasks, one should use the
+  Note that, as above, when working with distributed tasks, one should use the
   `Task.Supervisor.async/5` function that expects explicit module, function,
   and arguments, instead of `Task.Supervisor.async/3` that works with anonymous
   functions. That's because anonymous functions expect the same module version
@@ -161,6 +192,12 @@ defmodule Task do
   and `Task.start_link/1` are for fire-and-forget tasks, where you don't
   care about the results or if it completes successfully or not.
 
+  > #### `use Task` {: .info}
+  >
+  > When you `use Task`, the `Task` module will define a
+  > `child_spec/1` function, so your module can be used
+  > as a child in a supervision tree.
+
   `use Task` defines a `child_spec/1` function, allowing the
   defined module to be put under a supervision tree. The generated
   `child_spec/1` can be customized with the following options:
@@ -212,7 +249,7 @@ defmodule Task do
 
   The list of callers of the current process can be retrieved from the Process
   dictionary with `Process.get(:"$callers")`. This will return either `nil` or
-  a list `[pid_n, ..., pid2, pid1]` with at least one entry Where `pid_n` is
+  a list `[pid_n, ..., pid2, pid1]` with at least one entry where `pid_n` is
   the PID that called the current process, `pid2` called `pid_n`, and `pid2` was
   called by `pid1`.
 
@@ -230,10 +267,10 @@ defmodule Task do
 
     * `:owner` - the PID of the process that started the task
 
-    * `:pid` - the PID of the task process; `nil` if the task does
-      not use a task process
+    * `:pid` - the PID of the task process; `nil` if there is no process
+      specifically assigned for the task
 
-    * `:ref` - the task monitor reference
+    * `:ref` - an opaque term used as the task monitor reference
 
   """
   @enforce_keys [:mfa, :owner, :pid, :ref]
@@ -248,9 +285,25 @@ defmodule Task do
           mfa: mfa(),
           owner: pid(),
           pid: pid() | nil,
-          ref: reference()
+          ref: ref()
         }
 
+  @typedoc """
+  The task opaque reference.
+  """
+  @opaque ref :: reference()
+
+  @typedoc """
+  Options given to `async_stream` functions.
+  """
+  @typedoc since: "1.17.0"
+  @type async_stream_option ::
+          {:max_concurrency, pos_integer()}
+          | {:ordered, boolean()}
+          | {:timeout, timeout()}
+          | {:on_timeout, :exit | :kill_task}
+          | {:zip_input_on_exit, boolean()}
+
   defguardp is_timeout(timeout)
             when timeout == :infinity or (is_integer(timeout) and timeout >= 0)
 
@@ -276,7 +329,7 @@ defmodule Task do
   @doc false
   defmacro __using__(opts) do
     quote location: :keep, bind_quoted: [opts: opts] do
-      unless Module.has_attribute?(__MODULE__, :doc) do
+      if not Module.has_attribute?(__MODULE__, :doc) do
         @doc """
         Returns a specification to start this module under a supervisor.
 
@@ -309,7 +362,7 @@ defmodule Task do
 
   This is used to start a statically supervised task under a supervision tree.
   """
-  @spec start_link((() -> any)) :: {:ok, pid}
+  @spec start_link((-> any)) :: {:ok, pid}
   def start_link(fun) when is_function(fun, 0) do
     start_link(:erlang, :apply, [fun, []])
   end
@@ -341,7 +394,7 @@ defmodule Task do
   to use `Task.Supervisor.start_child/2` instead, which allows
   you to control the shutdown time via the `:shutdown` option.
   """
-  @spec start((() -> any)) :: {:ok, pid}
+  @spec start((-> any)) :: {:ok, pid}
   def start(fun) when is_function(fun, 0) do
     start(:erlang, :apply, [fun, []])
   end
@@ -371,8 +424,14 @@ defmodule Task do
   `fun` must be a zero-arity anonymous function. This function
   spawns a process that is linked to and monitored by the caller
   process. A `Task` struct is returned containing the relevant
-  information. Developers must eventually call `Task.await/2` or
-  `Task.yield/2` followed by `Task.shutdown/2` on the returned task.
+  information.
+
+  If you start an `async`, you **must await**. This is either done
+  by calling `Task.await/2` or `Task.yield/2` followed by
+  `Task.shutdown/2` on the returned task. Alternatively, if you
+  spawn a task inside a `GenServer`, then the `GenServer` will
+  automatically await for you and call `c:GenServer.handle_info/2`
+  with the task response and associated `:DOWN` message.
 
   Read the `Task` module documentation for more information about
   the general usage of async tasks.
@@ -431,14 +490,11 @@ defmodule Task do
   the anonymous function. Use `async/3` if you want another function
   to be used as metadata.
   """
-  @spec async((() -> any)) :: t
+  @spec async((-> any)) :: t
   def async(fun) when is_function(fun, 0) do
     async(:erlang, :apply, [fun, []])
   end
 
-  # TODO: Remove conditional on Erlang/OTP 24
-  @compile {:no_warn_undefined, {:erlang, :monitor, 3}}
-
   @doc """
   Starts a task that must be awaited on.
 
@@ -452,18 +508,12 @@ defmodule Task do
       when is_atom(module) and is_atom(function_name) and is_list(args) do
     mfargs = {module, function_name, args}
     owner = self()
+    # No need to monitor because the processes are linked
     {:ok, pid} = Task.Supervised.start_link(get_owner(owner), :nomonitor)
 
-    {reply_to, ref} =
-      if function_exported?(:erlang, :monitor, 3) do
-        ref = :erlang.monitor(:process, pid, alias: :demonitor)
-        {ref, ref}
-      else
-        {owner, Process.monitor(pid)}
-      end
-
-    send(pid, {owner, ref, reply_to, get_callers(owner), mfargs})
-    %Task{pid: pid, ref: ref, owner: owner, mfa: {module, function_name, length(args)}}
+    alias = build_alias(pid)
+    send(pid, {owner, alias, alias, get_callers(owner), mfargs})
+    %Task{pid: pid, ref: alias, owner: owner, mfa: {module, function_name, length(args)}}
   end
 
   @doc """
@@ -560,7 +610,7 @@ defmodule Task do
     * `:zip_input_on_exit` - (since v1.14.0) adds the original
       input to `:exit` tuples. The value emitted for that task is
       `{:exit, {input, reason}}`, where `input` is the collection element
-      that caused an exited during processing. Defaults to `false`.
+      that caused an exit during processing. Defaults to `false`.
 
   ## Example
 
@@ -642,7 +692,8 @@ defmodule Task do
   example above.
   """
   @doc since: "1.4.0"
-  @spec async_stream(Enumerable.t(), module, atom, [term], keyword) :: Enumerable.t()
+  @spec async_stream(Enumerable.t(), module, atom, [term], [async_stream_option]) ::
+          Enumerable.t()
   def async_stream(enumerable, module, function_name, args, options \\ [])
       when is_atom(module) and is_atom(function_name) and is_list(args) do
     build_stream(enumerable, {module, function_name, args}, options)
@@ -665,23 +716,26 @@ defmodule Task do
 
       iex> strings = ["long string", "longer string", "there are many of these"]
       iex> stream = Task.async_stream(strings, fn text -> text |> String.codepoints() |> Enum.count() end)
-      iex> Enum.reduce(stream, 0, fn {:ok, num}, acc -> num + acc end)
+      iex> Enum.sum_by(stream, fn {:ok, num} -> num end)
       47
 
   See `async_stream/5` for discussion, options, and more examples.
   """
   @doc since: "1.4.0"
-  @spec async_stream(Enumerable.t(), (term -> term), keyword) :: Enumerable.t()
+  @spec async_stream(Enumerable.t(), (term -> term), [async_stream_option]) :: Enumerable.t()
   def async_stream(enumerable, fun, options \\ [])
       when is_function(fun, 1) and is_list(options) do
     build_stream(enumerable, fun, options)
   end
 
   defp build_stream(enumerable, fun, options) do
+    options = Task.Supervised.validate_stream_options(options)
+
     fn acc, acc_fun ->
       owner = get_owner(self())
 
       Task.Supervised.stream(enumerable, acc, acc_fun, get_callers(self()), fun, options, fn ->
+        # No need to monitor because the processes are linked
         {:ok, pid} = Task.Supervised.start_link(owner, :nomonitor)
         {:ok, :link, pid}
       end)
@@ -784,18 +838,18 @@ defmodule Task do
 
         # If the task succeeds...
         def handle_info({ref, result}, state) do
-          # The task succeed so we can cancel the monitoring and discard the DOWN message
+          # The task succeed so we can demonitor its reference
           Process.demonitor(ref, [:flush])
 
           {url, state} = pop_in(state.tasks[ref])
-          IO.puts "Got #{inspect(result)} for URL #{inspect url}"
+          IO.puts("Got #{inspect(result)} for URL #{inspect url}")
           {:noreply, state}
         end
 
         # If the task fails...
         def handle_info({:DOWN, ref, _, _, reason}, state) do
           {url, state} = pop_in(state.tasks[ref])
-          IO.puts "URL #{inspect url} failed with reason #{inspect(reason)}"
+          IO.puts("URL #{inspect url} failed with reason #{inspect(reason)}")
           {:noreply, state}
         end
       end
@@ -817,16 +871,20 @@ defmodule Task do
       raise ArgumentError, invalid_owner_error(task)
     end
 
+    await_receive(ref, task, timeout)
+  end
+
+  defp await_receive(ref, task, timeout) do
     receive do
       {^ref, reply} ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
         reply
 
       {:DOWN, ^ref, _, proc, reason} ->
         exit({reason(reason, proc), {__MODULE__, :await, [task, timeout]}})
     after
       timeout ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
         exit({:timeout, {__MODULE__, :await, [task, timeout]}})
     end
   end
@@ -843,23 +901,22 @@ defmodule Task do
   Important: avoid using [`Task.async/1,3`](`async/1`) and then immediately ignoring
   the task. If you want to start tasks you don't care about their
   results, use `Task.Supervisor.start_child/2` instead.
-
-  Requires Erlang/OTP 24+.
   """
   @doc since: "1.13.0"
+  @spec ignore(t) :: {:ok, term} | {:exit, term} | nil
   def ignore(%Task{ref: ref, pid: pid, owner: owner} = task) do
-    unless function_exported?(:erlang, :monitor, 3) do
-      raise "Task.ignore/1 requires Erlang/OTP 24+"
-    end
-
     if owner != self() do
       raise ArgumentError, invalid_owner_error(task)
     end
 
+    ignore_receive(ref, pid, task)
+  end
+
+  defp ignore_receive(ref, pid, task) do
     receive do
       {^ref, reply} ->
-        Process.unlink(pid)
-        Process.demonitor(ref, [:flush])
+        pid && Process.unlink(pid)
+        demonitor(ref)
         {:ok, reply}
 
       {:DOWN, ^ref, _, proc, :noconnection} ->
@@ -869,8 +926,8 @@ defmodule Task do
         {:exit, reason}
     after
       0 ->
-        Process.unlink(pid)
-        Process.demonitor(ref, [:flush])
+        pid && Process.unlink(pid)
+        demonitor(ref)
         nil
     end
   end
@@ -919,15 +976,13 @@ defmodule Task do
   @spec await_many([t], timeout) :: [term]
   def await_many(tasks, timeout \\ 5000) when is_timeout(timeout) do
     awaiting =
-      for task <- tasks, into: %{} do
-        %Task{ref: ref, owner: owner} = task
-
+      Map.new(tasks, fn %Task{ref: ref, owner: owner} = task ->
         if owner != self() do
           raise ArgumentError, invalid_owner_error(task)
         end
 
         {ref, true}
-      end
+      end)
 
     timeout_ref = make_ref()
 
@@ -960,7 +1015,7 @@ defmodule Task do
         exit({reason(reason, proc), {__MODULE__, :await_many, [tasks, timeout]}})
 
       {ref, reply} when is_map_key(awaiting, ref) ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
 
         await_many(
           tasks,
@@ -974,7 +1029,7 @@ defmodule Task do
 
   defp demonitor_pending_tasks(awaiting) do
     Enum.each(awaiting, fn {ref, _} ->
-      Process.demonitor(ref, [:flush])
+      demonitor(ref)
     end)
   end
 
@@ -983,7 +1038,7 @@ defmodule Task do
   def find(tasks, {ref, reply}) when is_reference(ref) do
     Enum.find_value(tasks, fn
       %Task{ref: ^ref} = task ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
         {reply, task}
 
       %Task{} ->
@@ -1035,7 +1090,7 @@ defmodule Task do
           result
 
         nil ->
-          Logger.warn("Failed to get a result in #{timeout}ms")
+          Logger.warning("Failed to get a result in #{timeout}ms")
           nil
       end
 
@@ -1047,7 +1102,7 @@ defmodule Task do
           result
 
         nil ->
-          Logger.warn("Failed to get a result in #{timeout}ms")
+          Logger.warning("Failed to get a result in #{timeout}ms")
           nil
       end
 
@@ -1061,9 +1116,13 @@ defmodule Task do
       raise ArgumentError, invalid_owner_error(task)
     end
 
+    yield_receive(ref, task, timeout)
+  end
+
+  defp yield_receive(ref, task, timeout) do
     receive do
       {^ref, reply} ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
         {:ok, reply}
 
       {:DOWN, ^ref, _, proc, :noconnection} ->
@@ -1092,17 +1151,15 @@ defmodule Task do
     * `{:ok, term}` if the task has successfully reported its
       result back in the given time interval
     * `{:exit, reason}` if the task has died
-    * `nil` if the task keeps running past the timeout
-
-  A timeout, in milliseconds or `:infinity`, can be given with a default value
-  of `5000`.
+    * `nil` if the task keeps running, either because a limit
+      has been reached or past the timeout
 
   Check `yield/2` for more information.
 
   ## Example
 
   `Task.yield_many/2` allows developers to spawn multiple tasks
-  and retrieve the results received in a given timeframe.
+  and retrieve the results received in a given time frame.
   If we combine it with `Task.shutdown/2` (or `Task.ignore/1`),
   it allows us to gather those results and cancel (or ignore)
   the tasks that have not replied in time.
@@ -1117,7 +1174,7 @@ defmodule Task do
           end)
         end
 
-      tasks_with_results = Task.yield_many(tasks, 5000)
+      tasks_with_results = Task.yield_many(tasks, timeout: 5000)
 
       results =
         Enum.map(tasks_with_results, fn {task, res} ->
@@ -1137,9 +1194,57 @@ defmodule Task do
   printed, as those were the tasks that have replied in the
   given time. All other tasks will have been shut down using
   the `Task.shutdown/2` call.
+
+  As a convenience, you can achieve a similar behavior to above
+  by specifying the `:on_timeout` option to be `:kill_task` (or
+  `:ignore`). See `Task.await_many/2` if you would rather exit
+  the caller process on timeout.
+
+  ## Options
+
+  The second argument is either a timeout or options, which defaults
+  to this:
+
+    * `:limit` - the maximum amount of tasks to wait for.
+      If the limit is reached before the timeout, this function
+      returns immediately without triggering the `:on_timeout` behaviour
+
+    * `:timeout` - the maximum amount of time (in milliseconds or `:infinity`)
+      each task is allowed to execute for. Defaults to `5000`.
+
+    * `:on_timeout` - what to do when a task times out. The possible
+      values are:
+      * `:nothing` - do nothing (default). The tasks can still be
+        awaited on, yielded on, ignored, or shut down later.
+      * `:ignore` - the results of the task will be ignored.
+      * `:kill_task` - the task that timed out is killed.
   """
   @spec yield_many([t], timeout) :: [{t, {:ok, term} | {:exit, term} | nil}]
-  def yield_many(tasks, timeout \\ 5000) when is_timeout(timeout) do
+  @spec yield_many([t],
+          limit: pos_integer(),
+          timeout: timeout,
+          on_timeout: :nothing | :ignore | :kill_task
+        ) ::
+          [{t, {:ok, term} | {:exit, term} | nil}]
+  def yield_many(tasks, opts \\ [])
+
+  def yield_many(tasks, timeout) when is_timeout(timeout) do
+    yield_many(tasks, timeout: timeout)
+  end
+
+  def yield_many(tasks, opts) when is_list(opts) do
+    refs =
+      Map.new(tasks, fn %Task{ref: ref, owner: owner} = task ->
+        if owner != self() do
+          raise ArgumentError, invalid_owner_error(task)
+        end
+
+        {ref, nil}
+      end)
+
+    on_timeout = Keyword.get(opts, :on_timeout, :nothing)
+    timeout = Keyword.get(opts, :timeout, 5_000)
+    limit = Keyword.get(opts, :limit, map_size(refs))
     timeout_ref = make_ref()
 
     timer_ref =
@@ -1148,44 +1253,52 @@ defmodule Task do
       end
 
     try do
-      yield_many(tasks, timeout_ref, :infinity)
+      yield_many(limit, refs, timeout_ref, timer_ref)
     catch
       {:noconnection, reason} ->
         exit({reason, {__MODULE__, :yield_many, [tasks, timeout]}})
-    after
-      timer_ref && Process.cancel_timer(timer_ref)
-      receive do: (^timeout_ref -> :ok), after: (0 -> :ok)
+    else
+      {timed_out?, refs} ->
+        for task <- tasks do
+          value =
+            with nil <- Map.fetch!(refs, task.ref) do
+              case on_timeout do
+                _ when not timed_out? -> nil
+                :nothing -> nil
+                :kill_task -> shutdown(task, :brutal_kill)
+                :ignore -> ignore(task)
+              end
+            end
+
+          {task, value}
+        end
     end
   end
 
-  defp yield_many([%Task{ref: ref, owner: owner} = task | rest], timeout_ref, timeout) do
-    if owner != self() do
-      raise ArgumentError, invalid_owner_error(task)
-    end
+  defp yield_many(0, refs, timeout_ref, timer_ref) do
+    timer_ref && Process.cancel_timer(timer_ref)
+    receive do: (^timeout_ref -> :ok), after: (0 -> :ok)
+    {false, refs}
+  end
 
+  defp yield_many(limit, refs, timeout_ref, timer_ref) do
     receive do
-      {^ref, reply} ->
-        Process.demonitor(ref, [:flush])
-        [{task, {:ok, reply}} | yield_many(rest, timeout_ref, timeout)]
+      {ref, reply} when is_map_key(refs, ref) ->
+        demonitor(ref)
+        yield_many(limit - 1, %{refs | ref => {:ok, reply}}, timeout_ref, timer_ref)
 
-      {:DOWN, ^ref, _, proc, :noconnection} ->
-        throw({:noconnection, reason(:noconnection, proc)})
-
-      {:DOWN, ^ref, _, _, reason} ->
-        [{task, {:exit, reason}} | yield_many(rest, timeout_ref, timeout)]
+      {:DOWN, ref, _, proc, reason} when is_map_key(refs, ref) ->
+        if reason == :noconnection do
+          throw({:noconnection, reason(:noconnection, proc)})
+        else
+          yield_many(limit - 1, %{refs | ref => {:exit, reason}}, timeout_ref, timer_ref)
+        end
 
       ^timeout_ref ->
-        [{task, nil} | yield_many(rest, timeout_ref, 0)]
-    after
-      timeout ->
-        [{task, nil} | yield_many(rest, timeout_ref, 0)]
+        {true, refs}
     end
   end
 
-  defp yield_many([], _timeout_ref, _timeout) do
-    []
-  end
-
   @doc """
   Unlinks and shuts down the task, and then checks for a reply.
 
@@ -1206,6 +1319,10 @@ defmodule Task do
   `:shutdown` to shut down all of its linked processes, including tasks, that
   are not trapping exits without generating any log messages.
 
+  If there is no process linked to the task, such as tasks started by
+  `Task.completed/1`, we check for a response or error accordingly, but without
+  shutting a process down.
+
   If a task's monitor has already been demonitored or received and there is not
   a response waiting in the message queue this function will return
   `{:exit, :noproc}` as the result or exit reason can not be determined.
@@ -1214,18 +1331,18 @@ defmodule Task do
   def shutdown(task, shutdown \\ 5000)
 
   def shutdown(%Task{pid: nil} = task, _) do
-    raise ArgumentError, "task #{inspect(task)} does not have an associated task process"
+    ignore(task)
   end
 
   def shutdown(%Task{owner: owner} = task, _) when owner != self() do
     raise ArgumentError, invalid_owner_error(task)
   end
 
-  def shutdown(%Task{pid: pid} = task, :brutal_kill) do
-    mon = Process.monitor(pid)
-    exit(pid, :kill)
+  def shutdown(%Task{pid: pid, ref: ref} = task, :brutal_kill) do
+    mon = build_monitor(pid)
+    shutdown_send(pid, :kill)
 
-    case shutdown_receive(task, mon, :brutal_kill, :infinity) do
+    case shutdown_receive(ref, mon, task, :brutal_kill, :infinity) do
       {:down, proc, :noconnection} ->
         exit({reason(:noconnection, proc), {__MODULE__, :shutdown, [task, :brutal_kill]}})
 
@@ -1237,11 +1354,11 @@ defmodule Task do
     end
   end
 
-  def shutdown(%Task{pid: pid} = task, timeout) when is_timeout(timeout) do
-    mon = Process.monitor(pid)
-    exit(pid, :shutdown)
+  def shutdown(%Task{pid: pid, ref: ref} = task, timeout) when is_timeout(timeout) do
+    mon = build_monitor(pid)
+    shutdown_send(pid, :shutdown)
 
-    case shutdown_receive(task, mon, :shutdown, timeout) do
+    case shutdown_receive(ref, mon, task, :shutdown, timeout) do
       {:down, proc, :noconnection} ->
         exit({reason(:noconnection, proc), {__MODULE__, :shutdown, [task, timeout]}})
 
@@ -1253,27 +1370,19 @@ defmodule Task do
     end
   end
 
-  ## Helpers
-
-  defp reason(:noconnection, proc), do: {:nodedown, monitor_node(proc)}
-  defp reason(reason, _), do: reason
-
-  defp monitor_node(pid) when is_pid(pid), do: node(pid)
-  defp monitor_node({_, node}), do: node
-
-  # spawn a process to ensure task gets exit signal if process dies from exit signal
-  # between unlink and exit.
-  defp exit(task, reason) do
+  # Spawn a process to ensure task gets exit signal
+  # if process dies from exit signal between unlink and exit.
+  defp shutdown_send(pid, reason) do
     caller = self()
     ref = make_ref()
-    enforcer = spawn(fn -> enforce_exit(task, reason, caller, ref) end)
-    Process.unlink(task)
-    Process.exit(task, reason)
+    enforcer = spawn(fn -> shutdown_send(pid, reason, caller, ref) end)
+    Process.unlink(pid)
+    Process.exit(pid, reason)
     send(enforcer, {:done, ref})
     :ok
   end
 
-  defp enforce_exit(pid, reason, caller, ref) do
+  defp shutdown_send(pid, reason, caller, ref) do
     mon = Process.monitor(caller)
 
     receive do
@@ -1282,14 +1391,14 @@ defmodule Task do
     end
   end
 
-  defp shutdown_receive(%{ref: ref} = task, mon, type, timeout) do
+  defp shutdown_receive(ref, mon, task, type, timeout) do
     receive do
       {:DOWN, ^mon, _, _, :shutdown} when type in [:shutdown, :timeout_kill] ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
         flush_reply(ref)
 
       {:DOWN, ^mon, _, _, :killed} when type == :brutal_kill ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
         flush_reply(ref)
 
       {:DOWN, ^mon, _, proc, :noproc} ->
@@ -1297,12 +1406,12 @@ defmodule Task do
         flush_reply(ref) || reason
 
       {:DOWN, ^mon, _, proc, reason} ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
         flush_reply(ref) || {:down, proc, reason}
     after
       timeout ->
         Process.exit(task.pid, :kill)
-        shutdown_receive(task, mon, :timeout_kill, :infinity)
+        shutdown_receive(ref, mon, task, :timeout_kill, :infinity)
     end
   end
 
@@ -1326,11 +1435,53 @@ defmodule Task do
         {:down, proc, reason}
     after
       0 ->
-        Process.demonitor(ref, [:flush])
+        demonitor(ref)
         {:down, proc, :noproc}
     end
   end
 
+  ## Optimizations
+
+  defp build_monitor(pid) do
+    :erlang.monitor(:process, pid)
+  end
+
+  defp build_alias(pid) do
+    :erlang.monitor(:process, pid, alias: :demonitor)
+  end
+
+  @doc false
+  # This instructs the Erlang compiler to apply selective
+  # receive optimizations to several functions in this module.
+  # This function is never invoked directly, it is only here
+  # for compiler optimization purposes.
+  #
+  # To verify which functions have been optimized, run the
+  # following command after Elixir is compiled from the project
+  # root:
+  #
+  #     ERL_COMPILER_OPTIONS=recv_opt_info elixir lib/elixir/lib/task.ex
+  #
+  def __recv_opt_info__(pid, task) do
+    await_receive(build_alias(pid), task, :infinity)
+    shutdown_receive(build_alias(pid), build_monitor(pid), task, :shutdown, :infinity)
+    yield_receive(build_alias(pid), task, :infinity)
+    ignore_receive(build_alias(pid), pid, task)
+  end
+
+  ## Helpers
+
+  defp demonitor(ref) when is_reference(ref) do
+    Process.demonitor(ref, [:flush])
+    :ok
+  end
+
+  defp reason(:noconnection, proc), do: {:nodedown, monitor_node(proc)}
+  defp reason(reason, _), do: reason
+
+  defp monitor_node(pid) when is_pid(pid), do: node(pid)
+  defp monitor_node({_, node}), do: node
+
   defp invalid_owner_error(task) do
     "task #{inspect(task)} must be queried from the owner but was queried from #{inspect(self())}"
   end
