@@ -5,6 +5,9 @@ defmodule URI do
   This module provides functions for working with URIs (for example, parsing
   URIs or encoding query strings). The functions in this module are implemented
   according to [RFC 3986](https://tools.ietf.org/html/rfc3986).
+
+  Additionally, the Erlang [`:uri_string` module](`:uri_string`) provides certain functionalities,
+  such as RFC 3986 compliant URI normalization.
   """
 
   @doc """
@@ -38,6 +41,12 @@ defmodule URI do
   @opaque authority :: nil | binary
 
   defmodule Error do
+    @moduledoc """
+    An exception raised when an error occurs when a `URI` is invalid.
+
+    For example, see `URI.new!/1`.
+    """
+
     defexception [:action, :reason, :part]
 
     @doc false
@@ -48,7 +57,7 @@ defmodule URI do
 
   import Bitwise
 
-  @reserved_characters ':/?#[]@!$&\'()*+,;='
+  @reserved_characters ~c":/?#[]@!$&'()*+,;="
   @formatted_reserved_characters Enum.map_join(@reserved_characters, ", ", &<<?`, &1, ?`>>)
 
   @doc """
@@ -229,7 +238,7 @@ defmodule URI do
 
       {{key, value}, rest} ->
         # Avoid warnings about Dict being deprecated
-        dict_module = Dict
+        dict_module = String.to_atom("Dict")
         decode_query_into_dict(rest, dict_module.put(dict, key, value), encoding)
     end
   end
@@ -334,7 +343,7 @@ defmodule URI do
   """
   @spec char_unreserved?(byte) :: boolean
   def char_unreserved?(character) do
-    character in ?0..?9 or character in ?a..?z or character in ?A..?Z or character in '~_-.'
+    character in ?0..?9 or character in ?a..?z or character in ?A..?Z or character in ~c"~_-."
   end
 
   @doc """
@@ -356,21 +365,29 @@ defmodule URI do
   end
 
   @doc """
-  Percent-escapes all characters that require escaping in `string`.
+  Percent-encodes all characters that require escaping in `string`.
 
-  This means reserved characters, such as `:` and `/`, and the
-  so-called unreserved characters, which have the same meaning both
-  escaped and unescaped, won't be escaped by default.
+  The optional `predicate` argument specifies a function used to detect whether
+  a byte in the `string` should be escaped:
 
-  See `encode_www_form/1` if you are interested in escaping reserved
-  characters too.
+    * if the function returns a truthy value, the byte should be kept as-is.
+    * if the function returns a falsy value, the byte should be escaped.
 
-  This function also accepts a `predicate` function as an optional
-  argument. If passed, this function will be called with each byte
-  in `string` as its argument and should return a truthy value (anything other
-  than `false` or `nil`) if the given byte should be left as is, or return a
-  falsy value (`false` or `nil`) if the character should be escaped. Defaults
-  to `URI.char_unescaped?/1`.
+  The `predicate` argument can use some built-in functions:
+
+    * `URI.char_unescaped?/1` (default) - reserved characters (such as `:`
+      and `/`) or unreserved (such as letters and numbers) are kept as-is.
+      It's typically used to encode the whole URI.
+    * `URI.char_unreserved?/1` - unreserved characters (such as letters and
+      numbers) are kept as-is. It's typically used to encode components in
+      a URI, such as query or fragment.
+    * `URI.char_reserved?/1` - Reserved characters (such as `:` and `/`) are
+      kept as-is.
+
+  And, you can also use custom functions.
+
+  See `encode_www_form/1` if you are interested in encoding `string` as
+  "x-www-form-urlencoded".
 
   ## Examples
 
@@ -648,16 +665,16 @@ defmodule URI do
         scheme = String.downcase(scheme, :ascii)
 
         case map do
-          %{port: _} ->
+          %{port: port} when is_integer(port) ->
             %{uri | scheme: scheme}
 
           %{} ->
-            case default_port(scheme) do
-              nil -> %{uri | scheme: scheme}
-              port -> %{uri | scheme: scheme, port: port}
-            end
+            %{uri | scheme: scheme, port: default_port(scheme)}
         end
 
+      %{port: :undefined} ->
+        %{uri | port: nil}
+
       %{} ->
         uri
     end
@@ -680,8 +697,9 @@ defmodule URI do
   If a `%URI{}` struct is given to this function, this function returns it
   unmodified.
 
-  > Note: this function sets the field :authority for backwards
-  > compatibility reasons but it is deprecated.
+  > #### `:authority` field {: .info}
+  >
+  > This function sets the deprecated field `:authority` for backwards-compatibility reasons.
 
   ## Examples
 
@@ -889,7 +907,7 @@ defmodule URI do
     %{rel | path: remove_dot_segments_from_path(rel.path)}
   end
 
-  def merge(base, %URI{host: host} = rel) when host != nil do
+  def merge(%URI{} = base, %URI{host: host} = rel) when host != nil do
     %{rel | scheme: base.scheme, path: remove_dot_segments_from_path(rel.path)}
   end
 
@@ -910,37 +928,42 @@ defmodule URI do
   defp merge_paths(_, "/" <> _ = rel_path), do: remove_dot_segments_from_path(rel_path)
 
   defp merge_paths(base_path, rel_path) do
-    [_ | base_segments] = path_to_segments(base_path)
-
-    path_to_segments(rel_path)
-    |> Kernel.++(base_segments)
+    (path_to_segments(base_path) ++ [:+] ++ path_to_segments(rel_path))
     |> remove_dot_segments([])
-    |> Enum.join("/")
+    |> join_reversed_segments()
   end
 
-  defp remove_dot_segments_from_path(nil) do
-    nil
-  end
+  defp remove_dot_segments_from_path(nil), do: nil
 
   defp remove_dot_segments_from_path(path) do
-    path
-    |> path_to_segments()
+    path_to_segments(path)
     |> remove_dot_segments([])
-    |> Enum.join("/")
+    |> join_reversed_segments()
+  end
+
+  defp path_to_segments(path) do
+    case String.split(path, "/") do
+      ["" | tail] -> [:/ | tail]
+      segments -> segments
+    end
   end
 
-  defp remove_dot_segments([], [head, ".." | acc]), do: remove_dot_segments([], [head | acc])
   defp remove_dot_segments([], acc), do: acc
+  defp remove_dot_segments([:/ | tail], acc), do: remove_dot_segments(tail, [:/ | acc])
+  defp remove_dot_segments(["."], acc), do: remove_dot_segments([], ["" | acc])
   defp remove_dot_segments(["." | tail], acc), do: remove_dot_segments(tail, acc)
-
-  defp remove_dot_segments([head | tail], ["..", ".." | _] = acc),
-    do: remove_dot_segments(tail, [head | acc])
-
-  defp remove_dot_segments(segments, [_, ".." | acc]), do: remove_dot_segments(segments, acc)
+  defp remove_dot_segments([".." | tail], [:/]), do: remove_dot_segments(tail, [:/])
+  defp remove_dot_segments([".."], [_ | acc]), do: remove_dot_segments([], ["" | acc])
+  defp remove_dot_segments([".." | tail], [_ | acc]), do: remove_dot_segments(tail, acc)
+  defp remove_dot_segments([_, :+ | tail], acc), do: remove_dot_segments(tail, acc)
   defp remove_dot_segments([head | tail], acc), do: remove_dot_segments(tail, [head | acc])
 
-  defp path_to_segments(path) do
-    path |> String.split("/") |> Enum.reverse()
+  defp join_reversed_segments(segments) do
+    case Enum.reverse(segments) do
+      [:/ | tail] -> ["" | tail]
+      list -> list
+    end
+    |> Enum.join("/")
   end
 
   @doc """
@@ -972,6 +995,40 @@ defmodule URI do
       %{uri | query: uri.query <> "&" <> query}
     end
   end
+
+  @doc """
+  Appends `path` to the given `uri`.
+
+  Path must start with `/` and cannot contain additional URL components like
+  fragments or query strings. This function further assumes the path is valid and
+  it does not contain a query string or fragment parts.
+
+  ## Examples
+
+      iex> URI.append_path(URI.parse("http://example.com/foo/?x=1"), "/my-path") |> URI.to_string()
+      "http://example.com/foo/my-path?x=1"
+
+      iex> URI.append_path(URI.parse("http://example.com"), "my-path")
+      ** (ArgumentError) path must start with "/", got: "my-path"
+
+  """
+  @doc since: "1.15.0"
+  @spec append_path(t(), String.t()) :: t()
+  def append_path(%URI{}, "//" <> _ = path) do
+    raise ArgumentError, ~s|path cannot start with "//", got: #{inspect(path)}|
+  end
+
+  def append_path(%URI{path: path} = uri, "/" <> rest = all) do
+    cond do
+      path == nil -> %{uri | path: all}
+      path != "" and :binary.last(path) == ?/ -> %{uri | path: path <> rest}
+      true -> %{uri | path: path <> all}
+    end
+  end
+
+  def append_path(%URI{}, path) when is_binary(path) do
+    raise ArgumentError, ~s|path must start with "/", got: #{inspect(path)}|
+  end
 end
 
 defimpl String.Chars, for: URI do
