@@ -18,9 +18,9 @@
 'MACRO-defmacro'(Caller, Call, Expr) -> define(Caller, defmacro, Call, Expr).
 'MACRO-defmacrop'(Caller, Call, Expr) -> define(Caller, defmacrop, Call, Expr).
 
-'MACRO-defmodule'(_Caller, Alias, [{do, Block}]) ->
+'MACRO-defmodule'({Line, _S, _E} = _Caller, Alias, [{do, Block}]) ->
   Escaped = elixir_quote:escape(Block, none, false),
-  Args = [Alias, Escaped, [], env()],
+  Args = [[{line, Line}], Alias, Escaped, [], false, env()],
   {{'.', [], [elixir_module, compile]}, [], Args}.
 
 '__info__'(functions) ->
@@ -35,18 +35,28 @@
    {defmodule, 2},
    {defp, 2}].
 
-define({Line, _S, E}, Kind, Call, Expr) ->
+define({Line, _S, #{module := Module} = E}, Kind, Call, Expr) ->
   UC = elixir_quote:has_unquotes(Call),
   UE = elixir_quote:has_unquotes(Expr),
-  EscapedCall = elixir_quote:escape(Call, none, true),
-  EscapedExpr = elixir_quote:escape(Expr, none, true),
-  Args = [Kind, not(UC or UE), EscapedCall, EscapedExpr, elixir_locals:cache_env(E#{line := Line})],
+
+  Store =
+    case UC or UE of
+      true ->
+        elixir_quote:escape({Call, Expr}, none, true);
+
+      false ->
+        Key = erlang:unique_integer(),
+        elixir_module:write_cache(Module, Key, {Call, Expr}),
+        Key
+    end,
+
+  Args = [Kind, Store, elixir_module:cache_env(E#{line := Line})],
   {{'.', [], [elixir_def, store_definition]}, [], Args}.
 
 unless_loaded(Fun, Args, Callback) ->
-  case code:is_loaded(?kernel) of
-    {_, _} -> apply(?kernel, Fun, Args);
-    false  -> Callback()
+  case erlang:module_loaded(?kernel) of
+    true -> apply(?kernel, Fun, Args);
+    false -> Callback()
   end.
 
 env() ->
