@@ -1,10 +1,10 @@
 -module(elixir_env).
 -include("elixir.hrl").
 -export([
-  new/0, to_caller/1, with_vars/2, reset_vars/1, env_to_ex/1,
-  reset_unused_vars/1, check_unused_vars/2, merge_and_check_unused_vars/3,
+  new/0, to_caller/1, merge_vars/2, with_vars/2, reset_vars/1, env_to_ex/1,
+  reset_unused_vars/1, check_unused_vars/2, merge_and_check_unused_vars/3, calculate_span/2,
   trace/2, format_error/1,
-  reset_read/2, prepare_write/1, close_write/2
+  reset_read/2, prepare_write/1, prepare_write/2, close_write/2
 ]).
 
 new() ->
@@ -32,7 +32,7 @@ trace(Event, #{tracers := Tracers} = E) ->
 
 to_caller({Line, #elixir_ex{vars={Read, _}}, Env}) ->
   Env#{line := Line, versioned_vars := Read};
-to_caller(#{} = Env) ->
+to_caller(#{'__struct__' := 'Elixir.Macro.Env'} = Env) ->
   Env.
 
 with_vars(Env, Vars) when is_list(Vars) ->
@@ -49,15 +49,27 @@ reset_vars(Env) ->
 
 env_to_ex(#{context := match, versioned_vars := Vars}) ->
   Counter = map_size(Vars),
-  #elixir_ex{prematch={Vars, Counter}, vars={Vars, false}, unused={#{}, Counter}};
+  #elixir_ex{
+    prematch={Vars, {#{}, []}, Counter},
+    vars={Vars, false},
+    unused={#{}, Counter}
+  };
 env_to_ex(#{versioned_vars := Vars}) ->
-  #elixir_ex{vars={Vars, false}, unused={#{}, map_size(Vars)}}.
+  #elixir_ex{
+    vars={Vars, false},
+    unused={#{}, map_size(Vars)}
+  }.
 
 %% VAR HANDLING
 
 reset_read(#elixir_ex{vars={_, Write}} = S, #elixir_ex{vars={Read, _}}) ->
   S#elixir_ex{vars={Read, Write}}.
 
+prepare_write(S, #{context := nil}) ->
+  prepare_write(S);
+prepare_write(S, _) ->
+  S.
+
 prepare_write(#elixir_ex{vars={Read, _}} = S) ->
   S#elixir_ex{vars={Read, Read}}.
 
@@ -82,10 +94,19 @@ reset_unused_vars(#elixir_ex{unused={_Unused, Version}} = S) ->
   S#elixir_ex{unused={#{}, Version}}.
 
 check_unused_vars(#elixir_ex{unused={Unused, _Version}}, E) ->
-  [elixir_errors:form_warn([{line, Line}], E, ?MODULE, {unused_var, Name, Overridden}) ||
-    {{Name, _}, {Line, Overridden}} <- maps:to_list(Unused), is_unused_var(Name)],
+  [elixir_errors:file_warn(calculate_span(Meta, Name), E, ?MODULE, {unused_var, Name, Overridden}) ||
+    {{{Name, _Kind}, _Count}, {Meta, Overridden}} <- maps:to_list(Unused), is_unused_var(Name)],
   E.
 
+calculate_span(Meta, Name) ->
+  case lists:keyfind(column, 1, Meta) of
+    {column, Column} ->
+      [{span, {?line(Meta), Column + string:length(atom_to_binary(Name))}} | Meta];
+
+    _ ->
+      Meta
+  end.
+
 merge_and_check_unused_vars(S, #elixir_ex{vars={Read, Write}, unused={Unused, _Version}}, E) ->
   #elixir_ex{unused={ClauseUnused, Version}} = S,
   NewUnused = merge_and_check_unused_vars(Read, Unused, ClauseUnused, E),
@@ -93,23 +114,21 @@ merge_and_check_unused_vars(S, #elixir_ex{vars={Read, Write}, unused={Unused, _V
 
 merge_and_check_unused_vars(Current, Unused, ClauseUnused, E) ->
   maps:fold(fun
-    ({Name, Count} = Key, false, Acc) ->
-      Var = {Name, nil},
-
-      %% The parent knows it, so we have to propagate it was used up.
+    ({Var, Count} = Key, false, Acc) ->
       case Current of
         #{Var := CurrentCount} when Count =< CurrentCount ->
+          %% The parent knows it, so we have to propagate it was used up.
           Acc#{Key => false};
 
         #{} ->
           Acc
       end;
 
-    ({Name, _Count}, {Line, Overridden}, Acc) ->
+    ({{Name, _Kind}, _Count}, {Meta, Overridden}, Acc) ->
       case is_unused_var(Name) of
         true ->
           Warn = {unused_var, Name, Overridden},
-          elixir_errors:form_warn([{line, Line}], E, ?MODULE, Warn);
+          elixir_errors:file_warn(Meta, E, ?MODULE, Warn);
 
         false ->
           ok
@@ -132,6 +151,8 @@ format_error({unused_var, Name, Overridden}) ->
   case atom_to_list(Name) of
     "_" ++ _ ->
       io_lib:format("unknown compiler variable \"~ts\" (expected one of __MODULE__, __ENV__, __DIR__, __CALLER__, __STACKTRACE__)", [Name]);
+    "&" ++ _ ->
+      io_lib:format("variable \"~ts\" is unused (this might happen when using a capture argument as a pattern)", [Name]);
     _ when Overridden ->
       io_lib:format("variable \"~ts\" is unused (there is a variable with the same name in the context, use the pin operator (^) to match on it or prefix this variable with underscore if it is not meant to be used)", [Name]);
     _ ->
