@@ -1,34 +1,56 @@
 -module(elixir_erl_compiler).
--export([spawn/1, forms/3, noenv_forms/3, erl_to_core/2, format_error/1]).
+-export([spawn/1, noenv_forms/3, erl_to_core/2, env_compiler_options/0]).
 -include("elixir.hrl").
 
 spawn(Fun) ->
   CompilerInfo = get(elixir_compiler_info),
 
+  CodeDiagnostics =
+    case get(elixir_code_diagnostics) of
+      undefined -> undefined;
+      {_Tail, Log} -> {[], Log}
+    end,
+
   {_, Ref} =
     spawn_monitor(fun() ->
       put(elixir_compiler_info, CompilerInfo),
+      put(elixir_code_diagnostics, CodeDiagnostics),
 
       try Fun() of
-        Result -> exit({ok, Result})
+        Result -> exit({ok, Result, get(elixir_code_diagnostics)})
       catch
         Kind:Reason:Stack ->
-          exit({Kind, Reason, Stack})
+          exit({Kind, Reason, Stack, get(elixir_code_diagnostics)})
       end
     end),
 
   receive
-    {'DOWN', Ref, process, _, {ok, Result}} ->
+    {'DOWN', Ref, process, _, {ok, Result, Diagnostics}} ->
+      copy_diagnostics(Diagnostics),
       Result;
-    {'DOWN', Ref, process, _, {Kind, Reason, Stack}} ->
+    {'DOWN', Ref, process, _, {Kind, Reason, Stack, Diagnostics}} ->
+      copy_diagnostics(Diagnostics),
       erlang:raise(Kind, Reason, Stack)
   end.
 
-forms(Forms, File, Opts) ->
-  compile(Forms, File, Opts ++ compile:env_compiler_options()).
+copy_diagnostics(undefined) ->
+  ok;
+copy_diagnostics({Head, _}) ->
+  case get(elixir_code_diagnostics) of
+    undefined -> ok;
+    {Tail, Log} -> put(elixir_code_diagnostics, {Head ++ Tail, Log})
+  end.
 
-noenv_forms(Forms, File, Opts) ->
-  compile(Forms, File, Opts).
+env_compiler_options() ->
+  case persistent_term:get(?MODULE, undefined) of
+    undefined ->
+      Options = compile:env_compiler_options() -- [warnings_as_errors],
+      persistent_term:put(?MODULE, Options),
+      Options;
+
+    Options ->
+      Options
+  end.
 
 erl_to_core(Forms, Opts) ->
   %% TODO: Remove parse transform handling on Elixir v2.0
@@ -42,7 +64,7 @@ erl_to_core(Forms, Opts) ->
       end
   end.
 
-compile(Forms, File, Opts) when is_list(Forms), is_list(Opts), is_binary(File) ->
+noenv_forms(Forms, File, Opts) when is_list(Forms), is_list(Opts), is_binary(File) ->
   Source = elixir_utils:characters_to_list(File),
 
   case erl_to_core(Forms, Opts) of
@@ -56,12 +78,18 @@ compile(Forms, File, Opts) when is_list(Forms), is_list(Opts), is_binary(File) -
           format_warnings(Opts, Warnings),
           {Module, Binary};
 
-        {ok, Module, _Binary, _Warnings} ->
-          elixir_errors:form_error([], File, ?MODULE, {invalid_compilation, Module});
+        {ok, Module, _, _} ->
+          incompatible_options("could not compile module ~ts", [elixir_aliases:inspect(Module)], File);
+
+        {ok, Module, _} ->
+          incompatible_options("could not compile module ~ts", [elixir_aliases:inspect(Module)], File);
 
         {error, Errors, Warnings} ->
           format_warnings(Opts, Warnings),
-          format_errors(Errors)
+          format_errors(Errors);
+
+        _ ->
+          incompatible_options("could not compile module", [], File)
       end;
 
     {error, CoreErrors, CoreWarnings} ->
@@ -69,6 +97,16 @@ compile(Forms, File, Opts) when is_list(Forms), is_list(Opts), is_binary(File) -
       format_errors(CoreErrors)
   end.
 
+incompatible_options(Prefix, Args, File) ->
+  Message = io_lib:format(
+    Prefix ++ ". We expected the compiler to return a .beam binary but "
+    "got something else. This usually happens because ERL_COMPILER_OPTIONS or @compile "
+    "was set to change the compilation outcome in a way that is incompatible with Elixir",
+    Args
+  ),
+
+  elixir_errors:compile_error([], File, Message).
+
 format_errors([]) ->
   exit({nocompile, "compilation failed but no error was raised"});
 format_errors(Errors) ->
@@ -95,11 +133,11 @@ format_warnings(Opts, Warnings) ->
 handle_file_warning(_, _File, {_Line, v3_core, {map_key_repeated, _}}) -> ok;
 handle_file_warning(_, _File, {_Line, sys_core_fold, {ignored, useless_building}}) -> ok;
 
-%% TODO: remove when we require Erlang/OTP 24
-handle_file_warning(_, _File, {_Line, sys_core_fold, useless_building}) -> ok;
-handle_file_warning(true, _File, {_Line, sys_core_fold, nomatch_guard}) -> ok;
-handle_file_warning(true, _File, {_Line, sys_core_fold, {nomatch_shadow, _}}) -> ok;
-%%
+%% We skip all of no_match related to no_clause, clause_type, guard, shadow.
+%% Those have too little information and they overlap with the type system.
+%% We keep the remaining ones because the Erlang compiler performs analyses
+%% on literals (including numbers), which the type system does not do.
+handle_file_warning(_, _File, {_Line, sys_core_fold, {nomatch, Reason}}) when is_atom(Reason) -> ok;
 
 %% Ignore all linting errors (only come up on parse transforms)
 handle_file_warning(_, _File, {_Line, erl_lint, _}) -> ok;
@@ -110,15 +148,15 @@ handle_file_warning(_, File, {Line, Module, Desc}) ->
 
 %% Handle warnings
 
-handle_file_error(File, {beam_validator, Rest}) ->
-  elixir_errors:form_error([{line, 0}], File, beam_validator, Rest);
+handle_file_error(File, {beam_validator, Desc}) ->
+  elixir_errors:compile_error([{line, 0}], File, beam_validator:format_error(Desc));
 handle_file_error(File, {Line, Module, Desc}) ->
   Message = custom_format(Module, Desc),
   elixir_errors:compile_error([{line, Line}], File, Message).
 
 %% Mention the capture operator in make_fun
 custom_format(sys_core_fold, {ignored, {no_effect, {erlang, make_fun, 3}}}) ->
-  "the result of the capture operator & (:erlang.make_fun/3) is never used";
+  "the result of the capture operator & (Function.capture/3) is never used";
 
 %% Make no_effect clauses pretty
 custom_format(sys_core_fold, {ignored, {no_effect, {erlang, F, A}}}) ->
@@ -132,10 +170,6 @@ custom_format(sys_core_fold, {ignored, {no_effect, {erlang, F, A}}}) ->
   end,
   io_lib:format(Fmt, Args);
 
-%% Rewrite nomatch to be more generic, it can happen inside if, unless, and the like
-custom_format(sys_core_fold, {nomatch, X}) when X == guard; X == no_clause ->
-  "this check/guard will always yield the same result";
-
 custom_format(sys_core_fold, {nomatch, {shadow, Line, {ErlName, ErlArity}}}) ->
   {Name, Arity} = elixir_utils:erl_fa_to_elixir_fa(ErlName, ErlArity),
 
@@ -144,41 +178,8 @@ custom_format(sys_core_fold, {nomatch, {shadow, Line, {ErlName, ErlArity}}}) ->
     [Name, Arity, Line]
   );
 
-%% Handle literal eval failures
-custom_format(sys_core_fold, {failed, {eval_failure, {Mod, Name, Arity}, Error}}) ->
-  #{'__struct__' := Struct} = 'Elixir.Exception':normalize(error, Error),
-  {ExMod, ExName, ExArgs} = elixir_rewrite:erl_to_ex(Mod, Name, lists:duplicate(Arity, nil)),
-  Call = 'Elixir.Exception':format_mfa(ExMod, ExName, length(ExArgs)),
-  Trimmed = case Call of
-              <<"Kernel.", Rest/binary>> -> Rest;
-              _ -> Call
-            end,
-  ["the call to ", Trimmed, " will fail with ", elixir_aliases:inspect(Struct)];
-
-%% TODO: remove when we require Erlang/OTP 24
-custom_format(sys_core_fold, {nomatch_shadow, Line, FA}) ->
-  custom_format(sys_core_fold, {nomatch, {shadow, Line, FA}});
-custom_format(sys_core_fold, nomatch_guard) ->
-  custom_format(sys_core_fold, {nomatch, guard});
-custom_format(sys_core_fold, {no_effect, X}) ->
-  custom_format(sys_core_fold, {ignored, {no_effect, X}});
-custom_format(sys_core_fold, {eval_failure, Error}) ->
-  #{'__struct__' := Struct} = 'Elixir.Exception':normalize(error, Error),
-  ["this expression will fail with ", elixir_aliases:inspect(Struct)];
-%%
-
 custom_format([], Desc) ->
   io_lib:format("~p", [Desc]);
 
 custom_format(Module, Desc) ->
   Module:format_error(Desc).
-
-%% Error formatting
-
-format_error({invalid_compilation, Module}) ->
-  io_lib:format(
-    "could not compile module ~ts. We expected the compiler to return a .beam binary but "
-    "got something else. This usually happens because ERL_COMPILER_OPTIONS or @compile "
-    "was set to change the compilation outcome in a way that is incompatible with Elixir",
-    [elixir_aliases:inspect(Module)]
-  ).
