@@ -20,13 +20,7 @@ reduce_clauses([], Acc, OldStack, SAcc, _S) ->
   {lists:reverse(Acc), SAcc#elixir_erl{stacktrace=OldStack}}.
 
 each_clause({'catch', Meta, Raw, Expr}, S) ->
-  {Args, Guards} = elixir_utils:extract_splat_guards(Raw),
-
-  Match =
-    case Args of
-      [X] -> [throw, X];
-      [X, Y] -> [X, Y]
-    end,
+  {Match, Guards} = elixir_utils:extract_splat_guards(Raw),
 
   {{clause, Line, [TKind, TMatches], TGuards, TBody}, TS} =
     elixir_erl_clauses:clause(?ann(Meta), fun elixir_erl_pass:translate_args/3, Match, Expr, Guards, S),
@@ -55,7 +49,7 @@ normalize_rescue(Meta, Var, Pattern, Expr, ErlangAliases) ->
         dynamic_normalize(Meta, Var, ?REQUIRES_STACKTRACE);
 
       false ->
-        case lists:splitwith(fun is_normalized_with_stacktrace/1, ErlangAliases) of
+        case lists:partition(fun is_normalized_with_stacktrace/1, ErlangAliases) of
           {[], _} -> [];
           {_, []} -> {'__STACKTRACE__', Meta, nil};
           {Some, _} -> dynamic_normalize(Meta, Var, Some)
@@ -171,7 +165,7 @@ erl_rescue_guard_for(Meta, Var, 'Elixir.CaseClauseError') ->
 erl_rescue_guard_for(Meta, Var, 'Elixir.WithClauseError') ->
   erl_and(Meta,
           erl_tuple_size(Meta, Var, 2),
-          erl_record_compare(Meta, Var, with_clause));
+          erl_record_compare(Meta, Var, else_clause));
 
 erl_rescue_guard_for(Meta, Var, 'Elixir.TryClauseError') ->
   erl_and(Meta,
