@@ -2,7 +2,7 @@
 -module(elixir_erl_var).
 -export([
   translate/4, assign/2, build/2,
-  load_binding/1, dump_binding/3,
+  load_binding/2, dump_binding/4,
   from_env/1, from_env/2
 ]).
 -include("elixir.hrl").
@@ -65,16 +65,16 @@ to_erl_vars([], _Counter) ->
 to_erl_var(Counter) ->
   list_to_atom("_@" ++ integer_to_list(Counter)).
 
-load_binding(Binding) ->
-  load_binding(Binding, #{}, [], [], 0).
+load_binding(Binding, Prune) ->
+  load_binding(Binding, #{}, [], [], 0, Prune).
 
-load_binding([Binding | NextBindings], ExVars, ErlVars, Normalized, Counter) ->
+load_binding([Binding | NextBindings], ExVars, ErlVars, Normalized, Counter, Prune) ->
   {Pair, Value} = load_pair(Binding),
 
   case ExVars of
     #{Pair := VarCounter} ->
       ErlVar = to_erl_var(VarCounter),
-      load_binding(NextBindings, ExVars, ErlVars, [{ErlVar, Value} | Normalized], Counter);
+      load_binding(NextBindings, ExVars, ErlVars, [{ErlVar, Value} | Normalized], Counter, Prune);
 
     #{} ->
       ErlVar = to_erl_var(Counter),
@@ -84,46 +84,40 @@ load_binding([Binding | NextBindings], ExVars, ErlVars, Normalized, Counter) ->
         ExVars#{Pair => Counter},
         [{Counter, ErlVar} | ErlVars],
         [{ErlVar, Value} | Normalized],
-        Counter + 1
+        Counter + 1,
+        Prune
       )
   end;
-load_binding([], ExVars, ErlVars, Normalized, _Counter) ->
-  %% TODO: Remove me once we require Erlang/OTP 24+
-  %% Also revisit dump_binding below and remove the vars field for simplicity.
-  Mod =
-    case erlang:system_info(otp_release) >= "24" of
-      true -> maps;
-      false -> orddict
-    end,
-
-  {ExVars, maps:from_list(ErlVars), Mod:from_list(lists:reverse(Normalized))}.
+load_binding([], ExVars, ErlVars, Normalized, Counter, true) ->
+  load_binding([{{elixir, prune_binding}, true}], ExVars, ErlVars, Normalized, Counter, false);
+load_binding([], ExVars, ErlVars, Normalized, _Counter, false) ->
+  {ExVars, maps:from_list(ErlVars), maps:from_list(lists:reverse(Normalized))}.
 
 load_pair({Key, Value}) when is_atom(Key) -> {{Key, nil}, Value};
 load_pair({Pair, Value}) -> {Pair, Value}.
 
-dump_binding(Binding, #elixir_ex{vars={ExVars, _}}, #elixir_erl{var_names=ErlVars}) ->
+dump_binding(Binding, ErlS, ExS, PruneBefore) ->
+  #elixir_erl{var_names=ErlVars} = ErlS,
+  #elixir_ex{vars={ExVars, _}, unused={Unused, _}} = ExS,
+
   maps:fold(fun
-    ({Var, Kind} = Pair, Version, Acc) when is_atom(Kind) ->
+    ({Var, Kind} = Pair, Version, {B, V})
+    when is_atom(Kind),
+         %% If the variable is part of the pruning (usually the input binding)
+         %% and is unused, we removed it from vars.
+         Version > PruneBefore orelse is_map_key({Pair, Version}, Unused) ->
       Key = case Kind of
         nil -> Var;
         _ -> Pair
       end,
 
       ErlName = maps:get(Version, ErlVars),
-      Value = find_binding(ErlName, Binding),
-      [{Key, Value} | Acc];
+      Value = maps:get(ErlName, Binding, nil),
+      {[{Key, Value} | B], V};
+
+    (Pair, _, {B, V}) when PruneBefore >= 0 ->
+      {B, maps:remove(Pair, V)};
 
     (_, _, Acc) ->
       Acc
-  end, [], ExVars).
-
-find_binding(ErlName, Binding = #{}) ->
-  case Binding of
-    #{ErlName := V} -> V;
-    _ -> nil
-  end;
-find_binding(ErlName, Binding) ->
-  case orddict:find(ErlName, Binding) of
-    {ok, V} -> V;
-    error -> nil
-  end.
+  end, {[], ExVars}, ExVars).
