@@ -1,6 +1,6 @@
 -module(elixir_map).
--export([expand_map/4, expand_struct/5, format_error/1, load_struct/5]).
--import(elixir_errors, [form_error/4, form_warn/4]).
+-export([expand_map/4, expand_struct/5, format_error/1, maybe_load_struct_info/5]).
+-import(elixir_errors, [function_error/4, file_error/4, file_warn/4]).
 -include("elixir.hrl").
 
 expand_map(Meta, [{'|', UpdateMeta, [Left, Right]}], S, #{context := nil} = E) ->
@@ -8,7 +8,7 @@ expand_map(Meta, [{'|', UpdateMeta, [Left, Right]}], S, #{context := nil} = E) -
   validate_kv(Meta, ERight, Right, E),
   {{'%{}', Meta, [{'|', UpdateMeta, [ELeft, ERight]}]}, SE, EE};
 expand_map(Meta, [{'|', _, [_, _]}] = Args, _S, #{context := Context, file := File}) ->
-  form_error(Meta, File, ?MODULE, {update_syntax_in_wrong_context, Context, {'%{}', Meta, Args}});
+  file_error(Meta, File, ?MODULE, {update_syntax_in_wrong_context, Context, {'%{}', Meta, Args}});
 expand_map(Meta, Args, S, E) ->
   {EArgs, SE, EE} = elixir_expand:expand_args(Args, S, E),
   validate_kv(Meta, EArgs, Args, E),
@@ -23,14 +23,14 @@ expand_struct(Meta, Left, {'%{}', MapMeta, MapArgs}, S, #{context := Context} =
       case extract_struct_assocs(Meta, ERight, E) of
         {expand, MapMeta, Assocs} when Context /= match -> %% Expand
           AssocKeys = [K || {K, _} <- Assocs],
-          Struct = load_struct(Meta, ELeft, [Assocs], AssocKeys, EE),
+          Struct = load_struct(Meta, ELeft, Assocs, EE),
           Keys = ['__struct__'] ++ AssocKeys,
-          WithoutKeys = maps:to_list(maps:without(Keys, Struct)),
+          WithoutKeys = lists:sort(maps:to_list(maps:without(Keys, Struct))),
           StructAssocs = elixir_quote:escape(WithoutKeys, none, false),
           {{'%', Meta, [ELeft, {'%{}', MapMeta, StructAssocs ++ Assocs}]}, SE, EE};
 
         {_, _, Assocs} -> %% Update or match
-          _ = load_struct(Meta, ELeft, [], [K || {K, _} <- Assocs], EE),
+          _ = load_struct_info(Meta, ELeft, Assocs, EE),
           {{'%', Meta, [ELeft, ERight]}, SE, EE}
       end;
 
@@ -38,13 +38,13 @@ expand_struct(Meta, Left, {'%{}', MapMeta, MapArgs}, S, #{context := Context} =
       {{'%', Meta, [ELeft, ERight]}, SE, EE};
 
     false when Context == match ->
-      form_error(Meta, E, ?MODULE, {invalid_struct_name_in_match, ELeft});
+      file_error(Meta, E, ?MODULE, {invalid_struct_name_in_match, ELeft});
 
     false ->
-      form_error(Meta, E, ?MODULE, {invalid_struct_name, ELeft})
+      file_error(Meta, E, ?MODULE, {invalid_struct_name, ELeft})
   end;
 expand_struct(Meta, _Left, Right, _S, E) ->
-  form_error(Meta, E, ?MODULE, {non_map_after_struct, Right}).
+  file_error(Meta, E, ?MODULE, {non_map_after_struct, Right}).
 
 clean_struct_key_from_map_args(Meta, [{'|', PipeMeta, [Left, MapAssocs]}], E) ->
   [{'|', PipeMeta, [Left, clean_struct_key_from_map_assocs(Meta, MapAssocs, E)]}];
@@ -54,14 +54,16 @@ clean_struct_key_from_map_args(Meta, MapAssocs, E) ->
 clean_struct_key_from_map_assocs(Meta, Assocs, E) ->
   case lists:keytake('__struct__', 1, Assocs) of
     {value, _, CleanAssocs} ->
-      form_warn(Meta, ?key(E, file), ?MODULE, ignored_struct_key_in_struct),
+      file_warn(Meta, ?key(E, file), ?MODULE, ignored_struct_key_in_struct),
       CleanAssocs;
     false ->
       Assocs
   end.
 
 validate_match_key(Meta, {Name, _, Context}, E) when is_atom(Name), is_atom(Context) ->
-  form_error(Meta, E, ?MODULE, {invalid_variable_in_map_key_match, Name});
+  file_error(Meta, E, ?MODULE, {invalid_variable_in_map_key_match, Name});
+validate_match_key(Meta, {'::', _, [Left, _]}, E) ->
+  validate_match_key(Meta, Left, E);
 validate_match_key(_, {'^', _, [{Name, _, Context}]}, _) when is_atom(Name), is_atom(Context) ->
   ok;
 validate_match_key(_, {'%{}', _, [_ | _]}, _) ->
@@ -81,12 +83,10 @@ validate_not_repeated(Meta, Key, Used, E) ->
   case is_literal(Key) andalso Used of
     #{Key := true} ->
       case E of
-        #{context := match} ->
-          form_error(Meta, ?key(E, file), ?MODULE, {repeated_key, Key});
-        _ ->
-          form_warn(Meta, ?key(E, file), ?MODULE, {repeated_key, Key}),
-          Used
-      end;
+        #{context := match} -> function_error(Meta, ?key(E, file), ?MODULE, {repeated_key, Key});
+        _ -> file_warn(Meta, ?key(E, file), ?MODULE, {repeated_key, Key})
+      end,
+      Used;
 
     #{} ->
       Used#{Key => true};
@@ -107,7 +107,7 @@ validate_kv(Meta, KV, Original, #{context := Context} = E) ->
       NewUsed = validate_not_repeated(Meta, K, Used, E),
       {Index + 1, NewUsed};
     (_, {Index, _Used}) ->
-      form_error(Meta, E, ?MODULE, {not_kv_pair, lists:nth(Index, Original)})
+      file_error(Meta, E, ?MODULE, {not_kv_pair, lists:nth(Index, Original)})
   end, {1, #{}}, KV).
 
 extract_struct_assocs(_, {'%{}', Meta, [{'|', _, [_, Assocs]}]}, _) ->
@@ -115,7 +115,7 @@ extract_struct_assocs(_, {'%{}', Meta, [{'|', _, [_, Assocs]}]}, _) ->
 extract_struct_assocs(_, {'%{}', Meta, Assocs}, _) ->
   {expand, Meta, delete_struct_key(Assocs)};
 extract_struct_assocs(Meta, Other, E) ->
-  form_error(Meta, E, ?MODULE, {non_map_after_struct, Other}).
+  file_error(Meta, E, ?MODULE, {non_map_after_struct, Other}).
 
 delete_struct_key(Assocs) ->
   lists:keydelete('__struct__', 1, Assocs).
@@ -125,31 +125,75 @@ validate_struct({Var, _Meta, Ctx}, match) when is_atom(Var), is_atom(Ctx) -> tru
 validate_struct(Atom, _) when is_atom(Atom) -> true;
 validate_struct(_, _) -> false.
 
-load_struct(Meta, Name, Args, Keys, E) ->
+load_struct_info(Meta, Name, Assocs, E) ->
+  assert_struct_assocs(Meta, Assocs, E),
+
+  case maybe_load_struct_info(Meta, Name, Assocs, true, E) of
+    {ok, Info} -> Info;
+    {error, Desc} -> file_error(Meta, E, ?MODULE, Desc)
+  end.
+
+maybe_load_struct_info(Meta, Name, Assocs, Trace, E) ->
   try
-    wrapped_load_struct(Meta, Name, Args, Keys, E)
+    case is_open(Name, E) andalso lookup_struct_info_from_data_tables(Name) of
+      %% If I am accessing myself and there is no attribute,
+      %% don't invoke the fallback to avoid calling loaded code.
+      false when ?key(E, module) =:= Name -> nil;
+      false -> Name:'__info__'(struct);
+      InfoList -> InfoList
+    end
+  of
+    nil ->
+      {error, detail_undef(Name, E)};
+
+    Info ->
+      Keys = [begin
+        lists:any(fun(Field) -> ?key(Field, field) =:= Key end, Info) orelse
+          function_error(Meta, E, ?MODULE, {unknown_key_for_struct, Name, Key}),
+        Key
+      end || {Key, _} <- Assocs],
+      Trace andalso elixir_env:trace({struct_expansion, Meta, Name, Keys}, E),
+      {ok, Info}
+  catch
+    error:undef -> {error, detail_undef(Name, E)}
+  end.
+
+lookup_struct_info_from_data_tables(Module) ->
+  try
+    {Set, _} = elixir_module:data_tables(Module),
+    ets:lookup_element(Set, {elixir, struct}, 2)
+  catch
+    _:_ -> false
+  end.
+
+load_struct(Meta, Name, Assocs, E) ->
+  assert_struct_assocs(Meta, Assocs, E),
+
+  try
+    maybe_load_struct(Meta, Name, Assocs, E)
+  of
+    {ok, Struct} -> Struct;
+    {error, Desc} -> file_error(Meta, E, ?MODULE, Desc)
   catch
     Kind:Reason ->
-      Info = [{Name, '__struct__', length(Args), [{file, "expanding struct"}]},
+      Info = [{Name, '__struct__', 1, [{file, "expanding struct"}]},
               elixir_utils:caller(?line(Meta), ?key(E, file), ?key(E, module), ?key(E, function))],
       erlang:raise(Kind, Reason, Info)
   end.
 
-wrapped_load_struct(Meta, Name, Args, Keys, E) ->
-  %% We also include the current module because it won't be present
-  %% in context module in case the module name is defined dynamically.
-  InContext = lists:member(Name, [?key(E, module) | ?key(E, context_modules)]),
-
-  Arity = length(Args),
-  External = InContext orelse (not(ensure_loaded(Name)) andalso wait_for_struct(Name)),
-
+maybe_load_struct(Meta, Name, Assocs, E) ->
   try
-    case External andalso elixir_def:external_for(Meta, Name, '__struct__', Arity, [def]) of
+    case is_open(Name, E) andalso elixir_def:external_for(Meta, Name, '__struct__', 1, [def]) of
+      %% If I am accessing myself and there is no __struct__ function,
+      %% don't invoke the fallback to avoid calling loaded code.
+      false when ?key(E, module) =:= Name ->
+        error(undef);
+
       false ->
-        apply(Name, '__struct__', Args);
+        Name:'__struct__'(Assocs);
 
       ExternalFun ->
-        %% There is an inherent race condition when using local_for.
+        %% There is an inherent race condition when using external_for.
         %% By the time we got to execute the function, the ETS table
         %% with temporary definitions for the given module may no longer
         %% be available, so any function invocation happening inside the
@@ -158,43 +202,51 @@ wrapped_load_struct(Meta, Name, Args, Keys, E) ->
         %% the table has not been deleted (unless compilation of that
         %% module failed which should then cause this call to fail too).
         try
-          apply(ExternalFun, Args)
+          ExternalFun(Assocs)
         catch
-          error:undef -> apply(Name, '__struct__', Args)
+          error:undef -> Name:'__struct__'(Assocs)
         end
     end
   of
     #{'__struct__' := Name} = Struct ->
-      assert_struct_keys(Meta, Name, Struct, Keys, E),
+      Keys = [begin
+        maps:is_key(Key, Struct) orelse
+          function_error(Meta, E, ?MODULE, {unknown_key_for_struct, Name, Key}),
+        Key
+      end || {Key, _} <- Assocs],
       elixir_env:trace({struct_expansion, Meta, Name, Keys}, E),
-      Struct;
+      {ok, Struct};
 
     #{'__struct__' := StructName} when is_atom(StructName) ->
-      form_error(Meta, E, ?MODULE, {struct_name_mismatch, Name, Arity, StructName});
+      {error, {struct_name_mismatch, Name, StructName}};
 
     Other ->
-      form_error(Meta, E, ?MODULE, {invalid_struct_return_value, Name, Arity, Other})
+      {error, {invalid_struct_return_value, Name, Other}}
   catch
-    error:undef ->
-      case InContext andalso (?key(E, function) == nil) of
-        true ->
-          form_error(Meta, E, ?MODULE, {inaccessible_struct, Name});
-        false ->
-          form_error(Meta, E, ?MODULE, {undefined_struct, Name, Arity})
-      end
+    error:undef -> {error, detail_undef(Name, E)}
   end.
 
-ensure_loaded(Module) ->
-  code:ensure_loaded(Module) == {module, Module}.
+assert_struct_assocs(Meta, Assocs, E) ->
+  [function_error(Meta, E, ?MODULE, {invalid_key_for_struct, K})
+   || {K, _} <- Assocs, not is_atom(K)].
+
+is_open(Name, E) ->
+  in_context(Name, E) orelse ((code:ensure_loaded(Name) /= {module, Name}) andalso wait_for_struct(Name)).
+
+in_context(Name, E) ->
+  %% We also include the current module because it won't be present
+  %% in context module in case the module name is defined dynamically.
+  lists:member(Name, [?key(E, module) | ?key(E, context_modules)]).
 
 wait_for_struct(Module) ->
   (erlang:get(elixir_compiler_info) /= undefined) andalso
     ('Elixir.Kernel.ErrorHandler':ensure_compiled(Module, struct, hard) =:= found).
 
-assert_struct_keys(Meta, Name, Struct, Keys, E) ->
-  [begin
-     form_error(Meta, E, ?MODULE, {unknown_key_for_struct, Name, Key})
-   end || Key <- Keys, not maps:is_key(Key, Struct)].
+detail_undef(Name, E) ->
+  case in_context(Name, E) andalso (?key(E, function) == nil) of
+    true -> {inaccessible_struct, Name};
+    false -> {undefined_struct, Name}
+  end.
 
 format_error({update_syntax_in_wrong_context, Context, Expr}) ->
   io_lib:format("cannot use map/struct update syntax in ~ts, got: ~ts",
@@ -221,29 +273,32 @@ format_error({not_kv_pair, Expr}) ->
 format_error({non_map_after_struct, Expr}) ->
   io_lib:format("expected struct to be followed by a map, got: ~ts",
                 ['Elixir.Macro':to_string(Expr)]);
-format_error({struct_name_mismatch, Module, Arity, StructName}) ->
+format_error({struct_name_mismatch, Module, StructName}) ->
   Name = elixir_aliases:inspect(Module),
-  Message = "expected struct name returned by ~ts.__struct__/~p to be ~ts, got: ~ts",
-  io_lib:format(Message, [Name, Arity, Name, elixir_aliases:inspect(StructName)]);
-format_error({invalid_struct_return_value, Module, Arity, Value}) ->
+  Message = "expected struct name returned by ~ts.__struct__/1 to be ~ts, got: ~ts",
+  io_lib:format(Message, [Name, Name, elixir_aliases:inspect(StructName)]);
+format_error({invalid_struct_return_value, Module, Value}) ->
   Message =
-    "expected ~ts.__struct__/~p to return a map with a :__struct__ key that holds the "
+    "expected ~ts.__struct__/1 to return a map with a :__struct__ key that holds the "
     "name of the struct (atom), got: ~ts",
-  io_lib:format(Message, [elixir_aliases:inspect(Module), Arity, 'Elixir.Kernel':inspect(Value)]);
+  io_lib:format(Message, [elixir_aliases:inspect(Module), 'Elixir.Kernel':inspect(Value)]);
 format_error({inaccessible_struct, Module}) ->
   Message =
     "cannot access struct ~ts, the struct was not yet defined or the struct is "
     "being accessed in the same context that defines it",
   io_lib:format(Message, [elixir_aliases:inspect(Module)]);
-format_error({undefined_struct, Module, Arity}) ->
+format_error({undefined_struct, Module}) ->
   Name = elixir_aliases:inspect(Module),
   io_lib:format(
-    "~ts.__struct__/~p is undefined, cannot expand struct ~ts. "
+    "~ts.__struct__/1 is undefined, cannot expand struct ~ts. "
     "Make sure the struct name is correct. If the struct name exists and is correct "
     "but it still cannot be found, you likely have cyclic module usage in your code",
-    [Name, Arity, Name]);
+    [Name, Name]);
 format_error({unknown_key_for_struct, Module, Key}) ->
   io_lib:format("unknown key ~ts for struct ~ts",
                 ['Elixir.Macro':to_string(Key), elixir_aliases:inspect(Module)]);
+format_error({invalid_key_for_struct, Key}) ->
+  io_lib:format("invalid key ~ts for struct, struct keys must be atoms, got: ",
+                ['Elixir.Macro':to_string(Key)]);
 format_error(ignored_struct_key_in_struct) ->
   "key :__struct__ is ignored when using structs".
