@@ -1,9 +1,10 @@
 -module(elixir_module).
 -export([file/1, data_tables/1, is_open/1, mode/1, delete_definition_attributes/6,
-         compile/4, expand_callback/6, format_error/1, compiler_modules/0,
-         write_cache/3, read_cache/2, next_counter/1]).
+         compile/6, expand_callback/6, format_error/1, compiler_modules/0,
+         write_cache/3, read_cache/2, next_counter/1, taint/1, cache_env/1, get_cached_env/1]).
 -include("elixir.hrl").
 -define(counter_attr, {elixir, counter}).
+-define(cache_key, {elixir, cache_env}).
 
 %% Stores modules currently being defined by the compiler
 
@@ -62,10 +63,43 @@ next_counter(Module) ->
     _:_ -> erlang:unique_integer()
   end.
 
+taint(Module) ->
+  try
+    {DataSet, _} = data_tables(Module),
+    ets:insert(DataSet, [{{elixir, taint}}]),
+    true
+  catch
+    _:_ -> false
+  end.
+
+cache_env(#{line := Line, module := Module} = E) ->
+  {Set, _} = data_tables(Module),
+  Cache = elixir_env:reset_vars(E#{line := nil}),
+  PrevKey = ets:lookup_element(Set, ?cache_key, 2),
+
+  Pos =
+    case ets:lookup(Set, {cache_env, PrevKey}) of
+      [{_, Cache}] ->
+        PrevKey;
+      _ ->
+        NewKey = PrevKey + 1,
+        ets:insert(Set, [{{cache_env, NewKey}, Cache}, {?cache_key, NewKey}]),
+        NewKey
+    end,
+
+  {Module, {Line, Pos}}.
+
+get_cached_env({Module, {Line, Pos}}) ->
+  {Set, _} = data_tables(Module),
+  (ets:lookup_element(Set, {cache_env, Pos}, 2))#{line := Line};
+get_cached_env(Env) ->
+  Env.
+
 %% Compilation hook
 
-compile(Module, Block, Vars, Env) when is_atom(Module) ->
-  #{line := Line, function := Function, versioned_vars := OldVerVars} = Env,
+compile(Meta, Module, Block, Vars, Prune, Env) ->
+  ModuleAsCharlist = validate_module_name(Module),
+  #{function := Function, versioned_vars := OldVerVars} = Env,
 
   {VerVars, _} =
     lists:mapfoldl(fun({Var, _}, I) -> {{Var, I}, I + 1} end, 0, maps:to_list(OldVerVars)),
@@ -82,75 +116,103 @@ compile(Module, Block, Vars, Env) when is_atom(Module) ->
     #{lexical_tracker := nil} ->
       elixir_lexical:run(
         MaybeLexEnv,
-        fun(LexEnv) -> compile(Line, Module, Block, Vars, LexEnv) end,
+        fun(LexEnv) -> compile(Meta, Module, ModuleAsCharlist, Block, Vars, Prune, LexEnv) end,
         fun(_LexEnv) -> ok end
       );
     _ ->
-      compile(Line, Module, Block, Vars, MaybeLexEnv)
-  end;
-compile(Module, _Block, _Vars, #{line := Line, file := File}) ->
-  elixir_errors:form_error([{line, Line}], File, ?MODULE, {invalid_module, Module}).
+      compile(Meta, Module, ModuleAsCharlist, Block, Vars, Prune, MaybeLexEnv)
+  end.
+
+validate_module_name(Module) when Module == nil; is_boolean(Module); not is_atom(Module) ->
+  invalid_module_name(Module);
+validate_module_name(Module) ->
+  Charlist = atom_to_list(Module),
+  case lists:any(fun(Char) -> (Char =:= $/) or (Char =:= $\\) end, Charlist) of
+    true -> invalid_module_name(Module);
+    false -> Charlist
+  end.
+
+invalid_module_name(Module) ->
+  %% We raise an argument error to keep it close to Elixir errors before it starts.
+  erlang:error('Elixir.ArgumentError':exception(
+    <<"invalid module name: ",
+      ('Elixir.Kernel':inspect(Module))/binary>>
+  )).
+
+compile(Meta, Module, ModuleAsCharlist, Block, Vars, Prune, E) ->
+  Anno = ?ann(Meta),
+  Line = erl_anno:line(Anno),
 
-compile(Line, Module, Block, Vars, E) ->
   File = ?key(E, file),
-  check_module_availability(Line, File, Module),
-  ModuleAsCharlist = validate_module_name(Line, File, Module),
+  check_module_availability(Module, Line, E),
+  elixir_env:trace(defmodule, E),
 
   CompilerModules = compiler_modules(),
-  {Tables, Ref} = build(Line, File, Module),
+  {Tables, Ref} = build(Module, Line, File, E),
   {DataSet, DataBag} = Tables,
 
   try
     put_compiler_modules([Module | CompilerModules]),
-    {Result, NE} = eval_form(Line, Module, DataBag, Block, Vars, E),
+    {Result, ModuleE, CallbackE} = eval_form(Line, Module, DataBag, Block, Vars, Prune, E),
     CheckerInfo = checker_info(),
 
     {Binary, PersistedAttributes, Autoload} =
       elixir_erl_compiler:spawn(fun() ->
         PersistedAttributes = ets:lookup_element(DataBag, persisted_attributes, 2),
         Attributes = attributes(DataSet, DataBag, PersistedAttributes),
-        {AllDefinitions, Private} = elixir_def:fetch_definitions(File, Module),
+        {AllDefinitions, Private} = elixir_def:fetch_definitions(Module, E),
 
         OnLoadAttribute = lists:keyfind(on_load, 1, Attributes),
-        NewPrivate = validate_on_load_attribute(OnLoadAttribute, AllDefinitions, Private, File, Line),
+        validate_on_load_attribute(OnLoadAttribute, AllDefinitions, DataBag, Line, E),
 
         DialyzerAttribute = lists:keyfind(dialyzer, 1, Attributes),
-        validate_dialyzer_attribute(DialyzerAttribute, AllDefinitions, File, Line),
+        validate_dialyzer_attribute(DialyzerAttribute, AllDefinitions, Line, E),
 
-        Unreachable = elixir_locals:warn_unused_local(File, Module, AllDefinitions, NewPrivate),
-        elixir_locals:ensure_no_undefined_local(File, Module, AllDefinitions),
-        elixir_locals:ensure_no_import_conflict(File, Module, AllDefinitions),
-
-        %% We stop tracking locals here to avoid race conditions in case after_load
-        %% evaluates code in a separate process that may write to locals table.
-        elixir_locals:stop({DataSet, DataBag}),
+        NifsAttribute = lists:keyfind(nifs, 1, Attributes),
+        validate_nifs_attribute(NifsAttribute, AllDefinitions, Line, E),
+        elixir_import:ensure_no_local_conflict(Module, AllDefinitions, E),
         make_readonly(Module),
 
         (not elixir_config:is_bootstrap()) andalso
-         'Elixir.Module':check_derive_behaviours_and_impls(E, DataSet, DataBag, AllDefinitions),
-
-        RawCompileOpts = bag_lookup_element(DataBag, {accumulate, compile}, 2),
-        CompileOpts = validate_compile_opts(RawCompileOpts, AllDefinitions, Unreachable, File, Line),
+         'Elixir.Module':'__check_attributes__'(E, DataSet, DataBag),
 
         AfterVerify = bag_lookup_element(DataBag, {accumulate, after_verify}, 2),
-        [elixir_env:trace({remote_function, [], VerifyMod, VerifyFun, 1}, E) ||
+        [elixir_env:trace({remote_function, [], VerifyMod, VerifyFun, 1}, CallbackE) ||
          {VerifyMod, VerifyFun} <- AfterVerify],
 
+        %% Ensure there are no errors before we infer types
+        compile_error_if_tainted(DataSet, E),
+
+        {Signatures, Unreachable} =
+          case elixir_config:is_bootstrap() of
+            true -> {#{}, []};
+            false ->
+              UsedPrivate = bag_lookup_element(DataBag, used_private, 2),
+              'Elixir.Module.Types':infer(Module, File, AllDefinitions, Private, UsedPrivate, E, CheckerInfo)
+          end,
+
+        RawCompileOpts = bag_lookup_element(DataBag, {accumulate, compile}, 2),
+        CompileOpts = validate_compile_opts(RawCompileOpts, AllDefinitions, Unreachable, Line, E),
+        Impls = bag_lookup_element(DataBag, impls, 2),
+
         ModuleMap = #{
           struct => get_struct(DataSet),
           module => Module,
-          line => Line,
+          anno => Anno,
           file => File,
           relative_file => elixir_utils:relative_to_cwd(File),
           attributes => Attributes,
           definitions => AllDefinitions,
-          unreachable => Unreachable,
           after_verify => AfterVerify,
           compile_opts => CompileOpts,
           deprecated => get_deprecated(DataBag),
-          is_behaviour => is_behaviour(DataBag)
+          defines_behaviour => defines_behaviour(DataBag),
+          impls => Impls,
+          unreachable => Unreachable,
+          signatures => Signatures
         },
 
+        compile_error_if_tainted(DataSet, E),
         Binary = elixir_erl:compile(ModuleMap),
         Autoload = proplists:get_value(autoload, CompileOpts, true),
         spawn_parallel_checker(CheckerInfo, Module, ModuleMap),
@@ -158,11 +220,12 @@ compile(Line, Module, Block, Vars, E) ->
       end),
 
     Autoload andalso code:load_binary(Module, beam_location(ModuleAsCharlist), Binary),
-    eval_callbacks(Line, DataBag, after_compile, [NE, Binary], NE),
-    elixir_env:trace({on_module, Binary, none}, E),
-    warn_unused_attributes(File, DataSet, DataBag, PersistedAttributes),
+    put_compiler_modules(CompilerModules),
+    eval_callbacks(Line, DataBag, after_compile, [CallbackE, Binary], CallbackE),
+    elixir_env:trace({on_module, Binary, none}, ModuleE),
+    warn_unused_attributes(DataSet, DataBag, PersistedAttributes, E),
     make_module_available(Module, Binary),
-    (CheckerInfo == undefined) andalso
+    (element(2, CheckerInfo) == nil) andalso
       [VerifyMod:VerifyFun(Module) ||
        {VerifyMod, VerifyFun} <- bag_lookup_element(DataBag, {accumulate, after_verify}, 2)],
     {module, Module, Binary, Result}
@@ -183,27 +246,40 @@ compile(Line, Module, Block, Vars, E) ->
     elixir_code_server:call({undefmodule, Ref})
   end.
 
-validate_compile_opts(Opts, Defs, Unreachable, File, Line) ->
-  lists:flatmap(fun (Opt) -> validate_compile_opt(Opt, Defs, Unreachable, File, Line) end, Opts).
+compile_error_if_tainted(DataSet, E) ->
+  case ets:member(DataSet, {elixir, taint}) of
+    true -> elixir_errors:compile_error(E);
+    false -> ok
+  end.
+
+validate_compile_opts(Opts, Defs, Unreachable, Line, E) ->
+  lists:flatmap(fun (Opt) -> validate_compile_opt(Opt, Defs, Unreachable, Line, E) end, Opts).
 
 %% TODO: Make this an error on v2.0
-validate_compile_opt({parse_transform, Module} = Opt, _Defs, _Unreachable, File, Line) ->
-  elixir_errors:form_warn([{line, Line}], File, ?MODULE, {parse_transform, Module}),
+validate_compile_opt({parse_transform, Module} = Opt, _Defs, _Unreachable, Line, E) ->
+  elixir_errors:file_warn([{line, Line}], E, ?MODULE, {parse_transform, Module}),
   [Opt];
-validate_compile_opt({inline, Inlines}, Defs, Unreachable, File, Line) ->
+validate_compile_opt({inline, Inlines}, Defs, Unreachable, Line, E) ->
   case validate_inlines(Inlines, Defs, Unreachable, []) of
-    {ok, []} -> [];
-    {ok, FilteredInlines} -> [{inline, FilteredInlines}];
-    {error, Def} -> elixir_errors:form_error([{line, Line}], File, ?MODULE, {bad_inline, Def})
+    {ok, []} ->
+      [];
+    {ok, FilteredInlines} ->
+      [{inline, FilteredInlines}];
+    {error, Reason} ->
+      elixir_errors:module_error([{line, Line}], E, ?MODULE, Reason),
+      []
   end;
-validate_compile_opt(Opt, Defs, Unreachable, File, Line) when is_list(Opt) ->
-  validate_compile_opts(Opt, Defs, Unreachable, File, Line);
-validate_compile_opt(Opt, _Defs, _Unreachable, _File, _Line) ->
+validate_compile_opt(Opt, Defs, Unreachable, Line, E) when is_list(Opt) ->
+  validate_compile_opts(Opt, Defs, Unreachable, Line, E);
+validate_compile_opt(Opt, _Defs, _Unreachable, _Line, _E) ->
   [Opt].
 
 validate_inlines([Inline | Inlines], Defs, Unreachable, Acc) ->
   case lists:keyfind(Inline, 1, Defs) of
-    false -> {error, Inline};
+    false ->
+      {error, {undefined_function, {compile, inline}, Inline}};
+    {_Def, Kind, _Meta, _Clauses} when Kind == defmacro; Kind == defmacrop ->
+      {error, {bad_macro, {compile, inline}, Inline}};
     _ ->
       case lists:member(Inline, Unreachable) of
         true -> validate_inlines(Inlines, Defs, Unreachable, Acc);
@@ -212,28 +288,39 @@ validate_inlines([Inline | Inlines], Defs, Unreachable, Acc) ->
   end;
 validate_inlines([], _Defs, _Unreachable, Acc) -> {ok, Acc}.
 
-validate_on_load_attribute({on_load, Def}, Defs, Private, File, Line) ->
+validate_on_load_attribute({on_load, Def}, Defs, Bag, Line, E) ->
   case lists:keyfind(Def, 1, Defs) of
     false ->
-      elixir_errors:form_error([{line, Line}], File, ?MODULE, {undefined_on_load, Def});
-    {_, Kind, _, _} when Kind == def; Kind == defp ->
-      lists:keydelete(Def, 1, Private);
-    {_, WrongKind, _, _} ->
-      elixir_errors:form_error([{line, Line}], File, ?MODULE, {wrong_kind_on_load, Def, WrongKind})
+      elixir_errors:module_error([{line, Line}], E, ?MODULE, {undefined_function, on_load, Def});
+    {_Def, Kind, _Meta, _Clauses} when Kind == defmacro; Kind == defmacrop ->
+      elixir_errors:module_error([{line, Line}], E, ?MODULE, {bad_macro, on_load, Def});
+    {_Def, Kind, _Meta, _Clauses} ->
+      (Kind == defp) andalso ets:insert(Bag, {used_private, Def})
   end;
-validate_on_load_attribute(false, _Module, Private, _File, _Line) -> Private.
+validate_on_load_attribute(false, _Defs, _Bag, _Line, _E) -> ok.
 
-validate_dialyzer_attribute({dialyzer, Dialyzer}, Defs, File, Line) ->
-  [case lists:keyfind(Fun, 1, Defs) of
-    false ->
-      elixir_errors:form_error([{line, Line}], File, ?MODULE, {bad_dialyzer, Key, Fun});
-    _ ->
-      ok
-   end || {Key, Funs} <- lists:flatten([Dialyzer]), Fun <- lists:flatten([Funs])];
-validate_dialyzer_attribute(false, _Defs, _File, _Line) ->
+validate_dialyzer_attribute({dialyzer, Dialyzer}, Defs, Line, E) ->
+  [validate_definition({dialyzer, Key}, Fun, Defs, Line, E)
+   || {Key, Funs} <- lists:flatten([Dialyzer]), Fun <- lists:flatten([Funs])];
+validate_dialyzer_attribute(false, _Defs, _Line, _E) ->
   ok.
 
-is_behaviour(DataBag) ->
+validate_nifs_attribute({nifs, Funs}, Defs, Line, E) ->
+  [validate_definition(nifs, Fun, Defs, Line, E) || Fun <- lists:flatten([Funs])];
+validate_nifs_attribute(false, _Defs, _Line, _E) ->
+  ok.
+
+validate_definition(Key, Fun, Defs, Line, E) ->
+  case lists:keyfind(Fun, 1, Defs) of
+    false ->
+      elixir_errors:module_error([{line, Line}], E, ?MODULE, {undefined_function, Key, Fun});
+    {Fun, Type, _Meta, _Clauses} when Type == defmacro; Type == defmacrop ->
+      elixir_errors:module_error([{line, Line}], E, ?MODULE, {bad_macro, Key, Fun});
+    _ ->
+      ok
+   end.
+
+defines_behaviour(DataBag) ->
   ets:member(DataBag, {accumulate, callback}) orelse ets:member(DataBag, {accumulate, macrocallback}).
 
 %% An undef error for a function in the module being compiled might result in an
@@ -252,13 +339,13 @@ compile_undef(Module, Fun, Arity, Stack) ->
 
 %% Handle reserved modules and duplicates.
 
-check_module_availability(Line, File, Module) ->
+check_module_availability(Module, Line, E) ->
   Reserved = ['Elixir.True', 'Elixir.False', 'Elixir.Nil',
               'Elixir.Any', 'Elixir.BitString', 'Elixir.PID',
               'Elixir.Reference', 'Elixir.Elixir', 'Elixir'],
 
   case lists:member(Module, Reserved) of
-    true  -> elixir_errors:form_error([{line, Line}], File, ?MODULE, {module_reserved, Module});
+    true  -> elixir_errors:file_error([{line, Line}], E, ?MODULE, {module_reserved, Module});
     false -> ok
   end,
 
@@ -266,7 +353,7 @@ check_module_availability(Line, File, Module) ->
     false ->
       case code:ensure_loaded(Module) of
         {module, _} ->
-          elixir_errors:form_warn([{line, Line}], File, ?MODULE, {module_defined, Module});
+          elixir_errors:file_warn([{line, Line}], E, ?MODULE, {module_defined, Module});
         {error, _}  ->
           ok
       end;
@@ -274,20 +361,9 @@ check_module_availability(Line, File, Module) ->
       ok
   end.
 
-validate_module_name(Line, File, Module) when Module == nil; is_boolean(Module) ->
-  elixir_errors:form_error([{line, Line}], File, ?MODULE, {invalid_module, Module});
-validate_module_name(Line, File, Module) ->
-  Charlist = atom_to_list(Module),
-  case lists:any(fun(Char) -> (Char =:= $/) or (Char =:= $\\) end, Charlist) of
-    true ->
-      elixir_errors:form_error([{line, Line}], File, ?MODULE, {invalid_module, Module});
-    false ->
-      Charlist
-  end.
-
 %% Hook that builds both attribute and functions and set up common hooks.
 
-build(Line, File, Module) ->
+build(Module, Line, File, E) ->
   %% In the set table we store:
   %%
   %% * {Attribute, Value, AccumulateOrUnsetOrReadOrUnreadline, TraceLineOrNil}
@@ -297,7 +373,6 @@ build(Line, File, Module) ->
   %% * {{type, Tuple}, ...}, {{opaque, Tuple}, ...}
   %% * {{callback, Tuple}, ...}, {{macrocallback, Tuple}, ...}
   %% * {{def, Tuple}, ...} (from elixir_def)
-  %% * {{import, Tuple}, ...} (from elixir_locals)
   %% * {{overridable, Tuple}, ...} (from elixir_overridable)
   %%
   DataSet = ets:new(Module, [set, public]),
@@ -313,8 +388,6 @@ build(Line, File, Module) ->
   %% * {overridables, ...} (from elixir_overridable)
   %% * {{default, Name}, ...} (from elixir_def)
   %% * {{clauses, Tuple}, ...} (from elixir_def)
-  %% * {reattach, ...} (from elixir_locals)
-  %% * {{local, Tuple}, ...} (from elixir_locals)
   %%
   DataBag = ets:new(Module, [duplicate_bag, public]),
 
@@ -341,10 +414,11 @@ build(Line, File, Module) ->
     {optional_callbacks, [], accumulate, []},
 
     % Others
+    {?cache_key, 0},
     {?counter_attr, 0}
   ]),
 
-  Persisted = [behaviour, on_load, external_resource, dialyzer, vsn],
+  Persisted = [behaviour, dialyzer, external_resource, on_load, vsn, nifs],
   ets:insert(DataBag, [{persisted_attributes, Attr} || Attr <- Persisted]),
 
   OnDefinition =
@@ -357,7 +431,6 @@ build(Line, File, Module) ->
   %% Setup definition related modules
   Tables = {DataSet, DataBag},
   elixir_def:setup(Tables),
-  elixir_locals:setup(Tables),
   Tuple = {Module, Tables, Line, File, all},
 
   Ref =
@@ -368,20 +441,36 @@ build(Line, File, Module) ->
         ets:delete(DataSet),
         ets:delete(DataBag),
         Error = {module_in_definition, Module, OldFile, OldLine},
-        elixir_errors:form_error([{line, Line}], File, ?MODULE, Error)
+        elixir_errors:file_error([{line, Line}], E, ?MODULE, Error)
     end,
 
   {Tables, Ref}.
 
 %% Handles module and callback evaluations.
 
-eval_form(Line, Module, DataBag, Block, Vars, E) ->
-  {Value, EE} = elixir_compiler:compile(Block, Vars, E),
+eval_form(Line, Module, DataBag, Block, Vars, Prune, E) ->
+  %% Given Elixir modules can get very long to compile due to metaprogramming,
+  %% we disable expansions that take linear time to code size.
+  {Value, ExS, EE} = elixir_compiler:compile(Block, Vars, [no_bool_opt, no_ssa_opt], E),
   elixir_overridable:store_not_overridden(Module),
   EV = (elixir_env:reset_vars(EE))#{line := Line},
   EC = eval_callbacks(Line, DataBag, before_compile, [EV], EV),
   elixir_overridable:store_not_overridden(Module),
-  {Value, EC}.
+  {Value, maybe_prune_versioned_vars(Prune, Vars, ExS, E), EC}.
+
+maybe_prune_versioned_vars(false, _Vars, _Exs, E) ->
+  E;
+maybe_prune_versioned_vars(true, Vars, ExS, E) ->
+  PruneBefore = length(Vars),
+  #elixir_ex{vars={ExVars, _}, unused={Unused, _}} = ExS,
+
+  VersionedVars =
+    maps:filter(fun
+      (Pair, Version) when Version < PruneBefore, not is_map_key({Pair, Version}, Unused) -> false;
+      (_, _) -> true
+    end, ExVars),
+
+  E#{versioned_vars := VersionedVars}.
 
 eval_callbacks(Line, DataBag, Name, Args, E) ->
   Callbacks = bag_lookup_element(DataBag, {accumulate, Name}, 2),
@@ -395,8 +484,8 @@ expand_callback(Line, M, F, Args, Acc, Fun) ->
   Meta = [{line, Line}, {required, true}],
 
   {EE, _S, ET} =
-    elixir_dispatch:dispatch_require(Meta, M, F, Args, S, E, fun(AM, AF, AA) ->
-      Fun(AM, AF, AA),
+    elixir_dispatch:dispatch_require(Meta, M, F, Args, S, E, fun(AM, AF) ->
+      Fun(AM, AF, Args),
       {ok, S, E}
     end),
 
@@ -427,19 +516,19 @@ lookup_attribute(DataSet, DataBag, Key) when is_atom(Key) ->
     [] -> []
   end.
 
-warn_unused_attributes(File, DataSet, DataBag, PersistedAttrs) ->
+warn_unused_attributes(DataSet, DataBag, PersistedAttrs, E) ->
   StoredAttrs = bag_lookup_element(DataBag, warn_attributes, 2),
   %% This is the same list as in Module.put_attribute
   %% without moduledoc which are never warned on.
   Attrs = [doc, typedoc, impl, deprecated | StoredAttrs -- PersistedAttrs],
   Query = [{{Attr, '_', '$1', '_'}, [{is_integer, '$1'}], [[Attr, '$1']]} || Attr <- Attrs],
-  [elixir_errors:form_warn([{line, Line}], File, ?MODULE, {unused_attribute, Key})
+  [elixir_errors:file_warn([{line, Line}], E, ?MODULE, {unused_attribute, Key})
    || [Key, Line] <- ets:select(DataSet, Query)].
 
 get_struct(Set) ->
   case ets:lookup(Set, {elixir, struct}) of
     [] -> nil;
-    [{_, Struct}] -> Struct
+    [{_, Fields}] -> Fields
   end.
 
 get_deprecated(Bag) ->
@@ -464,14 +553,19 @@ beam_location(ModuleAsCharlist) ->
 
 checker_info() ->
   case get(elixir_checker_info) of
-    undefined -> undefined;
+    undefined -> {self(), nil};
     _ -> 'Elixir.Module.ParallelChecker':get()
   end.
 
-spawn_parallel_checker(undefined, _Module, _ModuleMap) ->
-  nil;
+spawn_parallel_checker({_, nil}, _Module, _ModuleMap) ->
+  ok;
 spawn_parallel_checker(CheckerInfo, Module, ModuleMap) ->
-  'Elixir.Module.ParallelChecker':spawn(CheckerInfo, Module, ModuleMap).
+  Log =
+    case erlang:get(elixir_code_diagnostics) of
+      {_, false} -> false;
+      _ -> true
+    end,
+  'Elixir.Module.ParallelChecker':spawn(CheckerInfo, Module, ModuleMap, Log).
 
 make_module_available(Module, Binary) ->
   case get(elixir_module_binaries) of
@@ -515,8 +609,6 @@ format_error({unused_attribute, deprecated}) ->
   "module attribute @deprecated was set but no definition follows it";
 format_error({unused_attribute, Attr}) ->
   io_lib:format("module attribute @~ts was set but never used", [Attr]);
-format_error({invalid_module, Module}) ->
-  io_lib:format("invalid module name: ~ts", ['Elixir.Kernel':inspect(Module)]);
 format_error({module_defined, Module}) ->
   Extra =
     case code:which(Module) of
@@ -533,15 +625,14 @@ format_error({module_reserved, Module}) ->
 format_error({module_in_definition, Module, File, Line}) ->
   io_lib:format("cannot define module ~ts because it is currently being defined in ~ts:~B",
     [elixir_aliases:inspect(Module), elixir_utils:relative_to_cwd(File), Line]);
-format_error({bad_inline, {Name, Arity}}) ->
-  io_lib:format("inlined function ~ts/~B undefined", [Name, Arity]);
-format_error({bad_dialyzer, Key, {Name, Arity}}) ->
-  io_lib:format("undefined function ~ts/~B given to @dialyzer :~ts", [Name, Arity, Key]);
-format_error({undefined_on_load, {Name, Arity}}) ->
-  io_lib:format("undefined function ~ts/~B given to @on_load", [Name, Arity]);
-format_error({wrong_kind_on_load, {Name, Arity}, WrongKind}) ->
-  io_lib:format("expected @on_load function ~ts/~B to be a function, got \"~ts\"",
-                [Name, Arity, WrongKind]);
+format_error({undefined_function, {Attr, Key}, {Name, Arity}}) ->
+  io_lib:format("undefined function ~ts/~B given to @~ts :~ts", [Name, Arity, Attr, Key]);
+format_error({undefined_function, Attr, {Name, Arity}}) ->
+  io_lib:format("undefined function ~ts/~B given to @~ts", [Name, Arity, Attr]);
+format_error({bad_macro, {Attr, Key}, {Name, Arity}}) ->
+  io_lib:format("macro ~ts/~B given to @~ts :~ts (only functions are supported)", [Name, Arity, Attr, Key]);
+format_error({bad_macro, Attr, {Name, Arity}}) ->
+  io_lib:format("macro ~ts/~B given to @~ts (only functions are supported)", [Name, Arity, Attr]);
 format_error({parse_transform, Module}) ->
   io_lib:format("@compile {:parse_transform, ~ts} is deprecated. Elixir will no longer support "
                 "Erlang-based transforms in future versions", [elixir_aliases:inspect(Module)]).
