@@ -1,12 +1,18 @@
 % Holds the logic responsible for defining overridable functions and handling super.
 -module(elixir_overridable).
--export([overridable_for/2, record_overridable/4, super/4, store_not_overridden/1, format_error/1]).
+-export([overridables_for/1, overridable_for/2,
+         record_overridable/3, super/4,
+         store_not_overridden/1, format_error/1]).
 -include("elixir.hrl").
--define(overridden_pos, 5).
+-define(overridden_pos, 4).
 
 overridables_for(Module) ->
   {_, Bag} = elixir_module:data_tables(Module),
-  ets:lookup(Bag, overridables).
+  try
+    ets:lookup_element(Bag, overridables, 2)
+  catch
+    _:_ -> []
+  end.
 
 overridable_for(Module, Tuple) ->
   {Set, _} = elixir_module:data_tables(Module),
@@ -16,22 +22,22 @@ overridable_for(Module, Tuple) ->
     [] -> not_overridable
   end.
 
-record_overridable(Module, Tuple, Def, Neighbours) ->
+record_overridable(Module, Tuple, Def) ->
   {Set, Bag} = elixir_module:data_tables(Module),
 
-  case ets:insert_new(Set, {{overridable, Tuple}, 1, Def, Neighbours, false}) of
+  case ets:insert_new(Set, {{overridable, Tuple}, 1, Def, false}) of
     true ->
       ets:insert(Bag, {overridables, Tuple});
     false ->
-      [{_, Count, PreviousDef, _, _}] = ets:lookup(Set, {overridable, Tuple}),
+      [{_, Count, PreviousDef, _}] = ets:lookup(Set, {overridable, Tuple}),
       {{_, Kind, Meta, File, _, _}, _} = Def,
       {{_, PreviousKind, _, _, _, _}, _} = PreviousDef,
 
       case is_valid_kind(Kind, PreviousKind) of
         true ->
-          ets:insert(Set, {{overridable, Tuple}, Count + 1, Def, Neighbours, false});
+          ets:insert(Set, {{overridable, Tuple}, Count + 1, Def, false});
         false ->
-          elixir_errors:form_error(Meta, File, ?MODULE, {bad_kind, Module, Tuple, Kind})
+          elixir_errors:file_error(Meta, File, ?MODULE, {bad_kind, Module, Tuple, Kind})
       end
   end,
 
@@ -44,7 +50,7 @@ super(Meta, Module, Tuple, E) ->
     [Overridable] ->
       store(Set, Module, Tuple, Overridable, true);
     [] ->
-      elixir_errors:form_error(Meta, E, ?MODULE, {no_super, Module, Tuple})
+      elixir_errors:file_error(Meta, E, ?MODULE, {no_super, Module, Tuple})
   end.
 
 store_not_overridden(Module) ->
@@ -61,16 +67,17 @@ store_not_overridden(Module) ->
 
         case is_valid_kind(Kind, OverridableKind) of
           true -> ok;
-          false -> elixir_errors:form_error(Meta, File, ?MODULE, {bad_kind, Module, Tuple, Kind})
+          false -> elixir_errors:file_error(Meta, File, ?MODULE, {bad_kind, Module, Tuple, Kind})
         end
     end
   end, ets:lookup(Bag, overridables)).
 
 %% Private
 
-store(Set, Module, Tuple, {_, Count, Def, Neighbours, Overridden}, Hidden) ->
-  {{{def, {Name, Arity}}, Kind, Meta, File, _Check,
+store(Set, Module, Tuple, {_, Count, Def, Overridden}, Hidden) ->
+  {{{def, {Name, Arity}}, Kind, BaseMeta, File, _Check,
    {Defaults, _HasBody, _LastDefaults}}, Clauses} = Def,
+  Meta = [{from_super, Hidden} | BaseMeta],
 
   {FinalKind, FinalName, FinalArity, FinalClauses} =
     case Hidden of
@@ -85,9 +92,8 @@ store(Set, Module, Tuple, {_, Count, Def, Neighbours, Overridden}, Hidden) ->
   case Overridden of
     false ->
       ets:update_element(Set, {overridable, Tuple}, {?overridden_pos, true}),
-      elixir_def:store_definition(false, FinalKind, Meta, FinalName, FinalArity,
-                                  File, Module, Defaults, FinalClauses),
-      elixir_locals:reattach({FinalName, FinalArity}, FinalKind, Module, Tuple, Neighbours, Meta);
+      elixir_def:store_definition(none, FinalKind, Meta, FinalName, FinalArity,
+                                  File, Module, Defaults, FinalClauses);
     true ->
       ok
   end,
@@ -116,12 +122,12 @@ format_error({bad_kind, Module, {Name, Arity}, Kind}) ->
   end;
 
 format_error({no_super, Module, {Name, Arity}}) ->
-  Bins   = [format_fa(Tuple) || {_, Tuple} <- overridables_for(Module)],
+  Bins   = [format_fa(Tuple) || Tuple <- overridables_for(Module)],
   Joined = 'Elixir.Enum':join(Bins, <<", ">>),
   io_lib:format("no super defined for ~ts/~B in module ~ts. Overridable functions available are: ~ts",
     [Name, Arity, elixir_aliases:inspect(Module), Joined]).
 
 format_fa({Name, Arity}) ->
-  A = 'Elixir.Code.Identifier':inspect_as_function(Name),
+  A = 'Elixir.Macro':inspect_atom(remote_call, Name),
   B = integer_to_binary(Arity),
   <<A/binary, $/, B/binary>>.
