@@ -63,6 +63,15 @@ defmodule Code.Formatter.OperatorsTest do
       assert_format "not !1", "not (!1)"
       assert_format "not(!1)", "not (!1)"
       assert_format "not(1 + 1)", "not (1 + 1)"
+      assert_format "-(2**2)", "-(2 ** 2)"
+    end
+
+    test "wraps operand in ambiguous calls" do
+      assert_same "def -(2 ** 2)"
+      assert_same "def -var"
+      assert_format "def (-(2 ** 2))", "def -(2 ** 2)"
+      assert_format "def --var", "def -- var"
+      assert_format "def -+var", "def -(+var)"
     end
 
     test "does not wrap operand if it is a nestable operator" do
@@ -414,6 +423,9 @@ defmodule Code.Formatter.OperatorsTest do
 
     test "with multiple of the same entry and right associative" do
       assert_same "foo ++ bar ++ baz"
+      assert_format "foo -- bar -- baz", "foo -- (bar -- baz)"
+      assert_same "foo +++ bar +++ baz"
+      assert_format "foo --- bar --- baz", "foo --- (bar --- baz)"
 
       bad = "a ++ b ++ c"
 
@@ -677,11 +689,12 @@ defmodule Code.Formatter.OperatorsTest do
       """
 
       good = """
-      var = [
-        one,
-        two,
-        three
-      ]
+      var =
+        [
+          one,
+          two,
+          three
+        ]
       """
 
       assert_format bad, good, @short_length
@@ -709,19 +722,19 @@ defmodule Code.Formatter.OperatorsTest do
     end
 
     test "with heredoc" do
-      heredoc = ~S"""
-      var = '''
+      heredoc = ~S'''
+      var = """
       one
-      '''
       """
+      '''
 
       assert_same heredoc, @short_length
 
-      heredoc = ~S"""
-      var = '''
+      heredoc = ~S'''
+      var = """
       #{one}
-      '''
       """
+      '''
 
       assert_same heredoc, @short_length
     end
@@ -807,19 +820,19 @@ defmodule Code.Formatter.OperatorsTest do
     end
 
     test "with next break fits" do
-      attribute = """
-      @doc '''
+      attribute = ~S'''
+      @doc """
       foo
-      '''
       """
+      '''
 
       assert_same attribute
 
-      attribute = """
-      @doc foo: '''
+      attribute = ~S'''
+      @doc foo: """
            bar
-           '''
-      """
+           """
+      '''
 
       assert_same attribute
     end
@@ -871,6 +884,7 @@ defmodule Code.Formatter.OperatorsTest do
 
     test "with operators inside" do
       assert_format "& +1", "&(+1)"
+      assert_format "& 1[:foo]", "& 1[:foo]"
       assert_format "& not &1", "&(not &1)"
       assert_format "& a ++ b", "&(a ++ b)"
       assert_format "& &1 && &2", "&(&1 && &2)"
@@ -934,6 +948,7 @@ defmodule Code.Formatter.OperatorsTest do
       assert_format "&(Mod.foo/1)", "&Mod.foo/1"
       assert_format "&(Mod.++/1)", "&Mod.++/1"
       assert_format ~s[&(Mod."foo bar"/1)], ~s[&Mod."foo bar"/1]
+      assert_format ~S[&(Mod."foo\nbar"/1)], ~S[&Mod."foo\nbar"/1]
 
       # Invalid
       assert_format "& Mod.foo/bar", "&(Mod.foo() / bar)"
