@@ -9,6 +9,11 @@ defmodule Code.Normalizer.QuotedASTTest do
       assert quoted_to_string({:{}, [], nil}) == "{}"
     end
 
+    test "variable with colors" do
+      opts = [syntax_colors: [variable: :blue]]
+      assert quoted_to_string(quote(do: foo), opts) == "\e[34mfoo\e[0m"
+    end
+
     test "local call" do
       assert quoted_to_string(quote(do: foo(1, 2, 3))) == "foo(1, 2, 3)"
       assert quoted_to_string(quote(do: foo([1, 2, 3]))) == "foo([1, 2, 3])"
@@ -25,6 +30,13 @@ defmodule Code.Normalizer.QuotedASTTest do
       assert quoted_to_string(quote(line: __ENV__.line, do: foo(1, b))) == "foo(1, b)"
     end
 
+    test "local call with colors" do
+      opts = [syntax_colors: [call: :blue, number: :yellow, variable: :red]]
+
+      assert quoted_to_string(quote(do: foo(1, a)), opts) ==
+               "\e[34mfoo\e[0m(\e[33m1\e[0m, \e[31ma\e[0m)"
+    end
+
     test "local call no parens" do
       assert quoted_to_string({:def, [], [1, 2]}) == "def 1, 2"
       assert quoted_to_string({:def, [closing: []], [1, 2]}) == "def(1, 2)"
@@ -77,6 +89,19 @@ defmodule Code.Normalizer.QuotedASTTest do
       assert quoted_to_string(quote(do: Foo.bar(<<>>, []))) == "Foo.bar(<<>>, [])"
     end
 
+    test "remote call with colors" do
+      opts = [syntax_colors: [call: :blue, number: :yellow, variable: :red, atom: :green]]
+
+      assert quoted_to_string(quote(do: foo.bar(1, 2)), opts) ==
+               "\e[31mfoo\e[0m.\e[34mbar\e[0m(\e[33m1\e[0m, \e[33m2\e[0m)"
+
+      assert quoted_to_string(quote(do: :foo.bar(1, 2)), opts) ==
+               "\e[32m:foo\e[0m.\e[34mbar\e[0m(\e[33m1\e[0m, \e[33m2\e[0m)"
+
+      assert quoted_to_string(quote(do: Foo.Bar.bar(1, 2)), opts) ==
+               "\e[32mFoo.Bar\e[0m.\e[34mbar\e[0m(\e[33m1\e[0m, \e[33m2\e[0m)"
+    end
+
     test "keyword call" do
       assert quoted_to_string(quote(do: Foo.bar(foo: :bar))) == "Foo.bar(foo: :bar)"
       assert quoted_to_string(quote(do: Foo.bar("Elixir.Foo": :bar))) == "Foo.bar([{Foo, :bar}])"
@@ -147,6 +172,20 @@ defmodule Code.Normalizer.QuotedASTTest do
              ) == ~s[~S"""\n"123"\n"""]
     end
 
+    test "regression: invalid sigil calls" do
+      assert quoted_to_string(quote do: sigil_r(<<"foo", 123>>, [])) ==
+               "sigil_r(<<\"foo\", 123>>, [])"
+
+      assert quoted_to_string(quote do: sigil_r(<<"foo">>, :invalid_modifiers)) ==
+               "sigil_r(\"foo\", :invalid_modifiers)"
+
+      assert quoted_to_string(quote do: sigil_r(<<"foo">>, [:invalid_modifier])) ==
+               "sigil_r(\"foo\", [:invalid_modifier])"
+
+      assert quoted_to_string(quote do: sigil_r(<<"foo">>, [])) == "~r\"foo\""
+      assert quoted_to_string(quote do: sigil_r(<<"foo">>, [?a, ?b, ?c])) == "~r\"foo\"abc"
+    end
+
     test "tuple" do
       assert quoted_to_string(quote do: {1, 2}) == "{1, 2}"
       assert quoted_to_string(quote do: {1}) == "{1}"
@@ -154,6 +193,13 @@ defmodule Code.Normalizer.QuotedASTTest do
       assert quoted_to_string(quote do: {1, 2, 3, foo: :bar}) == "{1, 2, 3, foo: :bar}"
     end
 
+    test "tuple with colors" do
+      opts = [syntax_colors: [tuple: :blue, number: :yellow]]
+
+      assert quoted_to_string(quote(do: {1, 2, 3}), opts) ==
+               "\e[34m{\e[0m\e[33m1\e[0m, \e[33m2\e[0m, \e[33m3\e[0m\e[34m}\e[0m"
+    end
+
     test "tuple call" do
       assert quoted_to_string(quote(do: alias(Foo.{Bar, Baz, Bong}))) ==
                "alias Foo.{Bar, Baz, Bong}"
@@ -359,16 +405,20 @@ defmodule Code.Normalizer.QuotedASTTest do
     end
 
     test "range" do
-      assert quoted_to_string(quote(do: unquote(-1..+2))) == "-1..2"
+      assert quoted_to_string(quote(do: -1..+2)) == "-1..+2"
       assert quoted_to_string(quote(do: Foo.integer()..3)) == "Foo.integer()..3"
-      assert quoted_to_string(quote(do: unquote(-1..+2//-3))) == "-1..2//-3"
+      assert quoted_to_string(quote(do: -1..+2//-3)) == "-1..+2//-3"
 
       assert quoted_to_string(quote(do: Foo.integer()..3//Bar.bat())) ==
                "Foo.integer()..3//Bar.bat()"
+
+      # invalid AST
+      assert quoted_to_string(-1..+2) == "-1..2"
+      assert quoted_to_string(-1..+2//-3) == "-1..2//-3"
     end
 
     test "when" do
-      assert quoted_to_string(quote(do: (() -> x))) == "(() -> x)"
+      assert quoted_to_string(quote(do: (-> x))) == "(-> x)"
       assert quoted_to_string(quote(do: (x when y -> z))) == "(x when y -> z)"
       assert quoted_to_string(quote(do: (x, y when z -> w))) == "(x, y when z -> w)"
       assert quoted_to_string(quote(do: (x, y when z -> w))) == "(x, y when z -> w)"
@@ -463,6 +513,12 @@ defmodule Code.Normalizer.QuotedASTTest do
       assert quoted_to_string(quote(do: identity(&1))) == "identity(&1)"
     end
 
+    test "operators with colors" do
+      opts = [syntax_colors: [operator: :blue, number: :yellow]]
+      assert quoted_to_string(quote(do: !!1), opts) == "\e[34m!\e[0m\e[34m!\e[0m\e[33m1\e[0m"
+      assert quoted_to_string(quote(do: 1 + 2), opts) == "\e[33m1\e[0m\e[34m +\e[0m \e[33m2\e[0m"
+    end
+
     test "access" do
       assert quoted_to_string(quote(do: a[b])) == "a[b]"
       assert quoted_to_string(quote(do: a[1 + 2])) == "a[1 + 2]"
@@ -483,6 +539,17 @@ defmodule Code.Normalizer.QuotedASTTest do
       assert quoted_to_string(quote(do: [{binary(), integer()}])) == "[{binary(), integer()}]"
     end
 
+    test "keyword list with colors" do
+      opts = [syntax_colors: [list: :blue, atom: :green, number: :yellow]]
+
+      assert quoted_to_string(quote(do: [a: 1, b: 2]), opts) ==
+               "\e[34m[\e[0m\e[32ma:\e[0m \e[33m1\e[0m, \e[32mb:\e[0m \e[33m2\e[0m\e[34m]\e[0m"
+    end
+
+    test "keyword list with :do as operand" do
+      assert quoted_to_string(quote(do: a = [do: 1])) == "a = [do: 1]"
+    end
+
     test "interpolation" do
       assert quoted_to_string(quote(do: "foo#{bar}baz")) == ~S["foo#{bar}baz"]
     end
@@ -510,11 +577,29 @@ defmodule Code.Normalizer.QuotedASTTest do
     test "integer/float" do
       assert quoted_to_string(1) == "1"
       assert quoted_to_string({:__block__, [], [1]}) == "1"
+      assert quoted_to_string(1.23) == "1.23"
+    end
+
+    test "integer/float with colors" do
+      opts = [syntax_colors: [number: :yellow]]
+      assert quoted_to_string(1, opts) == "\e[33m1\e[0m"
+      assert quoted_to_string(1.23, opts) == "\e[33m1.23\e[0m"
     end
 
     test "charlist" do
       assert quoted_to_string(quote(do: [])) == "[]"
-      assert quoted_to_string(quote(do: 'abc')) == "'abc'"
+      assert quoted_to_string(quote(do: ~c"abc")) == ~S/~c"abc"/
+
+      # False positive
+      assert quoted_to_string(
+               quote do
+                 :"Elixir.List".to_charlist([
+                   case var do
+                     var -> var
+                   end
+                 ])
+               end
+             ) =~ "List.to_charlist([\n  case var do\n    var -> var\n  end\n])"
     end
 
     test "string" do
@@ -523,6 +608,11 @@ defmodule Code.Normalizer.QuotedASTTest do
       assert quoted_to_string(quote(do: "#{"abc"}")) == ~S/"#{"abc"}"/
     end
 
+    test "string with colors" do
+      opts = [syntax_colors: [string: :green]]
+      assert quoted_to_string(quote(do: "abc"), opts) == "\e[32m\"abc\"\e[0m"
+    end
+
     test "catch-all" do
       assert quoted_to_string(quote do: {unquote(self())}) == "{#{inspect(self())}}"
       assert quoted_to_string(quote do: foo(unquote(self()))) == "foo(#{inspect(self())})"
@@ -543,6 +633,36 @@ defmodule Code.Normalizer.QuotedASTTest do
       assert quoted_to_string(quote(do: foo |> [bar: :baz])) == "foo |> [bar: :baz]"
     end
 
+    test "keyword arg with cursor" do
+      input = "def foo, do: :bar, __cursor__()"
+      expected = "def foo, [{:do, :bar}, __cursor__()]"
+
+      ast = Code.string_to_quoted!(input)
+      assert quoted_to_string(ast) == expected
+
+      encoder = &{:ok, {:__block__, &2, [&1]}}
+      ast = Code.string_to_quoted!(input, literal_encoder: encoder)
+      assert quoted_to_string(ast) == expected
+
+      ast = Code.string_to_quoted!(input, token_metadata: true)
+      assert quoted_to_string(ast) == expected
+
+      ast = Code.string_to_quoted!(input, literal_encoder: encoder, token_metadata: true)
+      assert quoted_to_string(ast) == expected
+    end
+
+    test "keyword arg with literal encoder and no metadata" do
+      input = """
+      foo(Bar) do
+        :ok
+      end
+      """
+
+      encoder = &{:ok, {:__block__, &2, [&1]}}
+      ast = Code.string_to_quoted!(input, literal_encoder: encoder)
+      assert quoted_to_string(ast) == "foo(Bar, do: :ok)"
+    end
+
     test "list in module attribute" do
       assert quoted_to_string(
                quote do
@@ -591,32 +711,22 @@ defmodule Code.Normalizer.QuotedASTTest do
     end
 
     test "charlists with slash escapes" do
-      assert quoted_to_string(quote(do: '\a\b\e\n\r\t\v'), escape: false) ==
-               ~s/'\a\b\e\n\r\t\v'/
+      assert quoted_to_string(~c"\a\b\e\n\r\t\v", escape: false) ==
+               ~s/~c"\a\b\e\n\r\t\v"/
 
-      assert quoted_to_string(quote(do: '\a\b\e\n\r\t\v')) ==
-               ~s/'\\a\\b\\e\\n\\r\\t\\v'/
+      assert quoted_to_string(~c"\a\b\e\n\r\t\v") ==
+               ~s/~c"\\a\\b\\e\\n\\r\\t\\v"/
 
-      assert quoted_to_string({:__block__, [], ['\a\b\e\n\r\t\v']}, escape: false) ==
-               ~s/'\a\b\e\n\r\t\v'/
+      assert quoted_to_string({:__block__, [], [~c"\a\b\e\n\r\t\v"]}, escape: false) ==
+               ~s/~c"\a\b\e\n\r\t\v"/
 
-      assert quoted_to_string({:__block__, [], ['\a\b\e\n\r\t\v']}) ==
-               ~s/'\\a\\b\\e\\n\\r\\t\\v'/
+      assert quoted_to_string({:__block__, [], [~c"\a\b\e\n\r\t\v"]}) ==
+               ~s/~c"\\a\\b\\e\\n\\r\\t\\v"/
     end
 
     test "charlists with non printable characters" do
-      assert quoted_to_string(quote(do: '\x00\x01\x10'), escape: false) == ~S/[0, 1, 16]/
-      assert quoted_to_string(quote(do: '\x00\x01\x10')) == ~S/[0, 1, 16]/
-    end
-
-    test "charlists with interpolations" do
-      assert quoted_to_string(quote(do: 'one #{2} three'), escape: false) == ~S/'one #{2} three'/
-      assert quoted_to_string(quote(do: 'one #{2} three')) == ~S/'one #{2} three'/
-
-      assert quoted_to_string(quote(do: 'one\n\'#{2}\'\nthree'), escape: false) ==
-               ~s['one\n\\'\#{2}\\'\nthree']
-
-      assert quoted_to_string(quote(do: 'one\n\'#{2}\'\nthree')) == ~S['one\n\'#{2}\'\nthree']
+      assert quoted_to_string(~c"\x00\x01\x10", escape: false) == ~S/[0, 1, 16]/
+      assert quoted_to_string(~c"\x00\x01\x10") == ~S/[0, 1, 16]/
     end
 
     test "atoms" do
@@ -656,6 +766,30 @@ defmodule Code.Normalizer.QuotedASTTest do
 
       assert quoted_to_string(quote(do: :"one\n\"#{2}\"\nthree")) == ~S[:"one\n\"#{2}\"\nthree"]
     end
+
+    test ":erlang.binary_to_atom/2 edge cases" do
+      assert quoted_to_string(quote(do: :erlang.binary_to_atom(<<>>, :utf8))) == ~S[:""]
+
+      assert quoted_to_string(quote(do: :erlang.binary_to_atom(<<1>>, :utf8))) ==
+               ~S":erlang.binary_to_atom(<<1>>, :utf8)"
+    end
+  end
+
+  describe "quoted_to_algebra/2 with invalid" do
+    test "block" do
+      assert quoted_to_string({:__block__, [], {:bar, [], []}}) ==
+               "{:__block__, [], {:bar, [], []}}"
+
+      assert quoted_to_string({:foo, [], [{:do, :ok}, :not_keyword]}) ==
+               "foo({:do, :ok}, :not_keyword)"
+
+      assert quoted_to_string({:foo, [], [[{:do, :ok}, :not_keyword]]}) ==
+               "foo([{:do, :ok}, :not_keyword])"
+    end
+
+    test "ode" do
+      assert quoted_to_string(1..3) == "1..3"
+    end
   end
 
   describe "quoted_to_algebra/2 does not escape" do
