@@ -99,6 +99,9 @@ defmodule DynamicSupervisorTest do
       assert DynamicSupervisor.start_link(Simple, {:ok, %{extra_arguments: -1}}) ==
                {:error, {:supervisor_data, {:invalid_extra_arguments, -1}}}
 
+      assert DynamicSupervisor.start_link(Simple, {:ok, %{auto_shutdown: :any_significant}}) ==
+               {:error, {:supervisor_data, {:invalid_auto_shutdown, :any_significant}}}
+
       assert DynamicSupervisor.start_link(Simple, :unknown) ==
                {:error, {:bad_return, {Simple, :init, :unknown}}}
 
@@ -137,14 +140,6 @@ defmodule DynamicSupervisorTest do
       {:ok, pid} = DynamicSupervisor.start_link(strategy: :one_for_one)
       assert :proc_lib.initial_call(pid) == {:supervisor, Supervisor.Default, [:Argument__1]}
     end
-
-    test "returns the callback module" do
-      {:ok, pid} = Supervisor.start_link([], strategy: :one_for_one)
-      assert :supervisor.get_callback_module(pid) == Supervisor.Default
-
-      {:ok, pid} = DynamicSupervisor.start_link(strategy: :one_for_one)
-      assert :supervisor.get_callback_module(pid) == Supervisor.Default
-    end
   end
 
   ## Code change
@@ -229,6 +224,20 @@ defmodule DynamicSupervisorTest do
 
       assert DynamicSupervisor.start_child(:not_used, %{id: 1, start: {Task, :foo, :bar}}) ==
                {:error, {:invalid_mfa, {Task, :foo, :bar}}}
+
+      assert DynamicSupervisor.start_child(:not_used, %{
+               id: 1,
+               start: {Task, :foo, [:bar]},
+               shutdown: -1
+             }) ==
+               {:error, {:invalid_shutdown, -1}}
+
+      assert DynamicSupervisor.start_child(:not_used, %{
+               id: 1,
+               start: {Task, :foo, [:bar]},
+               significant: true
+             }) ==
+               {:error, {:invalid_significant, true}}
     end
 
     test "with different returns" do
@@ -451,6 +460,31 @@ defmodule DynamicSupervisorTest do
       assert_receive {:EXIT, ^pid, :shutdown}
     end
 
+    test "with valid shutdown" do
+      Process.flag(:trap_exit, true)
+
+      {:ok, pid} = DynamicSupervisor.start_link(strategy: :one_for_one)
+
+      for n <- 0..1 do
+        assert {:ok, child_pid} =
+                 DynamicSupervisor.start_child(pid, %{
+                   id: n,
+                   start: {Task, :start_link, [fn -> Process.sleep(:infinity) end]},
+                   shutdown: n
+                 })
+
+        assert_kill(child_pid, :shutdown)
+      end
+    end
+
+    test "with invalid valid shutdown" do
+      assert DynamicSupervisor.start_child(:not_used, %{
+               id: 1,
+               start: {Task, :start_link, [fn -> :ok end]},
+               shutdown: -1
+             }) == {:error, {:invalid_shutdown, -1}}
+    end
+
     def start_link(:ok3), do: {:ok, spawn_link(fn -> Process.sleep(:infinity) end), :extra}
     def start_link(:ok2), do: {:ok, spawn_link(fn -> Process.sleep(:infinity) end)}
     def start_link(:error), do: {:error, :found}
