@@ -18,13 +18,20 @@ defmodule ExceptionTest do
       end
     end
 
-    assert Exception.message(%{__struct__: BadException, __exception__: true, raise: true}) =~
-             "got RuntimeError with message \"oops\" while retrieving Exception.message/1 " <>
-               "for %{__exception__: true, __struct__: ExceptionTest.BadException, raise: true}"
+    assert "got RuntimeError with message \"oops\" while retrieving Exception.message/1 for %{" <>
+             inspected =
+             Exception.message(%{__struct__: BadException, __exception__: true, raise: true})
 
-    assert Exception.message(%{__struct__: BadException, __exception__: true, raise: false}) =~
-             "got nil while retrieving Exception.message/1 " <>
-               "for %{__exception__: true, __struct__: ExceptionTest.BadException, raise: false}"
+    assert inspected =~ "raise: true"
+    assert inspected =~ "__exception__: true"
+    assert inspected =~ "__struct__: ExceptionTest.BadException"
+
+    assert "got nil while retrieving Exception.message/1 for %{" <> inspected =
+             Exception.message(%{__struct__: BadException, __exception__: true, raise: false})
+
+    assert inspected =~ "raise: false"
+    assert inspected =~ "__exception__: true"
+    assert inspected =~ "__struct__: ExceptionTest.BadException"
   end
 
   test "normalize/2" do
@@ -104,35 +111,37 @@ defmodule ExceptionTest do
   end
 
   test "format_stacktrace_entry/1 with file and line" do
-    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: 'file.ex', line: 10]}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: ~c"file.ex", line: 10]}) ==
              "file.ex:10: Foo.bar()"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, [1, 2, 3], [file: 'file.ex', line: 10]}) ==
+    assert Exception.format_stacktrace_entry(
+             {Foo, :bar, [1, 2, 3], [file: ~c"file.ex", line: 10]}
+           ) ==
              "file.ex:10: Foo.bar(1, 2, 3)"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, 1, [file: 'file.ex', line: 10]}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, 1, [file: ~c"file.ex", line: 10]}) ==
              "file.ex:10: Foo.bar/1"
   end
 
   test "format_stacktrace_entry/1 with file no line" do
-    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: 'file.ex']}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: ~c"file.ex"]}) ==
              "file.ex: Foo.bar()"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: 'file.ex', line: 0]}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, [], [file: ~c"file.ex", line: 0]}) ==
              "file.ex: Foo.bar()"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, [1, 2, 3], [file: 'file.ex']}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, [1, 2, 3], [file: ~c"file.ex"]}) ==
              "file.ex: Foo.bar(1, 2, 3)"
 
-    assert Exception.format_stacktrace_entry({Foo, :bar, 1, [file: 'file.ex']}) ==
+    assert Exception.format_stacktrace_entry({Foo, :bar, 1, [file: ~c"file.ex"]}) ==
              "file.ex: Foo.bar/1"
   end
 
   test "format_stacktrace_entry/1 with application" do
-    assert Exception.format_stacktrace_entry({Exception, :bar, [], [file: 'file.ex']}) ==
+    assert Exception.format_stacktrace_entry({Exception, :bar, [], [file: ~c"file.ex"]}) ==
              "(elixir #{System.version()}) file.ex: Exception.bar()"
 
-    assert Exception.format_stacktrace_entry({Exception, :bar, [], [file: 'file.ex', line: 10]}) ==
+    assert Exception.format_stacktrace_entry({Exception, :bar, [], [file: ~c"file.ex", line: 10]}) ==
              "(elixir #{System.version()}) file.ex:10: Exception.bar()"
   end
 
@@ -412,6 +421,35 @@ defmodule ExceptionTest do
       assert Exception.blame(:exit, :function_clause, stack) == {:function_clause, stack}
     end
 
+    test "handles operators precedence" do
+      import PathHelpers
+
+      write_beam(
+        defmodule OperatorPrecedence do
+          def test!(x, y) when x in [1, 2, 3] and y >= 4, do: :ok
+        end
+      )
+
+      :code.purge(OperatorPrecedence)
+      :code.delete(OperatorPrecedence)
+
+      assert blame_message(OperatorPrecedence, & &1.test!(1, 2)) =~ """
+             no function clause matching in ExceptionTest.OperatorPrecedence.test!/2
+
+             The following arguments were given to ExceptionTest.OperatorPrecedence.test!/2:
+
+                 # 1
+                 1
+
+                 # 2
+                 2
+
+             Attempted function clauses (showing 1 out of 1):
+
+                 def test!(x, y) when (x === 1 or -x === 2- or -x === 3-) and -y >= 4-
+             """
+    end
+
     test "reverts is_struct macro on guards for blaming" do
       import PathHelpers
 
@@ -426,11 +464,13 @@ defmodule ExceptionTest do
               when is_binary(url) or (is_struct(url) and is_struct(url, url_module) and false) do
             url
           end
+
+          def sub_get!(url) when is_struct(url.sub, URI), do: url.sub
         end
       )
 
-      :code.delete(Req)
       :code.purge(Req)
+      :code.delete(Req)
 
       assert blame_message(Req, & &1.get!(url: "https://elixir-lang.org")) =~ """
              no function clause matching in ExceptionTest.Req.get!/1
@@ -475,30 +515,34 @@ defmodule ExceptionTest do
 
                  def get!(url) when -is_binary(url)- or is_struct(url) and is_struct(url, URI) and -false-
              """
+
+      assert blame_message(Req, & &1.sub_get!(%{})) =~ """
+             no function clause matching in ExceptionTest.Req.sub_get!/1
+
+             The following arguments were given to ExceptionTest.Req.sub_get!/1:
+
+                 # 1
+                 %{}
+
+             Attempted function clauses (showing 1 out of 1):
+
+                 def sub_get!(url) when -is_struct(url.sub, URI)-
+             """
     end
 
     test "annotates badarg on apply" do
       assert blame_message([], & &1.foo()) ==
                "you attempted to apply a function named :foo on []. If you are using Kernel.apply/3, make sure " <>
                  "the module is an atom. If you are using the dot syntax, such as " <>
-                 "module.function(), make sure the left-hand side of the dot is a module atom"
+                 "module.function(), make sure the left-hand side of the dot is an atom representing a module"
 
       assert blame_message([], &apply(&1, :foo, [])) ==
                "you attempted to apply a function named :foo on []. If you are using Kernel.apply/3, make sure " <>
                  "the module is an atom. If you are using the dot syntax, such as " <>
-                 "module.function(), make sure the left-hand side of the dot is a module atom"
+                 "module.function(), make sure the left-hand side of the dot is an atom representing a module"
 
-      assert blame_message([], &apply(Kernel, &1, [1, 2])) ==
-               "you attempted to apply a function named [] on module Kernel. However [] is not a valid function name. " <>
-                 "Function names (the second argument of apply) must always be an atom"
-
-      assert blame_message(123, &apply(Kernel, :+, &1)) ==
-               "you attempted to apply a function named :+ on module Kernel with arguments 123. " <>
-                 "Arguments (the third argument of apply) must always be a proper list"
-
-      assert blame_message(123, &apply(Kernel, :+, [&1 | 456])) ==
-               "you attempted to apply a function named :+ on module Kernel with arguments [123 | 456]. " <>
-                 "Arguments (the third argument of apply) must always be a proper list"
+      assert blame_message([], &apply(&1, :foo, [1, 2])) ==
+               "you attempted to apply a function on []. Modules (the first argument of apply) must always be an atom"
     end
 
     test "annotates function clause errors" do
@@ -542,9 +586,52 @@ defmodule ExceptionTest do
       assert message =~ "* set_cookie/2"
     end
 
+    test "annotates undefined function error with module suggestions" do
+      import PathHelpers
+
+      modules = [
+        Namespace.A.One,
+        Namespace.A.Two,
+        Namespace.A.Three,
+        Namespace.B.One,
+        Namespace.B.Two,
+        Namespace.B.Three
+      ]
+
+      for module <- modules do
+        write_beam(
+          defmodule module do
+            def foo, do: :bar
+          end
+        )
+      end
+
+      assert blame_message(ENUM, & &1.map(&1, 1)) == """
+             function ENUM.map/2 is undefined (module ENUM is not available). Did you mean:
+
+                   * Enum.map/2
+             """
+
+      assert blame_message(ENUM, & &1.not_a_function(&1, 1)) ==
+               "function ENUM.not_a_function/2 is undefined (module ENUM is not available). " <>
+                 "Make sure the module name is correct and has been specified in full (or that an alias has been defined)"
+
+      assert blame_message(One, & &1.foo()) == """
+             function One.foo/0 is undefined (module One is not available). Did you mean:
+
+                   * Namespace.A.One.foo/0
+                   * Namespace.B.One.foo/0
+             """
+
+      for module <- modules do
+        :code.purge(module)
+        :code.delete(module)
+      end
+    end
+
     test "annotates undefined function clause error with macro hints" do
       assert blame_message(Integer, & &1.is_odd(1)) ==
-               "function Integer.is_odd/1 is undefined or private. However there is " <>
+               "function Integer.is_odd/1 is undefined or private. However, there is " <>
                  "a macro with the same name and arity. Be sure to require Integer if " <>
                  "you intend to invoke this macro"
     end
@@ -590,12 +677,7 @@ defmodule ExceptionTest do
 
     test "annotates undefined key error with nil hints" do
       assert blame_message(nil, & &1.foo) ==
-               "key :foo not found in: nil. If you are using the dot syntax, " <>
-                 "such as map.field, make sure the left-hand side of the dot is a map"
-
-      # we use `Code.eval_string/1` to escape the formatter and warnings
-      assert blame_message("nil.foo", &Code.eval_string/1) ==
-               "key :foo not found in: nil. If you are using the dot syntax, " <>
+               "key :foo not found in: nil\n\nIf you are using the dot syntax, " <>
                  "such as map.field, make sure the left-hand side of the dot is a map"
     end
 
@@ -758,8 +840,8 @@ defmodule ExceptionTest do
         end
       )
 
-      :code.delete(Blaming)
       :code.purge(Blaming)
+      :code.delete(Blaming)
 
       {:ok, :def, clauses} = Exception.blame_mfa(Blaming, :with_elem, [1, 2])
 
@@ -854,7 +936,9 @@ defmodule ExceptionTest do
              |> message == "function Kernel.bar/1 is undefined or private"
 
       assert %UndefinedFunctionError{module: Foo, function: :bar, arity: 1}
-             |> message == "function Foo.bar/1 is undefined (module Foo is not available)"
+             |> message ==
+               "function Foo.bar/1 is undefined (module Foo is not available). " <>
+                 "Make sure the module name is correct and has been specified in full (or that an alias has been defined)"
 
       assert %UndefinedFunctionError{module: nil, function: :bar, arity: 3}
              |> message == "function nil.bar/3 is undefined"
@@ -873,52 +957,63 @@ defmodule ExceptionTest do
     test "ErlangError" do
       assert %ErlangError{original: :sample} |> message == "Erlang error: :sample"
     end
-  end
 
-  if System.otp_release() >= "24" do
-    describe "error_info" do
-      test "badarg on erlang" do
-        assert message(:erlang, & &1.element("foo", "bar")) == """
-               errors were found at the given arguments:
+    test "MissingApplicationsError" do
+      assert %MissingApplicationsError{
+               apps: [{:logger, "~> 1.18"}, {:ex_unit, Version.parse_requirement!(">= 0.0.0")}],
+               description: "applications are required"
+             }
+             |> message == """
+             applications are required
 
-                 * 1st argument: not an integer
-                 * 2nd argument: not a tuple
-               """
-      end
+             To address this, include these applications as your dependencies:
 
-      test "badarg on ets" do
-        ets = :ets.new(:foo, [])
-        :ets.delete(ets)
-
-        assert message(:ets, & &1.insert(ets, 1)) == """
-               errors were found at the given arguments:
-
-                 * 1st argument: the table identifier does not refer to an existing ETS table
-                 * 2nd argument: not a tuple
-               """
-      end
-
-      test "system_limit on counters" do
-        assert message(:counters, & &1.new(123_456_789_123_456_789_123_456_789, [])) == """
-               a system limit has been reached due to errors at the given arguments:
-
-                 * 1st argument: counters array size reached a system limit
-               """
-      end
+               {:logger, "~> 1.18"}
+               {:ex_unit, ">= 0.0.0"}\
+             """
     end
   end
 
-  if System.otp_release() >= "25" do
-    describe "binary constructor error info" do
-      defp concat(a, b), do: a <> b
+  describe "error_info" do
+    test "badarg on erlang" do
+      assert message(:erlang, & &1.element("foo", "bar")) == """
+             errors were found at the given arguments:
 
-      test "on binary concatenation" do
-        assert message(123, &concat(&1, "bar")) ==
-                 "construction of binary failed: segment 1 of type 'binary': expected a binary but got: 123"
+               * 1st argument: not an integer
+               * 2nd argument: not a tuple
+             """
+    end
 
-        assert message(~D[0001-02-03], &concat(&1, "bar")) ==
-                 "construction of binary failed: segment 1 of type 'binary': expected a binary but got: ~D[0001-02-03]"
-      end
+    test "badarg on ets" do
+      ets = :ets.new(:foo, [])
+      :ets.delete(ets)
+
+      assert message(:ets, & &1.insert(ets, 1)) == """
+             errors were found at the given arguments:
+
+               * 1st argument: the table identifier does not refer to an existing ETS table
+               * 2nd argument: not a tuple
+             """
+    end
+
+    test "system_limit on counters" do
+      assert message(:counters, & &1.new(123_456_789_123_456_789_123_456_789, [])) == """
+             a system limit has been reached due to errors at the given arguments:
+
+               * 1st argument: counters array size reached a system limit
+             """
+    end
+  end
+
+  describe "binary constructor error info" do
+    defp concat(a, b), do: a <> b
+
+    test "on binary concatenation" do
+      assert message(123, &concat(&1, "bar")) ==
+               "construction of binary failed: segment 1 of type 'binary': expected a binary but got: 123"
+
+      assert message(~D[0001-02-03], &concat(&1, "bar")) ==
+               "construction of binary failed: segment 1 of type 'binary': expected a binary but got: ~D[0001-02-03]"
     end
   end
 
