@@ -14,7 +14,7 @@ defmodule FileTest do
     # Following Erlang's underlying implementation
     #
     # Renaming files
-    # :ok               -> rename file to existing file default behaviour
+    # :ok               -> rename file to existing file default behavior
     # {:error, :eisdir} -> rename file to existing empty dir
     # {:error, :eisdir} -> rename file to existing non-empty dir
     # :ok               -> rename file to non-existing location
@@ -26,7 +26,7 @@ defmodule FileTest do
     # :ok                -> rename dir to non-existing leaf location
     # {:error, ??}       -> rename dir to non-existing parent location
     # :ok                -> rename dir to itself
-    # :ok                -> rename dir to existing empty dir default behaviour
+    # :ok                -> rename dir to existing empty dir default behavior
     # {:error, :eexist}  -> rename dir to existing empty dir
     # {:error, :einval}  -> rename parent dir to existing sub dir
     # {:error, :einval}  -> rename parent dir to non-existing sub dir
@@ -35,7 +35,7 @@ defmodule FileTest do
     # other tests
     # {:error, :enoent} -> rename unknown source
     # :ok               -> rename preserves mode
-    test "rename file to existing file default behaviour" do
+    test "rename file to existing file default behavior" do
       src = tmp_fixture_path("file.txt")
       dest = tmp_path("tmp.file")
 
@@ -128,7 +128,7 @@ defmodule FileTest do
       end
     end
 
-    test "rename! file to existing file default behaviour" do
+    test "rename! file to existing file default behavior" do
       src = tmp_fixture_path("file.txt")
       dest = tmp_path("tmp.file")
 
@@ -256,7 +256,7 @@ defmodule FileTest do
       end
     end
 
-    test "rename dir to existing empty dir default behaviour" do
+    test "rename dir to existing empty dir default behavior" do
       src = tmp_fixture_path("cp_r")
       dest = tmp_path("tmp")
 
@@ -853,12 +853,12 @@ defmodule FileTest do
     end
 
     test "read with list" do
-      assert {:ok, "FOO\n"} = File.read(Path.expand('fixtures/file.txt', __DIR__))
-      assert {:error, :enoent} = File.read(Path.expand('fixtures/missing.txt', __DIR__))
+      assert {:ok, "FOO\n"} = File.read(Path.expand(~c"fixtures/file.txt", __DIR__))
+      assert {:error, :enoent} = File.read(Path.expand(~c"fixtures/missing.txt", __DIR__))
     end
 
     test "read with UTF-8" do
-      assert {:ok, "Русский\n日\n"} = File.read(Path.expand('fixtures/utf8.txt', __DIR__))
+      assert {:ok, "Русский\n日\n"} = File.read(Path.expand(~c"fixtures/utf8.txt", __DIR__))
     end
 
     test "read!" do
@@ -875,7 +875,7 @@ defmodule FileTest do
 
       try do
         refute File.exists?(fixture)
-        assert File.write(fixture, 'test text') == :ok
+        assert File.write(fixture, ~c"test text") == :ok
         assert File.read(fixture) == {:ok, "test text"}
       after
         File.rm(fixture)
@@ -915,7 +915,7 @@ defmodule FileTest do
 
     test "open file with charlist" do
       {:ok, file} = File.open(fixture_path("file.txt"), [:charlist])
-      assert IO.gets(file, "") == 'FOO\n'
+      assert IO.gets(file, "") == ~c"FOO\n"
       assert File.close(file) == :ok
     end
 
@@ -951,6 +951,7 @@ defmodule FileTest do
         {:ok, file} = File.open(fixture, [:write])
         assert IO.binwrite(file, "Русский") == :ok
         assert File.close(file) == :ok
+        assert_raise ErlangError, fn -> IO.binwrite(file, "Русский") end
         assert File.read(fixture) == {:ok, "Русский"}
       after
         File.rm(fixture)
@@ -975,7 +976,7 @@ defmodule FileTest do
     end
 
     test "open a missing file" do
-      assert File.open('missing.txt') == {:error, :enoent}
+      assert File.open(~c"missing.txt") == {:error, :enoent}
     end
 
     test "open a file with function" do
@@ -987,7 +988,7 @@ defmodule FileTest do
       message = "could not open \"missing.txt\": no such file or directory"
 
       assert_raise File.Error, message, fn ->
-        File.open!('missing.txt')
+        File.open!(~c"missing.txt")
       end
     end
 
@@ -1166,7 +1167,7 @@ defmodule FileTest do
     end
 
     test "rm nonexistent file" do
-      assert File.rm('missing.txt') == {:error, :enoent}
+      assert File.rm(~c"missing.txt") == {:error, :enoent}
     end
 
     test "rm!" do
@@ -1311,6 +1312,14 @@ defmodule FileTest do
       assert File.rm_rf(fixture) == {:ok, [fixture]}
     end
 
+    test "rm_rf with write-only subdir" do
+      dir = tmp_path("tmp")
+      subdir = Path.join(dir, "write-only")
+      File.mkdir_p!(subdir)
+      File.chmod!(subdir, 0o222)
+      assert File.rm_rf(dir) == {:ok, [dir, subdir]}
+    end
+
     test "rm_rf with unknown" do
       fixture = tmp_path("tmp.unknown")
       assert File.rm_rf(fixture) == {:ok, []}
@@ -1451,256 +1460,6 @@ defmodule FileTest do
     end
   end
 
-  describe "file stream" do
-    test "returns a struct" do
-      src = fixture_path("file.txt")
-      stream = File.stream!(src)
-      assert %File.Stream{} = stream
-      assert stream.modes == [:raw, :read_ahead, :binary]
-      assert stream.raw
-      assert stream.line_or_bytes == :line
-
-      stream = File.stream!(src, read_ahead: false)
-      assert %File.Stream{} = stream
-      assert stream.modes == [:raw, :binary]
-      assert stream.raw
-
-      stream = File.stream!(src, read_ahead: 5000)
-      assert %File.Stream{} = stream
-      assert stream.modes == [:raw, {:read_ahead, 5000}, :binary]
-      assert stream.raw
-
-      stream = File.stream!(src, [:utf8], 10)
-      assert %File.Stream{} = stream
-      assert stream.modes == [{:encoding, :utf8}, :binary]
-      refute stream.raw
-      assert stream.line_or_bytes == 10
-    end
-
-    test "counts bytes/characters" do
-      src = fixture_path("file.txt")
-      stream = File.stream!(src)
-      assert Enum.count(stream) == 1
-
-      stream = File.stream!(src, [:utf8])
-      assert Enum.count(stream) == 1
-
-      stream = File.stream!(src, [], 2)
-      assert Enum.count(stream) == 2
-    end
-
-    test "reads and writes lines" do
-      src = fixture_path("file.txt")
-      dest = tmp_path("tmp_test.txt")
-
-      try do
-        stream = File.stream!(src)
-
-        File.open(dest, [:write], fn target ->
-          Enum.each(stream, fn line ->
-            IO.write(target, String.replace(line, "O", "A"))
-          end)
-        end)
-
-        assert File.read(dest) == {:ok, "FAA\n"}
-      after
-        File.rm(dest)
-      end
-    end
-
-    test "reads and writes bytes" do
-      src = fixture_path("file.txt")
-      dest = tmp_path("tmp_test.txt")
-
-      try do
-        stream = File.stream!(src, [], 1)
-
-        File.open(dest, [:write], fn target ->
-          Enum.each(stream, fn <<char>> ->
-            IO.write(target, <<char + 1>>)
-          end)
-        end)
-
-        assert File.read(dest) == {:ok, "GPP\v"}
-      after
-        File.rm(dest)
-      end
-    end
-
-    test "is collectable" do
-      src = fixture_path("file.txt")
-      dest = tmp_path("tmp_test.txt")
-
-      try do
-        refute File.exists?(dest)
-        original = File.stream!(dest)
-
-        stream =
-          File.stream!(src)
-          |> Stream.map(&String.replace(&1, "O", "A"))
-          |> Enum.into(original)
-
-        assert stream == original
-        assert File.read(dest) == {:ok, "FAA\n"}
-      after
-        File.rm(dest)
-      end
-    end
-
-    test "is collectable with append" do
-      src = fixture_path("file.txt")
-      dest = tmp_path("tmp_test.txt")
-
-      try do
-        refute File.exists?(dest)
-        original = File.stream!(dest, [:append])
-
-        File.stream!(src, [:append])
-        |> Stream.map(&String.replace(&1, "O", "A"))
-        |> Enum.into(original)
-
-        File.stream!(src, [:append])
-        |> Enum.into(original)
-
-        assert File.read(dest) == {:ok, "FAA\nFOO\n"}
-      after
-        File.rm(dest)
-      end
-    end
-
-    test "keeps BOM when raw" do
-      src = fixture_path("utf8_bom.txt")
-
-      assert src
-             |> File.stream!([])
-             |> Enum.take(1) == [<<239, 187, 191>> <> "Русский\n"]
-
-      assert src
-             |> File.stream!([], 1)
-             |> Enum.take(5) == [<<239>>, <<187>>, <<191>>, <<208>>, <<160>>]
-
-      assert src |> File.stream!([]) |> Enum.count() == 2
-      assert src |> File.stream!([], 1) |> Enum.count() == 22
-    end
-
-    test "trims BOM via option when raw" do
-      src = fixture_path("utf8_bom.txt")
-
-      assert src
-             |> File.stream!([:trim_bom])
-             |> Enum.take(1) == ["Русский\n"]
-
-      assert src
-             |> File.stream!([:trim_bom], 1)
-             |> Enum.take(5) == [<<208>>, <<160>>, <<209>>, <<131>>, <<209>>]
-
-      assert src |> File.stream!([:trim_bom]) |> Enum.count() == 2
-      assert src |> File.stream!([:trim_bom], 1) |> Enum.count() == 19
-    end
-
-    test "keeps BOM with utf8 encoding" do
-      src = fixture_path("utf8_bom.txt")
-
-      assert src
-             |> File.stream!([{:encoding, :utf8}])
-             |> Enum.take(1) == [<<239, 187, 191>> <> "Русский\n"]
-
-      assert src
-             |> File.stream!([{:encoding, :utf8}], 1)
-             |> Enum.take(9) == ["\uFEFF", "Р", "у", "с", "с", "к", "и", "й", "\n"]
-    end
-
-    test "trims BOM via option with utf8 encoding" do
-      src = fixture_path("utf8_bom.txt")
-
-      assert src
-             |> File.stream!([{:encoding, :utf8}, :trim_bom])
-             |> Enum.take(1) == ["Русский\n"]
-
-      assert src
-             |> File.stream!([{:encoding, :utf8}, :trim_bom], 1)
-             |> Enum.take(8) == ["Р", "у", "с", "с", "к", "и", "й", "\n"]
-    end
-
-    test "keeps BOM with UTF16 BE" do
-      src = fixture_path("utf16_be_bom.txt")
-
-      assert src
-             |> File.stream!([{:encoding, {:utf16, :big}}])
-             |> Enum.take(1) == ["\uFEFFРусский\n"]
-    end
-
-    test "keeps BOM with UTF16 LE" do
-      src = fixture_path("utf16_le_bom.txt")
-
-      assert src
-             |> File.stream!([{:encoding, {:utf16, :little}}])
-             |> Enum.take(1) == ["\uFEFFРусский\n"]
-    end
-
-    test "trims BOM via option with utf16 BE encoding" do
-      src = fixture_path("utf16_be_bom.txt")
-
-      assert src
-             |> File.stream!([{:encoding, {:utf16, :big}}, :trim_bom])
-             |> Enum.take(1) == ["Русский\n"]
-
-      assert src
-             |> File.stream!([{:encoding, {:utf16, :big}}, :trim_bom], 1)
-             |> Enum.take(8) == ["Р", "у", "с", "с", "к", "и", "й", "\n"]
-    end
-
-    test "trims BOM via option with utf16 LE encoding" do
-      src = fixture_path("utf16_le_bom.txt")
-
-      assert src
-             |> File.stream!([{:encoding, {:utf16, :little}}, :trim_bom])
-             |> Enum.take(1) == ["Русский\n"]
-
-      assert src
-             |> File.stream!([{:encoding, {:utf16, :little}}, :trim_bom], 1)
-             |> Enum.take(8) == ["Р", "у", "с", "с", "к", "и", "й", "\n"]
-    end
-
-    test "reads and writes line by line in UTF-8" do
-      src = fixture_path("file.txt")
-      dest = tmp_path("tmp_test.txt")
-
-      try do
-        stream = File.stream!(src)
-
-        File.open(dest, [:write, :utf8], fn target ->
-          Enum.each(stream, fn line ->
-            IO.write(target, String.replace(line, "O", "A"))
-          end)
-        end)
-
-        assert File.read(dest) == {:ok, "FAA\n"}
-      after
-        File.rm(dest)
-      end
-    end
-
-    test "reads and writes character in UTF-8" do
-      src = fixture_path("file.txt")
-      dest = tmp_path("tmp_test.txt")
-
-      try do
-        stream = File.stream!(src, [:utf8], 1)
-
-        File.open(dest, [:write], fn target ->
-          Enum.each(stream, fn <<char::utf8>> ->
-            IO.write(target, <<char + 1::utf8>>)
-          end)
-        end)
-
-        assert File.read(dest) == {:ok, "GPP\v"}
-      after
-        File.rm(dest)
-      end
-    end
-  end
-
   describe "links" do
     test "read_link with regular file" do
       dest = tmp_path("symlink")
@@ -2035,7 +1794,7 @@ defmodule FileTest do
         stat = File.stat!(fixture)
         assert stat.mode == 0o100666
 
-        unless windows?() do
+        if not windows?() do
           assert File.chmod(fixture, 0o100777) == :ok
           stat = File.stat!(fixture)
           assert stat.mode == 0o100777
@@ -2055,7 +1814,7 @@ defmodule FileTest do
         stat = File.stat!(fixture)
         assert stat.mode == 0o100666
 
-        unless windows?() do
+        if not windows?() do
           assert File.chmod!(fixture, 0o100777) == :ok
           stat = File.stat!(fixture)
           assert stat.mode == 0o100777
