@@ -1,29 +1,5 @@
 Code.require_file("test_helper.exs", __DIR__)
 
-# This is to temporarily test some inconsistencies in
-# the error ArgumentError messages.
-# Remove MyArgumentError and replace the calls to:
-# - MyArgumentError with ArgumentError
-# - MyArgumentError.culprit() with Atom.to_string("Foo")
-# in Erlang/OTP 25
-defmodule MyArgumentError do
-  defexception message: "argument error"
-
-  @impl true
-  def message(_) do
-    """
-    errors were found at the given arguments:
-
-      * 1st argument: not an atom
-    """
-  end
-
-  def culprit() do
-    raise = fn -> raise(MyArgumentError) end
-    raise.()
-  end
-end
-
 defmodule Inspect.AtomTest do
   use ExUnit.Case, async: true
 
@@ -155,6 +131,8 @@ defmodule Inspect.BitStringTest do
     assert inspect(" ゆんゆん") == "\" ゆんゆん\""
     # BOM
     assert inspect("\uFEFFhello world") == "\"\\uFEFFhello world\""
+    # Invisible characters
+    assert inspect("\u2063") == "\"\\u2063\""
   end
 
   test "infer" do
@@ -169,6 +147,7 @@ defmodule Inspect.BitStringTest do
     assert inspect(<<"john", 193, "doe">>, binaries: :as_strings) == ~s("john\\xC1doe")
     assert inspect(<<"john">>, binaries: :as_strings) == ~s("john")
     assert inspect(<<193>>, binaries: :as_strings) == ~s("\\xC1")
+    assert inspect(<<193>>, base: :hex, binaries: :as_strings) == ~s("\\xC1")
   end
 
   test "as binaries" do
@@ -294,13 +273,13 @@ defmodule Inspect.ListTest do
   end
 
   test "printable" do
-    assert inspect('abc') == "'abc'"
+    assert inspect(~c"abc") == ~s(~c"abc")
   end
 
   test "printable limit" do
-    assert inspect('hello world', printable_limit: 4) == ~s('hell' ++ ...)
+    assert inspect(~c"hello world", printable_limit: 4) == ~s(~c"hell" ++ ...)
     # Non printable characters after the limit don't matter
-    assert inspect('hello world' ++ [0], printable_limit: 4) == ~s('hell' ++ ...)
+    assert inspect(~c"hello world" ++ [0], printable_limit: 4) == ~s(~c"hell" ++ ...)
     # Non printable strings aren't affected by printable limit
     assert inspect([0, 1, 2, 3, 4], printable_limit: 3) == ~s([0, 1, 2, 3, 4])
   end
@@ -320,24 +299,24 @@ defmodule Inspect.ListTest do
   end
 
   test "opt infer" do
-    assert inspect('john' ++ [0] ++ 'doe', charlists: :infer) ==
+    assert inspect(~c"john" ++ [0] ++ ~c"doe", charlists: :infer) ==
              "[106, 111, 104, 110, 0, 100, 111, 101]"
 
-    assert inspect('john', charlists: :infer) == "'john'"
+    assert inspect(~c"john", charlists: :infer) == ~s(~c"john")
     assert inspect([0], charlists: :infer) == "[0]"
   end
 
   test "opt as strings" do
-    assert inspect('john' ++ [0] ++ 'doe', charlists: :as_charlists) == "'john\\0doe'"
-    assert inspect('john', charlists: :as_charlists) == "'john'"
-    assert inspect([0], charlists: :as_charlists) == "'\\0'"
+    assert inspect(~c"john" ++ [0] ++ ~c"doe", charlists: :as_charlists) == ~s(~c"john\\0doe")
+    assert inspect(~c"john", charlists: :as_charlists) == ~s(~c"john")
+    assert inspect([0], charlists: :as_charlists) == ~s(~c"\\0")
   end
 
   test "opt as lists" do
-    assert inspect('john' ++ [0] ++ 'doe', charlists: :as_lists) ==
+    assert inspect(~c"john" ++ [0] ++ ~c"doe", charlists: :as_lists) ==
              "[106, 111, 104, 110, 0, 100, 111, 101]"
 
-    assert inspect('john', charlists: :as_lists) == "[106, 111, 104, 110]"
+    assert inspect(~c"john", charlists: :as_lists) == "[106, 111, 104, 110]"
     assert inspect([0], charlists: :as_lists) == "[0]"
   end
 
@@ -361,7 +340,7 @@ defmodule Inspect.ListTest do
   end
 
   test "codepoints" do
-    assert inspect('é') == "[233]"
+    assert inspect(~c"é") == "[233]"
   end
 
   test "empty" do
@@ -417,14 +396,20 @@ defmodule Inspect.MapTest do
   test "basic" do
     assert inspect(%{1 => "b"}) == "%{1 => \"b\"}"
 
-    assert inspect(%{1 => "b", 2 => "c"}, pretty: true, width: 1) ==
+    assert inspect(%{1 => "b", 2 => "c"},
+             pretty: true,
+             width: 1,
+             custom_options: [sort_maps: true]
+           ) ==
              "%{\n  1 => \"b\",\n  2 => \"c\"\n}"
   end
 
   test "keyword" do
     assert inspect(%{a: 1}) == "%{a: 1}"
-    assert inspect(%{a: 1, b: 2}) == "%{a: 1, b: 2}"
-    assert inspect(%{a: 1, b: 2, c: 3}) == "%{a: 1, b: 2, c: 3}"
+    assert inspect(%{a: 1, b: 2}, custom_options: [sort_maps: true]) == "%{a: 1, b: 2}"
+
+    assert inspect(%{a: 1, b: 2, c: 3}, custom_options: [sort_maps: true]) ==
+             "%{a: 1, b: 2, c: 3}"
   end
 
   test "with limit" do
@@ -446,12 +431,12 @@ defmodule Inspect.MapTest do
   test "public modified struct" do
     public = %Public{key: 1}
 
-    assert inspect(Map.put(public, :foo, :bar)) ==
+    assert inspect(Map.put(public, :foo, :bar), custom_options: [sort_maps: true]) ==
              "%{__struct__: Inspect.MapTest.Public, foo: :bar, key: 1}"
   end
 
   test "private struct" do
-    assert inspect(%{__struct__: Private, key: 1}) ==
+    assert inspect(%{__struct__: Private, key: 1}, custom_options: [sort_maps: true]) ==
              "%{__struct__: Inspect.MapTest.Private, key: 1}"
   end
 
@@ -460,20 +445,20 @@ defmodule Inspect.MapTest do
     defstruct @enforce_keys
 
     defimpl Inspect do
-      def inspect(%Failing{name: _name}, _) do
-        MyArgumentError.culprit()
+      def inspect(%Failing{name: name}, _) do
+        Atom.to_string(name)
       end
     end
   end
 
   test "safely inspect bad implementation" do
-    assert_raise MyArgumentError, ~r/errors were found at the given arguments:/, fn ->
-      raise(MyArgumentError)
+    assert_raise ArgumentError, ~r/argument error/, fn ->
+      raise(ArgumentError)
     end
 
     message = ~s'''
     #Inspect.Error<
-      got MyArgumentError with message:
+      got ArgumentError with message:
 
           """
           errors were found at the given arguments:
@@ -486,13 +471,13 @@ defmodule Inspect.MapTest do
           %{__struct__: Inspect.MapTest.Failing, name: "Foo"}
     '''
 
-    assert inspect(%Failing{name: "Foo"}) =~ message
+    assert inspect(%Failing{name: "Foo"}, custom_options: [sort_maps: true]) =~ message
   end
 
   test "safely inspect bad implementation disables colors" do
     message = ~s'''
     #Inspect.Error<
-      got MyArgumentError with message:
+      got ArgumentError with message:
 
           """
           errors were found at the given arguments:
@@ -505,12 +490,15 @@ defmodule Inspect.MapTest do
           %{__struct__: Inspect.MapTest.Failing, name: "Foo"}
     '''
 
-    assert inspect(%Failing{name: "Foo"}, syntax_colors: [atom: [:green]]) =~ message
+    assert inspect(%Failing{name: "Foo"},
+             syntax_colors: [atom: [:green]],
+             custom_options: [sort_maps: true]
+           ) =~ message
   end
 
   test "unsafely inspect bad implementation" do
     exception_message = ~s'''
-    got MyArgumentError with message:
+    got ArgumentError with message:
 
         """
         errors were found at the given arguments:
@@ -524,14 +512,11 @@ defmodule Inspect.MapTest do
     '''
 
     try do
-      inspect(%Failing{name: "Foo"}, safe: false)
+      inspect(%Failing{name: "Foo"}, safe: false, custom_options: [sort_maps: true])
     rescue
       exception in Inspect.Error ->
         assert Exception.message(exception) =~ exception_message
-        assert [{MyArgumentError, fun_name, 0, [{:file, _}, {:line, _} | _]} | _] = __STACKTRACE__
-
-        assert fun_name in [:"-culprit/0-fun-0-", :culprit]
-        assert Exception.message(exception) =~ exception_message
+        assert [{:erlang, :atom_to_binary, [_], [_ | _]} | _] = __STACKTRACE__
     else
       _ -> flunk("expected failure")
     end
@@ -541,25 +526,31 @@ defmodule Inspect.MapTest do
     # Inspect.Error is raised here when we tried to print the error message
     # called by another exception (Protocol.UndefinedError in this case)
     exception_message = ~s'''
-    protocol Enumerable not implemented for #Inspect.Error<
-      got MyArgumentError with message:
+    protocol Enumerable not implemented for type Inspect.MapTest.Failing (a struct)
 
-          """
-          errors were found at the given arguments:
+    Got value:
 
-            * 1st argument: not an atom
-          """
+        #Inspect.Error<
+          got ArgumentError with message:
 
-      while inspecting:
+              """
+              errors were found at the given arguments:
+
+                * 1st argument: not an atom
+              """
+
+          while inspecting:
 
-          %{__struct__: Inspect.MapTest.Failing, name: "Foo"}
     '''
 
     try do
       Enum.to_list(%Failing{name: "Foo"})
     rescue
       exception in Protocol.UndefinedError ->
-        assert Exception.message(exception) =~ exception_message
+        message = Exception.message(exception)
+        assert message =~ exception_message
+        assert message =~ "__struct__: Inspect.MapTest.Failing"
+        assert message =~ "name: \"Foo\""
 
         assert [
                  {Enumerable, :impl_for!, 1, _} | _
@@ -585,9 +576,11 @@ defmodule Inspect.MapTest do
   end
 
   test "Exception.message/1 with bad implementation" do
+    failing = %Failing{name: "Foo"}
+
     message = ~s'''
     #Inspect.Error<
-      got MyArgumentError with message:
+      got ArgumentError with message:
 
           """
           errors were found at the given arguments:
@@ -602,7 +595,7 @@ defmodule Inspect.MapTest do
 
     {my_argument_error, stacktrace} =
       try do
-        MyArgumentError.culprit()
+        atom_to_string(failing.name)
       rescue
         e ->
           {e, __STACKTRACE__}
@@ -617,10 +610,14 @@ defmodule Inspect.MapTest do
         )
       )
 
-    assert inspect(%Failing{name: "Foo"}) =~ message
+    assert inspect(failing, custom_options: [sort_maps: true]) =~ message
     assert inspected =~ message
   end
 
+  defp atom_to_string(atom) do
+    Atom.to_string(atom)
+  end
+
   test "exception" do
     assert inspect(%RuntimeError{message: "runtime error"}) ==
              "%RuntimeError{message: \"runtime error\"}"
@@ -808,8 +805,8 @@ defmodule Inspect.OthersTest do
     Application.put_env(:elixir, :anony, V.fun())
     Application.put_env(:elixir, :named, &V.fun/0)
 
-    :code.delete(V)
     :code.purge(V)
+    :code.delete(V)
 
     anony = Application.get_env(:elixir, :anony)
     named = Application.get_env(:elixir, :named)
@@ -853,6 +850,7 @@ defmodule Inspect.OthersTest do
 
   test "regex" do
     assert inspect(~r(foo)m) == "~r/foo/m"
+    assert inspect(~r[\\\#{2,}]iu) == ~S"~r/\\\#{2,}/iu"
 
     assert inspect(Regex.compile!("a\\/b")) == "~r/a\\/b/"
 
@@ -864,7 +862,7 @@ defmodule Inspect.OthersTest do
     assert inspect(~r/hi/, syntax_colors: [regex: :red]) == "\e[31m~r/hi/\e[0m"
 
     assert inspect(Regex.compile!("foo", "i")) == "~r/foo/i"
-    assert inspect(Regex.compile!("foo", [:caseless])) == ~S'Regex.compile!("foo", [:caseless])'
+    assert inspect(Regex.compile!("foo", [:ucp])) == ~S'Regex.compile!("foo", [:ucp])'
   end
 
   test "inspect_fun" do
@@ -932,7 +930,11 @@ defmodule Inspect.CustomProtocolTest do
     got Protocol.UndefinedError with message:
 
         """
-        protocol Inspect.CustomProtocolTest.CustomInspect not implemented for %Inspect.CustomProtocolTest.MissingImplementation{} of type Inspect.CustomProtocolTest.MissingImplementation (a struct)
+        protocol Inspect.CustomProtocolTest.CustomInspect not implemented for type Inspect.CustomProtocolTest.MissingImplementation (a struct)
+
+        Got value:
+
+            %Inspect.CustomProtocolTest.MissingImplementation{}
         """
 
     while inspecting:
@@ -959,7 +961,11 @@ defmodule Inspect.CustomProtocolTest do
       got Protocol.UndefinedError with message:
 
           """
-          protocol Inspect.CustomProtocolTest.CustomInspect not implemented for %Inspect.CustomProtocolTest.MissingImplementation{} of type Inspect.CustomProtocolTest.MissingImplementation (a struct)
+          protocol Inspect.CustomProtocolTest.CustomInspect not implemented for type Inspect.CustomProtocolTest.MissingImplementation (a struct)
+
+          Got value:
+
+              %Inspect.CustomProtocolTest.MissingImplementation{}
           """
 
       while inspecting:
