@@ -27,6 +27,13 @@ defmodule Kernel.AliasTest do
     assert Nested.flatten([[13]]) == [13]
   end
 
+  test "non-recursive alias" do
+    alias Billing, as: BillingLib
+    alias MyApp.Billing
+    assert BillingLib == :"Elixir.Billing"
+    assert Billing == :"Elixir.MyApp.Billing"
+  end
+
   test "lexical" do
     if true_fun() do
       alias OMG, as: List, warn: false
@@ -80,10 +87,10 @@ end
 defmodule Kernel.AliasNestingTest do
   use ExUnit.Case, async: true
 
-  require Kernel.AliasNestingGenerator
-  Kernel.AliasNestingGenerator.create()
-
   test "aliases nesting" do
+    require Kernel.AliasNestingGenerator
+    Kernel.AliasNestingGenerator.create()
+
     assert Parent.a() == :a
     assert Parent.Child.b() == :a
   end
@@ -95,6 +102,32 @@ defmodule Kernel.AliasNestingTest do
   test "aliases nesting with previous alias" do
     assert Nested.value() == 2
   end
+
+  alias Another.AliasEnv, warn: false
+  def aliases_before, do: __ENV__.aliases
+
+  defmodule Elixir.AliasEnv do
+    def aliases_nested, do: __ENV__.aliases
+  end
+
+  def aliases_after, do: __ENV__.aliases
+
+  test "keeps env after overriding nested Elixir module of the same name" do
+    assert aliases_before() == [
+             {Elixir.Nested, Kernel.AliasNestingTest.Nested},
+             {Elixir.AliasEnv, Another.AliasEnv}
+           ]
+
+    assert Elixir.AliasEnv.aliases_nested() == [
+             {Elixir.Nested, Kernel.AliasNestingTest.Nested},
+             {Elixir.AliasEnv, Another.AliasEnv}
+           ]
+
+    assert aliases_after() == [
+             {Elixir.Nested, Kernel.AliasNestingTest.Nested},
+             {Elixir.AliasEnv, Another.AliasEnv}
+           ]
+  end
 end
 
 # Test case extracted from using records with aliases
