@@ -52,7 +52,6 @@ defmodule Kernel.LexicalTrackerTest do
   end
 
   test "can add aliases", config do
-    D.add_alias(config[:pid], String, 1, true)
     D.alias_dispatch(config[:pid], String)
     assert D.references(config[:pid]) == {[], [], [], []}
   end
@@ -93,18 +92,19 @@ defmodule Kernel.LexicalTrackerTest do
   end
 
   test "unused aliases", config do
-    D.add_alias(config[:pid], String, 1, true)
-    assert D.collect_unused_aliases(config[:pid]) == [{String, 1}]
+    D.warn_alias(config[:pid], [], String, String)
+    assert D.collect_unused_aliases(config[:pid]) == [{String, []}]
   end
 
   test "used aliases are not unused", config do
-    D.add_alias(config[:pid], String, 1, true)
+    D.warn_alias(config[:pid], [], String, String)
     D.alias_dispatch(config[:pid], String)
     assert D.collect_unused_aliases(config[:pid]) == []
   end
 
-  test "aliases with no warn are not unused", config do
-    D.add_alias(config[:pid], String, 1, false)
+  test "used aliases are not unused in reverse order", config do
+    D.alias_dispatch(config[:pid], String)
+    D.warn_alias(config[:pid], [], String, String)
     assert D.collect_unused_aliases(config[:pid]) == []
   end
 
@@ -162,7 +162,7 @@ defmodule Kernel.LexicalTrackerTest do
       {{compile, _, _, _}, _binding} =
         Code.eval_string("""
         defmodule Kernel.LexicalTrackerTest.Attribute1 do
-          @example [String, Enum]
+          @example [String, Enum, 3 + 10]
           def foo(atom) when atom in @example, do: atom
           Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
         end |> elem(3)
@@ -226,6 +226,33 @@ defmodule Kernel.LexicalTrackerTest do
 
       assert String in compile
       assert Enum in compile
+
+      {{compile, _, _, _}, _binding} =
+        Code.eval_string("""
+        defmodule Kernel.LexicalTrackerTest.Attribute6 do
+          @example %{foo: Application.compile_env(:elixir, Enum, String)}
+          def foo(atom) when atom == @example.foo, do: atom
+          Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
+        end |> elem(3)
+        """)
+
+      refute String in compile
+      refute Enum in compile
+
+      {{compile, _, _, _}, _binding} =
+        Code.eval_string("""
+        defmodule Kernel.LexicalTrackerTest.Attribute7 do
+          Module.register_attribute(__MODULE__, :example, accumulate: true)
+          @example String
+          @example Enum
+          _ = Module.get_last_attribute(__MODULE__, :example)
+
+          Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
+        end |> elem(3)
+        """)
+
+      assert String in compile
+      assert Enum in compile
     end
 
     test "@compile adds a runtime dependency" do
@@ -307,6 +334,31 @@ defmodule Kernel.LexicalTrackerTest do
       assert URI in runtime
     end
 
+    test "dbg adds a compile dependency" do
+      {{compile, exports, runtime, _}, _binding} =
+        Code.eval_string("""
+        defmodule Kernel.LexicalTrackerTest.Dbg do
+          def foo, do: dbg(:ok)
+          Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
+        end |> elem(3)
+        """)
+
+      assert Macro in compile
+      refute Macro in exports
+      refute Macro in runtime
+
+      {{compile, exports, runtime, _}, _binding} =
+        Code.eval_string("""
+        defmodule Kernel.LexicalTrackerTest.NoDbg do
+          Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
+        end |> elem(3)
+        """)
+
+      refute Macro in compile
+      refute Macro in exports
+      refute Macro in runtime
+    end
+
     test "imports adds an export dependency" do
       {{compile, exports, runtime, _}, _binding} =
         Code.eval_string("""
@@ -345,9 +397,21 @@ defmodule Kernel.LexicalTrackerTest do
       assert URI in compile
       assert URI in exports
       refute URI in runtime
+
+      {{compile, exports, runtime, _}, _binding} =
+        Code.eval_string("""
+        defmodule Kernel.LexicalTrackerTest.StructPattern do
+          def is_uri(%URI{}), do: true
+          Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
+        end |> elem(3)
+        """)
+
+      refute URI in compile
+      assert URI in exports
+      assert URI in runtime
     end
 
-    test "Macro.struct! adds an export dependency" do
+    test "Macro.struct_info! adds an export dependency" do
       {{compile, exports, runtime, _}, _binding} =
         Code.eval_string("""
         defmodule Kernel.LexicalTrackerTest.MacroStruct do
@@ -356,7 +420,7 @@ defmodule Kernel.LexicalTrackerTest do
           # mechanism to make this expansion become a compile-time one.
           # However, in some cases, such as typespecs, we don't necessarily
           # want the compile-time dependency to happen.
-          Macro.struct!(:"Elixir.URI", __ENV__)
+          Macro.struct_info!(:"Elixir.URI", __ENV__)
           Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
         end |> elem(3)
         """)
@@ -366,6 +430,44 @@ defmodule Kernel.LexicalTrackerTest do
       refute URI in runtime
     end
 
+    test "aliases in patterns and guards inside functions do not add runtime dependency" do
+      {{compile, exports, runtime, _}, _binding} =
+        Code.eval_string("""
+        defmodule Kernel.LexicalTrackerTest.PatternGuardsRuntime do
+          def is_uri_atom(URI), do: true
+          def is_range_struct(range) when is_struct(range, Range), do: true
+          Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
+        end |> elem(3)
+        """)
+
+      refute URI in compile
+      refute URI in exports
+      refute URI in runtime
+      refute Range in compile
+      refute Range in exports
+      refute Range in runtime
+    end
+
+    test "aliases in patterns and guards outside functions do add compile dependency" do
+      {{compile, exports, runtime, _}, _binding} =
+        Code.eval_string("""
+        defmodule Kernel.LexicalTrackerTest.PatternGuardsCompile do
+          %URI{} = URI.parse("/")
+          case 1..3 do
+            range when is_struct(range, Range) -> :ok
+          end
+          Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
+        end |> elem(3)
+        """)
+
+      assert URI in compile
+      assert URI in exports
+      refute URI in runtime
+      assert Range in compile
+      refute Range in exports
+      refute Range in runtime
+    end
+
     test "compile_env! does not add a compile dependency" do
       {{compile, exports, runtime, _}, _binding} =
         Code.eval_string("""
@@ -399,12 +501,12 @@ defmodule Kernel.LexicalTrackerTest do
       {{compile, exports, runtime, _}, _binding} =
         Code.eval_string("""
         defmodule Kernel.LexicalTrackerTest.Defmacro do
-          defmacro uri() do
-            Macro.escape(URI.parse("/hello"))
+          defmacro uri(path) do
+            Macro.escape(URI.parse(path))
           end
 
           def fun() do
-            uri()
+            uri("/hello")
           end
 
           Kernel.LexicalTracker.references(__ENV__.lexical_tracker)
