@@ -44,7 +44,7 @@ defmodule Kernel.ParallelCompilerTest do
         end)
 
       assert profile =~
-               ~r"\[profile\] [\s\d]{6}ms compiling \+      0ms waiting for .*tmp/profile_time/bar.ex"
+               ~r"\[profile\] [\s\d]{6}ms compiling \+      0ms waiting while compiling .*tmp/profile_time/bar.ex"
 
       assert profile =~ ~r"\[profile\] Finished compilation cycle of 1 modules in \d+ms"
       assert profile =~ ~r"\[profile\] Finished group pass check of 1 modules in \d+ms"
@@ -146,11 +146,12 @@ defmodule Kernel.ParallelCompilerTest do
 
       expected_msg = "Undef.__struct__/1 is undefined, cannot expand struct Undef"
 
-      assert capture_io(fn ->
-               assert {:error, [{^fixture, 3, msg}], []} =
+      assert capture_io(:stderr, fn ->
+               assert {:error, [{^fixture, {3, 5}, msg}, {^fixture, 0, compile_msg}], []} =
                         Kernel.ParallelCompiler.compile([fixture])
 
                assert msg =~ expected_msg
+               assert compile_msg =~ "cannot compile module Undef (errors have been logged)"
              end) =~ expected_msg
     end
 
@@ -169,7 +170,7 @@ defmodule Kernel.ParallelCompilerTest do
 
       expected_msg = "** (KeyError) key :invalid_key not found"
 
-      assert capture_io(fn ->
+      assert capture_io(:stderr, fn ->
                assert {:error, [{^fixture, 3, msg}], []} =
                         Kernel.ParallelCompiler.compile([fixture])
 
@@ -177,6 +178,46 @@ defmodule Kernel.ParallelCompilerTest do
              end) =~ expected_msg
     end
 
+    test "does not crash with pending monitor message" do
+      {pid, ref} = spawn_monitor(fn -> :ok end)
+
+      [fixture] =
+        write_tmp(
+          "quick_example",
+          quick_example: """
+          defmodule QuickExample do
+          end
+          """
+        )
+
+      assert {:ok, [QuickExample], []} = Kernel.ParallelCompiler.compile([fixture])
+      assert_received {:DOWN, ^ref, _, ^pid, :normal}
+    after
+      purge([QuickExample])
+    end
+
+    test "does not crash on external reports" do
+      [fixture] =
+        write_tmp(
+          "compile_quoted",
+          quick_example: """
+          defmodule CompileQuoted do
+            try do
+              Code.compile_quoted({:fn, [], [{:->, [], [[], quote(do: unknown_var)]}]})
+            rescue
+              _ -> :ok
+            end
+          end
+          """
+        )
+
+      assert capture_io(:stderr, fn ->
+               assert {:ok, [CompileQuoted], []} = Kernel.ParallelCompiler.compile([fixture])
+             end) =~ "undefined variable \"unknown_var\""
+    after
+      purge([CompileQuoted])
+    end
+
     test "does not hang on missing dependencies" do
       [fixture] =
         write_tmp(
@@ -196,12 +237,15 @@ defmodule Kernel.ParallelCompilerTest do
       expected_msg =
         "ThisModuleWillNeverBeAvailable.__struct__/1 is undefined, cannot expand struct ThisModuleWillNeverBeAvailable"
 
-      assert capture_io(fn ->
-               assert {:error, [{^fixture, 7, msg}], []} =
+      assert capture_io(:stderr, fn ->
+               assert {:error, [{^fixture, {7, 3}, msg}, {^fixture, 0, compile_msg}], []} =
                         Kernel.ParallelCompiler.compile([fixture])
 
                assert msg =~ expected_msg
-             end) =~ "== Compilation error"
+
+               assert compile_msg =~
+                        "cannot compile module WithBehaviourAndStruct (errors have been logged)"
+             end) =~ expected_msg
     end
 
     test "does not deadlock on missing dependencies" do
@@ -222,12 +266,17 @@ defmodule Kernel.ParallelCompilerTest do
       expected_msg =
         "ThisModuleWillNeverBeAvailable.__struct__/1 is undefined, cannot expand struct ThisModuleWillNeverBeAvailable"
 
-      assert capture_io(fn ->
-               assert {:error, [{^missing_struct, 2, msg}], []} =
+      assert capture_io(:stderr, fn ->
+               assert {:error,
+                       [{^missing_struct, {2, 3}, msg}, {^missing_struct, 0, compile_msg}],
+                       []} =
                         Kernel.ParallelCompiler.compile([missing_struct, depends_on])
 
                assert msg =~ expected_msg
-             end) =~ "== Compilation error"
+
+               assert compile_msg =~
+                        "cannot compile module MissingStruct (errors have been logged)"
+             end) =~ expected_msg
     end
 
     test "does not deadlock on missing import/struct dependencies" do
@@ -244,12 +293,19 @@ defmodule Kernel.ParallelCompilerTest do
           """
         )
 
-      assert capture_io(fn ->
-               assert {:error, [{^missing_import, 2, msg}], []} =
+      expected_msg = "module Unknown.Module is not loaded and could not be found"
+
+      assert capture_io(:stderr, fn ->
+               assert {:error,
+                       [{^missing_import, {2, 3}, msg}, {^missing_import, 0, compile_msg}],
+                       []} =
                         Kernel.ParallelCompiler.compile([missing_import, depends_on])
 
-               assert msg =~ "module Unknown.Module is not loaded and could not be found"
-             end) =~ "== Compilation error"
+               assert msg =~ expected_msg
+
+               assert compile_msg =~
+                        "cannot compile module MissingStruct (errors have been logged)"
+             end) =~ expected_msg
     end
 
     test "handles deadlocks" do
@@ -269,7 +325,7 @@ defmodule Kernel.ParallelCompilerTest do
         )
 
       msg =
-        capture_io(fn ->
+        capture_io(:stderr, fn ->
           fixtures = [foo, bar]
           assert {:error, [bar_error, foo_error], []} = Kernel.ParallelCompiler.compile(fixtures)
           assert bar_error == {bar, nil, "deadlocked waiting on module FooDeadlock"}
@@ -280,9 +336,9 @@ defmodule Kernel.ParallelCompilerTest do
       assert msg =~ "parallel_deadlock/foo.ex => BarDeadlock"
       assert msg =~ "parallel_deadlock/bar.ex => FooDeadlock"
       assert msg =~ ~r"== Compilation error in file .+parallel_deadlock/foo\.ex =="
-      assert msg =~ "** (CompileError)  deadlocked waiting on module BarDeadlock"
+      assert msg =~ "** (CompileError) deadlocked waiting on module BarDeadlock"
       assert msg =~ ~r"== Compilation error in file .+parallel_deadlock/bar\.ex =="
-      assert msg =~ "** (CompileError)  deadlocked waiting on module FooDeadlock"
+      assert msg =~ "** (CompileError) deadlocked waiting on module FooDeadlock"
     end
 
     test "does not deadlock from Code.ensure_compiled" do
@@ -307,7 +363,7 @@ defmodule Kernel.ParallelCompilerTest do
       purge([FooCircular, BarCircular])
     end
 
-    test "handles async compilation" do
+    test "handles pmap compilation" do
       [foo, bar] =
         write_tmp(
           "async_compile",
@@ -315,10 +371,11 @@ defmodule Kernel.ParallelCompilerTest do
           defmodule FooAsync do
             true = Code.can_await_module_compilation?()
 
-            Kernel.ParallelCompiler.async(fn ->
-              true = Code.can_await_module_compilation?()
-              BarAsync.__info__(:module)
-            end)
+            [BarAsync] =
+              Kernel.ParallelCompiler.pmap([:ok], fn :ok ->
+                true = Code.can_await_module_compilation?()
+                BarAsync.__info__(:module)
+              end)
           end
           """,
           bar: """
@@ -328,9 +385,9 @@ defmodule Kernel.ParallelCompilerTest do
           """
         )
 
-      capture_io(fn ->
+      capture_io(:stderr, fn ->
         fixtures = [foo, bar]
-        assert assert {:ok, modules, []} = Kernel.ParallelCompiler.compile(fixtures)
+        assert {:ok, modules, []} = Kernel.ParallelCompiler.compile(fixtures)
         assert FooAsync in modules
         assert BarAsync in modules
       end)
@@ -338,17 +395,15 @@ defmodule Kernel.ParallelCompilerTest do
       purge([FooAsync, BarAsync])
     end
 
-    test "handles async deadlocks" do
+    test "handles pmap deadlocks" do
       [foo, bar] =
         write_tmp(
           "async_deadlock",
           foo: """
           defmodule FooAsyncDeadlock do
-            Kernel.ParallelCompiler.async(fn ->
+            Kernel.ParallelCompiler.pmap([:ok], fn :ok ->
               BarAsyncDeadlock.__info__(:module)
             end)
-
-            BarAsyncDeadlock.__info__(:module)
           end
           """,
           bar: """
@@ -358,51 +413,14 @@ defmodule Kernel.ParallelCompilerTest do
           """
         )
 
-      capture_io(fn ->
+      capture_io(:stderr, fn ->
         fixtures = [foo, bar]
         assert {:error, [bar_error, foo_error], []} = Kernel.ParallelCompiler.compile(fixtures)
-        assert bar_error == {bar, nil, "deadlocked waiting on module FooAsyncDeadlock"}
-        assert foo_error == {foo, nil, "deadlocked waiting on module BarAsyncDeadlock"}
+        assert {^bar, nil, "deadlocked waiting on module FooAsyncDeadlock"} = bar_error
+        assert {^foo, nil, "deadlocked waiting on pmap [#PID<" <> _} = foo_error
       end)
     end
 
-    test "supports warnings as errors" do
-      warnings_as_errors = Code.get_compiler_option(:warnings_as_errors)
-
-      [fixture] =
-        write_tmp(
-          "warnings_as_errors",
-          warnings_as_errors: """
-          defmodule WarningsSample do
-            def hello(a), do: a
-            def hello(b), do: b
-          end
-          """
-        )
-
-      output = tmp_path("not_to_be_used")
-
-      try do
-        Code.compiler_options(warnings_as_errors: true)
-
-        msg =
-          capture_io(:stderr, fn ->
-            assert {:error, [error], []} =
-                     Kernel.ParallelCompiler.compile_to_path([fixture], output)
-
-            assert {^fixture, 3, "this clause " <> _} = error
-          end)
-
-        assert msg =~
-                 "Compilation failed due to warnings while using the --warnings-as-errors option\n"
-      after
-        Code.compiler_options(warnings_as_errors: warnings_as_errors)
-        purge([WarningsSample])
-      end
-
-      refute File.exists?(output)
-    end
-
     test "does not use incorrect line number when error originates in another file" do
       File.mkdir_p!(tmp_path())
 
@@ -410,22 +428,22 @@ defmodule Kernel.ParallelCompilerTest do
         write_tmp(
           "error_line",
           a: """
-          defmodule A do
+          defmodule PCA do
             def fun(arg), do: arg / 2
           end
           """,
           b: """
-          defmodule B do
+          defmodule PCB do
             def fun(arg) do
-              A.fun(arg)
+              PCA.fun(arg)
               :ok
             end
           end
-          B.fun(:not_a_number)
+          PCB.fun(:not_a_number)
           """
         )
 
-      capture_io(fn ->
+      capture_io(:stderr, fn ->
         assert {:error, [{^b, 0, _}], _} = Kernel.ParallelCompiler.compile([a, b])
       end)
     end
@@ -442,11 +460,29 @@ defmodule Kernel.ParallelCompilerTest do
           """
         )
 
-      capture_io(fn ->
+      capture_io(:stderr, fn ->
         assert {:error, [{^fixture, 2, _}], _} = Kernel.ParallelCompiler.compile([fixture])
       end)
     end
 
+    test "gets correct file+line+column number for SyntaxError" do
+      File.mkdir_p!(tmp_path())
+
+      [fixture] =
+        write_tmp("error",
+          error: """
+          raise SyntaxError, file: "foo/bar.ex", line: 3, column: 10
+          """
+        )
+
+      file = Path.absname("foo/bar.ex")
+
+      capture_io(:stderr, fn ->
+        assert {:error, [%{file: ^file, source: ^fixture, position: {3, 10}}], _} =
+                 Kernel.ParallelCompiler.compile([fixture], return_diagnostics: true)
+      end)
+    end
+
     test "gets proper beam destinations from dynamic modules" do
       fixtures =
         write_tmp(
@@ -479,11 +515,12 @@ defmodule Kernel.ParallelCompilerTest do
 
       expected_msg = "Undef.__struct__/1 is undefined, cannot expand struct Undef"
 
-      assert capture_io(fn ->
-               assert {:error, [{^fixture, 3, msg}], []} =
+      assert capture_io(:stderr, fn ->
+               assert {:error, [{^fixture, {3, 5}, msg}, {^fixture, 0, compile_msg}], []} =
                         Kernel.ParallelCompiler.require([fixture])
 
                assert msg =~ expected_msg
+               assert compile_msg =~ "cannot compile module Undef (errors have been logged)"
              end) =~ expected_msg
     end
 
@@ -506,44 +543,15 @@ defmodule Kernel.ParallelCompilerTest do
       expected_msg =
         "ThisModuleWillNeverBeAvailable.__struct__/1 is undefined, cannot expand struct ThisModuleWillNeverBeAvailable"
 
-      assert capture_io(fn ->
-               assert {:error, [{^fixture, 7, msg}], []} =
+      assert capture_io(:stderr, fn ->
+               assert {:error, [{^fixture, {7, 3}, msg}, {^fixture, 0, compile_msg}], []} =
                         Kernel.ParallelCompiler.require([fixture])
 
                assert msg =~ expected_msg
-             end) =~ "== Compilation error"
-    end
 
-    test "supports warnings as errors" do
-      warnings_as_errors = Code.get_compiler_option(:warnings_as_errors)
-
-      [fixture] =
-        write_tmp(
-          "warnings_as_errors",
-          warnings_as_errors: """
-          defmodule WarningsSample do
-            def hello(a), do: a
-            def hello(b), do: b
-          end
-          """
-        )
-
-      try do
-        Code.compiler_options(warnings_as_errors: true)
-
-        msg =
-          capture_io(:stderr, fn ->
-            assert {:error, [error], []} = Kernel.ParallelCompiler.require([fixture])
-
-            assert {^fixture, 3, "this clause " <> _} = error
-          end)
-
-        assert msg =~
-                 "Compilation failed due to warnings while using the --warnings-as-errors option\n"
-      after
-        Code.compiler_options(warnings_as_errors: warnings_as_errors)
-        purge([WarningsSample])
-      end
+               assert compile_msg =~
+                        "cannot compile module WithBehaviourAndStruct (errors have been logged)"
+             end) =~ expected_msg
     end
   end
 end
