@@ -17,7 +17,7 @@ defmodule Kernel.RaiseTest do
       try do
         raise "a"
       rescue
-        _ -> hd(__STACKTRACE__)
+        _ -> Enum.fetch!(__STACKTRACE__, 0)
       end
 
     file = __ENV__.file |> Path.relative_to_cwd() |> String.to_charlist()
@@ -71,21 +71,19 @@ defmodule Kernel.RaiseTest do
     end
   end
 
-  if System.otp_release() >= "24" do
-    test "raise with error_info" do
-      {exception, stacktrace} =
-        try do
-          raise "a"
-        rescue
-          e -> {e, __STACKTRACE__}
-        end
+  test "raise with error_info" do
+    {exception, stacktrace} =
+      try do
+        raise "a"
+      rescue
+        e -> {e, __STACKTRACE__}
+      end
 
-      assert [{__MODULE__, _, _, meta} | _] = stacktrace
-      assert meta[:error_info] == %{module: Exception}
+    assert [{__MODULE__, _, _, meta} | _] = stacktrace
+    assert meta[:error_info] == %{module: Exception}
 
-      assert Exception.format_error(exception, stacktrace) ==
-               %{general: "a", reason: "#Elixir.RuntimeError"}
-    end
+    assert Exception.format_error(exception, stacktrace) ==
+             %{general: "a", reason: "#Elixir.RuntimeError"}
   end
 
   test "reraise message" do
@@ -165,16 +163,6 @@ defmodule Kernel.RaiseTest do
     end
   end
 
-  test "reraise with invalid stacktrace" do
-    try do
-      reraise %RuntimeError{message: "message"}, {:oops, @trace}
-    rescue
-      ArgumentError ->
-        {name, arity} = __ENV__.function
-        assert [{__MODULE__, ^name, ^arity, _} | _] = __STACKTRACE__
-    end
-  end
-
   describe "rescue" do
     test "runtime error" do
       result =
@@ -192,7 +180,7 @@ defmodule Kernel.RaiseTest do
         try do
           raise "an exception"
         rescue
-          AnotherError -> true
+          ArgumentError -> true
         catch
           :error, _ -> false
         end
@@ -226,6 +214,17 @@ defmodule Kernel.RaiseTest do
       assert result == "an exception"
     end
 
+    test "named function clause (stacktrace) or runtime (no stacktrace) error" do
+      result =
+        try do
+          Access.get("foo", 0)
+        rescue
+          x in [FunctionClauseError, CaseClauseError] -> Exception.message(x)
+        end
+
+      assert result == "no function clause matching in Access.get/3"
+    end
+
     test "with higher precedence than catch" do
       result =
         try do
@@ -305,6 +304,21 @@ defmodule Kernel.RaiseTest do
 
       assert result == "an exception"
     end
+
+    defmacrop argerr(e) do
+      quote(do: unquote(e) in ArgumentError)
+    end
+
+    test "with rescue macro" do
+      result =
+        try do
+          raise ArgumentError, "oops, badarg"
+        rescue
+          argerr(e) -> Exception.message(e)
+        end
+
+      assert result == "oops, badarg"
+    end
   end
 
   describe "normalize" do
@@ -328,18 +342,19 @@ defmodule Kernel.RaiseTest do
         end
 
       assert result ==
-               "function DoNotExist.for_sure/0 is undefined (module DoNotExist is not available)"
+               "function DoNotExist.for_sure/0 is undefined (module DoNotExist is not available). " <>
+                 "Make sure the module name is correct and has been specified in full (or that an alias has been defined)"
     end
 
     test "function clause error" do
       result =
         try do
-          zero(1)
+          Access.get(:ok, :error)
         rescue
           x in [FunctionClauseError] -> Exception.message(x)
         end
 
-      assert result == "no function clause matching in Kernel.RaiseTest.zero/1"
+      assert result == "no function clause matching in Access.get/3"
     end
 
     test "badarg error" do
@@ -410,6 +425,7 @@ defmodule Kernel.RaiseTest do
 
       fun = BadFunction.Missing.fun()
 
+      :code.purge(BadFunction.Missing)
       :code.delete(BadFunction.Missing)
 
       defmodule BadFunction.Missing do
@@ -418,15 +434,15 @@ defmodule Kernel.RaiseTest do
 
       :code.purge(BadFunction.Missing)
 
-      result =
-        try do
-          fun.()
-        rescue
-          x in [BadFunctionError] -> Exception.message(x)
-        end
-
-      assert result =~
-               ~r/function #Function<[0-9]\.[0-9]*\/0[^>]*> is invalid, likely because it points to an old version of the code/
+      try do
+        fun.()
+      rescue
+        x in [BadFunctionError] ->
+          assert Exception.message(x) =~
+                   ~r/function #Function<[0-9]\.[0-9]*\/0[^>]*> is invalid, likely because it points to an old version of the code/
+      else
+        _ -> flunk("this should not be invoked")
+      end
     end
 
     test "badmatch error" do
@@ -434,7 +450,7 @@ defmodule Kernel.RaiseTest do
 
       result =
         try do
-          ^x = zero(0)
+          ^x = Process.get(:unused, 0)
         rescue
           x in [MatchError] -> Exception.message(x)
         end
@@ -442,12 +458,10 @@ defmodule Kernel.RaiseTest do
       assert result == "no match of right hand side value: 0"
     end
 
-    defp empty_map(), do: %{}
-
     test "bad key error" do
       result =
         try do
-          %{empty_map() | foo: :bar}
+          %{Process.get(:unused, %{}) | foo: :bar}
         rescue
           x in [KeyError] -> Exception.message(x)
         end
@@ -456,7 +470,7 @@ defmodule Kernel.RaiseTest do
 
       result =
         try do
-          empty_map().foo
+          Process.get(:unused, %{}).foo
         rescue
           x in [KeyError] -> Exception.message(x)
         end
@@ -467,7 +481,7 @@ defmodule Kernel.RaiseTest do
     test "bad map error" do
       result =
         try do
-          %{zero(0) | foo: :bar}
+          %{Process.get(:unused, 0) | foo: :bar}
         rescue
           x in [BadMapError] -> Exception.message(x)
         end
@@ -478,7 +492,7 @@ defmodule Kernel.RaiseTest do
     test "bad boolean error" do
       result =
         try do
-          1 and true
+          Process.get(:unused, 1) and true
         rescue
           x in [BadBooleanError] -> Exception.message(x)
         end
@@ -491,7 +505,7 @@ defmodule Kernel.RaiseTest do
 
       result =
         try do
-          case zero(0) do
+          case Process.get(:unused, 0) do
             ^x -> nil
           end
         rescue
@@ -505,7 +519,7 @@ defmodule Kernel.RaiseTest do
       result =
         try do
           cond do
-            !zero(0) -> :ok
+            !Process.get(:unused, 0) -> :ok
           end
         rescue
           x in [CondClauseError] -> Exception.message(x)
@@ -544,7 +558,8 @@ defmodule Kernel.RaiseTest do
         end
 
       assert result ==
-               "function DoNotExist.for_sure/0 is undefined (module DoNotExist is not available)"
+               "function DoNotExist.for_sure/0 is undefined (module DoNotExist is not available). " <>
+                 "Make sure the module name is correct and has been specified in full (or that an alias has been defined)"
     end
   end
 
@@ -561,8 +576,7 @@ defmodule Kernel.RaiseTest do
       end
 
     assert result ==
-             "function DoNotExist.for_sure/0 is undefined (module DoNotExist is not available)"
+             "function DoNotExist.for_sure/0 is undefined (module DoNotExist is not available). " <>
+               "Make sure the module name is correct and has been specified in full (or that an alias has been defined)"
   end
-
-  defp zero(0), do: 0
 end
