@@ -6,8 +6,7 @@ defmodule MapTest do
   doctest Map
 
   @sample %{a: 1, b: 2}
-
-  defp sample, do: @sample
+  defp sample, do: Process.get(:unused, %{a: 1, b: 2})
 
   test "maps in attributes" do
     assert @sample == %{a: 1, b: 2}
@@ -82,7 +81,7 @@ defmodule MapTest do
 
   test "map_size/1" do
     assert map_size(%{}) == 0
-    assert map_size(@sample) == 2
+    assert map_size(sample()) == 2
   end
 
   test "new/1" do
@@ -112,6 +111,22 @@ defmodule MapTest do
     assert_raise BadMapError, fn -> Map.split(:foo, []) end
   end
 
+  test "split_with/2" do
+    assert Map.split_with(%{}, fn {_k, v} -> rem(v, 2) == 0 end) == {%{}, %{}}
+
+    assert Map.split_with(%{a: 1, b: 2, c: 3}, fn {_k, v} -> rem(v, 2) == 0 end) ==
+             {%{b: 2}, %{a: 1, c: 3}}
+
+    assert Map.split_with(%{a: 2, b: 4, c: 6}, fn {_k, v} -> rem(v, 2) == 0 end) ==
+             {%{a: 2, b: 4, c: 6}, %{}}
+
+    assert Map.split_with(%{1 => 1, 2 => 2, 3 => 3}, fn {k, _v} -> rem(k, 2) == 0 end) ==
+             {%{2 => 2}, %{1 => 1, 3 => 3}}
+
+    assert Map.split_with(%{1 => 2, 3 => 4, 5 => 6}, fn {k, _v} -> rem(k, 2) == 0 end) ==
+             {%{}, %{1 => 2, 3 => 4, 5 => 6}}
+  end
+
   test "get_and_update/3" do
     message = "the given function must return a two-element tuple or :pop, got: 1"
 
@@ -206,7 +221,7 @@ defmodule MapTest do
     assert Map.replace!(map, :b, 10) == %{c: 3, b: 10, a: 1}
     assert Map.replace!(map, :a, 1) == map
 
-    assert_raise KeyError, "key :x not found in: %{a: 1, b: 2, c: 3}", fn ->
+    assert_raise KeyError, ~r/key :x not found in: %{.*a: 1.*}/, fn ->
       Map.replace!(map, :x, 10)
     end
 
@@ -215,6 +230,29 @@ defmodule MapTest do
     end
   end
 
+  test "intersect/2" do
+    map = %{a: 1, b: 2}
+
+    assert Map.intersect(map, %{a: 2}) == %{a: 2}
+    assert Map.intersect(map, %{c: 3}) == %{}
+    assert Map.intersect(%{a: 2}, map) == %{a: 1}
+    assert Map.intersect(%{c: 3}, map) == %{}
+
+    assert Map.intersect(map, %{a: 2}) |> Map.intersect(%{a: 3, c: 3}) == %{a: 3}
+    assert Map.intersect(map, %{c: 3}) |> Map.intersect(%{c: 4}) == %{}
+    assert Map.intersect(map, %{a: 3, c: 3}) |> Map.intersect(%{a: 2}) == %{a: 2}
+  end
+
+  test "intersect/3" do
+    # When first map is bigger
+    assert Map.intersect(%{a: 1, b: 2, c: 3}, %{c: 4, d: 5}, fn :c, 3, 4 -> :x end) ==
+             %{c: :x}
+
+    # When second map is bigger
+    assert Map.intersect(%{b: 2, c: 3}, %{a: 1, c: 4, d: 5}, fn :c, 3, 4 -> :x end) ==
+             %{c: :x}
+  end
+
   test "implements (almost) all functions in Keyword" do
     assert Keyword.__info__(:functions) -- Map.__info__(:functions) == [
              delete: 3,
@@ -240,17 +278,9 @@ defmodule MapTest do
   end
 
   defmodule ExternalUser do
-    def __struct__ do
-      %{__struct__: __MODULE__, name: "john", age: 27}
-    end
-
-    def __struct__(kv) do
-      Enum.reduce(kv, __struct__(), fn {k, v}, acc -> :maps.update(k, v, acc) end)
-    end
+    defstruct name: "john", age: 27
   end
 
-  defp empty_map(), do: %{}
-
   test "structs" do
     assert %ExternalUser{} == %{__struct__: ExternalUser, name: "john", age: 27}
 
@@ -261,10 +291,6 @@ defmodule MapTest do
 
     %ExternalUser{name: name} = %ExternalUser{}
     assert name == "john"
-
-    assert_raise BadStructError, "expected a struct named MapTest.ExternalUser, got: %{}", fn ->
-      %ExternalUser{empty_map() | name: "meg"}
-    end
   end
 
   describe "structs with variable name" do
@@ -292,12 +318,11 @@ defmodule MapTest do
       end
     end
 
-    defp foo(), do: "foo"
     defp destruct1(%module{}), do: module
     defp destruct2(%_{}), do: :ok
 
     test "does not match" do
-      invalid_struct = %{__struct__: foo()}
+      invalid_struct = Process.get(:unused, %{__struct__: "foo"})
 
       assert_raise CaseClauseError, fn ->
         case invalid_struct do
@@ -312,7 +337,7 @@ defmodule MapTest do
       end
 
       assert_raise CaseClauseError, fn ->
-        foo = foo()
+        foo = Process.get(:unused, "foo")
 
         case invalid_struct do
           %^foo{} -> :ok
@@ -337,7 +362,7 @@ defmodule MapTest do
       end
 
       assert_raise MatchError, fn ->
-        foo = foo()
+        foo = Process.get(:unused, "foo")
         %^foo{} = invalid_struct
       end
     end
@@ -393,6 +418,10 @@ defmodule MapTest do
       defmodule TestMod do
         @enforce_keys :foo
         defstruct [:foo]
+
+        # Verify it remain set afterwards
+        :foo = @enforce_keys
+
         def foo do
           %TestMod{}
         end
