@@ -18,8 +18,10 @@ defmodule ModuleTest.ToBeUsed do
 
   defmacro __after_compile__(%Macro.Env{module: ModuleTest.ToUse} = env, bin)
            when is_binary(bin) do
+    # Ensure module is not longer tracked as being loaded
+    false = __MODULE__ in :elixir_module.compiler_modules()
     [] = Macro.Env.vars(env)
-    # IO.puts "HELLO"
+    :ok
   end
 
   defmacro callback(env) do
@@ -33,7 +35,7 @@ end
 
 defmodule ModuleTest.ToUse do
   # Moving the next line around can make tests fail
-  36 = __ENV__.line
+  38 = __ENV__.line
   var = 1
   # Not available in callbacks
   _ = var
@@ -52,13 +54,6 @@ defmodule ModuleTest do
   @register_example :it_works
   @register_example :still_works
 
-  contents =
-    quote do
-      def eval_quoted_info, do: {__MODULE__, __ENV__.file, __ENV__.line}
-    end
-
-  Module.eval_quoted(__MODULE__, contents, [], file: "sample.ex", line: 13)
-
   defp purge(module) do
     :code.purge(module)
     :code.delete(module)
@@ -121,37 +116,15 @@ defmodule ModuleTest do
   end
 
   test "in memory modules are tagged as so" do
-    assert :code.which(__MODULE__) == ''
-  end
-
-  ## Eval
-
-  test "executes eval_quoted definitions" do
-    assert eval_quoted_info() == {ModuleTest, "sample.ex", 13}
-  end
-
-  test "resets last definition information on eval" do
-    # This should not emit any warning
-    defmodule LastDefinition do
-      def foo(0), do: 0
-
-      Module.eval_quoted(
-        __ENV__,
-        quote do
-          def bar, do: :ok
-        end
-      )
-
-      def foo(1), do: 1
-    end
-  end
-
-  test "retrieves line from use callsite" do
-    assert ModuleTest.ToUse.line() == 41
+    assert :code.which(__MODULE__) == ~c""
   end
 
   ## Callbacks
 
+  test "retrieves line from use callsite" do
+    assert ModuleTest.ToUse.line() == 43
+  end
+
   test "executes custom before_compile callback" do
     assert ModuleTest.ToUse.callback_value(true) == true
     assert ModuleTest.ToUse.callback_value(false) == false
@@ -231,7 +204,7 @@ defmodule ModuleTest do
       @compile {:autoload, false}
     end
 
-    refute :code.is_loaded(NoAutoload)
+    refute Code.loaded?(NoAutoload)
   end
 
   ## Naming
@@ -332,6 +305,23 @@ defmodule ModuleTest do
     assert Elixir.ModuleTest.NonAtomAlias.hello() == :world
   end
 
+  test "does not leak alias from Elixir root alias" do
+    defmodule Elixir.ModuleTest.ElixirRootAlias do
+      def hello, do: :world
+    end
+
+    refute __ENV__.aliases[Elixir.ModuleTest]
+    refute __ENV__.aliases[Elixir.ElixirRootAlias]
+    assert Elixir.ModuleTest.ElixirRootAlias.hello() == :world
+  end
+
+  test "does not warn on captured underscored vars" do
+    _unused = 123
+
+    defmodule __MODULE__.NoVarWarning do
+    end
+  end
+
   @compile {:no_warn_undefined, ModuleCreateSample}
 
   test "create" do
@@ -350,8 +340,10 @@ defmodule ModuleTest do
         def world, do: true
       end
 
-    assert_raise CompileError, fn ->
-      {:module, Elixir, _, _} = Module.create(Elixir, contents, __ENV__)
+    assert_raise CompileError, ~r/cannot compile module Elixir/, fn ->
+      Code.with_diagnostics(fn ->
+        Module.create(Elixir, contents, __ENV__)
+      end)
     end
   end
 
@@ -435,6 +427,13 @@ defmodule ModuleTest do
     assert backend.debug_info(:elixir_v1, ModuleCreateNoDebugInfo, data, []) == {:error, :missing}
   end
 
+  test "compiles to core" do
+    {:ok, {Atom, [{~c"Dbgi", dbgi}]}} = Atom |> :code.which() |> :beam_lib.chunks([~c"Dbgi"])
+    {:debug_info_v1, backend, data} = :erlang.binary_to_term(dbgi)
+    {:ok, core} = backend.debug_info(:core_v1, Atom, data, [])
+    assert is_tuple(core)
+  end
+
   test "no function in module body" do
     in_module do
       assert __ENV__.function == nil
@@ -495,12 +494,14 @@ defmodule ModuleTest do
     in_module do
       def foo(a, b), do: a + b
 
-      assert {:v1, :def, _,
+      assert {:v1, :def, def_meta,
               [
-                {_, [{:a, _, nil}, {:b, _, nil}], [],
+                {clause_meta, [{:a, _, nil}, {:b, _, nil}], [],
                  {{:., _, [:erlang, :+]}, _, [{:a, _, nil}, {:b, _, nil}]}}
               ]} = Module.get_definition(__MODULE__, {:foo, 2})
 
+      assert [line: _, column: _] = Keyword.take(def_meta, [:line, :column])
+      assert [line: _, column: _] = Keyword.take(clause_meta, [:line, :column])
       assert {:v1, :def, _, []} = Module.get_definition(__MODULE__, {:foo, 2}, skip_clauses: true)
 
       assert Module.delete_definition(__MODULE__, {:foo, 2})
@@ -537,9 +538,10 @@ defmodule ModuleTest do
   end
 
   describe "get_attribute/3" do
-    test "returns a list when the attribute is marked as `accummulate: true`" do
+    test "returns a list when the attribute is marked as `accumulate: true`" do
       in_module do
         Module.register_attribute(__MODULE__, :value, accumulate: true)
+        assert Module.get_attribute(__MODULE__, :value) == []
         Module.put_attribute(__MODULE__, :value, 1)
         assert Module.get_attribute(__MODULE__, :value) == [1]
         Module.put_attribute(__MODULE__, :value, 2)
@@ -552,6 +554,21 @@ defmodule ModuleTest do
         Module.put_attribute(__MODULE__, :attribute, 1)
         assert Module.get_attribute(__MODULE__, :attribute) == 1
         assert Module.get_attribute(__MODULE__, :attribute, :default) == 1
+        Module.put_attribute(__MODULE__, :attribute, nil)
+        assert Module.get_attribute(__MODULE__, :attribute, :default) == nil
+      end
+    end
+
+    test "returns the value of the attribute if persisted" do
+      in_module do
+        Module.register_attribute(__MODULE__, :value, persist: true)
+        assert Module.get_attribute(__MODULE__, :value, 123) == 123
+        Module.put_attribute(__MODULE__, :value, 1)
+        assert Module.get_attribute(__MODULE__, :value) == 1
+        Module.put_attribute(__MODULE__, :value, 2)
+        assert Module.get_attribute(__MODULE__, :value) == 2
+        Module.delete_attribute(__MODULE__, :value)
+        assert Module.get_attribute(__MODULE__, :value, 123) == 123
       end
     end
 
@@ -562,6 +579,41 @@ defmodule ModuleTest do
     end
   end
 
+  describe "get_last_attribute/3" do
+    test "returns the last set value when the attribute is marked as `accumulate: true`" do
+      in_module do
+        Module.register_attribute(__MODULE__, :value, accumulate: true)
+        Module.put_attribute(__MODULE__, :value, 1)
+        assert Module.get_last_attribute(__MODULE__, :value) == 1
+        Module.put_attribute(__MODULE__, :value, 2)
+        assert Module.get_last_attribute(__MODULE__, :value) == 2
+      end
+    end
+
+    test "returns the value of the non-accumulate attribute if it exists" do
+      in_module do
+        Module.put_attribute(__MODULE__, :attribute, 1)
+        assert Module.get_last_attribute(__MODULE__, :attribute) == 1
+        Module.put_attribute(__MODULE__, :attribute, nil)
+        assert Module.get_last_attribute(__MODULE__, :attribute, :default) == nil
+      end
+    end
+
+    test "returns the passed default if the accumulate attribute has not yet been set" do
+      in_module do
+        Module.register_attribute(__MODULE__, :value, accumulate: true)
+        assert Module.get_last_attribute(__MODULE__, :value) == nil
+        assert Module.get_last_attribute(__MODULE__, :value, :default) == :default
+      end
+    end
+
+    test "returns the passed default if the non-accumulate attribute does not exist" do
+      in_module do
+        assert Module.get_last_attribute(__MODULE__, :value, :default) == :default
+      end
+    end
+  end
+
   describe "has_attribute?/2 and attributes_in/2" do
     test "returns true when attribute has been defined" do
       in_module do
