@@ -38,6 +38,21 @@ defmodule PathTest do
     File.rm_rf(Path.join(config.tmp_dir, "wildcard"))
   end
 
+  @tag :tmp_dir
+  test "wildcard/2 follows ..", config do
+    hello = Path.join(config.tmp_dir, "wildcard/hello")
+    world = Path.join(config.tmp_dir, "wildcard/world")
+    File.mkdir_p(hello)
+    File.touch(world)
+
+    assert Path.wildcard(Path.join(config.tmp_dir, "wildcard/w*/../h*")) == []
+
+    assert Path.wildcard(Path.join(config.tmp_dir, "wildcard/h*/../w*")) ==
+             [Path.join(config.tmp_dir, "wildcard/hello/../world")]
+  after
+    File.rm_rf(Path.join(config.tmp_dir, "wildcard"))
+  end
+
   test "wildcard/2 raises on null byte" do
     assert_raise ArgumentError, ~r/null byte/, fn -> Path.wildcard("foo\0bar") end
   end
@@ -72,23 +87,46 @@ defmodule PathTest do
       assert Path.relative("../usr/local/bin") == "../usr/local/bin"
     end
 
-    test "relative_to/2" do
-      assert Path.relative_to("D:/usr/local/foo", "D:/usr/") == "local/foo"
-      assert Path.relative_to("D:/usr/local/foo", "d:/usr/") == "local/foo"
-      assert Path.relative_to("d:/usr/local/foo", "D:/usr/") == "local/foo"
-      assert Path.relative_to("D:/usr/local/foo", "d:/") == "usr/local/foo"
-      assert Path.relative_to("D:/usr/local/foo", "D:/") == "usr/local/foo"
-      assert Path.relative_to("D:/usr/local/foo", "d:") == "D:/usr/local/foo"
-      assert Path.relative_to("D:/usr/local/foo", "D:") == "D:/usr/local/foo"
+    test "relative_to/3" do
+      # should give same relative paths for both force true and false
+      for force <- [true, false] do
+        assert Path.relative_to("//usr/local/foo", "//usr/", force: force) == "local/foo"
+
+        assert Path.relative_to("D:/usr/local/foo", "D:/usr/", force: force) == "local/foo"
+        assert Path.relative_to("D:/usr/local/foo", "d:/usr/", force: force) == "local/foo"
+        assert Path.relative_to("d:/usr/local/foo", "D:/usr/", force: force) == "local/foo"
+        assert Path.relative_to("D:/usr/local/foo", "d:/", force: force) == "usr/local/foo"
+        assert Path.relative_to("D:/usr/local/foo", "D:/", force: force) == "usr/local/foo"
+
+        assert Path.relative_to("d:/usr/local/foo/..", "d:/usr/local", force: force) == "."
+        assert Path.relative_to("d:/usr/local/../foo", "d:/usr/foo", force: force) == "."
+        assert Path.relative_to("d:/usr/local/../foo/bar", "d:/usr/foo", force: force) == "bar"
+        assert Path.relative_to("d:/usr/local/../foo/./bar", "d:/usr/foo", force: force) == "bar"
+
+        assert Path.relative_to("d:/usr/local/../foo/bar/..", "d:/usr/foo", force: force) == "."
+        assert Path.relative_to("d:/usr/local/foo/..", "d:/usr/local/..", force: force) == "local"
+        assert Path.relative_to("d:/usr/local/foo/..", "d:/usr/local/.", force: force) == "."
+      end
+
+      # different results for force: true
+      assert Path.relative_to("d:/usr/local/../foo", "d:/usr/local") == "d:/usr/foo"
+      assert Path.relative_to("d:/usr/local/../foo", "d:/usr/local", force: true) == "../foo"
+
+      assert Path.relative_to("d:/usr/local/../foo/../bar", "d:/usr/foo") == "d:/usr/bar"
+      assert Path.relative_to("d:/usr/local/../foo/../bar", "d:/usr/foo", force: true) == "../bar"
+
+      # on different volumes with force: true it should return the original path
+      assert Path.relative_to("d:/usr/local", "c:/usr/local", force: true) == "d:/usr/local"
+      assert Path.relative_to("d:/usr/local", "c:/another/local", force: true) == "d:/usr/local"
     end
 
     test "type/1" do
       assert Path.type("C:/usr/local/bin") == :absolute
-      assert Path.type('C:\\usr\\local\\bin') == :absolute
+      assert Path.type(~c"C:\\usr\\local\\bin") == :absolute
       assert Path.type("C:usr\\local\\bin") == :volumerelative
 
       assert Path.type("/usr/local/bin") == :volumerelative
-      assert Path.type('usr/local/bin') == :relative
+      assert Path.type(~c"usr/local/bin") == :relative
       assert Path.type("../usr/local/bin") == :relative
 
       assert Path.type("//host/path") == :absolute
@@ -109,12 +147,12 @@ defmodule PathTest do
       assert Path.safe_relative("foo/../..") == :error
     end
 
-    test "safe_relative_to/2" do
-      assert Path.safe_relative_to("local/foo/bar", "local") == {:ok, "local/foo/bar"}
-      assert Path.safe_relative_to("foo/..", "local") == {:ok, ""}
-      assert Path.safe_relative_to("..", "local/foo") == :error
-      assert Path.safe_relative_to("d:/usr/local/foo", "D:/") == :error
-      assert Path.safe_relative_to("D:/usr/local/foo", "d:/") == :error
+    test "safe_relative/2" do
+      assert Path.safe_relative("local/foo/bar", "local") == {:ok, "local/foo/bar"}
+      assert Path.safe_relative("foo/..", "local") == {:ok, ""}
+      assert Path.safe_relative("..", "local/foo") == :error
+      assert Path.safe_relative("d:/usr/local/foo", "D:/") == :error
+      assert Path.safe_relative("D:/usr/local/foo", "d:/") == :error
     end
   end
 
@@ -126,8 +164,51 @@ defmodule PathTest do
       assert Path.relative("usr/local/bin") == "usr/local/bin"
       assert Path.relative("../usr/local/bin") == "../usr/local/bin"
       assert Path.relative("/") == "."
-      assert Path.relative('/') == "."
-      assert Path.relative(['/usr', ?/, "local/bin"]) == "usr/local/bin"
+      assert Path.relative(~c"/") == "."
+      assert Path.relative([~c"/usr", ?/, "local/bin"]) == "usr/local/bin"
+    end
+
+    test "relative_to/3" do
+      # subpaths of cwd, should give the same result for both force true and false
+      for force <- [false, true] do
+        assert Path.relative_to("/usr/local/foo", "/usr/local", force: force) == "foo"
+        assert Path.relative_to("/usr/local/foo", "/", force: force) == "usr/local/foo"
+        assert Path.relative_to("/usr/local/foo", "/usr/local/foo", force: force) == "."
+        assert Path.relative_to("/usr/local/foo/", "/usr/local/foo", force: force) == "."
+        assert Path.relative_to("/usr/local/foo", "/usr/local/foo/", force: force) == "."
+
+        assert Path.relative_to("/usr/local/foo/..", "/usr/local", force: force) == "."
+        assert Path.relative_to("/usr/local/../foo", "/usr/foo", force: force) == "."
+        assert Path.relative_to("/usr/local/../foo/bar", "/usr/foo", force: force) == "bar"
+        assert Path.relative_to("/usr/local/../foo/./bar", "/usr/foo", force: force) == "bar"
+        assert Path.relative_to("/usr/local/../foo/bar/..", "/usr/foo", force: force) == "."
+
+        assert Path.relative_to("/usr/local/foo/..", "/usr/local/..", force: force) == "local"
+        assert Path.relative_to("/usr/local/foo/..", "/usr/local/.", force: force) == "."
+      end
+
+      # With relative second argument
+      assert Path.relative_to("/usr/local/foo", "etc") == "/usr/local/foo"
+      assert Path.relative_to("/usr/local/foo", "etc", force: true) == "/usr/local/foo"
+
+      # Different relative paths for force true/false
+      assert Path.relative_to("/usr/local/foo", "/etc") == "/usr/local/foo"
+      assert Path.relative_to("/usr/local/foo", "/etc", force: true) == "../usr/local/foo"
+
+      assert Path.relative_to("/usr/local/../foo", "/usr/local") == "/usr/foo"
+      assert Path.relative_to("/usr/local/../foo", "/usr/local", force: true) == "../foo"
+
+      assert Path.relative_to("/usr/local/../foo/../bar", "/usr/foo") == "/usr/bar"
+      assert Path.relative_to("/usr/local/../foo/../bar", "/usr/foo", force: true) == "../bar"
+
+      # More tests with force: true
+      assert Path.relative_to("/etc", "/usr/local/foo", force: true) == "../../../etc"
+      assert Path.relative_to(~c"/usr/local/foo", "/etc", force: true) == "../usr/local/foo"
+      assert Path.relative_to("/usr/local", "/usr/local/foo", force: true) == ".."
+      assert Path.relative_to("/usr/local/..", "/usr/local", force: true) == ".."
+
+      assert Path.relative_to("/usr/../etc/foo/../../bar", "/log/foo/../../usr/", force: true) ==
+               "../bar"
     end
 
     test "type/1" do
@@ -135,21 +216,29 @@ defmodule PathTest do
       assert Path.type("usr/local/bin") == :relative
       assert Path.type("../usr/local/bin") == :relative
 
-      assert Path.type('/usr/local/bin') == :absolute
-      assert Path.type('usr/local/bin') == :relative
-      assert Path.type('../usr/local/bin') == :relative
+      assert Path.type(~c"/usr/local/bin") == :absolute
+      assert Path.type(~c"usr/local/bin") == :relative
+      assert Path.type(~c"../usr/local/bin") == :relative
 
-      assert Path.type(['/usr/', 'local/bin']) == :absolute
-      assert Path.type(['usr/', 'local/bin']) == :relative
-      assert Path.type(['../usr', '/local/bin']) == :relative
+      assert Path.type([~c"/usr/", ~c"local/bin"]) == :absolute
+      assert Path.type([~c"usr/", ~c"local/bin"]) == :relative
+      assert Path.type([~c"../usr", ~c"/local/bin"]) == :relative
     end
   end
 
-  test "relative_to_cwd/1" do
+  test "relative_to_cwd/2" do
     assert Path.relative_to_cwd(__ENV__.file) == Path.relative_to(__ENV__.file, File.cwd!())
 
     assert Path.relative_to_cwd(to_charlist(__ENV__.file)) ==
              Path.relative_to(to_charlist(__ENV__.file), to_charlist(File.cwd!()))
+
+    assert Path.relative_to_cwd(Path.dirname(File.cwd!()), force: true) == ".."
+
+    [slash | split_cwd] = Path.split(File.cwd!())
+    relative_to_root = List.duplicate("..", length(split_cwd))
+
+    assert Path.relative_to_cwd(slash) == slash
+    assert Path.relative_to_cwd(slash, force: true) == Path.join(relative_to_root)
   end
 
   test "absname/1,2" do
@@ -172,9 +261,9 @@ defmodule PathTest do
     home = System.user_home!() |> Path.absname()
 
     assert home == Path.expand("~")
-    assert home == Path.expand('~')
+    assert home == Path.expand(~c"~")
     assert is_binary(Path.expand("~/foo"))
-    assert is_binary(Path.expand('~/foo'))
+    assert is_binary(Path.expand(~c"~/foo"))
 
     assert Path.expand("~/file") == Path.join(home, "file")
     assert Path.expand("~/file", "whatever") == Path.join(home, "file")
@@ -205,7 +294,8 @@ defmodule PathTest do
     assert drive_letter == "/bar"
 
     drive_letter =
-      Path.expand(['..', ?/, "bar/../bar"], '/foo/../foo/../foo') |> strip_drive_letter_if_windows
+      Path.expand([~c"..", ?/, "bar/../bar"], ~c"/foo/../foo/../foo")
+      |> strip_drive_letter_if_windows
 
     assert "/bar" == drive_letter
 
@@ -214,20 +304,25 @@ defmodule PathTest do
     assert Path.expand("bar/../bar", "foo") == Path.expand("foo/bar")
   end
 
-  test "relative_to/2" do
-    assert Path.relative_to("/usr/local/foo", "/usr/local") == "foo"
-    assert Path.relative_to("/usr/local/foo", "/") == "usr/local/foo"
-    assert Path.relative_to("/usr/local/foo", "/etc") == "/usr/local/foo"
-    assert Path.relative_to("/usr/local/foo", "/usr/local/foo") == "."
-    assert Path.relative_to("/usr/local/foo/", "/usr/local/foo") == "."
-    assert Path.relative_to("/usr/local/foo", "/usr/local/foo/") == "."
+  test "relative_to/3 (with relative paths)" do
+    # on cwd
+    assert Path.relative_to("foo", File.cwd!()) == "foo"
+    assert Path.relative_to("./foo", File.cwd!()) == "foo"
+    assert Path.relative_to("./foo/.", File.cwd!()) == "foo"
+    assert Path.relative_to("./foo/./bar/.", File.cwd!()) == "foo/bar"
+    assert Path.relative_to("../foo/./bar/.", File.cwd!()) == "../foo/bar"
+    assert Path.relative_to("../foo/./bar/..", File.cwd!()) == "../foo"
+    assert Path.relative_to("../foo/../bar/..", File.cwd!()) == ".."
+    assert Path.relative_to("./foo/../bar/..", File.cwd!()) == "."
 
+    # both relative
+    assert Path.relative_to("usr/local/foo", ".") == "usr/local/foo"
+    assert Path.relative_to(".", "usr/local/foo") == "."
     assert Path.relative_to("usr/local/foo", "usr/local") == "foo"
-    assert Path.relative_to("usr/local/foo", "etc") == "usr/local/foo"
-    assert Path.relative_to('usr/local/foo', "etc") == "usr/local/foo"
-
+    assert Path.relative_to("usr/local/foo", "etc") == "../usr/local/foo"
+    assert Path.relative_to(~c"usr/local/foo", "etc") == "../usr/local/foo"
     assert Path.relative_to("usr/local/foo", "usr/local") == "foo"
-    assert Path.relative_to(["usr", ?/, 'local/foo'], 'usr/local') == "foo"
+    assert Path.relative_to(["usr", ?/, ~c"local/foo"], ~c"usr/local") == "foo"
   end
 
   test "safe_relative/1" do
@@ -239,26 +334,26 @@ defmodule PathTest do
     assert Path.safe_relative("foo/../..") == :error
   end
 
-  test "safe_relative_to/2" do
-    assert Path.safe_relative_to("/usr/local/foo", "/usr/local") == :error
-    assert Path.safe_relative_to("../../..", "foo/bar") == :error
-    assert Path.safe_relative_to("../../..", "foo/bar") == :error
-    assert Path.safe_relative_to("/usr/local/foo", "/") == :error
+  test "safe_relative/2" do
+    assert Path.safe_relative("/usr/local/foo", "/usr/local") == :error
+    assert Path.safe_relative("../../..", "foo/bar") == :error
+    assert Path.safe_relative("../../..", "foo/bar") == :error
+    assert Path.safe_relative("/usr/local/foo", "/") == :error
   end
 
   test "rootname/2" do
     assert Path.rootname("~/foo/bar.ex", ".ex") == "~/foo/bar"
     assert Path.rootname("~/foo/bar.exs", ".ex") == "~/foo/bar.exs"
     assert Path.rootname("~/foo/bar.old.ex", ".ex") == "~/foo/bar.old"
-    assert Path.rootname([?~, '/foo/bar', ".old.ex"], '.ex') == "~/foo/bar.old"
+    assert Path.rootname([?~, ~c"/foo/bar", ".old.ex"], ~c".ex") == "~/foo/bar.old"
   end
 
   test "extname/1" do
     assert Path.extname("foo.erl") == ".erl"
     assert Path.extname("~/foo/bar") == ""
 
-    assert Path.extname('foo.erl') == ".erl"
-    assert Path.extname('~/foo/bar') == ""
+    assert Path.extname(~c"foo.erl") == ".erl"
+    assert Path.extname(~c"~/foo/bar") == ""
   end
 
   test "dirname/1" do
@@ -268,7 +363,7 @@ defmodule PathTest do
     assert Path.dirname("~/foo/bar.ex") == "~/foo"
     assert Path.dirname("/foo/bar/baz/") == "/foo/bar/baz"
 
-    assert Path.dirname([?~, "/foo", '/bar.ex']) == "~/foo"
+    assert Path.dirname([?~, "/foo", ~c"/bar.ex"]) == "~/foo"
   end
 
   test "basename/1,2" do
@@ -280,7 +375,7 @@ defmodule PathTest do
     assert Path.basename("~/foo/bar.exs", ".ex") == "bar.exs"
     assert Path.basename("~/for/bar.old.ex", ".ex") == "bar.old"
 
-    assert Path.basename([?~, "/for/bar", '.old.ex'], ".ex") == "bar.old"
+    assert Path.basename([?~, "/for/bar", ~c".old.ex"], ".ex") == "bar.old"
   end
 
   test "join/1" do
@@ -289,11 +384,11 @@ defmodule PathTest do
     assert Path.join(["/", "foo", "bar"]) == "/foo/bar"
     assert Path.join(["/", "foo", "bar", "/"]) == "/foo/bar"
     assert Path.join(["~", "foo", "bar"]) == "~/foo/bar"
-    assert Path.join(['/foo/', "/bar/"]) == "/foo/bar"
+    assert Path.join([~c"/foo/", "/bar/"]) == "/foo/bar"
     assert Path.join(["/", ""]) == "/"
     assert Path.join(["/", "", "bar"]) == "/bar"
-    assert Path.join(['foo', [?b, "a", ?r]]) == "foo/bar"
-    assert Path.join([[?f, 'o', "o"]]) == "foo"
+    assert Path.join([~c"foo", [?b, "a", ?r]]) == "foo/bar"
+    assert Path.join([[?f, ~c"o", "o"]]) == "foo"
   end
 
   test "join/2" do
