@@ -26,11 +26,12 @@ defmodule ProtocolTest do
   @with_any_binary with_any_binary
 
   defprotocol Derivable do
-    def ok(a)
-  end
+    @undefined_impl_description "you should try harder"
+
+    @impl true
+    defmacro __deriving__(module, options) do
+      struct = Macro.struct!(module, __CALLER__)
 
-  defimpl Derivable, for: Any do
-    defmacro __deriving__(module, struct, options) do
       quote do
         defimpl Derivable, for: unquote(module) do
           def ok(arg) do
@@ -40,6 +41,10 @@ defmodule ProtocolTest do
       end
     end
 
+    def ok(a)
+  end
+
+  defimpl Derivable, for: Any do
     def ok(arg) do
       {:ok, arg}
     end
@@ -74,6 +79,14 @@ defmodule ProtocolTest do
     end
   end
 
+  defmodule ImplStructExplicitFor do
+    defstruct a: 0, b: 0
+
+    defimpl Sample, for: __MODULE__ do
+      def ok(_struct), do: true
+    end
+  end
+
   test "protocol implementations without any" do
     assert is_nil(Sample.impl_for(:foo))
     assert is_nil(Sample.impl_for(fn x -> x end))
@@ -91,23 +104,34 @@ defmodule ProtocolTest do
     assert is_nil(Sample.impl_for(make_ref()))
 
     assert Sample.impl_for(%ImplStruct{}) == Sample.ProtocolTest.ImplStruct
+    assert Sample.impl_for(%ImplStructExplicitFor{}) == Sample.ProtocolTest.ImplStructExplicitFor
     assert Sample.impl_for(%NoImplStruct{}) == nil
+    assert is_nil(Sample.impl_for(%{__struct__: nil}))
   end
 
   test "protocol implementation with Any and struct fallbacks" do
     assert WithAny.impl_for(%NoImplStruct{}) == WithAny.Any
-    # Derived
-    assert WithAny.impl_for(%ImplStruct{}) == WithAny.Any
+    assert WithAny.impl_for(%{__struct__: nil}) == WithAny.Any
     assert WithAny.impl_for(%{__struct__: "foo"}) == WithAny.Map
     assert WithAny.impl_for(%{}) == WithAny.Map
     assert WithAny.impl_for(self()) == WithAny.Any
+
+    # Derived
+    assert WithAny.impl_for(%ImplStruct{}) == ProtocolTest.WithAny.ProtocolTest.ImplStruct
   end
 
   test "protocol not implemented" do
-    message = "protocol ProtocolTest.Sample not implemented for :foo of type Atom"
+    message =
+      """
+      protocol ProtocolTest.Sample not implemented for type Atom
+
+      Got value:
+
+          :foo
+      """
 
     assert_raise Protocol.UndefinedError, message, fn ->
-      sample = Sample
+      sample = String.to_atom("Elixir.ProtocolTest.Sample")
       sample.ok(:foo)
     end
   end
@@ -117,7 +141,6 @@ defmodule ProtocolTest do
 
     write_beam(
       defprotocol SampleDocsProto do
-        @type t :: any
         @doc "Ok"
         @deprecated "Reason"
         @spec ok(t) :: boolean
@@ -125,13 +148,34 @@ defmodule ProtocolTest do
       end
     )
 
+    write_beam(
+      defimpl SampleDocsProto, for: List do
+        def ok(_), do: true
+      end
+    )
+
+    write_beam(
+      defimpl SampleDocsProto, for: Map do
+        @moduledoc "for map"
+
+        def ok(_), do: true
+      end
+    )
+
     {:docs_v1, _, _, _, _, _, docs} = Code.fetch_docs(SampleDocsProto)
 
+    assert {{:type, :t, 0}, _, [], %{"en" => type_doc}, _} = List.keyfind(docs, {:type, :t, 0}, 0)
+    assert type_doc =~ "All the types that implement this protocol"
+
     assert {{:function, :ok, 1}, _, ["ok(term)"], %{"en" => "Ok"}, _} =
              List.keyfind(docs, {:function, :ok, 1}, 0)
 
     deprecated = SampleDocsProto.__info__(:deprecated)
     assert [{{:ok, 1}, "Reason"}] = deprecated
+
+    {:docs_v1, _, _, _, :hidden, _, _} = Code.fetch_docs(SampleDocsProto.List)
+    {:docs_v1, _, _, _, moduledoc, _, _} = Code.fetch_docs(SampleDocsProto.Map)
+    assert moduledoc == %{"en" => "for map"}
   end
 
   @compile {:no_warn_undefined, WithAll}
@@ -143,13 +187,21 @@ defmodule ProtocolTest do
   end
 
   test "protocol defines callbacks" do
-    assert [{:type, 13, :fun, args}] = get_callbacks(@sample_binary, :ok, 1)
-    assert args == [{:type, 13, :product, [{:user_type, 13, :t, []}]}, {:type, 13, :boolean, []}]
+    assert [{:type, {13, 13}, :fun, args}] = get_callbacks(@sample_binary, :ok, 1)
+
+    assert args == [
+             {:type, {13, 13}, :product, [{:user_type, {13, 16}, :t, []}]},
+             {:type, {13, 22}, :boolean, []}
+           ]
 
     assert [{:type, 23, :fun, args}] = get_callbacks(@with_any_binary, :ok, 1)
     assert args == [{:type, 23, :product, [{:user_type, 23, :t, []}]}, {:type, 23, :term, []}]
   end
 
+  test "protocol defines t/0 type with documentation" do
+    assert {:type, {:t, {_, _, :any, []}, []}} = get_type(@sample_binary, :t, 0)
+  end
+
   test "protocol defines functions and attributes" do
     assert Sample.__protocol__(:module) == Sample
     assert Sample.__protocol__(:functions) == [ok: 1]
@@ -167,7 +219,6 @@ defmodule ProtocolTest do
   test "defimpl" do
     module = Module.concat(Sample, ImplStruct)
     assert module.__impl__(:for) == ImplStruct
-    assert module.__impl__(:target) == module
     assert module.__impl__(:protocol) == Sample
     assert module.__info__(:attributes)[:__impl__] == [protocol: Sample, for: ImplStruct]
   end
@@ -175,7 +226,6 @@ defmodule ProtocolTest do
   test "defimpl with implicit derive" do
     module = Module.concat(WithAny, ImplStruct)
     assert module.__impl__(:for) == ImplStruct
-    assert module.__impl__(:target) == WithAny.Any
     assert module.__impl__(:protocol) == WithAny
     assert module.__info__(:attributes)[:__impl__] == [protocol: WithAny, for: ImplStruct]
   end
@@ -183,7 +233,6 @@ defmodule ProtocolTest do
   test "defimpl with explicit derive" do
     module = Module.concat(Derivable, ImplStruct)
     assert module.__impl__(:for) == ImplStruct
-    assert module.__impl__(:target) == module
     assert module.__impl__(:protocol) == Derivable
     assert module.__info__(:attributes)[:__impl__] == [protocol: Derivable, for: ImplStruct]
   end
@@ -221,6 +270,15 @@ defmodule ProtocolTest do
     List.keyfind(callbacks, {name, arity}, 0) |> elem(1)
   end
 
+  defp get_type(beam, name, arity) do
+    {:ok, types} = Code.Typespec.fetch_types(beam)
+
+    assert {:value, value} =
+             :lists.search(&match?({_, {^name, _, args}} when length(args) == arity, &1), types)
+
+    value
+  end
+
   test "derives protocol implicitly" do
     struct = %ImplStruct{a: 1, b: 1}
     assert WithAny.ok(struct) == {:ok, struct}
@@ -233,10 +291,12 @@ defmodule ProtocolTest do
     struct = %ImplStruct{a: 1, b: 1}
     assert Derivable.ok(struct) == {:ok, struct, %ImplStruct{}, []}
 
-    assert_raise Protocol.UndefinedError, fn ->
-      struct = %NoImplStruct{a: 1, b: 1}
-      Derivable.ok(struct)
-    end
+    assert_raise Protocol.UndefinedError,
+                 ~r"protocol ProtocolTest.Derivable not implemented for type ProtocolTest.NoImplStruct \(a struct\), you should try harder",
+                 fn ->
+                   struct = %NoImplStruct{a: 1, b: 1}
+                   Derivable.ok(struct)
+                 end
   end
 
   test "derives protocol explicitly with options" do
@@ -262,6 +322,7 @@ defmodule ProtocolTest do
     assert Derivable.ok(struct) == {:ok, struct, struct(InlineStruct), :oops}
   end
 
+  @tag :requires_source
   test "derived implementation keeps local file/line info" do
     assert ProtocolTest.WithAny.ProtocolTest.ImplStruct.__info__(:compile)[:source] ==
              String.to_charlist(__ENV__.file)
