@@ -184,7 +184,7 @@ defmodule RecordTest do
     call: MapSet.new(),
     string: "abc",
     binary: <<1, 2, 3>>,
-    charlist: 'abc'
+    charlist: ~c"abc"
   )
 
   test "records with literal defaults and on-the-fly record" do
@@ -199,7 +199,7 @@ defmodule RecordTest do
              call: MapSet.new(),
              string: "abc",
              binary: <<1, 2, 3>>,
-             charlist: 'abc'
+             charlist: ~c"abc"
            ]
 
     assert defaults(defaults(), :struct) == ~D[2016-01-01]
@@ -212,7 +212,7 @@ defmodule RecordTest do
     assert defaults(defaults(), :call) == MapSet.new()
     assert defaults(defaults(), :string) == "abc"
     assert defaults(defaults(), :binary) == <<1, 2, 3>>
-    assert defaults(defaults(), :charlist) == 'abc'
+    assert defaults(defaults(), :charlist) == ~c"abc"
   end
 
   test "records with literal defaults and record in a variable" do
@@ -229,7 +229,7 @@ defmodule RecordTest do
              call: MapSet.new(),
              string: "abc",
              binary: <<1, 2, 3>>,
-             charlist: 'abc'
+             charlist: ~c"abc"
            ]
 
     assert defaults(defaults, :struct) == ~D[2016-01-01]
@@ -242,7 +242,7 @@ defmodule RecordTest do
     assert defaults(defaults, :call) == MapSet.new()
     assert defaults(defaults, :string) == "abc"
     assert defaults(defaults, :binary) == <<1, 2, 3>>
-    assert defaults(defaults, :charlist) == 'abc'
+    assert defaults(defaults, :charlist) == ~c"abc"
   end
 
   test "records with dynamic arguments" do
@@ -341,6 +341,20 @@ defmodule RecordTest do
     end
   end
 
+  test "docs metadata" do
+    import PathHelpers
+
+    write_beam(
+      defmodule Metadata do
+        Record.defrecord(:user, foo: 0, bar: "baz")
+      end
+    )
+
+    {:docs_v1, 348, :elixir, "text/markdown", _, %{}, docs} = Code.fetch_docs(RecordTest.Metadata)
+    {{:macro, :user, 1}, _meta, _sig, _docs, metadata} = List.keyfind(docs, {:macro, :user, 1}, 0)
+    assert %{record: {:user, [foo: 0, bar: "baz"]}} = metadata
+  end
+
   describe "warnings" do
     import ExUnit.CaptureIO
 
@@ -374,8 +388,8 @@ defmodule RecordTest do
     end
 
     defp purge(module) when is_atom(module) do
-      :code.delete(module)
       :code.purge(module)
+      :code.delete(module)
     end
   end
 end
