@@ -43,7 +43,7 @@ defmodule SystemTest do
   end
 
   test "argv/0" do
-    list = elixir('-e "IO.inspect System.argv()" -- -o opt arg1 arg2 --long-opt 10')
+    list = elixir(~c"-e \"IO.inspect System.argv()\" -- -o opt arg1 arg2 --long-opt 10")
     {args, _} = Code.eval_string(list, [])
     assert args == ["-o", "opt", "arg1", "arg2", "--long-opt", "10"]
   end
@@ -92,7 +92,7 @@ defmodule SystemTest do
 
   test "cmd/3 raises with non-binary arguments" do
     assert_raise ArgumentError, ~r"all arguments for System.cmd/3 must be binaries", fn ->
-      System.cmd("ls", ['/usr'])
+      System.cmd("ls", [~c"/usr"])
     end
   end
 
@@ -110,7 +110,8 @@ defmodule SystemTest do
         env: %{"foo" => "bar", "baz" => nil},
         arg0: "echo",
         stderr_to_stdout: true,
-        parallelism: true
+        parallelism: true,
+        use_stdio: true
       ]
 
       assert {["hello\r\n"], 0} = System.cmd("cmd", ~w[/c echo hello], opts)
@@ -121,13 +122,13 @@ defmodule SystemTest do
     test "cmd/3 with absolute and relative paths", config do
       echo = Path.join(config.tmp_dir, @echo)
       File.mkdir_p!(Path.dirname(echo))
-      File.cp!(System.find_executable("cmd"), echo)
+      File.ln_s!(System.find_executable("cmd"), echo)
 
       File.cd!(Path.dirname(echo), fn ->
         # There is a bug in OTP where find_executable is finding
         # entries on the current directory. If this is the case,
         # we should avoid the assertion below.
-        unless System.find_executable(@echo) do
+        if !System.find_executable(@echo) do
           assert :enoent = catch_error(System.cmd(@echo, ~w[/c echo hello]))
         end
 
@@ -146,7 +147,8 @@ defmodule SystemTest do
         cd: File.cwd!(),
         env: %{"foo" => "bar", "baz" => nil},
         stderr_to_stdout: true,
-        parallelism: true
+        parallelism: true,
+        use_stdio: true
       ]
 
       assert {["bar\r\n"], 0} = System.shell("echo %foo%", opts)
@@ -167,24 +169,50 @@ defmodule SystemTest do
         env: %{"foo" => "bar", "baz" => nil},
         arg0: "echo",
         stderr_to_stdout: true,
-        parallelism: true
+        parallelism: true,
+        use_stdio: true
       ]
 
       assert {["hello\n"], 0} = System.cmd("echo", ["hello"], opts)
     end
 
+    test "cmd/3 (can't use `use_stdio: false, stderr_to_stdout: true`)" do
+      opts = [
+        into: [],
+        cd: File.cwd!(),
+        env: %{"foo" => "bar", "baz" => nil},
+        arg0: "echo",
+        stderr_to_stdout: true,
+        use_stdio: false
+      ]
+
+      message = ~r"cannot use \"stderr_to_stdout: true\" and \"use_stdio: false\""
+
+      assert_raise ArgumentError, message, fn ->
+        System.cmd("echo", ["hello"], opts)
+      end
+    end
+
+    test "cmd/3 by line" do
+      assert {["hello", "world"], 0} =
+               System.cmd("echo", ["hello\nworld"], into: [], lines: 1024)
+
+      assert {["hello", "world"], 0} =
+               System.cmd("echo", ["-n", "hello\nworld"], into: [], lines: 3)
+    end
+
     @echo "echo-elixir-test"
     @tag :tmp_dir
     test "cmd/3 with absolute and relative paths", config do
       echo = Path.join(config.tmp_dir, @echo)
       File.mkdir_p!(Path.dirname(echo))
-      File.cp!(System.find_executable("echo"), echo)
+      File.ln_s!(System.find_executable("echo"), echo)
 
       File.cd!(Path.dirname(echo), fn ->
         # There is a bug in OTP where find_executable is finding
         # entries on the current directory. If this is the case,
         # we should avoid the assertion below.
-        unless System.find_executable(@echo) do
+        if !System.find_executable(@echo) do
           assert :enoent = catch_error(System.cmd(@echo, ["hello"]))
         end
 
@@ -201,9 +229,14 @@ defmodule SystemTest do
       assert {"1\n2\n", 0} = System.shell("x=1; echo $x; echo '2'")
     end
 
+    test "shell/1 with empty string" do
+      assert {"", 0} = System.shell("")
+      assert {"", 0} = System.shell("  ")
+    end
+
     @tag timeout: 1_000
     test "shell/1 returns when command awaits input" do
-      assert {"", 0} = System.shell("cat")
+      assert {"", 0} = System.shell("cat", close_stdin: true)
     end
 
     test "shell/1 with comment" do
@@ -215,7 +248,8 @@ defmodule SystemTest do
         into: [],
         cd: File.cwd!(),
         env: %{"foo" => "bar", "baz" => nil},
-        stderr_to_stdout: true
+        stderr_to_stdout: true,
+        use_stdio: true
       ]
 
       assert {["bar\n"], 0} = System.shell("echo $foo", opts)
