@@ -319,23 +319,21 @@ defmodule Task.SupervisorTest do
   end
 
   describe "await/1" do
-    if System.otp_release() >= "24" do
-      test "demonitors and unalias on timeout", config do
-        task =
-          Task.Supervisor.async(config[:supervisor], fn ->
-            assert_receive :go
-            :done
-          end)
+    test "demonitors and unalias on timeout", config do
+      task =
+        Task.Supervisor.async(config[:supervisor], fn ->
+          assert_receive :go
+          :done
+        end)
 
-        assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
-        new_ref = Process.monitor(task.pid)
-        old_ref = task.ref
+      assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
+      new_ref = Process.monitor(task.pid)
+      old_ref = task.ref
 
-        send(task.pid, :go)
-        assert_receive {:DOWN, ^new_ref, _, _, _}
-        refute_received {^old_ref, :done}
-        refute_received {:DOWN, ^old_ref, _, _, _}
-      end
+      send(task.pid, :go)
+      assert_receive {:DOWN, ^new_ref, _, _, _}
+      refute_received {^old_ref, :done}
+      refute_received {:DOWN, ^old_ref, _, _, _}
     end
 
     test "exits on task throw", config do
@@ -385,7 +383,7 @@ defmodule Task.SupervisorTest do
       Process.flag(:trap_exit, true)
 
       assert supervisor
-             |> Task.Supervisor.async_stream(4..1, &sleep/1, @opts)
+             |> Task.Supervisor.async_stream(4..1//-1, &sleep/1, @opts)
              |> Enum.to_list() == [ok: 4, ok: 3, ok: 2, ok: 1]
     end
 
@@ -462,6 +460,31 @@ defmodule Task.SupervisorTest do
       assert_receive 2
       assert_receive 3
     end
+
+    test "with timeout and :zip_input_on_exit set to true", %{supervisor: supervisor} do
+      opts = Keyword.merge(@opts, zip_input_on_exit: true, on_timeout: :kill_task, timeout: 50)
+
+      assert supervisor
+             |> Task.Supervisor.async_stream([1, 100], &sleep/1, opts)
+             |> Enum.to_list() == [ok: 1, exit: {100, :timeout}]
+    end
+
+    test "with outer halt on failure and :zip_input_on_exit", %{supervisor: supervisor} do
+      Process.flag(:trap_exit, true)
+      opts = Keyword.merge(@opts, zip_input_on_exit: true)
+
+      assert supervisor
+             |> Task.Supervisor.async_stream(1..8, &exit/1, opts)
+             |> Enum.take(4) == [exit: {1, 1}, exit: {2, 2}, exit: {3, 3}, exit: {4, 4}]
+    end
+
+    test "does not allow streaming with invalid :shutdown", %{supervisor: supervisor} do
+      message = ":shutdown must be either a positive integer or :brutal_kill"
+
+      assert_raise ArgumentError, message, fn ->
+        Task.Supervisor.async_stream(supervisor, [], fn _ -> :ok end, shutdown: :unknown)
+      end
+    end
   end
 
   describe "async_stream_nolink" do
@@ -489,7 +512,7 @@ defmodule Task.SupervisorTest do
 
     test "streams an enumerable with slowest first", %{supervisor: supervisor} do
       assert supervisor
-             |> Task.Supervisor.async_stream_nolink(4..1, &sleep/1, @opts)
+             |> Task.Supervisor.async_stream_nolink(4..1//-1, &sleep/1, @opts)
              |> Enum.to_list() == [ok: 4, ok: 3, ok: 2, ok: 1]
     end
 
