@@ -211,65 +211,65 @@ defmodule TaskTest do
     assert_receive :done
   end
 
-  if System.otp_release() >= "24" do
-    describe "ignore/1" do
-      test "discards on time replies" do
-        task = Task.async(fn -> :ok end)
-        wait_until_down(task)
-        assert Task.ignore(task) == {:ok, :ok}
-        assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
-      end
+  describe "ignore/1" do
+    test "discards on time replies" do
+      task = Task.async(fn -> :ok end)
+      wait_until_down(task)
+      assert Task.ignore(task) == {:ok, :ok}
+      assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
+    end
 
-      test "discards late replies" do
-        task = Task.async(fn -> assert_receive(:go) && :ok end)
-        assert Task.ignore(task) == nil
-        send(task.pid, :go)
-        wait_until_down(task)
-        assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
-      end
+    test "discards late replies" do
+      task = Task.async(fn -> assert_receive(:go) && :ok end)
+      assert Task.ignore(task) == nil
+      send(task.pid, :go)
+      wait_until_down(task)
+      assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
+    end
 
-      test "discards on-time failures" do
-        Process.flag(:trap_exit, true)
-        task = Task.async(fn -> exit(:oops) end)
-        wait_until_down(task)
-        assert Task.ignore(task) == {:exit, :oops}
-        assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
-      end
+    test "discards on-time failures" do
+      Process.flag(:trap_exit, true)
+      task = Task.async(fn -> exit(:oops) end)
+      wait_until_down(task)
+      assert Task.ignore(task) == {:exit, :oops}
+      assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
+    end
 
-      test "discards late failures" do
-        task = Task.async(fn -> assert_receive(:go) && exit(:oops) end)
-        assert Task.ignore(task) == nil
-        send(task.pid, :go)
-        wait_until_down(task)
-        assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
-      end
+    test "discards late failures" do
+      task = Task.async(fn -> assert_receive(:go) && exit(:oops) end)
+      assert Task.ignore(task) == nil
+      send(task.pid, :go)
+      wait_until_down(task)
+      assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
+    end
 
-      test "exits on :noconnection" do
-        ref = make_ref()
-        task = %Task{ref: ref, pid: self(), owner: self(), mfa: {__MODULE__, :test, 1}}
-        send(self(), {:DOWN, ref, self(), self(), :noconnection})
-        assert catch_exit(Task.ignore(task)) |> elem(0) == {:nodedown, :nonode@nohost}
-      end
+    test "exits on :noconnection" do
+      ref = make_ref()
+      task = %Task{ref: ref, pid: self(), owner: self(), mfa: {__MODULE__, :test, 1}}
+      send(self(), {:DOWN, ref, self(), self(), :noconnection})
+      assert catch_exit(Task.ignore(task)) |> elem(0) == {:nodedown, node()}
+    end
+
+    test "can ignore completed tasks" do
+      assert Task.ignore(Task.completed(:done)) == {:ok, :done}
     end
   end
 
   describe "await/2" do
-    if System.otp_release() >= "24" do
-      test "demonitors and unalias on timeout" do
-        task =
-          Task.async(fn ->
-            assert_receive :go
-            :done
-          end)
+    test "demonitors and unalias on timeout" do
+      task =
+        Task.async(fn ->
+          assert_receive :go
+          :done
+        end)
 
-        assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
-        send(task.pid, :go)
-        ref = task.ref
+      assert catch_exit(Task.await(task, 0)) == {:timeout, {Task, :await, [task, 0]}}
+      send(task.pid, :go)
+      ref = task.ref
 
-        wait_until_down(task)
-        refute_received {^ref, :done}
-        refute_received {:DOWN, ^ref, _, _, _}
-      end
+      wait_until_down(task)
+      refute_received {^ref, :done}
+      refute_received {:DOWN, ^ref, _, _, _}
     end
 
     test "exits on timeout" do
@@ -327,7 +327,7 @@ defmodule TaskTest do
       ref = make_ref()
       task = %Task{ref: ref, pid: self(), owner: self(), mfa: {__MODULE__, :test, 1}}
       send(self(), {:DOWN, ref, :process, self(), :noconnection})
-      assert catch_exit(Task.await(task)) |> elem(0) == {:nodedown, :nonode@nohost}
+      assert catch_exit(Task.await(task)) |> elem(0) == {:nodedown, node()}
     end
 
     test "exits on :noconnection from named monitor" do
@@ -464,7 +464,7 @@ defmodule TaskTest do
       ]
 
       send(self(), {:DOWN, ref, :process, self(), :noconnection})
-      assert catch_exit(Task.await_many(tasks)) |> elem(0) == {:nodedown, :nonode@nohost}
+      assert catch_exit(Task.await_many(tasks)) |> elem(0) == {:nodedown, node()}
     end
 
     test "exits immediately on :noconnection from named monitor" do
@@ -514,7 +514,7 @@ defmodule TaskTest do
       ref = make_ref()
       task = %Task{ref: ref, pid: self(), owner: self(), mfa: {__MODULE__, :test, 1}}
       send(self(), {:DOWN, ref, self(), self(), :noconnection})
-      assert catch_exit(Task.yield(task)) |> elem(0) == {:nodedown, :nonode@nohost}
+      assert catch_exit(Task.yield(task)) |> elem(0) == {:nodedown, node()}
     end
 
     test "raises when invoked from a non-owner process" do
@@ -537,11 +537,36 @@ defmodule TaskTest do
       refute_received {:DOWN, _, _, _, _}
     end
 
-    test "returns nil on timeout" do
+    test "returns nil on timeout by default" do
       task = %Task{ref: make_ref(), owner: self(), pid: nil, mfa: {__MODULE__, :test, 1}}
       assert Task.yield_many([task], 0) == [{task, nil}]
     end
 
+    test "shuts down on timeout when configured" do
+      Process.flag(:trap_exit, true)
+      task = Task.async(fn -> Process.sleep(:infinity) end)
+      assert Task.yield_many([task], timeout: 0, on_timeout: :kill_task) == [{task, nil}]
+      refute Process.alive?(task.pid)
+    end
+
+    test "ignores on timeout when configured" do
+      task =
+        Task.async(fn ->
+          receive do
+            :done -> :ok
+          end
+        end)
+
+      assert Task.yield_many([task], timeout: 0, on_timeout: :ignore) == [{task, nil}]
+      assert Process.alive?(task.pid)
+
+      ref = Process.monitor(task.pid)
+      send(task.pid, :done)
+      assert_receive {:DOWN, ^ref, _, _, _}
+
+      assert Task.yield(task, 0) == nil
+    end
+
     test "return exit on normal exit" do
       task = Task.async(fn -> exit(:normal) end)
       assert Task.yield_many([task]) == [{task, {:exit, :normal}}]
@@ -551,7 +576,7 @@ defmodule TaskTest do
       ref = make_ref()
       task = %Task{ref: ref, pid: self(), owner: self(), mfa: {__MODULE__, :test, 1}}
       send(self(), {:DOWN, ref, :process, self(), :noconnection})
-      assert catch_exit(Task.yield_many([task])) |> elem(0) == {:nodedown, :nonode@nohost}
+      assert catch_exit(Task.yield_many([task])) |> elem(0) == {:nodedown, node()}
     end
 
     test "raises when invoked from a non-owner process" do
@@ -576,6 +601,37 @@ defmodule TaskTest do
                [{task1, {:ok, :result}}, {task2, nil}, {task3, {:exit, :normal}}]
     end
 
+    test "returns results from multiple tasks with limit" do
+      task1 = %Task{ref: make_ref(), owner: self(), pid: nil, mfa: {__MODULE__, :test, 1}}
+      task2 = %Task{ref: make_ref(), owner: self(), pid: nil, mfa: {__MODULE__, :test, 1}}
+      task3 = %Task{ref: make_ref(), owner: self(), pid: nil, mfa: {__MODULE__, :test, 1}}
+
+      send(self(), {task1.ref, :result})
+      send(self(), {:DOWN, task3.ref, :process, self(), :normal})
+
+      assert Task.yield_many([task1, task2, task3], limit: 1, timeout: :infinity) ==
+               [{task1, {:ok, :result}}, {task2, nil}, {task3, nil}]
+
+      assert Task.yield_many([task2, task3], limit: 1, timeout: :infinity) ==
+               [{task2, nil}, {task3, {:exit, :normal}}]
+    end
+
+    test "returns results from multiple tasks with limit and on timeout" do
+      Process.flag(:trap_exit, true)
+      task1 = Task.async(fn -> Process.sleep(:infinity) end)
+      task2 = Task.async(fn -> :done end)
+
+      assert Task.yield_many([task1, task2], timeout: :infinity, on_timeout: :kill_task, limit: 1) ==
+               [{task1, nil}, {task2, {:ok, :done}}]
+
+      assert Process.alive?(task1.pid)
+
+      assert Task.yield_many([task1], timeout: 0, on_timeout: :kill_task, limit: 1) ==
+               [{task1, nil}]
+
+      refute Process.alive?(task1.pid)
+    end
+
     test "returns results on infinity timeout" do
       task1 = %Task{ref: make_ref(), owner: self(), pid: nil, mfa: {__MODULE__, :test, 1}}
       task2 = %Task{ref: make_ref(), owner: self(), pid: nil, mfa: {__MODULE__, :test, 1}}
@@ -652,10 +708,18 @@ defmodule TaskTest do
                {{:nodedown, node()}, {Task, :shutdown, [task, 5000]}}
     end
 
-    test "raises if task PID is nil" do
-      task = %Task{ref: make_ref(), owner: nil, pid: nil, mfa: {__MODULE__, :test, 1}}
-      message = "task #{inspect(task)} does not have an associated task process"
-      assert_raise ArgumentError, message, fn -> Task.shutdown(task) end
+    test "ignores if task PID is nil" do
+      ref = make_ref()
+      send(self(), {ref, :done})
+
+      assert Task.shutdown(%Task{ref: ref, owner: self(), pid: nil, mfa: {__MODULE__, :test, 1}}) ==
+               {:ok, :done}
+
+      ref = make_ref()
+      send(self(), {:DOWN, ref, :process, self(), :done})
+
+      assert Task.shutdown(%Task{ref: ref, owner: self(), pid: nil, mfa: {__MODULE__, :test, 1}}) ==
+               {:exit, :done}
     end
 
     test "raises when invoked from a non-owner process" do
@@ -778,13 +842,13 @@ defmodule TaskTest do
     test "streams an enumerable with ordered: false" do
       opts = [max_concurrency: 1, ordered: false]
 
-      assert 4..1
+      assert 4..1//-1
              |> Task.async_stream(&sleep(&1 * 100), opts)
              |> Enum.to_list() == [ok: 400, ok: 300, ok: 200, ok: 100]
 
       opts = [max_concurrency: 4, ordered: false]
 
-      assert 4..1
+      assert 4..1//-1
              |> Task.async_stream(&sleep(&1 * 100), opts)
              |> Enum.to_list() == [ok: 100, ok: 200, ok: 300, ok: 400]
     end
@@ -805,7 +869,19 @@ defmodule TaskTest do
 
     test "does not allow streaming with max_concurrency = 0" do
       assert_raise ArgumentError, ":max_concurrency must be an integer greater than zero", fn ->
-        Task.async_stream([1], fn _ -> :ok end, max_concurrency: 0) |> Enum.to_list()
+        Task.async_stream([1], fn _ -> :ok end, max_concurrency: 0)
+      end
+    end
+
+    test "does not allow streaming with invalid :on_timeout" do
+      assert_raise ArgumentError, ":on_timeout must be either :exit or :kill_task", fn ->
+        Task.async_stream([1], fn _ -> :ok end, on_timeout: :unknown)
+      end
+    end
+
+    test "does not allow streaming with invalid :timeout" do
+      assert_raise ArgumentError, ":timeout must be either a positive integer or :infinity", fn ->
+        Task.async_stream([1], fn _ -> :ok end, timeout: :unknown)
       end
     end
 
@@ -879,7 +955,7 @@ defmodule TaskTest do
       test "streams an enumerable with slowest first" do
         Process.flag(:trap_exit, true)
 
-        assert 4..1
+        assert 4..1//-1
                |> Task.async_stream(&sleep/1, @opts)
                |> Enum.to_list() == [ok: 4, ok: 3, ok: 2, ok: 1]
       end
@@ -922,7 +998,7 @@ defmodule TaskTest do
       end
 
       test "is zippable with slowest first" do
-        task = 4..1 |> Task.async_stream(&sleep/1, @opts) |> Stream.map(&elem(&1, 1))
+        task = 4..1//-1 |> Task.async_stream(&sleep/1, @opts) |> Stream.map(&elem(&1, 1))
         assert Enum.zip(task, task) == [{4, 4}, {3, 3}, {2, 2}, {1, 1}]
       end
 
@@ -943,7 +1019,7 @@ defmodule TaskTest do
       end
 
       test "with inner halt and slowest first" do
-        assert 8..1
+        assert 8..1//-1
                |> Stream.take(4)
                |> Task.async_stream(&sleep/1, @opts)
                |> Enum.to_list() == [ok: 8, ok: 7, ok: 6, ok: 5]
@@ -964,7 +1040,7 @@ defmodule TaskTest do
       end
 
       test "with outer halt and slowest first" do
-        assert 8..1
+        assert 8..1//-1
                |> Task.async_stream(&sleep/1, @opts)
                |> Enum.take(4) == [ok: 8, ok: 7, ok: 6, ok: 5]
       end
@@ -1023,4 +1099,68 @@ defmodule TaskTest do
       end
     end
   end
+
+  describe "default :logger reporter" do
+    setup do
+      translator = :logger.get_primary_config().filters[:logger_translator]
+      assert :ok = :logger.remove_primary_filter(:logger_translator)
+      on_exit(fn -> :logger.add_primary_filter(:logger_translator, translator) end)
+    end
+
+    test "logs a terminated task" do
+      parent = self()
+      {:ok, pid} = Task.start_link(__MODULE__, :task, [parent])
+
+      assert ExUnit.CaptureLog.capture_log(fn ->
+               ref = Process.monitor(pid)
+               send(pid, :go)
+               receive do: ({:DOWN, ^ref, _, _, _} -> :ok)
+             end) =~ ~r"""
+             \[error\] \*\* Task #PID<\d+\.\d+\.\d+> terminating
+             \*\* Started from #PID<\d+\.\d+\.\d+>
+             \*\* When function  == &TaskTest.task/1
+             \*\*      arguments == \[#PID<\d+\.\d+\.\d+>\]
+             \*\* Reason for termination ==\s
+             \*\* {%RuntimeError{message: "oops"},
+             """
+    end
+
+    test "logs a terminated task with a process label" do
+      fun = fn ->
+        Process.set_label({:any, "term"})
+        raise "oops"
+      end
+
+      parent = self()
+      {:ok, pid} = Task.start_link(__MODULE__, :task, [parent, fun])
+
+      assert ExUnit.CaptureLog.capture_log(fn ->
+               ref = Process.monitor(pid)
+               send(pid, :go)
+               receive do: ({:DOWN, ^ref, _, _, _} -> :ok)
+             end) =~ ~r"""
+             \[error\] \*\* Task #PID<\d+\.\d+\.\d+> terminating
+             \*\* Process Label == {:any, "term"}
+             \*\* Started from #PID<\d+\.\d+\.\d+>
+             \*\* When function  == &TaskTest.task/2
+             \*\*      arguments == \[#PID<\d+\.\d+\.\d+>,
+              #Function<.+>\]
+             \*\* Reason for termination ==\s
+             \*\* {%RuntimeError{message: "oops"},
+             """
+    end
+  end
+
+  def task(parent, fun \\ fn -> raise "oops" end) do
+    mon = Process.monitor(parent)
+    Process.unlink(parent)
+
+    receive do
+      :go ->
+        fun.()
+
+      {:DOWN, ^mon, _, _, _} ->
+        exit(:shutdown)
+    end
+  end
 end
