@@ -4,6 +4,7 @@ File.rm_rf!(path)
 File.mkdir_p!(path)
 Code.prepend_path(path)
 
+Application.put_env(:elixir, :ansi_enabled, true)
 Code.compiler_options(debug_info: true)
 
 defmodule PathHelpers do
@@ -23,20 +24,28 @@ defmodule PathHelpers do
     Path.join(tmp_path(), extra)
   end
 
-  def elixir(args) do
-    run_cmd(elixir_executable(), args)
+  def elixir(args, executable_extension \\ "") do
+    run_cmd(elixir_executable(executable_extension), args)
   end
 
-  def elixir_executable do
-    executable_path("elixir")
+  def elixir_executable(extension \\ "") do
+    executable_path("elixir", extension)
   end
 
-  def elixirc(args) do
-    run_cmd(elixirc_executable(), args)
+  def elixirc(args, executable_extension \\ "") do
+    run_cmd(elixirc_executable(executable_extension), args)
   end
 
-  def elixirc_executable do
-    executable_path("elixirc")
+  def elixirc_executable(extension \\ "") do
+    executable_path("elixirc", extension)
+  end
+
+  def iex(args, executable_extension \\ "") do
+    run_cmd(iex_executable(executable_extension), args)
+  end
+
+  def iex_executable(extension \\ "") do
+    executable_path("iex", extension)
   end
 
   def write_beam({:module, name, bin, _} = res) do
@@ -47,13 +56,13 @@ defmodule PathHelpers do
   end
 
   defp run_cmd(executable, args) do
-    '#{executable} #{IO.chardata_to_string(args)}#{redirect_std_err_on_win()}'
+    ~c"#{executable} #{IO.chardata_to_string(args)}#{redirect_std_err_on_win()}"
     |> :os.cmd()
-    |> :binary.list_to_bin()
+    |> :unicode.characters_to_binary()
   end
 
-  defp executable_path(name) do
-    Path.expand("../../../../bin/#{name}#{executable_extension()}", __DIR__)
+  defp executable_path(name, extension) do
+    Path.expand("../../../../bin/#{name}#{extension}", __DIR__)
   end
 
   if match?({:win32, _}, :os.type()) do
@@ -87,8 +96,29 @@ assert_timeout = String.to_integer(System.get_env("ELIXIR_ASSERT_TIMEOUT") || "5
 epmd_exclude = if match?({:win32, _}, :os.type()), do: [epmd: true], else: []
 os_exclude = if PathHelpers.windows?(), do: [unix: true], else: [windows: true]
 
+{line_exclude, line_include} =
+  if line = System.get_env("LINE"), do: {[:test], [line: line]}, else: {[], []}
+
+distributed_exclude =
+  if Code.ensure_loaded?(:peer) and Node.alive?() do
+    {:ok, _pid, node} = :peer.start(%{name: :secondary})
+    true = :erpc.call(node, :code, :set_path, [:code.get_path()])
+    {:ok, _} = :erpc.call(node, :application, :ensure_all_started, [:elixir])
+    []
+  else
+    [distributed: true]
+  end
+
+source_exclude =
+  if :deterministic in :compile.env_compiler_options() do
+    [:requires_source]
+  else
+    []
+  end
+
 ExUnit.start(
-  trace: "--trace" in System.argv(),
+  trace: !!System.get_env("TRACE"),
   assert_receive_timeout: assert_timeout,
-  exclude: epmd_exclude ++ os_exclude
+  exclude: epmd_exclude ++ os_exclude ++ line_exclude ++ distributed_exclude ++ source_exclude,
+  include: line_include
 )
