@@ -1,10 +1,23 @@
 defmodule ExUnit.AssertionError do
   @moduledoc """
   Raised to signal an assertion error.
+
+  This is used by macros such as `ExUnit.Assertions.assert/1`.
   """
 
   @no_value :ex_unit_no_meaningful_value
 
+  @typedoc since: "1.16.0"
+  @type t :: %__MODULE__{
+          left: any,
+          right: any,
+          message: any,
+          expr: any,
+          args: any,
+          doctest: any,
+          context: any
+        }
+
   defexception left: @no_value,
                right: @no_value,
                message: @no_value,
@@ -16,6 +29,7 @@ defmodule ExUnit.AssertionError do
   @doc """
   Indicates no meaningful value for a field.
   """
+  @spec no_value :: atom
   def no_value do
     @no_value
   end
@@ -31,6 +45,11 @@ defmodule ExUnit.MultiError do
   Raised to signal multiple errors happened in a test case.
   """
 
+  @typedoc since: "1.16.0"
+  @type t :: %__MODULE__{
+          errors: [{Exception.kind(), any, Exception.stacktrace()}]
+        }
+
   defexception errors: []
 
   @impl true
@@ -102,6 +121,12 @@ defmodule ExUnit.Assertions do
 
   Even though the match works, `assert` still expects a truth
   value. In such cases, simply use `==/2` or `match?/2`.
+
+  If you need more complex pattern matching using guards, you
+  need to use `match?/2`:
+
+      assert match?([%{id: id} | _] when is_integer(id), records)
+
   """
   defmacro assert({:=, meta, [left, right]} = assertion) do
     code = escape_quoted(:assert, meta, assertion)
@@ -204,7 +229,7 @@ defmodule ExUnit.Assertions do
 
   defmacro refute(assertion) do
     if translated = translate_assertion(:refute, assertion, __CALLER__) do
-      {:!, [], [translated]}
+      {:!, [generated: true], [translated]}
     else
       {args, value} = extract_args(assertion, __CALLER__)
 
@@ -260,19 +285,21 @@ defmodule ExUnit.Assertions do
     quote do
       left = unquote(left)
       right = unquote(right)
+      expr = unquote(expr)
+      message = unquote(message)
 
       if ExUnit.Assertions.__equal__?(left, right) do
         ExUnit.Assertions.assert(false,
           left: left,
-          expr: unquote(expr),
-          message: unquote(message <> ", both sides are exactly equal")
+          expr: expr,
+          message: message <> ", both sides are exactly equal"
         )
       else
         ExUnit.Assertions.assert(unquote(call),
           left: left,
           right: right,
-          expr: unquote(expr),
-          message: unquote(message),
+          expr: expr,
+          message: message,
           context: unquote(context)
         )
       end
@@ -336,6 +363,23 @@ defmodule ExUnit.Assertions do
   end
 
   @doc false
+  def __match__({:when, _, _} = left, right, _, _, _) do
+    suggestion =
+      quote do
+        assert match?(unquote(left), unquote(right))
+      end
+
+    raise ArgumentError, """
+    invalid pattern in assert/1:
+
+    #{Macro.to_string(left) |> Inspect.Error.pad(2)}
+
+    To assert with guards, use match?/2:
+
+    #{Macro.to_string(suggestion) |> Inspect.Error.pad(2)}
+    """
+  end
+
   def __match__(left, right, code, check, caller) do
     left = __expand_pattern__(left, caller)
     vars = collect_vars_from_pattern(left)
@@ -389,7 +433,7 @@ defmodule ExUnit.Assertions do
   end
 
   def assert(value, opts) when is_list(opts) do
-    unless value, do: raise(ExUnit.AssertionError, opts)
+    if !value, do: raise(ExUnit.AssertionError, opts)
     true
   end
 
@@ -397,8 +441,9 @@ defmodule ExUnit.Assertions do
   Asserts that a message matching `pattern` was or is going to be received
   within the `timeout` period, specified in milliseconds.
 
-  Unlike `assert_received`, it has a default `timeout`
-  of 100 milliseconds.
+  Unlike `assert_received`, it has a configurable timeout.
+  The default timeout duration is determined by the `assert_receive_timeout` option,
+  which can be set using `ExUnit.configure/1`. This option defaults to 100 milliseconds.
 
   The `pattern` argument must be a match pattern. Flunks with `failure_message`
   if a message matching `pattern` is not received.
@@ -487,8 +532,12 @@ defmodule ExUnit.Assertions do
         end
       end
 
-    failure_message =
-      failure_message ||
+    on_timeout =
+      if failure_message do
+        quote do
+          flunk(unquote(failure_message))
+        end
+      else
         quote do
           ExUnit.Assertions.__timeout__(
             unquote(Macro.escape(expanded_pattern)),
@@ -498,6 +547,7 @@ defmodule ExUnit.Assertions do
             timeout
           )
         end
+      end
 
     quote do
       timeout = ExUnit.Assertions.__timeout__(unquote(timeout), :assert_receive_timeout)
@@ -506,7 +556,7 @@ defmodule ExUnit.Assertions do
         receive do
           unquote(pattern) -> {received, unquote(mark_as_generated(vars))}
         after
-          timeout -> flunk(unquote(failure_message))
+          timeout -> unquote(on_timeout)
         end
 
       received
@@ -595,6 +645,12 @@ defmodule ExUnit.Assertions do
   defp collect_pins_from_pattern(expr, vars) do
     {_, pins} =
       Macro.prewalk(expr, %{}, fn
+        {:quote, _, [_]}, acc ->
+          {:ok, acc}
+
+        {:quote, _, [_, _]}, acc ->
+          {:ok, acc}
+
         {:^, _, [var]}, acc ->
           identifier = var_context(var)
 
@@ -631,15 +687,15 @@ defmodule ExUnit.Assertions do
       {:"::", _, [left, right]}, acc ->
         {[left], collect_vars_from_binary(right, acc)}
 
-      {skip, _, [_]}, acc when skip in [:^, :@] ->
+      {skip, _, [_]}, acc when skip in [:^, :@, :quote] ->
+        {:ok, acc}
+
+      {skip, _, [_, _]}, acc when skip in [:quote] ->
         {:ok, acc}
 
       {:_, _, context}, acc when is_atom(context) ->
         {:ok, acc}
 
-      {_, [{:expanded, expanded} | _], _}, acc ->
-        {[expanded], acc}
-
       {name, meta, context}, acc when is_atom(name) and is_atom(context) ->
         {:ok, [{name, meta, context} | acc]}
 
@@ -673,34 +729,46 @@ defmodule ExUnit.Assertions do
 
   @doc false
   def __expand_pattern__({:when, meta, [left, right]}, caller) do
-    left = prewalk_expand_pattern(left, Macro.Env.to_match(caller))
-    right = prewalk_expand_pattern(right, %{caller | context: :guard})
+    left = expand_pattern(left, Macro.Env.to_match(caller))
+    right = expand_pattern(right, %{caller | context: :guard})
     {:when, meta, [left, right]}
   end
 
   def __expand_pattern__(expr, caller) do
-    prewalk_expand_pattern(expr, Macro.Env.to_match(caller))
+    expand_pattern(expr, Macro.Env.to_match(caller))
   end
 
-  defp prewalk_expand_pattern(expr, caller) do
-    Macro.prewalk(expr, fn
-      {:__aliases__, _, _} = expr ->
-        Macro.expand(expr, caller)
+  defp expand_pattern({:quote, _, [_]} = expr, _caller), do: expr
+  defp expand_pattern({:quote, _, [_, _]} = expr, _caller), do: expr
+  defp expand_pattern({:__aliases__, _, _} = expr, caller), do: Macro.expand(expr, caller)
 
-      {:@, _, [{attribute, _, _}]} ->
-        caller.module |> Module.get_attribute(attribute) |> Macro.escape()
+  defp expand_pattern({:@, _, [{attribute, _, _}]}, caller) do
+    caller.module |> Module.get_attribute(attribute) |> Macro.escape()
+  end
 
-      {left, meta, right} = expr ->
-        case Macro.expand(expr, caller) do
-          ^expr -> expr
-          other -> {left, [expanded: other] ++ meta, right}
-        end
+  defp expand_pattern({left, meta, right} = expr, caller) do
+    case Macro.expand(expr, caller) do
+      ^expr ->
+        {expand_pattern(left, caller), meta, expand_pattern(right, caller)}
+
+      {left, meta, right} ->
+        {expand_pattern(left, caller), [original: expr] ++ meta, expand_pattern(right, caller)}
 
       other ->
-        Macro.expand(other, caller)
-    end)
+        other
+    end
   end
 
+  defp expand_pattern({left, right}, caller) do
+    {expand_pattern(left, caller), expand_pattern(right, caller)}
+  end
+
+  defp expand_pattern([_ | _] = list, caller) do
+    Enum.map(list, &expand_pattern(&1, caller))
+  end
+
+  defp expand_pattern(other, _caller), do: other
+
   defp suppress_warning({name, meta, [expr, [do: clauses]]}) do
     clauses =
       Enum.map(clauses, fn {:->, meta, args} ->
@@ -718,7 +786,7 @@ defmodule ExUnit.Assertions do
   ## Examples
 
       assert_raise ArithmeticError, "bad argument in arithmetic expression", fn ->
-        1 + "test"
+        1 / 0
       end
 
       assert_raise RuntimeError, ~r/^today's lucky number is 0\.\d+!$/, fn ->
@@ -740,7 +808,7 @@ defmodule ExUnit.Assertions do
         "expected:\n  #{inspect(message)}\n" <>
         "actual:\n" <> "  #{inspect(Exception.message(error))}"
 
-    unless match?, do: flunk(message)
+    if not match?, do: flunk(message)
 
     error
   end
@@ -755,6 +823,10 @@ defmodule ExUnit.Assertions do
         1 + "test"
       end
 
+      assert_raise RuntimeError, fn ->
+        raise "assertion will pass due to this raise"
+      end
+
   """
   def assert_raise(exception, function) when is_function(function) do
     try do
