@@ -8,14 +8,17 @@ defmodule ExUnit.CLIFormatter do
   ## Callbacks
 
   def init(opts) do
-    print_filters(Keyword.take(opts, [:include, :exclude]))
+    IO.puts("Running ExUnit with seed: #{opts[:seed]}, max_cases: #{opts[:max_cases]}")
+    print_filters(opts, :exclude)
+    print_filters(opts, :include)
+    IO.puts("")
 
     config = %{
-      seed: opts[:seed],
       trace: opts[:trace],
       colors: colors(opts),
       width: get_terminal_width(),
       slowest: opts[:slowest],
+      slowest_modules: opts[:slowest_modules],
       test_counter: %{},
       test_timings: [],
       failure_counter: 0,
@@ -45,6 +48,10 @@ defmodule ExUnit.CLIFormatter do
       IO.puts(format_slowest_tests(config, times_us.run))
     end
 
+    if config.slowest_modules > 0 do
+      IO.puts(format_slowest_modules(config, times_us.run))
+    end
+
     print_summary(config, false)
     {:noreply, config}
   end
@@ -65,7 +72,8 @@ defmodule ExUnit.CLIFormatter do
     {:noreply, update_test_timings(config, test)}
   end
 
-  def handle_cast({:test_finished, %ExUnit.Test{state: {:excluded, _}} = test}, config) do
+  def handle_cast({:test_finished, %ExUnit.Test{state: {:excluded, reason}} = test}, config)
+      when is_binary(reason) do
     if config.trace, do: IO.puts(trace_test_excluded(test))
 
     test_counter = update_test_counter(config.test_counter, test)
@@ -74,7 +82,8 @@ defmodule ExUnit.CLIFormatter do
     {:noreply, config}
   end
 
-  def handle_cast({:test_finished, %ExUnit.Test{state: {:skipped, _}} = test}, config) do
+  def handle_cast({:test_finished, %ExUnit.Test{state: {:skipped, reason}} = test}, config)
+      when is_binary(reason) do
     if config.trace do
       IO.puts(skipped(trace_test_skipped(test), config))
     else
@@ -87,7 +96,11 @@ defmodule ExUnit.CLIFormatter do
     {:noreply, config}
   end
 
-  def handle_cast({:test_finished, %ExUnit.Test{state: {:invalid, _}} = test}, config) do
+  def handle_cast(
+        {:test_finished,
+         %ExUnit.Test{state: {:invalid, %ExUnit.TestModule{state: {:failed, _}}}} = test},
+        config
+      ) do
     if config.trace do
       IO.puts(invalid(trace_test_result(test), config))
     else
@@ -124,9 +137,14 @@ defmodule ExUnit.CLIFormatter do
     {:noreply, update_test_timings(config, test)}
   end
 
-  def handle_cast({:module_started, %ExUnit.TestModule{name: name, file: file}}, config) do
-    if config.trace() do
+  def handle_cast({:module_started, %ExUnit.TestModule{} = module}, config) do
+    if config.trace do
+      %{name: name, file: file, parameters: parameters} = module
       IO.puts("\n#{inspect(name)} [#{Path.relative_to_cwd(file)}]")
+
+      if parameters != %{} do
+        IO.puts("Parameters: #{inspect(parameters)}")
+      end
     end
 
     {:noreply, config}
@@ -260,13 +278,49 @@ defmodule ExUnit.CLIFormatter do
     ]
   end
 
+  defp format_slowest_modules(%{slowest_modules: slowest, test_timings: timings}, run_us) do
+    slowest_tests =
+      timings
+      |> Enum.group_by(
+        fn %{module: module, tags: tags} ->
+          {module, tags.file}
+        end,
+        fn %{time: time} -> time end
+      )
+      |> Enum.into([], fn {{module, trace_test_file_line}, timings} ->
+        {module, trace_test_file_line, Enum.sum(timings)}
+      end)
+      |> Enum.sort_by(fn {_module, _, sum_timings} -> sum_timings end, :desc)
+      |> Enum.take(slowest)
+
+    slowest_us =
+      Enum.reduce(slowest_tests, 0, fn {_module, _, sum_timings}, acc ->
+        acc + sum_timings
+      end)
+
+    slowest_time = slowest_us |> normalize_us() |> format_us()
+    percentage = Float.round(slowest_us / run_us * 100, 1)
+
+    [
+      "\nTop #{slowest} slowest (#{slowest_time}s), #{percentage}% of total time:\n\n"
+      | Enum.map(slowest_tests, &format_slow_module/1)
+    ]
+  end
+
   defp format_slow_test(%ExUnit.Test{time: time, module: module} = test) do
     "#{trace_test_started(test)} (#{inspect(module)}) (#{format_us(time)}ms) " <>
       "[#{trace_test_file_line(test)}]\n"
   end
 
-  defp update_test_timings(%{slowest: slowest} = config, %ExUnit.Test{} = test) do
-    if slowest > 0 do
+  defp format_slow_module({module, test_file_path, timings}) do
+    "#{inspect(module)} (#{format_us(timings)}ms)\n [#{Path.relative_to_cwd(test_file_path)}]\n"
+  end
+
+  defp update_test_timings(
+         %{slowest: slowest, slowest_modules: slowest_modules} = config,
+         %ExUnit.Test{} = test
+       ) do
+    if slowest > 0 or slowest_modules > 0 do
       # Do not store logs, as they are not used for timings and consume memory.
       update_in(config.test_timings, &[%{test | logs: ""} | &1])
     else
@@ -309,22 +363,16 @@ defmodule ExUnit.CLIFormatter do
       true ->
         IO.puts(success(message, config))
     end
-
-    IO.puts("\nRandomized with seed #{config.seed}")
   end
 
   defp if_true(value, false, _fun), do: value
   defp if_true(value, true, fun), do: fun.(value)
 
-  defp print_filters(include: [], exclude: []) do
-    :ok
-  end
-
-  defp print_filters(include: include, exclude: exclude) do
-    if exclude != [], do: IO.puts(format_filters(exclude, :exclude))
-    if include != [], do: IO.puts(format_filters(include, :include))
-    IO.puts("")
-    :ok
+  defp print_filters(opts, key) do
+    case opts[key] do
+      [] -> :ok
+      filters -> IO.puts(format_filters(filters, key))
+    end
   end
 
   defp print_failure(formatted, config) do
@@ -337,7 +385,9 @@ defmodule ExUnit.CLIFormatter do
   end
 
   defp format_test_type_counts(%{test_counter: test_counter} = _config) do
-    Enum.map(test_counter, fn {test_type, count} ->
+    test_counter
+    |> Enum.sort()
+    |> Enum.map(fn {test_type, count} ->
       type_pluralized = pluralize(count, test_type, ExUnit.plural_rule(test_type |> to_string()))
       "#{count} #{type_pluralized}, "
     end)
@@ -351,8 +401,8 @@ defmodule ExUnit.CLIFormatter do
 
   # Color styles
 
-  defp colorize(escape, string, %{colors: colors}) do
-    if colors[:enabled] do
+  defp colorize(key, string, %{colors: colors}) do
+    if escape = colors[:enabled] && colors[key] do
       [escape, string, :reset]
       |> IO.ANSI.format_fragment(true)
       |> IO.iodata_to_binary()
@@ -363,55 +413,54 @@ defmodule ExUnit.CLIFormatter do
 
   defp colorize_doc(escape, doc, %{colors: colors}) do
     if colors[:enabled] do
-      Inspect.Algebra.color(doc, escape, %Inspect.Opts{syntax_colors: colors})
+      Inspect.Algebra.color_doc(doc, escape, %Inspect.Opts{syntax_colors: colors})
     else
       doc
     end
   end
 
   defp success(msg, config) do
-    colorize(:green, msg, config)
+    colorize(:success, msg, config)
   end
 
   defp invalid(msg, config) do
-    colorize(:yellow, msg, config)
+    colorize(:invalid, msg, config)
   end
 
   defp skipped(msg, config) do
-    colorize(:yellow, msg, config)
+    colorize(:skipped, msg, config)
   end
 
   defp failure(msg, config) do
-    colorize(:red, msg, config)
+    colorize(:failure, msg, config)
   end
 
-  defp formatter(:diff_enabled?, _, %{colors: colors}), do: colors[:enabled]
+  # Diff formatting
 
-  defp formatter(:error_info, msg, config), do: colorize(:red, msg, config)
+  defp formatter(:diff_enabled?, _, %{colors: colors}),
+    do: colors[:enabled]
 
-  defp formatter(:extra_info, msg, config), do: colorize(:cyan, msg, config)
-
-  defp formatter(:location_info, msg, config), do: colorize([:bright, :black], msg, config)
-
-  defp formatter(:diff_delete, doc, config), do: colorize_doc(:diff_delete, doc, config)
+  defp formatter(:diff_delete, doc, config),
+    do: colorize_doc(:diff_delete, doc, config)
 
   defp formatter(:diff_delete_whitespace, doc, config),
     do: colorize_doc(:diff_delete_whitespace, doc, config)
 
-  defp formatter(:diff_insert, doc, config), do: colorize_doc(:diff_insert, doc, config)
+  defp formatter(:diff_insert, doc, config),
+    do: colorize_doc(:diff_insert, doc, config)
 
   defp formatter(:diff_insert_whitespace, doc, config),
     do: colorize_doc(:diff_insert_whitespace, doc, config)
 
   defp formatter(:blame_diff, msg, %{colors: colors} = config) do
     if colors[:enabled] do
-      colorize(:red, msg, config)
+      colorize(:diff_delete, msg, config)
     else
       "-" <> msg <> "-"
     end
   end
 
-  defp formatter(_, msg, _config), do: msg
+  defp formatter(key, msg, config), do: colorize(key, msg, config)
 
   defp pluralize(1, singular, _plural), do: singular
   defp pluralize(_, _singular, plural), do: plural
@@ -427,7 +476,16 @@ defmodule ExUnit.CLIFormatter do
     diff_delete: :red,
     diff_delete_whitespace: IO.ANSI.color_background(2, 0, 0),
     diff_insert: :green,
-    diff_insert_whitespace: IO.ANSI.color_background(0, 2, 0)
+    diff_insert_whitespace: IO.ANSI.color_background(0, 2, 0),
+
+    # CLI formatter
+    success: :green,
+    invalid: :yellow,
+    skipped: :yellow,
+    failure: :red,
+    error_info: :red,
+    extra_info: :cyan,
+    location_info: [:bright, :black]
   ]
 
   defp colors(opts) do
