@@ -1,27 +1,20 @@
 defmodule ExUnit.OnExitHandler do
   @moduledoc false
 
-  @name __MODULE__
-  @ets_opts [:public, :named_table, read_concurrency: true, write_concurrency: true]
+  # ETS table name stored in ExUnit.Server
+  @name ExUnit.Server
 
   # ETS column numbers
   @supervisor 2
   @on_exit 3
 
-  use Agent
-
-  @spec start_link(keyword()) :: {:ok, pid}
-  def start_link(_opts) do
-    Agent.start_link(fn -> :ets.new(@name, @ets_opts) end, name: @name)
-  end
-
   @spec register(pid) :: :ok
   def register(pid) when is_pid(pid) do
     :ets.insert(@name, {pid, nil, []})
     :ok
   end
 
-  @spec add(pid, term, (() -> term)) :: :ok | :error
+  @spec add(pid, term, (-> term)) :: :ok | :error
   def add(pid, name_or_ref, callback) when is_pid(pid) and is_function(callback, 0) do
     try do
       :ets.lookup_element(@name, pid, @on_exit)
