@@ -27,12 +27,15 @@ defmodule ExUnit.MixProject do
         formatters: [ExUnit.CLIFormatter],
         include: [],
         max_failures: :infinity,
+        rand_algorithm: :exsss,
         refute_receive_timeout: 100,
         slowest: 0,
+        slowest_modules: 0,
         stacktrace_depth: 20,
         timeout: 60000,
         trace: false,
-        after_suite: []
+        after_suite: [],
+        repeat_until_failure: 0
       ]
     ]
   end
