@@ -34,9 +34,6 @@ defmodule ExUnit.DocTestTest.GoodModule do
   """
   def test_sigil, do: :ok
 
-  @doc "    iex>1 + 2\n    3"
-  def no_trailing_new_line, do: :ok
-
   @doc """
   iex> a = 1
   iex> b = a + 2
@@ -67,6 +64,13 @@ defmodule ExUnit.DocTestTest.GoodModule do
   bar
   """
   def multiline_exception_test, do: :ok
+
+  @doc """
+  iex> num = 1
+  iex> ExUnit.DocTestTest.GoodModule.variable_in_expectation(num)
+  num + 1
+  """
+  def variable_in_expectation(num), do: num + 1
 end
 |> ExUnit.BeamHelpers.write_beam()
 
@@ -136,7 +140,13 @@ defmodule ExUnit.DocTestTest.NoImport do
 end
 |> ExUnit.BeamHelpers.write_beam()
 
-defmodule ExUnit.DocTestTest.Invalid do
+defmodule ExUnit.DocTestTest.NoTrailing do
+  @doc "    iex>1 + 2\n    3"
+  def no_trailing_new_line, do: :ok
+end
+|> ExUnit.BeamHelpers.write_beam()
+
+defmodule ExUnit.DocTestTest.Failure do
   def starting_line, do: __ENV__.line + 2
 
   @moduledoc """
@@ -167,6 +177,18 @@ defmodule ExUnit.DocTestTest.Invalid do
 
   """
 
+  @doc """
+      # This will fail to inspect
+      iex> ExUnit.DocTestTest.Haiku.new(:this, :is, {:not, :a, :haiku})
+      #Haiku<:this_wont_be_asserted>
+  """
+  def raising_inspect, do: :ok
+end
+|> ExUnit.BeamHelpers.write_beam()
+
+defmodule ExUnit.DocTestTest.Invalid do
+  def starting_line, do: __ENV__.line + 2
+
   @doc """
       iex> 1 + * 1
       1
@@ -179,14 +201,6 @@ defmodule ExUnit.DocTestTest.Invalid do
   """
   defmacro b(), do: :ok
 
-  @doc """
-    ```
-    iex> 1 + 2
-    3
-  ```
-  """
-  def indented_not_enough, do: :ok
-
   @doc ~S'''
   ```
   iex> 1 + 2
@@ -222,11 +236,25 @@ defmodule ExUnit.DocTestTest.Invalid do
   @type t :: any()
 
   @doc """
-      # This will fail to inspect
-      iex> ExUnit.DocTestTest.Haiku.new(:this, :is, {:not, :a, :haiku})
-      #Haiku<:this_wont_be_asserted>
+      iex> :foo
+      iex> :bar
+      1 + * 1
   """
-  def raising_inspect, do: :ok
+  def result, do: :ok
+
+  @doc """
+      iex> 123 +
+      :mixed
+  """
+  def mixed, do: :ok
+
+  @doc """
+      iex> 1 + 2
+      3
+      iex> 123 +
+      :mixed
+  """
+  def invalid_second, do: :ok
 end
 |> ExUnit.BeamHelpers.write_beam()
 
@@ -276,6 +304,28 @@ defmodule ExUnit.DocTestTest.IndentationNotEnough do
 end
 |> ExUnit.BeamHelpers.write_beam()
 
+defmodule ExUnit.DocTestTest.FencedNotEnough do
+  @doc ~S'''
+    ```
+    iex> 1 + 2
+    3
+  ```
+  '''
+  def test_fun, do: :ok
+end
+|> ExUnit.BeamHelpers.write_beam()
+
+defmodule ExUnit.DocTestTest.IndentationNotEnoughMultilineCode do
+  @doc ~S'''
+      iex> "hello
+      ...> world"
+      "hello
+  world"
+  '''
+  def test_fun, do: :ok
+end
+|> ExUnit.BeamHelpers.write_beam()
+
 defmodule ExUnit.DocTestTest.FencedHeredocs do
   @doc ~S'''
   Receives a test and formats its failure.
@@ -288,8 +338,7 @@ defmodule ExUnit.DocTestTest.FencedHeredocs do
   ```
 
       ```
-      iex> 1 + 2
-      3
+      iex> 3 = 1 + 2
       ```
 
   ```
@@ -438,7 +487,9 @@ defmodule ExUnit.DocTestTest.PatternMatching do
       iex> %{b: _, d: :e} = %{a: :c, d: :e}
 
       iex> %{year: 2001, day: 1} = ~D[2000-01-01]
+  """
 
+  @doc """
       iex> adder = fn int -> int + 1 end
       iex> num =
       ...>   adder.(0)
@@ -450,7 +501,7 @@ defmodule ExUnit.DocTestTest.PatternMatching do
       # false assertions do not accidentally raise
       iex> false = (List.flatten([]) != [])
   """
-  def doctest(), do: :ok
+  def passing(), do: :ok
 end
 |> ExUnit.BeamHelpers.write_beam()
 
@@ -462,7 +513,11 @@ defmodule ExUnit.DocTestTest do
   # doctest ExUnit.DocTest
 
   doctest ExUnit.DocTestTest.GoodModule, import: true
-  doctest ExUnit.DocTestTest.SomewhatGoodModuleWithOnly, only: [one: 0], import: true
+  doctest ExUnit.DocTestTest.NoTrailing
+
+  doctest ExUnit.DocTestTest.SomewhatGoodModuleWithOnly,
+    only: [:moduledoc, one: 0],
+    import: true
 
   doctest ExUnit.DocTestTest.SomewhatGoodModuleWithExcept,
     except: [:moduledoc, two: 0],
@@ -511,34 +566,39 @@ defmodule ExUnit.DocTestTest do
   test "doctest failures" do
     # When adding or removing lines above this line, the tests below will
     # fail because we are explicitly asserting some doctest lines from
-    # ActuallyCompiled in the format of "test/ex_unit/doc_test_test.exs:<LINE>".
-    defmodule ActuallyCompiled do
+    # FailureCompiled in the format of "test/ex_unit/doc_test_test.exs:<LINE>".
+    defmodule FailureCompiled do
       use ExUnit.Case
-      doctest ExUnit.DocTestTest.Invalid
+      doctest ExUnit.DocTestTest.Failure
     end
 
     doctest_line = __ENV__.line - 3
-    starting_line = ExUnit.DocTestTest.Invalid.starting_line() + 2
+    starting_line = ExUnit.DocTestTest.Failure.starting_line() + 2
 
     ExUnit.configure(seed: 0, colors: [enabled: false])
     output = capture_io(fn -> ExUnit.run() end)
+    output = String.replace(output, [IO.ANSI.red(), IO.ANSI.reset()], "")
+    location = location(ExUnit.DocTestTest.Failure)
 
     assert output =~ """
-             1) doctest module ExUnit.DocTestTest.Invalid (1) (ExUnit.DocTestTest.ActuallyCompiled)
+             1) doctest module ExUnit.DocTestTest.Failure (1) (ExUnit.DocTestTest.FailureCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line}:6: syntax error before: '*'
-                 #{line_placeholder(starting_line)} |
-                 #{starting_line} |  1 + * 1
-                 #{line_placeholder(starting_line)} |      ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on #{location}:#{starting_line}:6:
+                     error: syntax error before: '*'
+                     │
+                 #{starting_line} │  1 + * 1
+                     │      ^
+                     │
+                     └─ #{location}:#{starting_line}:6
                 doctest:
                   iex> 1 + * 1
                   1
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(starting_line)}ExUnit.DocTestTest.Failure (module)
            """
 
     assert output =~ """
-             2) doctest module ExUnit.DocTestTest.Invalid (2) (ExUnit.DocTestTest.ActuallyCompiled)
+             2) doctest module ExUnit.DocTestTest.Failure (2) (ExUnit.DocTestTest.FailureCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
                 Doctest failed
                 doctest:
@@ -548,11 +608,11 @@ defmodule ExUnit.DocTestTest do
                 left:  2
                 right: 3
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 3}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(starting_line + 3)}ExUnit.DocTestTest.Failure (module)
            """
 
     assert output =~ """
-             3) doctest module ExUnit.DocTestTest.Invalid (3) (ExUnit.DocTestTest.ActuallyCompiled)
+             3) doctest module ExUnit.DocTestTest.Failure (3) (ExUnit.DocTestTest.FailureCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
                 Doctest failed
                 doctest:
@@ -565,45 +625,45 @@ defmodule ExUnit.DocTestTest do
                 left:  "This is a slightly shorter text string."
                 right: "This is a much shorter text string."
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 6}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(starting_line + 6)}ExUnit.DocTestTest.Failure (module)
            """
 
     assert output =~ """
-             4) doctest module ExUnit.DocTestTest.Invalid (4) (ExUnit.DocTestTest.ActuallyCompiled)
+             4) doctest module ExUnit.DocTestTest.Failure (4) (ExUnit.DocTestTest.FailureCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
                 Doctest failed
                 doctest:
                   iex> :oops
-                  "#Inspect<[]>"
+                  #Inspect<[]>
                 code:  inspect(:oops) === "#Inspect<[]>"
                 left:  ":oops"
                 right: "#Inspect<[]>"
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 12}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(starting_line + 12)}ExUnit.DocTestTest.Failure (module)
            """
 
     assert output =~ """
-             5) doctest module ExUnit.DocTestTest.Invalid (5) (ExUnit.DocTestTest.ActuallyCompiled)
+             5) doctest module ExUnit.DocTestTest.Failure (5) (ExUnit.DocTestTest.FailureCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                ** (UndefinedFunctionError) function Hello.world/0 is undefined (module Hello is not available)
+                ** (UndefinedFunctionError) function Hello.world/0 is undefined (module Hello is not available). Make sure the module name is correct and has been specified in full (or that an alias has been defined)
                 stacktrace:
                   Hello.world()
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 15}: (test)
+                  (for doctest at) #{location}:#{starting_line + 15}: (test)
            """
 
     assert output =~ """
-             6) doctest module ExUnit.DocTestTest.Invalid (6) (ExUnit.DocTestTest.ActuallyCompiled)
+             6) doctest module ExUnit.DocTestTest.Failure (6) (ExUnit.DocTestTest.FailureCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
                 Doctest failed: expected exception WhatIsThis but got RuntimeError with message "oops"
                 doctest:
                   iex> raise "oops"
-                  ** (WhatIsThis) "oops"
+                  ** (WhatIsThis) oops
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 18}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(starting_line + 18)}ExUnit.DocTestTest.Failure (module)
            """
 
     assert output =~ """
-             7) doctest module ExUnit.DocTestTest.Invalid (7) (ExUnit.DocTestTest.ActuallyCompiled)
+             7) doctest module ExUnit.DocTestTest.Failure (7) (ExUnit.DocTestTest.FailureCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
                 Doctest failed: wrong message for RuntimeError
                 expected:
@@ -612,126 +672,224 @@ defmodule ExUnit.DocTestTest do
                   "oops"
                 doctest:
                   iex> raise "oops"
-                  ** (RuntimeError) "hello"
+                  ** (RuntimeError) hello
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 21}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(starting_line + 21)}ExUnit.DocTestTest.Failure (module)
            """
 
+    assert output =~ "8) doctest ExUnit.DocTestTest.Failure.raising_inspect/0"
+    assert output =~ "iex> ExUnit.DocTestTest.Haiku.new(:this, :is, {:not, :a, :haiku})"
+
+    assert output =~
+             "#{stack(starting_line + 28)}ExUnit.DocTestTest.Failure (module)"
+
+    assert output =~ "8 doctests, 8 failures"
+  end
+
+  test "doctest invalid" do
+    # When adding or removing lines above this line, the tests below will
+    # fail because we are explicitly asserting some doctest lines from
+    # InvalidCompiled in the format of "test/ex_unit/doc_test_test.exs:<LINE>".
+    defmodule InvalidCompiled do
+      use ExUnit.Case
+      doctest ExUnit.DocTestTest.Invalid
+    end
+
+    doctest_line = __ENV__.line - 3
+    starting_line = ExUnit.DocTestTest.Invalid.starting_line() + 1
+    location = location(ExUnit.DocTestTest.Invalid)
+
+    ExUnit.configure(seed: 0, colors: [enabled: false])
+    output = capture_io(fn -> ExUnit.run() end)
+    output = String.replace(output, [IO.ANSI.red(), IO.ANSI.reset()], "")
+
+    line = starting_line
+
     assert output =~ """
-             8) doctest ExUnit.DocTestTest.Invalid.a/0 (8) (ExUnit.DocTestTest.ActuallyCompiled)
+             1) doctest ExUnit.DocTestTest.Invalid.a/0 (1) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 27}:6: syntax error before: '*'
-                 #{line_placeholder(starting_line + 27)} |
-                 #{starting_line + 27} |  1 + * 1
-                 #{line_placeholder(starting_line + 27)} |      ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on #{location}:#{line}:6:
+                     error: syntax error before: '*'
+                     │
+                 #{line} │  1 + * 1
+                     │      ^
+                     │
+                     └─ #{location}:#{line}:6
                 doctest:
                   iex> 1 + * 1
                   1
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 27}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(line)}ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 6
+
     assert output =~ """
-             9) doctest ExUnit.DocTestTest.Invalid.dedented_past_fence/0 (9) (ExUnit.DocTestTest.ActuallyCompiled)
+             2) doctest ExUnit.DocTestTest.Invalid.b/0 (2) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 57}:5: unexpected token: "`" (column 5, code point U+0060)
-                 #{line_placeholder(starting_line + 57)} |
-                 #{starting_line + 57} |     ```
-                 #{line_placeholder(starting_line + 57)} |     ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on #{location}:#{line}:6:
+                     error: syntax error before: '*'
+                     │
+                 #{line} │  1 + * 1
+                     │      ^
+                     │
+                     └─ #{location}:#{line}:6
                 doctest:
-                  iex> 1 + 2
-                  3
-                      ```
+                  iex> 1 + * 1
+                  1
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 56}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(line)}ExUnit.DocTestTest.Invalid (module)
            """
 
-    assert output =~ """
-            10) doctest ExUnit.DocTestTest.Invalid.indented_not_enough/0 (10) (ExUnit.DocTestTest.ActuallyCompiled)
-                test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 41}:1: unexpected token: "`" (column 1, code point U+0060)
-                 #{line_placeholder(starting_line + 41)} |
-                 #{starting_line + 41} | `
-                 #{line_placeholder(starting_line + 41)} | ^
-                doctest:
-                  iex> 1 + 2
-                  3
-                  `
-                stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 40}: ExUnit.DocTestTest.Invalid (module)
-           """
+    line = starting_line + 15
 
     assert output =~ """
-            11) doctest ExUnit.DocTestTest.Invalid.indented_too_much/0 (11) (ExUnit.DocTestTest.ActuallyCompiled)
+             3) doctest ExUnit.DocTestTest.Invalid.indented_too_much/0 (3) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 49}:3: unexpected token: "`" (column 3, code point U+0060)
-                 #{line_placeholder(starting_line + 49)} |
-                 #{starting_line + 49} |   ```
-                 #{line_placeholder(starting_line + 49)} |   ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on #{location}:#{line}:3:
+                     error: unexpected token: "`" (column 3, code point U+0060)
+                     │
+                 #{line} │   ```
+                     │   ^
+                     │
+                     └─ #{location}:#{line}:3
                 doctest:
                   iex> 1 + 2
                   3
                     ```
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 48}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(line - 1)}ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 23
+
     assert output =~ """
-            12) doctest ExUnit.DocTestTest.Invalid.invalid_utf8/0 (12) (ExUnit.DocTestTest.ActuallyCompiled)
+             4) doctest ExUnit.DocTestTest.Invalid.dedented_past_fence/0 (4) (ExUnit.DocTestTest.InvalidCompiled)
+                test/ex_unit/doc_test_test.exs:#{doctest_line}
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on #{location}:#{line}:5:
+                     error: unexpected token: "`" (column 5, code point U+0060)
+                     │
+                 #{line} │     ```
+                     │     ^
+                     │
+                     └─ #{location}:#{line}:5
+                doctest:
+                  iex> 1 + 2
+                  3
+                      ```
+                stacktrace:
+                  #{stack(line - 1)}ExUnit.DocTestTest.Invalid (module)
+           """
+
+    line = starting_line + 28
+
+    assert output =~ """
+             5) doctest ExUnit.DocTestTest.Invalid.invalid_utf8/0 (5) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
                 Doctest did not compile, got: (UnicodeConversionError) invalid encoding starting at <<255, 34, 41>>
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 63}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(line)}ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 35
+
     assert output =~ """
-            13) doctest ExUnit.DocTestTest.Invalid.misplaced_opaque_type/0 (13) (ExUnit.DocTestTest.ActuallyCompiled)
+           6) doctest ExUnit.DocTestTest.Invalid.misplaced_opaque_type/0 (6) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (TokenMissingError) test/ex_unit/doc_test_test.exs:#{starting_line + 69}:7: missing terminator: } (for "{" starting at line #{starting_line + 69})
-                 #{line_placeholder(starting_line + 69)} |
-                 #{starting_line + 69} | {:ok, #Inspect<[]>}
-                 #{line_placeholder(starting_line + 69)} |       ^
-                If you are planning to assert on the result of an iex> expression which contains a value inspected as #Name<...>, please make sure the inspected value is placed at the beginning of the expression; otherwise Elixir will treat it as a comment due to the leading sign #.
+                Doctest did not compile, got: (TokenMissingError) token missing on #{location}:#{line}:20:
+                     error: missing terminator: }
+                     │
+                 228 │ {:ok, #Inspect<[]>}
+                     │ │                  └ missing closing delimiter (expected "}")
+                     │ └ unclosed delimiter
+                     │
+                     └─ #{location}:#{line}:20
+                If you are planning to assert on the result of an iex> expression which contains a value inspected as #Name<...>, please make sure the inspected value is placed at the beginning of the expression, otherwise Elixir will treat it as a comment due to the leading sign #.
                 doctest:
                   iex> {:ok, :oops}
                   {:ok, #Inspect<[]>}
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 69}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(line)}ExUnit.DocTestTest.Invalid (module)
            """
 
-    assert output =~ "14) doctest ExUnit.DocTestTest.Invalid.raising_inspect/0"
-    assert output =~ "iex> ExUnit.DocTestTest.Haiku.new(:this, :is, {:not, :a, :haiku})"
-
-    assert output =~
-             "test/ex_unit/doc_test_test.exs:#{starting_line + 82}: ExUnit.DocTestTest.Invalid (module)"
+    line = starting_line + 40
 
     assert output =~ """
-            15) doctest ExUnit.DocTestTest.Invalid.b/0 (15) (ExUnit.DocTestTest.ActuallyCompiled)
+             7) doctest ExUnit.DocTestTest.Invalid.t/0 (7) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 33}:6: syntax error before: '*'
-                 #{line_placeholder(starting_line + 33)} |
-                 #{starting_line + 33} |  1 + * 1
-                 #{line_placeholder(starting_line + 33)} |      ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on #{location}:#{line}:6:
+                     error: syntax error before: '*'
+                     │
+                 #{line} │  1 + * 1
+                     │      ^
+                     │
+                     └─ #{location}:#{line}:6
                 doctest:
                   iex> 1 + * 1
                   1
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 33}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(line)}ExUnit.DocTestTest.Invalid (module)
            """
 
+    line = starting_line + 48
+
     assert output =~ """
-            16) doctest ExUnit.DocTestTest.Invalid.t/0 (16) (ExUnit.DocTestTest.ActuallyCompiled)
+             8) doctest ExUnit.DocTestTest.Invalid.result/0 (8) (ExUnit.DocTestTest.InvalidCompiled)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
-                Doctest did not compile, got: (SyntaxError) test/ex_unit/doc_test_test.exs:#{starting_line + 75}:6: syntax error before: '*'
-                 #{line_placeholder(starting_line + 75)} |
-                 #{starting_line + 75} |  1 + * 1
-                 #{line_placeholder(starting_line + 75)} |      ^
+                Doctest did not compile, got: (SyntaxError) invalid syntax found on #{location}:#{line}:5:
+                     error: syntax error before: '*'
+                     │
+                 #{line} │ 1 + * 1
+                     │     ^
+                     │
+                     └─ #{location}:#{line}:5
                 doctest:
-                  iex> 1 + * 1
-                  1
+                  iex> :foo
+                  iex> :bar
+                  1 + * 1
                 stacktrace:
-                  test/ex_unit/doc_test_test.exs:#{starting_line + 75}: ExUnit.DocTestTest.Invalid (module)
+                  #{stack(line)}ExUnit.DocTestTest.Invalid (module)
            """
+
+    line = starting_line + 53
+
+    assert output =~ """
+             9) doctest ExUnit.DocTestTest.Invalid.mixed/0 (9) (ExUnit.DocTestTest.InvalidCompiled)
+                test/ex_unit/doc_test_test.exs:#{doctest_line}
+                Doctest did not compile, got: (TokenMissingError) token missing on #{location}:#{line}:6:
+                     error: syntax error: expression is incomplete
+                     │
+                 #{line} │  123 +
+                     │      ^
+                     │
+                     └─ #{location}:#{line}:6
+                doctest:
+                  iex> 123 +
+                  :mixed
+                stacktrace:
+                  #{stack(line)}ExUnit.DocTestTest.Invalid (module)
+           """
+
+    line = starting_line + 61
+
+    assert output =~ """
+            10) doctest ExUnit.DocTestTest.Invalid.invalid_second/0 (10) (ExUnit.DocTestTest.InvalidCompiled)
+                test/ex_unit/doc_test_test.exs:#{doctest_line}
+                Doctest did not compile, got: (TokenMissingError) token missing on #{location}:#{line}:6:
+                     error: syntax error: expression is incomplete
+                     │
+                 #{line} │  123 +
+                     │      ^
+                     │
+                     └─ #{location}:#{line}:6
+                doctest:
+                  iex> 123 +
+                  :mixed
+                stacktrace:
+                  #{stack(line)}ExUnit.DocTestTest.Invalid (module)
+           """
+
+    assert output =~ "10 doctests, 10 failures"
   end
 
   test "pattern matching assertions in doctests" do
@@ -742,6 +900,7 @@ defmodule ExUnit.DocTestTest do
 
     doctest_line = __ENV__.line - 3
     starting_line = ExUnit.DocTestTest.PatternMatching.starting_line()
+    location = location(ExUnit.DocTestTest.PatternMatching)
 
     ExUnit.configure(seed: 0, colors: [enabled: false])
     output = capture_io(fn -> ExUnit.run() end)
@@ -754,7 +913,7 @@ defmodule ExUnit.DocTestTest do
                 left:  {1, 2}
                 right: {1, 3}
                 stacktrace:
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 3}: (test)
+                  (for doctest at) #{location}:#{starting_line + 3}: (test)
            """
 
     assert output =~ """
@@ -765,7 +924,7 @@ defmodule ExUnit.DocTestTest do
                 left:  {1, 2}
                 right: {1, 3}
                 stacktrace:
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 6}: (test)
+                  (for doctest at) #{location}:#{starting_line + 6}: (test)
            """
 
     assert output =~ """
@@ -776,7 +935,7 @@ defmodule ExUnit.DocTestTest do
                 left:  "Hello, world"
                 right: "Hello world"
                 stacktrace:
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 8}: (test)
+                  (for doctest at) #{location}:#{starting_line + 8}: (test)
            """
 
     assert output =~ """
@@ -787,7 +946,7 @@ defmodule ExUnit.DocTestTest do
                 left:  "Hello, " <> _
                 right: "Hello world"
                 stacktrace:
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 10}: (test)
+                  (for doctest at) #{location}:#{starting_line + 10}: (test)
            """
 
     assert output =~ """
@@ -798,7 +957,7 @@ defmodule ExUnit.DocTestTest do
                 left:  [:a | _]
                 right: [:b, :a]
                 stacktrace:
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 12}: (test)
+                  (for doctest at) #{location}:#{starting_line + 12}: (test)
            """
 
     assert output =~ """
@@ -811,7 +970,7 @@ defmodule ExUnit.DocTestTest do
                 left:  [^atom | _]
                 right: [:b, :a]
                 stacktrace:
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 15}: (test)
+                  (for doctest at) #{location}:#{starting_line + 15}: (test)
            """
 
     assert output =~ """
@@ -822,18 +981,18 @@ defmodule ExUnit.DocTestTest do
                 left:  %{b: _, d: :e}
                 right: %{a: :c, d: :e}
                 stacktrace:
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 17}: (test)
+                  (for doctest at) #{location}:#{starting_line + 17}: (test)
            """
 
     assert output =~ """
              8) doctest module ExUnit.DocTestTest.PatternMatching (8) (ExUnit.DocTestTest.PatternMatchingRunner)
                 test/ex_unit/doc_test_test.exs:#{doctest_line}
                 match (=) failed
-                code:  %{year: 2001, day: 1} = ~D"2000-01-01"
+                code:  %{year: 2001, day: 1} = ~D[2000-01-01]
                 left:  %{year: 2001, day: 1}
                 right: ~D[2000-01-01]
                 stacktrace:
-                  (for doctest at) test/ex_unit/doc_test_test.exs:#{starting_line + 19}: (test)
+                  (for doctest at) #{location}:#{starting_line + 19}: (test)
            """
 
     assert output =~ "10 doctests, 8 failures"
@@ -852,9 +1011,7 @@ defmodule ExUnit.DocTestTest do
     message =
       ~s[unknown IEx prompt: "iex(foo@bar)1> 1 +".\nAccepted formats are: iex>, iex(1)>, ...>, ...(1)>]
 
-    regex = ~r[test/ex_unit/doc_test_test\.exs:\d+: #{Regex.escape(message)}]
-
-    assert_raise ExUnit.DocTest.Error, regex, fn ->
+    assert_raise ExUnit.DocTest.Error, ~r/#{Regex.escape(message)}/, fn ->
       defmodule HostUsage do
         use ExUnit.Case
         doctest ExUnit.DocTestTest.Host
@@ -862,137 +1019,212 @@ defmodule ExUnit.DocTestTest do
     end
   end
 
-  test "doctests type" do
-    defmodule DoctestType do
-      use ExUnit.Case
-      doctest ExUnit.DocTestTest.NoImport
+  test "doctests built-in tags" do
+    alias ExUnit.DocTestTest.{NoImport, NoTrailing}
 
-      setup test do
-        assert test.test_type == :doctest
-        :ok
+    defmodule DoctestTags do
+      use ExUnit.Case, register: false
+      doctest NoImport
+      doctest NoTrailing
+    end
+
+    assert %ExUnit.TestModule{tests: [test1, test2, test3]} = DoctestTags.__ex_unit__()
+
+    assert %{
+             test_type: :doctest,
+             doctest: NoImport,
+             doctest_line: 133,
+             doctest_data: %{end_line: 134}
+           } = test1.tags
+
+    assert %{
+             test_type: :doctest,
+             doctest: NoImport,
+             doctest_line: 136,
+             doctest_data: %{end_line: 137}
+           } = test2.tags
+
+    assert %{
+             test_type: :doctest,
+             doctest: NoTrailing,
+             doctest_line: 145,
+             doctest_data: %{end_line: 145}
+           } = test3.tags
+  end
+
+  describe "errors" do
+    test "multiple exceptions in one test case is not supported" do
+      message = ~r"multiple exceptions in the same doctest example are not supported"
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.MultipleExceptions
+        end
       end
     end
 
-    assert capture_io(fn -> ExUnit.run() end) =~ "2 doctests, 0 failures"
-  end
+    test "fails on invalid module" do
+      assert capture_io(:stderr, fn ->
+               assert_raise CompileError, fn ->
+                 defmodule NeverCompiled do
+                   import ExUnit.DocTest
+                   doctest ExUnit.DocTestTest.Unknown
+                 end
+               end
+             end) =~ "module ExUnit.DocTestTest.Unknown is not loaded and could not be found"
+    end
 
-  test "multiple exceptions in one test case is not supported" do
-    message = ~r"multiple exceptions in the same doctest example are not supported"
+    test "fails when testing functions are not found" do
+      message = """
+      #{stack(0)}undefined or private functions given to doctest:
 
-    assert_raise ExUnit.DocTest.Error, message, fn ->
-      defmodule NeverCompiled do
-        import ExUnit.DocTest
-        doctest ExUnit.DocTestTest.MultipleExceptions
+          ExUnit.DocTestTest.SomewhatGoodModuleWithOnly.three/0
+          ExUnit.DocTestTest.SomewhatGoodModuleWithOnly.four/1
+
+      """
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+
+          doctest ExUnit.DocTestTest.SomewhatGoodModuleWithOnly,
+            only: [three: 0, four: 1],
+            import: true
+        end
+      end
+    end
+
+    test "fails when there are no docs" do
+      message = ~r"could not retrieve the documentation for module ExUnit\.DocTestTest"
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest
+        end
+      end
+    end
+
+    test "fails in indentation mismatch" do
+      message =
+        ~r[indentation level mismatch on doctest line: \"   iex> bar = 2\".*is exactly 2 spaces]s
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.IndentationMismatchedPrompt
+        end
+      end
+
+      message = ~r[indentation level mismatch on doctest line: \"    3\".*is exactly 2 spaces]s
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.IndentationTooMuch
+        end
+      end
+
+      message = ~r[indentation level mismatch on doctest line: \"  3\".*is exactly 4 spaces]s
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.IndentationNotEnough
+        end
+      end
+
+      message = ~r[indentation level mismatch on doctest line: \"```\".*is exactly 2 spaces]s
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.FencedNotEnough
+        end
+      end
+
+      message =
+        ~r[indentation level mismatch on doctest line: \"world\\\"\".*is exactly 4 spaces]s
+
+      assert_raise ExUnit.DocTest.Error, message, fn ->
+        defmodule NeverCompiled do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.IndentationNotEnoughMultilineCode
+        end
+      end
+    end
+
+    test "fails on invalid use" do
+      assert_raise RuntimeError, ~r"cannot define doctest", fn ->
+        defmodule FunctionClashFail do
+          import ExUnit.DocTest
+          doctest ExUnit.DocTestTest.Failure
+        end
       end
     end
   end
 
-  test "fails on invalid module" do
-    message = ~r"module ExUnit\.DocTestTest\.Unknown is not loaded and could not be found"
-
-    assert_raise CompileError, message, fn ->
-      defmodule NeverCompiled do
-        import ExUnit.DocTest
-        doctest ExUnit.DocTestTest.Unknown
+  describe "doctest_file" do
+    test "passing" do
+      defmodule FilePassing do
+        use ExUnit.Case
+        doctest_file(Path.expand("../fixtures/passing.md", __DIR__))
       end
+
+      output = capture_io(fn -> ExUnit.run() end)
+      assert output =~ "2 doctests, 0 failures"
+    end
+
+    test "failing" do
+      defmodule FileFailing do
+        use ExUnit.Case
+        doctest_file(Path.expand("../fixtures/failing.md", __DIR__))
+      end
+
+      doctest_line = __ENV__.line - 3
+      ExUnit.configure(seed: 0, colors: [enabled: false])
+      output = capture_io(fn -> ExUnit.run() end)
+
+      assert output =~ """
+               1) doctest test/fixtures/failing.md (1) (ExUnit.DocTestTest.FileFailing)
+                  test/ex_unit/doc_test_test.exs:#{doctest_line}
+                  Doctest failed
+                  doctest:
+                    iex> 1 + 2
+                    4
+                  code:  1 + 2 === 4
+                  left:  3
+                  right: 4
+                  stacktrace:
+                    test/fixtures/failing.md:4: ExUnit.DocTestTest.FileFailing (module)
+             """
     end
   end
 
-  test "fails when testing single function not found" do
-    message = """
-    test/ex_unit/doc_test_test\.exs: undefined or private function given to doctest:
+  test "doctest direct invocation" do
+    defmodule Direct do
+      use ExUnit.Case, register: false
+      doctest ExUnit.DocTestTest.GoodModule, import: true
+    end
 
-        ExUnit.DocTestTest.SomewhatGoodModuleWithOnly.three/0
+    [head | _] = Direct.__ex_unit__().tests
+    assert apply(Direct, head.name, [%{}]) == {:ok, 2}
+  end
 
-    """
-
-    assert_raise ExUnit.DocTest.Error, message, fn ->
-      defmodule NeverCompiled do
-        import ExUnit.DocTest
-
-        doctest ExUnit.DocTestTest.SomewhatGoodModuleWithOnly, only: [three: 0], import: true
-      end
+  defp location(module) do
+    if __MODULE__.module_info(:compile)[:source] do
+      "test/ex_unit/doc_test_test.exs"
+    else
+      inspect(module)
     end
   end
 
-  test "fails when testing multiple functions not found" do
-    message = """
-    test/ex_unit/doc_test_test\.exs: undefined or private functions given to doctest:
-
-        ExUnit.DocTestTest.SomewhatGoodModuleWithOnly.three/0
-        ExUnit.DocTestTest.SomewhatGoodModuleWithOnly.four/1
-
-    """
-
-    assert_raise ExUnit.DocTest.Error, message, fn ->
-      defmodule NeverCompiled do
-        import ExUnit.DocTest
-
-        doctest ExUnit.DocTestTest.SomewhatGoodModuleWithOnly,
-          only: [three: 0, four: 1],
-          import: true
-      end
+  defp stack(line) do
+    if __MODULE__.module_info(:compile)[:source] do
+      [Exception.format_file_line("test/ex_unit/doc_test_test.exs", line), ?\s]
+    else
+      ""
     end
   end
-
-  test "fails when there are no docs" do
-    message = ~r"could not retrieve the documentation for module ExUnit\.DocTestTest"
-
-    assert_raise ExUnit.DocTest.Error, message, fn ->
-      defmodule NeverCompiled do
-        import ExUnit.DocTest
-        doctest ExUnit.DocTestTest
-      end
-    end
-  end
-
-  test "fails in indentation mismatch" do
-    message =
-      ~r[test/ex_unit/doc_test_test\.exs:\d+: indentation level mismatch on doctest line: \"   iex> bar = 2\".*is exactly 2 spaces]s
-
-    assert_raise ExUnit.DocTest.Error, message, fn ->
-      defmodule NeverCompiled do
-        import ExUnit.DocTest
-        doctest ExUnit.DocTestTest.IndentationMismatchedPrompt
-      end
-    end
-
-    message =
-      ~r[test/ex_unit/doc_test_test\.exs:\d+: indentation level mismatch on doctest line: \"    3\".*is exactly 2 spaces]s
-
-    assert_raise ExUnit.DocTest.Error, message, fn ->
-      defmodule NeverCompiled do
-        import ExUnit.DocTest
-        doctest ExUnit.DocTestTest.IndentationTooMuch
-      end
-    end
-
-    message =
-      ~r[test/ex_unit/doc_test_test\.exs:\d+: indentation level mismatch on doctest line: \"  3\".*is exactly 4 spaces]s
-
-    assert_raise ExUnit.DocTest.Error, message, fn ->
-      defmodule NeverCompiled do
-        import ExUnit.DocTest
-        doctest ExUnit.DocTestTest.IndentationNotEnough
-      end
-    end
-  end
-
-  test "fails on invalid use" do
-    assert_raise RuntimeError, ~r"cannot define doctest", fn ->
-      defmodule FunctionClashFail do
-        import ExUnit.DocTest
-        doctest ExUnit.DocTestTest.Invalid
-      end
-    end
-  end
-
-  defp line_placeholder(line_number) do
-    digits =
-      line_number
-      |> Integer.digits()
-      |> length()
-
-    String.duplicate(" ", digits)
-  end
 end
