@@ -15,7 +15,15 @@ defmodule ExUnit.RegisterTest do
     defmodule SingularTestTypeCase do
       use ExUnit.Case
 
-      :"property is true" = ExUnit.Case.register_test(__ENV__, :property, "is true", [])
+      :"property is true" =
+        ExUnit.Case.register_test(
+          __ENV__.module,
+          __ENV__.file,
+          __ENV__.line,
+          :property,
+          "is true",
+          []
+        )
 
       def unquote(:"property is true")(_) do
         assert succeed()
@@ -43,13 +51,29 @@ defmodule ExUnit.RegisterTest do
     defmodule PluralTestTypeCase do
       use ExUnit.Case
 
-      :"property is true" = ExUnit.Case.register_test(__ENV__, :property, "is true", [])
+      :"property is true" =
+        ExUnit.Case.register_test(
+          __ENV__.module,
+          __ENV__.file,
+          __ENV__.line,
+          :property,
+          "is true",
+          []
+        )
 
       def unquote(:"property is true")(_) do
         assert succeed()
       end
 
-      :"property is also true" = ExUnit.Case.register_test(__ENV__, :property, "is also true", [])
+      :"property is also true" =
+        ExUnit.Case.register_test(
+          __ENV__.module,
+          __ENV__.file,
+          __ENV__.line,
+          :property,
+          "is also true",
+          []
+        )
 
       def unquote(:"property is also true")(_) do
         assert succeed()
