@@ -24,7 +24,7 @@ defmodule ExUnitTest do
              assert ExUnit.run() == %{failures: 2, skipped: 0, total: 2, excluded: 0}
            end) =~ "\n2 tests, 2 failures\n"
 
-    ExUnit.Server.modules_loaded()
+    ExUnit.Server.modules_loaded(false)
 
     assert capture_io(fn ->
              assert ExUnit.async_run() |> ExUnit.await_run() ==
@@ -34,7 +34,7 @@ defmodule ExUnitTest do
 
   test "supports rerunning given modules" do
     defmodule SampleAsyncTest do
-      use ExUnit.Case, async: true
+      use ExUnit.Case, async: true, register: false
 
       test "true" do
         assert false
@@ -42,7 +42,7 @@ defmodule ExUnitTest do
     end
 
     defmodule SampleSyncTest do
-      use ExUnit.Case
+      use ExUnit.Case, register: false
 
       test "true" do
         assert false
@@ -61,15 +61,26 @@ defmodule ExUnitTest do
 
     assert capture_io(fn ->
              assert ExUnit.run() == %{
-                      failures: 3,
+                      failures: 1,
                       skipped: 0,
-                      total: 3,
+                      total: 1,
                       excluded: 0
                     }
-           end) =~ "\n3 tests, 3 failures\n"
+           end) =~ "\n1 test, 1 failure\n"
+
+    sample = [SampleSyncTest, SampleAsyncTest]
 
     assert capture_io(fn ->
-             assert ExUnit.run([SampleSyncTest, SampleAsyncTest]) == %{
+             assert ExUnit.run(sample) == %{
+                      failures: 2,
+                      skipped: 0,
+                      total: 2,
+                      excluded: 0
+                    }
+           end) =~ "\n2 tests, 2 failures\n"
+
+    assert capture_io(fn ->
+             assert ExUnit.run(sample ++ sample) == %{
                       failures: 2,
                       skipped: 0,
                       total: 2,
@@ -127,8 +138,6 @@ defmodule ExUnitTest do
            Showing results so far...
 
            0 failures
-
-           Randomized with seed 0
            """
   end
 
@@ -152,6 +161,24 @@ defmodule ExUnitTest do
            end) =~ "\n1 test, 1 failure\n"
   end
 
+  test "reports capture log crashes" do
+    defmodule CaptureLogTest do
+      use ExUnit.Case
+
+      test "capture log crash because logger stopped" do
+        Logger.App.stop()
+        Logger.App.start()
+        flunk("this should fail")
+      end
+    end
+
+    configure_and_reload_on_exit([])
+
+    assert capture_io(fn ->
+             assert ExUnit.run() == %{failures: 1, skipped: 0, total: 1, excluded: 0}
+           end) =~ "\n1 test, 1 failure\n"
+  end
+
   test "supports timeouts" do
     defmodule TimeoutTest do
       use ExUnit.Case
@@ -210,6 +237,41 @@ defmodule ExUnitTest do
     assert output =~ ~r"\* test delayed \(.+ms\)"
   end
 
+  test "reports slow test modules" do
+    defmodule SlowTestModule do
+      use ExUnit.Case
+
+      test "slow" do
+        refute false
+      end
+    end
+
+    defmodule SlowerTestModule do
+      use ExUnit.Case
+
+      test "slower" do
+        Process.sleep(5)
+        refute false
+      end
+    end
+
+    defmodule SlowestTestModule do
+      use ExUnit.Case
+
+      test "slowest" do
+        Process.sleep(10)
+        refute false
+      end
+    end
+
+    configure_and_reload_on_exit(slowest_modules: 2)
+
+    output = capture_io(fn -> ExUnit.run() end)
+    assert output =~ ~r"Top 2 slowest \(\d+\.\d+s\), \d+.\d% of total time:"
+    assert output =~ ~r"SlowestTestModule \(.+ms\)"
+    assert output =~ ~r"SlowerTestModule \(.+ms\)"
+  end
+
   test "sets max cases to one with trace enabled" do
     configure_and_reload_on_exit(trace: true, max_cases: 10)
     config = ExUnit.configuration()
@@ -287,14 +349,8 @@ defmodule ExUnitTest do
   test "log capturing" do
     defmodule LogCapturingTest do
       use ExUnit.Case
-
       require Logger
 
-      setup_all do
-        :ok = Logger.remove_backend(:console)
-        on_exit(fn -> Logger.add_backend(:console, flush: true) end)
-      end
-
       @tag :capture_log
       test "one" do
         Logger.debug("one")
@@ -312,18 +368,12 @@ defmodule ExUnitTest do
         Logger.debug("three")
         assert 1 == 2
       end
-
-      test "four" do
-        Logger.debug("four")
-        assert 1 == 2
-      end
     end
 
     output = capture_io(&ExUnit.run/0)
-    assert output =~ "[debug] two\n"
     refute output =~ "[debug] one\n"
+    assert output =~ "[debug] two\n"
     assert output =~ "[debug] three\n"
-    refute output =~ "[debug] four\n"
   end
 
   test "supports multi errors" do
@@ -381,7 +431,7 @@ defmodule ExUnitTest do
   test "raises friendly error for duplicate test names" do
     message = ~S("test duplicate" is already defined in ExUnitTest.TestWithSameNames)
 
-    assert_raise ExUnit.DuplicateTestError, message, fn ->
+    assert_raise ArgumentError, message, fn ->
       defmodule TestWithSameNames do
         use ExUnit.Case
 
@@ -572,11 +622,11 @@ defmodule ExUnitTest do
       use ExUnit.Case, async: true
 
       test "generated seed is always the same in the same module and test" do
-        assert :rand.uniform(1_000_000) == 622_983
+        assert :rand.uniform(1_000_000) == 253_740
       end
 
       test "generated seed is different for other test" do
-        assert :rand.uniform(1_000_000) == 783_982
+        assert :rand.uniform(1_000_000) == 462_665
       end
     end
 
@@ -584,7 +634,7 @@ defmodule ExUnitTest do
       use ExUnit.Case, async: true
 
       test "generated seed is always the same in the same module and test" do
-        assert :rand.uniform(1_000_000) == 358_099
+        assert :rand.uniform(1_000_000) == 563_517
       end
     end
 
@@ -633,6 +683,38 @@ defmodule ExUnitTest do
     assert output =~ "\n6 tests, 0 failures, 1 excluded, 4 invalid, 1 skipped\n"
   end
 
+  test "parameterized tests" do
+    Process.register(self(), :parameterized_tests)
+
+    defmodule ParameterizedTests do
+      use ExUnit.Case, async: true, parameterize: [%{value: true}, %{value: false}]
+
+      @tag :tmp_dir
+      test "hello world", %{value: value, tmp_dir: tmp_dir} do
+        send(:parameterized_tests, {:tmp_dir, tmp_dir})
+        assert value
+      end
+    end
+
+    configure_and_reload_on_exit(trace: true)
+
+    output = capture_io(fn -> ExUnit.run() end)
+
+    assert output =~ ~r"""
+           ExUnitTest.ParameterizedTests \[.*test/ex_unit_test.exs\]
+           Parameters: %\{value: false\}
+           """
+
+    assert output =~ """
+             1) test hello world (ExUnitTest.ParameterizedTests)
+                Parameters: %{value: false}
+           """
+
+    # Check that the temporary paths are different
+    assert_receive {:tmp_dir, tmp_dir1}
+    assert_receive {:tmp_dir, tmp_dir2} when tmp_dir1 != tmp_dir2
+  end
+
   describe "after_suite/1" do
     test "executes all callbacks set in reverse order" do
       Process.register(self(), :after_suite_test_process)
@@ -873,6 +955,109 @@ defmodule ExUnitTest do
     end
   end
 
+  describe ":repeat_until_failure" do
+    test "defaults to 0" do
+      configure_and_reload_on_exit([])
+      ExUnit.start(autorun: false)
+      config = ExUnit.configuration()
+      assert config[:repeat_until_failure] == 0
+    end
+
+    test "sets value of :repeat_until_failure" do
+      configure_and_reload_on_exit([])
+      ExUnit.start(repeat_until_failure: 5, autorun: false)
+      config = ExUnit.configuration()
+      assert config[:repeat_until_failure] == 5
+    end
+
+    test "repeats tests up to the configured number of times" do
+      defmodule TestRepeatUntilFailureReached do
+        use ExUnit.Case
+
+        @tag :skip
+        test "skipped #{__ENV__.line}", do: assert(false)
+
+        test __ENV__.line, do: assert(true)
+        test __ENV__.line, do: assert(true)
+        test __ENV__.line, do: assert(true)
+
+        @tag :exclude
+        test "excluded #{__ENV__.line}", do: assert(false)
+      end
+
+      configure_and_reload_on_exit(repeat_until_failure: 5)
+
+      output =
+        capture_io(fn ->
+          assert ExUnit.run() == %{total: 5, failures: 0, skipped: 1, excluded: 1}
+        end)
+
+      runs = String.split(output, "Running ExUnit", trim: true)
+      # 6 runs in total, 5 repeats
+      assert length(runs) == 6
+    end
+
+    test "repeats tests up to the configured number of times with groups" do
+      defmodule TestGroupedRepeatUntilFailureReached do
+        use ExUnit.Case, async: true, group: :example
+        test __ENV__.line, do: assert(true)
+      end
+
+      configure_and_reload_on_exit(repeat_until_failure: 5)
+
+      output =
+        capture_io(fn ->
+          assert ExUnit.run() == %{total: 1, failures: 0, skipped: 0, excluded: 0}
+        end)
+
+      runs = String.split(output, "Running ExUnit", trim: true)
+      # 6 runs in total, 5 repeats
+      assert length(runs) == 6
+    end
+
+    test "stops on failure" do
+      {:ok, pid} = Agent.start_link(fn -> 0 end)
+      Process.register(pid, :ex_unit_repeat_until_failure_count)
+
+      defmodule TestRepeatUntilFailureFailure do
+        use ExUnit.Case
+
+        @tag :skip
+        test "skipped #{__ENV__.line}", do: assert(true)
+
+        test "maybe pass #{__ENV__.line}" do
+          count = Agent.get(:ex_unit_repeat_until_failure_count, & &1)
+
+          if count < 3 do
+            Agent.update(:ex_unit_repeat_until_failure_count, &(&1 + 1))
+            assert(true)
+          else
+            assert(false)
+          end
+        end
+
+        @tag :exclude
+        test "excluded #{__ENV__.line}", do: assert(true)
+
+        @tag :exclude
+        test "excluded #{__ENV__.line}", do: assert(false)
+      end
+
+      configure_and_reload_on_exit(repeat_until_failure: 5)
+
+      output =
+        capture_io(fn ->
+          assert ExUnit.run() == %{total: 4, excluded: 2, failures: 1, skipped: 1}
+        end)
+
+      runs = String.split(output, "Running ExUnit", trim: true)
+      # four runs in total, the first two repeats work fine, the third repeat (4th run)
+      # fails, therefore we stop
+      assert length(runs) == 4
+      assert List.last(runs) =~ "Expected truthy, got false"
+    end
+  end
+
   test "prints warning when all tests are excluded" do
     defmodule OnlyExcludedTests do
       use ExUnit.Case
@@ -895,19 +1080,145 @@ defmodule ExUnitTest do
     assert output =~ "2 tests, 0 failures, 2 excluded\n"
   end
 
+  test "tests are run in compile order (FIFO)" do
+    defmodule FirstTestFIFO do
+      use ExUnit.Case
+
+      test "first test" do
+        assert true
+      end
+    end
+
+    defmodule SecondTestFIFO do
+      use ExUnit.Case
+
+      test "second test" do
+        assert true
+      end
+    end
+
+    defmodule ThirdTestFIFO do
+      use ExUnit.Case
+
+      test "third test" do
+        assert true
+      end
+    end
+
+    configure_and_reload_on_exit(trace: true)
+
+    output =
+      capture_io(fn ->
+        assert ExUnit.run() == %{total: 3, failures: 0, excluded: 0, skipped: 0}
+      end)
+
+    [_, first, second, third | _] = String.split(output, "\n\n")
+
+    assert first =~ "FirstTestFIFO"
+    assert second =~ "SecondTestFIFO"
+    assert third =~ "ThirdTestFIFO"
+  end
+
+  test "groups are run in compile order (FIFO)" do
+    defmodule RedOneFIFO do
+      use ExUnit.Case, async: true, group: :red
+
+      test "red one test" do
+        Process.sleep(5)
+        assert true
+      end
+    end
+
+    defmodule BlueOneFIFO do
+      use ExUnit.Case, async: true, group: :blue
+
+      test "blue one test" do
+        Process.sleep(5)
+        assert true
+      end
+    end
+
+    defmodule RedTwoFIFO do
+      use ExUnit.Case, async: true, group: :red
+
+      test "red two test" do
+        assert true
+      end
+    end
+
+    defmodule BlueTwoFIFO do
+      use ExUnit.Case, async: true, group: :blue
+
+      test "blue two test" do
+        assert true
+      end
+    end
+
+    configure_and_reload_on_exit(trace: true, max_cases: 2)
+
+    output =
+      capture_io(fn ->
+        assert ExUnit.run() == %{total: 4, failures: 0, excluded: 0, skipped: 0}
+      end)
+
+    [_, first, second, third, fourth | _] = String.split(output, "\n\n")
+
+    assert first =~ "RedOneFIFO"
+    assert second =~ "RedTwoFIFO"
+    assert third =~ "BlueOneFIFO"
+    assert fourth =~ "BlueTwoFIFO"
+  end
+
+  test "filters async tests" do
+    defmodule FirstTestAsyncTrue do
+      use ExUnit.Case, async: true
+
+      test "first test" do
+        assert true
+      end
+    end
+
+    defmodule SecondTestAsyncTrue do
+      use ExUnit.Case, async: true
+
+      test "second test" do
+        assert true
+      end
+    end
+
+    defmodule FirstTestAsyncFalse do
+      use ExUnit.Case, async: false
+
+      test "first test" do
+        assert true
+      end
+    end
+
+    assert {%{failures: 0, skipped: 0, total: 3, excluded: 1}, _} =
+             run_with_filter([include: [async: true], exclude: [:test]], [])
+
+    assert {%{failures: 0, skipped: 0, total: 3, excluded: 2}, _} =
+             run_with_filter(
+               [include: [async: false], exclude: [:test]],
+               [FirstTestAsyncTrue, SecondTestAsyncTrue, FirstTestAsyncFalse]
+             )
+  end
+
   ##  Helpers
 
   defp run_with_filter(filters, cases) do
-    Enum.each(cases, &ExUnit.Server.add_sync_module/1)
-    ExUnit.Server.modules_loaded()
+    Enum.each(cases, fn mod ->
+      ExUnit.Server.add_module(mod, mod.__ex_unit__(:config))
+    end)
+
+    ExUnit.Server.modules_loaded(false)
 
     opts =
       ExUnit.configuration()
       |> Keyword.merge(filters)
       |> Keyword.merge(colors: [enabled: false])
 
-    output = capture_io(fn -> Process.put(:capture_result, ExUnit.Runner.run(opts, nil)) end)
-    {Process.get(:capture_result), output}
+    with_io(fn -> ExUnit.Runner.run(opts, nil) |> elem(0) end)
   end
 
   defp next_message_in_mailbox() do
