@@ -13,7 +13,9 @@ defmodule IEx.Config do
     :alive_prompt,
     :alive_continuation_prompt,
     :width,
-    :parser
+    :parser,
+    :dot_iex,
+    :auto_reload
   ]
 
   # Read API
@@ -88,6 +90,14 @@ defmodule IEx.Config do
     end
   end
 
+  def dot_iex() do
+    Application.get_env(:iex, :dot_iex)
+  end
+
+  def auto_reload?() do
+    Application.fetch_env!(:iex, :auto_reload)
+  end
+
   # Used by default on evaluation cycle
   defp default_color(:eval_interrupt), do: [:yellow]
   defp default_color(:eval_result), do: [:yellow]
@@ -193,6 +203,8 @@ defmodule IEx.Config do
   defp validate_option({:alive_continuation_prompt, new}) when is_binary(new), do: :ok
   defp validate_option({:width, new}) when is_integer(new), do: :ok
   defp validate_option({:parser, tuple}) when tuple_size(tuple) == 3, do: :ok
+  defp validate_option({:dot_iex, path}) when is_binary(path), do: :ok
+  defp validate_option({:auto_reload, enabled}) when is_boolean(enabled), do: :ok
 
   defp validate_option(option) do
     raise ArgumentError, "invalid configuration #{inspect(option)}"
