@@ -22,33 +22,44 @@ defmodule IEx.Helpers do
 
   There are many other helpers available, here are some examples:
 
-    * `b/1`            - prints callbacks info and docs for a given module
-    * `c/1`            - compiles a file
-    * `c/2`            - compiles a file and writes bytecode to the given path
-    * `cd/1`           - changes the current directory
-    * `clear/0`        - clears the screen
-    * `exports/1`      - shows all exports (functions + macros) in a module
-    * `flush/0`        - flushes all messages sent to the shell
-    * `h/0`            - prints this help message
-    * `h/1`            - prints help for the given module, function or macro
-    * `i/0`            - prints information about the last value
-    * `i/1`            - prints information about the given term
-    * `ls/0`           - lists the contents of the current directory
-    * `ls/1`           - lists the contents of the specified directory
-    * `open/1`         - opens the source for the given module or function in your editor
-    * `pid/1`          - creates a PID from a string
-    * `pid/3`          - creates a PID with the 3 integer arguments passed
-    * `port/1`         - creates a port from a string
-    * `port/2`         - creates a port with the 2 non-negative integers passed
-    * `pwd/0`          - prints the current working directory
-    * `r/1`            - recompiles the given module's source file
-    * `recompile/0`    - recompiles the current project
-    * `ref/1`          - creates a reference from a string
-    * `ref/4`          - creates a reference with the 4 integer arguments passed
-    * `runtime_info/0` - prints runtime info (versions, memory usage, stats)
-    * `t/1`            - prints the types for the given module or function
-    * `v/0`            - retrieves the last value from the history
-    * `v/1`            - retrieves the nth value from the history
+    * `b/1`             - prints callbacks info and docs for a given module
+    * `c/1`             - compiles a file
+    * `c/2`             - compiles a file and writes bytecode to the given path
+    * `cd/1`            - changes the current directory
+    * `clear/0`         - clears the screen
+    * `exports/1`       - shows all exports (functions + macros) in a module
+    * `flush/0`         - flushes all messages sent to the shell
+    * `h/0`             - prints this help message
+    * `h/1`             - prints help for the given module, function or macro
+    * `i/0`             - prints information about the last value
+    * `i/1`             - prints information about the given term
+    * `ls/0`            - lists the contents of the current directory
+    * `ls/1`            - lists the contents of the specified directory
+    * `open/1`          - opens the source for the given module or function in your editor
+    * `pid/1`           - creates a PID from a string
+    * `pid/3`           - creates a PID with the 3 integer arguments passed
+    * `port/1`          - creates a port from a string
+    * `port/2`          - creates a port with the 2 non-negative integers passed
+    * `pwd/0`           - prints the current working directory
+    * `r/1`             - recompiles the given module's source file
+    * `recompile/0`     - recompiles the current project
+    * `ref/1`           - creates a reference from a string
+    * `ref/4`           - creates a reference with the 4 integer arguments passed
+    * `runtime_info/0`  - prints runtime info (versions, memory usage, stats)
+    * `t/1`             - prints the types for the given module or function
+    * `v/0`             - retrieves the last value from the history
+    * `v/1`             - retrieves the nth value from the history
+
+  There are also several helpers available when debugging, such as:
+
+    * `break!/2`        - sets a breakpoint at `Module.function/arity`
+    * `breaks/0`        - prints all breakpoints to the terminal
+    * `c/0`             - a shortcut for `continue/0`
+    * `continue/0`      - continues execution of the current process
+    * `n/0`             - a shortcut for `next/0`
+    * `next/0`          - goes to the next line of the current breakpoint
+    * `remove_breaks/0` - removes all breakpoints and instrumentation from all modules
+    * `whereami/1`      - prints the current location and stacktrace in a pry session
 
   Help for all of those functions can be consulted directly from
   the command line using the `h/1` helper itself. Try:
@@ -69,17 +80,22 @@ defmodule IEx.Helpers do
   import IEx, only: [dont_display_result: 0]
 
   @doc """
-  Recompiles the current Mix project.
+  Recompiles the current Mix project or Mix install
+  dependencies.
 
-  This helper only works when IEx is started with a Mix
-  project, for example, `iex -S mix`. Note this function
-  simply recompiles Elixir modules, without reloading
-  configuration, recompiling dependencies, or restarting
-  applications.
+  This helper requires either `Mix.install/2` to have been
+  called within the current IEx session or for IEx to be
+  started alongside, for example, `iex -S mix`.
 
-  Therefore, any long running process may crash on recompilation,
-  as changed modules will be temporarily removed and recompiled,
-  without going through the proper code change callback.
+  In the `Mix.install/1` case, it will recompile any outdated
+  path dependency declared during install. Within a project,
+  it will recompile any outdated module.
+
+  Note this function simply recompiles Elixir modules, without
+  reloading configuration or restarting applications. This means
+  any long running process may crash on recompilation, as changed
+  modules will be temporarily removed and recompiled, without
+  going through the proper code change callback.
 
   If you want to reload a single module, consider using
   `r(ModuleName)` instead.
@@ -93,31 +109,61 @@ defmodule IEx.Helpers do
 
   """
   def recompile(options \\ []) do
-    if mix_started?() do
-      config = Mix.Project.config()
-      consolidation = Mix.Project.consolidation_path(config)
-      reenable_tasks(config)
+    cond do
+      not mix_started?() ->
+        IO.puts(IEx.color(:eval_error, "Mix is not running. Please start IEx with: iex -S mix"))
+        :error
 
-      # No longer allow consolidations to be accessed.
-      Code.delete_path(consolidation)
-      purge_protocols(consolidation)
+      Mix.installed?() ->
+        Mix.in_install_project(fn ->
+          # TODO: remove this once Mix requires Hex with the fix from
+          # https://github.com/hexpm/hex/pull/1015
+          # Context: Mix.install/1 starts :hex if necessary and stops
+          # it afterwards. Calling compile here may require hex to be
+          # started and that should happen automatically, but because
+          # of a bug it is not (fixed in the linked PR).
+          _ = Application.ensure_all_started(:hex)
 
-      force? = Keyword.get(options, :force, false)
-      arguments = if force?, do: ["--force"], else: []
+          do_recompile(options)
+          # Just as with Mix.install/2 we clear all task invocations,
+          # so that we can recompile the dependencies again next time
+          Mix.Task.clear()
+          :ok
+        end)
 
-      {result, _} = Mix.Task.run("compile", arguments)
+      true ->
+        project = Mix.Project.get()
 
-      # Re-enable consolidation and allow them to be loaded.
-      Code.prepend_path(consolidation)
-      purge_protocols(consolidation)
+        if is_nil(project) or
+             project.__info__(:compile)[:source] == String.to_charlist(Path.absname("mix.exs")) do
+          Mix.Project.with_build_lock(fn ->
+            purge_result = IEx.MixListener.purge()
 
-      result
-    else
-      IO.puts(IEx.color(:eval_error, "Mix is not running. Please start IEx with: iex -S mix"))
-      :error
+            case do_recompile(options) do
+              :noop -> purge_result
+              compile_result -> compile_result
+            end
+          end)
+        else
+          message = "Cannot recompile because the current working directory changed"
+          IO.puts(IEx.color(:eval_error, message))
+        end
     end
   end
 
+  defp do_recompile(options) do
+    config = Mix.Project.config()
+    consolidation = Mix.Project.consolidation_path(config)
+    reenable_tasks(config)
+
+    force? = Keyword.get(options, :force, false)
+    args = ["--purge-consolidation-path-if-stale", consolidation, "--return-errors"]
+    args = if force?, do: ["--force" | args], else: args
+
+    {result, _} = Mix.Task.run("compile", args)
+    result
+  end
+
   defp mix_started? do
     List.keyfind(Application.started_applications(), :mix, 0) != nil
   end
@@ -130,32 +176,24 @@ defmodule IEx.Helpers do
     Enum.each(compilers, &Mix.Task.reenable("compile.#{&1}"))
   end
 
-  defp purge_protocols(path) do
-    case File.ls(path) do
-      {:ok, beams} ->
-        Enum.each(beams, fn beam ->
-          module = beam |> Path.rootname() |> String.to_atom()
-          :code.purge(module)
-          :code.delete(module)
-        end)
-
-      {:error, _} ->
-        :ok
-    end
-  end
-
   @doc """
   Compiles the given files.
 
   It expects a list of files to compile and an optional path to write
   the compiled code to. By default files are in-memory compiled.
-  To write compiled files to the current directory, an empty string
-  can be given.
+  To write compiled files to the current directory, `"."` can be given.
 
   It returns the names of the compiled modules.
 
   If you want to recompile an existing module, check `r/1` instead.
 
+  > #### Remote compilation {: .warning}
+  >
+  > When compiling code, warnings and errors may be printed to standard error.
+  > However, when connecting to a remote node, the standard error output is not
+  > redirected to the client. This means that compilation failures will be written
+  > to the logs or the output terminal of the machine you connect to.
+
   ## Examples
 
   In the example below, we pass a directory to where the `c/2` function will
@@ -174,13 +212,13 @@ defmodule IEx.Helpers do
   def c(files, path \\ :in_memory) when is_binary(path) or path == :in_memory do
     files = List.wrap(files)
 
-    unless Enum.all?(files, &is_binary/1) do
+    if not Enum.all?(files, &is_binary/1) do
       raise ArgumentError, "expected a binary or a list of binaries as argument"
     end
 
     {found, not_found} = Enum.split_with(files, &File.exists?/1)
 
-    unless Enum.empty?(not_found) do
+    if not Enum.empty?(not_found) do
       raise ArgumentError, "could not find files #{Enum.join(not_found, ", ")}"
     end
 
@@ -431,7 +469,7 @@ defmodule IEx.Helpers do
 
     sources =
       Enum.map(modules, fn module ->
-        unless Code.ensure_loaded?(module) do
+        if not Code.ensure_loaded?(module) do
           raise ArgumentError, "could not load nor find module: #{inspect(module)}"
         end
 
@@ -523,12 +561,15 @@ defmodule IEx.Helpers do
     :code.get_path()
     |> Protocol.extract_protocols()
     |> Enum.uniq()
-    |> Enum.reject(fn protocol -> is_nil(protocol.impl_for(term)) end)
+    |> Enum.filter(fn protocol ->
+      Code.ensure_loaded?(protocol) and function_exported?(protocol, :impl_for, 1) and
+        protocol.impl_for(term) != nil
+    end)
     |> Enum.sort()
     |> Enum.map_join(", ", &inspect/1)
   end
 
-  @runtime_info_topics [:system, :memory, :limits, :applications]
+  @runtime_info_topics [:system, :memory, :limits, :applications, :allocators]
   @doc """
   Prints VM/runtime information such as versions, memory usage and statistics.
 
@@ -567,9 +608,9 @@ defmodule IEx.Helpers do
   end
 
   defp print_topic_info(topics) when is_list(topics) do
-    IO.write(pad_key("Showing topics"))
+    IO.write(pad_key("Showing topics:"))
     IO.puts(inspect(topics))
-    IO.write(pad_key("Additional topics"))
+    IO.write(pad_key("Additional topics:"))
     IO.puts(inspect(@runtime_info_topics -- topics))
     IO.puts("")
     IO.puts("To view a specific topic call runtime_info(topic)")
@@ -588,12 +629,12 @@ defmodule IEx.Helpers do
 
   defp print_runtime_info_topic(:memory) do
     print_pane("Memory")
-    print_memory("Total", :total)
     print_memory("Atoms", :atom)
     print_memory("Binaries", :binary)
     print_memory("Code", :code)
     print_memory("ETS", :ets)
     print_memory("Processes", :processes)
+    print_memory("Total", :total)
   end
 
   defp print_runtime_info_topic(:limits) do
@@ -608,12 +649,12 @@ defmodule IEx.Helpers do
   end
 
   defp print_runtime_info_topic(:applications) do
-    print_pane("Loaded OTP Applications")
+    print_pane("Loaded OTP applications")
     started = Application.started_applications()
     loaded = Application.loaded_applications()
 
     for {app, _, version} = entry <- Enum.sort(loaded) do
-      IO.write(pad_key(app))
+      IO.write(pad_key(Atom.to_string(app)))
       IO.write(String.pad_trailing("#{version}", 20))
 
       if entry in started do
@@ -626,6 +667,38 @@ defmodule IEx.Helpers do
     :ok
   end
 
+  @allocator_column_padding 12
+  @allocator_size_padding 19
+
+  defp print_runtime_info_topic(:allocators) do
+    allocator_sizes_map = get_allocator_sizes()
+
+    if Enum.empty?(allocator_sizes_map) do
+      IO.puts(IEx.color(:eval_error, "Could not get allocator sizes."))
+    else
+      print_pane("Memory allocators")
+
+      IO.puts([
+        String.duplicate(" ", @allocator_column_padding),
+        String.pad_leading("Block size", @allocator_size_padding),
+        String.pad_leading("Carrier size", @allocator_size_padding),
+        String.pad_leading("Max carrier size", @allocator_size_padding)
+      ])
+
+      print_allocator("Temporary", allocator_sizes_map.temp_alloc)
+      print_allocator("Short-lived", allocator_sizes_map.sl_alloc)
+      print_allocator("STD", allocator_sizes_map.std_alloc)
+      print_allocator("Long-lived", allocator_sizes_map.ll_alloc)
+      print_allocator("Erlang heap", allocator_sizes_map.eheap_alloc)
+      print_allocator("ETS", allocator_sizes_map.ets_alloc)
+      print_allocator("Fix", allocator_sizes_map.fix_alloc)
+      print_allocator("Literal", allocator_sizes_map.literal_alloc)
+      print_allocator("Binary", allocator_sizes_map.binary_alloc)
+      print_allocator("Driver", allocator_sizes_map.driver_alloc)
+      print_allocator("Total", allocator_sizes_map.total)
+    end
+  end
+
   defp print_pane(msg) do
     IO.puts(IEx.color(:eval_result, ["\n## ", msg, " \n"]))
   end
@@ -653,6 +726,71 @@ defmodule IEx.Helpers do
     IO.puts("#{pad_key(key)}#{format_bytes(value)}")
   end
 
+  defp print_allocator(title, %{
+         block_size: block_size,
+         carrier_size: carriers_size,
+         max_carrier_size: max_carriers_size
+       }) do
+    IO.puts([
+      String.pad_trailing(title, @allocator_column_padding),
+      block_size |> format_bytes(:KB) |> String.pad_leading(@allocator_size_padding),
+      carriers_size |> format_bytes(:KB) |> String.pad_leading(@allocator_size_padding),
+      max_carriers_size |> format_bytes(:KB) |> String.pad_leading(@allocator_size_padding)
+    ])
+  end
+
+  defp get_allocator_sizes() do
+    alloc_util_allocators = :erlang.system_info(:alloc_util_allocators)
+
+    allocators_map =
+      Map.new(
+        [:total | alloc_util_allocators],
+        &{&1, %{block_size: 0, carrier_size: 0, max_carrier_size: 0}}
+      )
+
+    try do
+      for {allocator, allocator_instances} <-
+            :erlang.system_info({:allocator_sizes, alloc_util_allocators}),
+          {:instance, _, instance_info} <- allocator_instances,
+          {_, [{:blocks, blocks}, {:carriers_size, ccs, mcs, _}]} <-
+            instance_info,
+          reduce: allocators_map do
+        acc ->
+          cbs =
+            case blocks do
+              [{_, [{:size, bs, _, _}]}] -> bs
+              _ -> 0
+            end
+
+          %{
+            block_size: block_size,
+            carrier_size: carrier_size,
+            max_carrier_size: max_carrier_size
+          } = Map.fetch!(acc, allocator)
+
+          %{
+            block_size: total_block_size,
+            carrier_size: total_carrier_size,
+            max_carrier_size: total_max_carrier_size
+          } = Map.fetch!(acc, :total)
+
+          acc
+          |> Map.replace!(allocator, %{
+            block_size: block_size + cbs,
+            carrier_size: carrier_size + ccs,
+            max_carrier_size: max_carrier_size + mcs
+          })
+          |> Map.replace!(:total, %{
+            block_size: total_block_size + cbs,
+            carrier_size: total_carrier_size + ccs,
+            max_carrier_size: total_max_carrier_size + mcs
+          })
+      end
+    rescue
+      _ -> %{}
+    end
+  end
+
   defp format_bytes(bytes) when is_integer(bytes) do
     cond do
       bytes >= memory_unit(:GB) -> format_bytes(bytes, :GB)
@@ -677,7 +815,7 @@ defmodule IEx.Helpers do
   defp memory_unit(:MB), do: 1024 * 1024
   defp memory_unit(:KB), do: 1024
 
-  defp pad_key(key), do: String.pad_trailing("#{key}:", 20, " ")
+  defp pad_key(key), do: String.pad_trailing(key, 21, " ")
 
   @doc """
   Clears out all messages sent to the shell's inbox and prints them out.
@@ -895,6 +1033,14 @@ defmodule IEx.Helpers do
     next()
   end
 
+  @doc """
+  A shortcut for `continue/0`.
+  """
+  @doc since: "1.17.0"
+  def c do
+    continue()
+  end
+
   @doc """
   Sets up a breakpoint in the AST of shape `Module.function/arity`
   with the given number of `stops`.
@@ -995,7 +1141,7 @@ defmodule IEx.Helpers do
   end
 
   @doc """
-  Sets the number of pending stops in the breakpoint
+  Resets the number of pending stops in the breakpoint
   with the given `id` to zero.
 
   Returns `:ok` if there is such breakpoint ID. `:not_found`
@@ -1009,7 +1155,7 @@ defmodule IEx.Helpers do
   defdelegate reset_break(id), to: IEx.Pry
 
   @doc """
-  Sets the number of pending stops in the given module,
+  Resets the number of pending stops in the given module,
   function and arity to zero.
 
   If the module is not instrumented or if the given function
@@ -1241,12 +1387,20 @@ defmodule IEx.Helpers do
       iex> pid("0.21.32")
       #PID<0.21.32>
 
+      iex> pid("#PID<0.21.32>")
+      #PID<0.21.32>
+
       iex> pid(:init)
       #PID<0.0.0>
 
   """
+  @spec pid(binary | atom) :: pid()
+  def pid("#PID<" <> string) do
+    :erlang.list_to_pid(~c"<#{string}")
+  end
+
   def pid(string) when is_binary(string) do
-    :erlang.list_to_pid('<#{string}>')
+    :erlang.list_to_pid(~c"<#{string}>")
   end
 
   def pid(name) when is_atom(name) do
@@ -1268,12 +1422,13 @@ defmodule IEx.Helpers do
       #PID<0.64.2048>
 
   """
+  @spec pid(non_neg_integer, non_neg_integer, non_neg_integer) :: pid()
   def pid(x, y, z)
       when is_integer(x) and x >= 0 and is_integer(y) and y >= 0 and is_integer(z) and z >= 0 do
     :erlang.list_to_pid(
-      '<' ++
+      ~c"<" ++
         Integer.to_charlist(x) ++
-        '.' ++ Integer.to_charlist(y) ++ '.' ++ Integer.to_charlist(z) ++ '>'
+        ~c"." ++ Integer.to_charlist(y) ++ ~c"." ++ Integer.to_charlist(z) ++ ~c">"
     )
   end
 
@@ -1287,8 +1442,9 @@ defmodule IEx.Helpers do
 
   """
   @doc since: "1.8.0"
+  @spec port(binary) :: port()
   def port(string) when is_binary(string) do
-    :erlang.list_to_port('#Port<#{string}>')
+    :erlang.list_to_port(~c"#Port<#{string}>")
   end
 
   @doc """
@@ -1303,10 +1459,11 @@ defmodule IEx.Helpers do
 
   """
   @doc since: "1.8.0"
+  @spec port(non_neg_integer, non_neg_integer) :: port()
   def port(major, minor)
       when is_integer(major) and major >= 0 and is_integer(minor) and minor >= 0 do
     :erlang.list_to_port(
-      '#Port<' ++ Integer.to_charlist(major) ++ '.' ++ Integer.to_charlist(minor) ++ '>'
+      ~c"#Port<" ++ Integer.to_charlist(major) ++ ~c"." ++ Integer.to_charlist(minor) ++ ~c">"
     )
   end
 
@@ -1320,8 +1477,9 @@ defmodule IEx.Helpers do
 
   """
   @doc since: "1.6.0"
+  @spec ref(binary) :: reference()
   def ref(string) when is_binary(string) do
-    :erlang.list_to_ref('#Ref<#{string}>')
+    :erlang.list_to_ref(~c"#Ref<#{string}>")
   end
 
   @doc """
@@ -1334,15 +1492,16 @@ defmodule IEx.Helpers do
 
   """
   @doc since: "1.6.0"
+  @spec ref(non_neg_integer, non_neg_integer, non_neg_integer, non_neg_integer) :: reference()
   def ref(w, x, y, z)
       when is_integer(w) and w >= 0 and is_integer(x) and x >= 0 and is_integer(y) and y >= 0 and
              is_integer(z) and z >= 0 do
     :erlang.list_to_ref(
-      '#Ref<' ++
+      ~c"#Ref<" ++
         Integer.to_charlist(w) ++
-        '.' ++
+        ~c"." ++
         Integer.to_charlist(x) ++
-        '.' ++ Integer.to_charlist(y) ++ '.' ++ Integer.to_charlist(z) ++ '>'
+        ~c"." ++ Integer.to_charlist(y) ++ ~c"." ++ Integer.to_charlist(z) ++ ~c">"
     )
   end
 
@@ -1373,15 +1532,14 @@ defmodule IEx.Helpers do
   def nl(nodes \\ Node.list(), module) when is_list(nodes) and is_atom(module) do
     case get_beam_and_path(module) do
       {bin, beam_path} ->
+        call = :erpc.multicall(nodes, :code, :load_binary, [module, beam_path, bin])
+
         results =
-          for node <- nodes do
-            case :rpc.call(node, :code, :load_binary, [module, beam_path, bin]) do
-              {:module, _} -> {node, :loaded, module}
-              {:badrpc, message} -> {node, :badrpc, message}
-              {:error, message} -> {node, :error, message}
-              unexpected -> {node, :error, unexpected}
-            end
-          end
+          Enum.zip_with(nodes, call, fn
+            node, {:ok, {:module, _}} -> {node, :loaded, module}
+            node, {:ok, {:error, reason}} -> {node, :error, reason}
+            node, {:error, {:erpc, reason}} -> {node, :badrpc, reason}
+          end)
 
         {:ok, results}
 
@@ -1392,7 +1550,8 @@ defmodule IEx.Helpers do
 
   defp get_beam_and_path(module) do
     with {^module, beam, filename} <- :code.get_object_code(module),
-         {:ok, ^module} <- beam |> :beam_lib.info() |> Keyword.fetch(:module) do
+         info_pairs when is_list(info_pairs) <- :beam_lib.info(beam),
+         {:ok, ^module} <- Keyword.fetch(info_pairs, :module) do
       {beam, filename}
     else
       _ -> :error
