@@ -2,10 +2,10 @@ defprotocol IEx.Info do
   @fallback_to_any true
 
   @moduledoc """
-  A protocol to print information in IEx about the given datastructure.
+  A protocol to print information in IEx about the given data structure.
 
   `IEx.Helpers.i/1` uses this protocol to display a term-specific list
-  of of information.
+  of information.
 
   By default, an `Any` implementation will be used which returns
   the `"Data type"`, `"Description"` and `"Reference modules"` sections.
@@ -31,12 +31,12 @@ defprotocol IEx.Info do
     * `"Data type"`: Name of the data type. Usually the name of the module
        defining the data type.
     * `"Description"`: One or a few sentences describing what the data type represents.
-    * `"Reference modules`: One or a few comma-separated module names that focus
+    * `"Reference modules"`: One or a few comma-separated module names that focus
       on working with this datatype.
 
   Other recommended sections are:
 
-    * `"Raw representation`: showing another way of writing the passed `term`.
+    * `"Raw representation"`: showing another way of writing the passed `term`.
       This is mostly relevant for data-structures whose `String.Chars`-implementations
       make use of sigils or other syntactic sugar.
   """
@@ -89,7 +89,7 @@ defimpl IEx.Info, for: Atom do
 
       {^atom, beam, _path} ->
         info = :beam_lib.info(beam)
-        Keyword.fetch(info, :module) == {:ok, atom}
+        is_list(info) and Keyword.fetch(info, :module) == {:ok, atom}
     end
   end
 
@@ -181,10 +181,11 @@ defimpl IEx.Info, for: List do
 
   defp info_printable_charlist(charlist) do
     description = """
-    This is a list of integers that is printed as a sequence of characters
-    delimited by single quotes because all the integers in it represent printable
-    ASCII characters. Conventionally, a list of Unicode code points is known as a
-    charlist and a list of ASCII characters is a subset of it.
+    This is a list of integers that is printed using the `~c` sigil syntax,
+    defined by the `Kernel.sigil_c/2` macro, because all the integers in it
+    represent printable ASCII characters. Conventionally, a list of Unicode
+    code points is known as a charlist and a list of ASCII characters is a
+    subset of it.
     """
 
     [
@@ -206,9 +207,9 @@ defimpl IEx.Info, for: List do
   defp info_improper_list(_improper_list) do
     description = """
     This is what is referred to as an "improper list". An improper list is a
-    list which its last tail is not to an empty list. For example: [1, 2, 3]
-    is a proper list, as it is equivalent to [1, 2, 3 | []], as opposed to
-    [1, 2 | 3] which is an improper list since its last tail returns 3.
+    list whose last tail is not an empty list. For example: [1, 2, 3] is a
+    proper list, as it is equivalent to [1, 2, 3 | []], as opposed to
+    [1, 2 | 3] which is an improper list since its last tail is 3.
     """
 
     [
@@ -345,7 +346,11 @@ defimpl IEx.Info, for: PID do
 
   def info(pid) do
     extra_info =
-      case :rpc.pinfo(pid, @keys) do
+      try do
+        :erpc.call(node(pid), :erlang, :process_info, [pid, @keys])
+      catch
+        _, _ -> [{"Alive", false}]
+      else
         [_ | _] = info ->
           [
             {"Alive", true},
@@ -381,11 +386,19 @@ end
 
 defimpl IEx.Info, for: Port do
   def info(port) do
-    connected = :rpc.call(node(port), :erlang, :port_info, [port, :connected])
+    open? =
+      try do
+        :erpc.call(node(port), :erlang, :port_info, [port, :connected])
+      catch
+        _, _ -> false
+      else
+        {:connected, _} -> true
+        _ -> false
+      end
 
     [
       {"Data type", "Port"},
-      {"Open", match?({:connected, _}, connected)},
+      {"Open", open?},
       {"Reference modules", "Port"}
     ]
   end
@@ -397,12 +410,16 @@ defimpl IEx.Info, for: Reference do
   end
 end
 
-defimpl IEx.Info, for: [Date, Time, NaiveDateTime] do
-  {sigil, repr} =
+defimpl IEx.Info, for: [Date, Time, DateTime, NaiveDateTime, Regex] do
+  naive_datetime_repr = ~S{"naive" datetime (that is, a datetime without a time zone)}
+
+  {sigil, repr, modules} =
     case @for do
-      Date -> {"D", "date"}
-      Time -> {"T", "time"}
-      NaiveDateTime -> {"N", ~S{"naive" datetime (that is, a datetime without a time zone)}}
+      Date -> {"D", "date", [Calendar, Map]}
+      Time -> {"T", "time", [Calendar, Map]}
+      DateTime -> {"U", "datetime", [Calendar, Map]}
+      NaiveDateTime -> {"N", naive_datetime_repr, [Calendar, Map]}
+      Regex -> {"r", "regular expression", [:re]}
     end
 
   def info(value) do
@@ -416,7 +433,31 @@ defimpl IEx.Info, for: [Date, Time, NaiveDateTime] do
       {"Data type", inspect(@for)},
       {"Description", description},
       {"Raw representation", raw_inspect(value)},
-      {"Reference modules", inspect(@for) <> ", Calendar, Map"}
+      {"Reference modules", unquote(Enum.map_join([@for | modules], ", ", &inspect/1))}
+    ]
+  end
+
+  defp raw_inspect(value) do
+    value
+    |> Inspect.Any.inspect(%Inspect.Opts{})
+    |> Inspect.Algebra.format(:infinity)
+    |> IO.iodata_to_binary()
+  end
+end
+
+defimpl IEx.Info, for: Range do
+  def info(value) do
+    description = """
+    This is a struct representing a range of numbers. It is commonly
+    defined using the `first..last//step` syntax. The step is not
+    required and defaults to 1.
+    """
+
+    [
+      {"Data type", inspect(@for)},
+      {"Description", description},
+      {"Raw representation", raw_inspect(value)},
+      {"Reference modules", inspect(@for)}
     ]
   end
 
