@@ -1,25 +1,3 @@
-defmodule IEx.State do
-  @moduledoc false
-  # This state is exchanged between IEx.Server and
-  # IEx.Evaluator which is why it is a struct.
-  defstruct parser_state: "",
-            counter: 1,
-            prefix: "iex",
-            on_eof: :stop_evaluator,
-            evaluator_options: [],
-            previous_state: nil
-
-  @type t :: %{
-          __struct__: __MODULE__,
-          parser_state: binary(),
-          counter: pos_integer(),
-          prefix: binary(),
-          on_eof: :stop_evaluator | :halt,
-          evaluator_options: keyword(),
-          previous_state: nil | t()
-        }
-end
-
 defmodule IEx.Server do
   @moduledoc """
   The IEx.Server.
@@ -32,6 +10,14 @@ defmodule IEx.Server do
 
   """
 
+  @doc false
+  defstruct parser_state: "",
+            counter: 1,
+            prefix: "iex",
+            on_eof: :stop_evaluator,
+            evaluator_options: [],
+            expand_fun: nil
+
   @doc """
   Starts a new IEx server session.
 
@@ -41,13 +27,17 @@ defmodule IEx.Server do
     * `:env` - the `Macro.Env` used for the evaluator
     * `:binding` - an initial set of variables for the evaluator
     * `:on_eof` - if it should `:stop_evaluator` (default) or `:halt` the system
+    * `:register` - if this shell should be registered in the broker (default is `true`)
 
   """
   @doc since: "1.8.0"
   @spec run(keyword) :: :ok
   def run(opts) when is_list(opts) do
-    IEx.Broker.register(self())
-    run_without_registration(opts)
+    if Keyword.get(opts, :register, true) do
+      IEx.Broker.register(self())
+    end
+
+    run_without_registration(init_state(opts), opts, nil)
   end
 
   ## Private APIs
@@ -71,15 +61,16 @@ defmodule IEx.Server do
   defp shell_loop(opts, pid, ref) do
     receive do
       {:take_over, take_pid, take_ref, take_location, take_whereami, take_opts} ->
-        if take_over?(take_pid, take_ref, take_location, take_whereami, take_opts) do
-          take_opts = Keyword.put(take_opts, :previous_state, iex_state(opts))
-          run_without_registration(take_opts)
+        IO.puts(IEx.color(:eval_interrupt, "Break reached: #{take_location}#{take_whereami}"))
+
+        if take_over?(take_pid, take_ref, 1, true) do
+          run_without_registration(init_state(opts), take_opts, nil)
         else
           shell_loop(opts, pid, ref)
         end
 
       {:DOWN, ^ref, :process, ^pid, :normal} ->
-        run_without_registration(opts)
+        run_without_registration(init_state(opts), opts, nil)
 
       {:DOWN, ^ref, :process, ^pid, _reason} ->
         :ok
@@ -88,7 +79,7 @@ defmodule IEx.Server do
 
   # Since we want to register only once, this function is the
   # reentrant point for starting a new shell (instead of run/run_from_shell).
-  defp run_without_registration(opts) do
+  defp run_without_registration(state, opts, input) do
     Process.flag(:trap_exit, true)
     Process.link(Process.group_leader())
 
@@ -96,19 +87,16 @@ defmodule IEx.Server do
       "Interactive Elixir (#{System.version()}) - press Ctrl+C to exit (type h() ENTER for help)"
     )
 
-    evaluator = start_evaluator(opts)
-    loop(iex_state(opts), :ok, evaluator, Process.monitor(evaluator))
+    evaluator = start_evaluator(state.counter, Keyword.merge(state.evaluator_options, opts))
+    loop(state, :ok, evaluator, Process.monitor(evaluator), input)
   end
 
   # Starts an evaluator using the provided options.
   # Made public but undocumented for testing.
   @doc false
-  @spec start_evaluator(keyword) :: pid
-  def start_evaluator(opts) do
-    evaluator =
-      opts[:evaluator] ||
-        :proc_lib.start(IEx.Evaluator, :init, [:ack, self(), Process.group_leader(), opts])
-
+  def start_evaluator(counter, opts) do
+    args = [:ack, self(), Process.group_leader(), counter, opts]
+    evaluator = opts[:evaluator] || :proc_lib.start(IEx.Evaluator, :init, args)
     Process.put(:evaluator, evaluator)
     evaluator
   end
@@ -122,51 +110,48 @@ defmodule IEx.Server do
     :ok
   end
 
-  defp rerun(opts, evaluator, evaluator_ref, input) do
-    kill_input(input)
+  defp rerun(state, opts, evaluator, evaluator_ref, input) do
     IO.puts("")
     stop_evaluator(evaluator, evaluator_ref)
-    run_without_registration(opts)
+    state = reset_state(state)
+    run_without_registration(state, opts, input)
   end
 
-  defp loop(state, prompt, evaluator, evaluator_ref) do
-    self_pid = self()
-    counter = state.counter
-
-    {prompt_type, prefix} =
-      if prompt == :incomplete do
-        {:continuation_prompt, "..."}
-      else
-        {:prompt, state.prefix}
-      end
-
-    input = spawn(fn -> io_get(self_pid, prompt_type, prefix, counter) end)
+  defp loop(state, status, evaluator, evaluator_ref, input) do
+    :io.setopts(expand_fun: state.expand_fun)
+    input = input || io_get(prompt(status, state.prefix, state.counter))
     wait_input(state, evaluator, evaluator_ref, input)
   end
 
   defp wait_input(state, evaluator, evaluator_ref, input) do
     receive do
-      {:input, ^input, code} when is_binary(code) ->
-        send(evaluator, {:eval, self(), code, state})
+      {:io_reply, ^input, code} when is_binary(code) ->
+        :io.setopts(expand_fun: fn _ -> {:yes, [], []} end)
+        send(evaluator, {:eval, self(), code, state.counter, state.parser_state})
         wait_eval(state, evaluator, evaluator_ref)
 
-      {:input, ^input, :eof} ->
+      {:io_reply, ^input, :eof} ->
         case state.on_eof do
           :halt -> System.halt(0)
           :stop_evaluator -> stop_evaluator(evaluator, evaluator_ref)
         end
 
       # Triggered by pressing "i" as the job control switch
-      {:input, ^input, {:error, :interrupted}} ->
+      {:io_reply, ^input, {:error, :interrupted}} ->
         io_error("** (EXIT) interrupted")
-        loop(%{state | parser_state: ""}, :new, evaluator, evaluator_ref)
+        loop(%{state | parser_state: ""}, :ok, evaluator, evaluator_ref, nil)
+
+      # Unknown IO message
+      {:io_reply, ^input, msg} ->
+        io_error("** (EXIT) unknown IO message: #{inspect(msg)}")
+        loop(%{state | parser_state: ""}, :ok, evaluator, evaluator_ref, nil)
 
       # Triggered when IO dies while waiting for input
-      {:input, ^input, {:error, :terminated}} ->
+      {:DOWN, ^input, _, _, _} ->
         stop_evaluator(evaluator, evaluator_ref)
 
       msg ->
-        handle_take_over(msg, state, evaluator, evaluator_ref, input, fn state ->
+        handle_common(msg, state, evaluator, evaluator_ref, input, fn state ->
           wait_input(state, evaluator, evaluator_ref, input)
         end)
     end
@@ -174,21 +159,23 @@ defmodule IEx.Server do
 
   defp wait_eval(state, evaluator, evaluator_ref) do
     receive do
-      {:evaled, ^evaluator, status, new_state} ->
-        loop(new_state, status, evaluator, evaluator_ref)
+      {:evaled, ^evaluator, status, parser_state} ->
+        counter = if(status == :ok, do: state.counter + 1, else: state.counter)
+        state = %{state | counter: counter, parser_state: parser_state}
+        loop(state, status, evaluator, evaluator_ref, nil)
 
       msg ->
-        handle_take_over(msg, state, evaluator, evaluator_ref, nil, fn state ->
+        handle_common(msg, state, evaluator, evaluator_ref, nil, fn state ->
           wait_eval(state, evaluator, evaluator_ref)
         end)
     end
   end
 
-  defp wait_take_over(state, evaluator, evaluator_ref) do
+  defp wait_common(state, evaluator, evaluator_ref, input) do
     receive do
       msg ->
-        handle_take_over(msg, state, evaluator, evaluator_ref, nil, fn state ->
-          wait_take_over(state, evaluator, evaluator_ref)
+        handle_common(msg, state, evaluator, evaluator_ref, input, fn state ->
+          wait_common(state, evaluator, evaluator_ref, input)
         end)
     end
   end
@@ -197,7 +184,7 @@ defmodule IEx.Server do
   #
   # A take process may also happen if the evaluator dies,
   # then a new evaluator is created to replace the dead one.
-  defp handle_take_over(
+  defp handle_common(
          {:take_over, take_pid, take_ref, take_location, take_whereami, take_opts},
          state,
          evaluator,
@@ -209,17 +196,16 @@ defmodule IEx.Server do
       evaluator == take_opts[:evaluator] ->
         IO.puts(IEx.color(:eval_interrupt, "Break reached: #{take_location}#{take_whereami}"))
 
-        if take_over?(take_pid, take_ref, true) do
-          kill_input(input)
-          take_opts = Keyword.put(take_opts, :previous_state, state)
-          loop(iex_state(take_opts), :ok, evaluator, evaluator_ref)
+        if take_over?(take_pid, take_ref, state.counter + 1, true) do
+          # Since we are in process, also bump the counter
+          state = reset_state(bump_counter(state))
+          loop(state, :ok, evaluator, evaluator_ref, input)
         else
           callback.(state)
         end
 
-      take_over?(take_pid, take_ref, take_location, take_whereami, take_opts) ->
-        take_opts = Keyword.put(take_opts, :previous_state, state)
-        rerun(take_opts, evaluator, evaluator_ref, input)
+      take_over?(take_pid, take_ref, take_location, take_whereami, take_opts, state.counter) ->
+        rerun(state, take_opts, evaluator, evaluator_ref, input)
 
       true ->
         callback.(state)
@@ -227,7 +213,7 @@ defmodule IEx.Server do
   end
 
   # User did ^G while the evaluator was busy or stuck
-  defp handle_take_over(
+  defp handle_common(
          {:EXIT, _pid, :interrupt},
          state,
          evaluator,
@@ -235,16 +221,12 @@ defmodule IEx.Server do
          input,
          _callback
        ) do
-    kill_input(input)
     io_error("** (EXIT) interrupted")
-    Process.delete(:evaluator)
     Process.exit(evaluator, :kill)
-    Process.demonitor(evaluator_ref, [:flush])
-    evaluator = start_evaluator(state.evaluator_options)
-    loop(%{state | parser_state: ""}, :ok, evaluator, Process.monitor(evaluator))
+    rerun(state, [], evaluator, evaluator_ref, input)
   end
 
-  defp handle_take_over(
+  defp handle_common(
          {:EXIT, pid, reason},
          state,
          evaluator,
@@ -260,16 +242,11 @@ defmodule IEx.Server do
     end
   end
 
-  defp handle_take_over({:respawn, evaluator}, state, evaluator, evaluator_ref, input, _callback) do
-    opts =
-      if state.previous_state,
-        do: state.previous_state.evaluator_options,
-        else: state.evaluator_options
-
-    rerun(opts, evaluator, evaluator_ref, input)
+  defp handle_common({:respawn, evaluator}, state, evaluator, evaluator_ref, input, _callback) do
+    rerun(bump_counter(state), [], evaluator, evaluator_ref, input)
   end
 
-  defp handle_take_over(
+  defp handle_common(
          {:continue, evaluator, next?},
          state,
          evaluator,
@@ -277,28 +254,11 @@ defmodule IEx.Server do
          input,
          _callback
        ) do
-    kill_input(input)
     send(evaluator, {:done, self(), next?})
-    wait_take_over(state, evaluator, evaluator_ref)
+    wait_common(state, evaluator, evaluator_ref, input)
   end
 
-  defp handle_take_over(
-         {:DOWN, evaluator_ref, :process, evaluator, :normal},
-         state,
-         evaluator,
-         evaluator_ref,
-         input,
-         _callback
-       ) do
-    opts =
-      if state.previous_state,
-        do: state.previous_state.evaluator_options,
-        else: state.evaluator_options
-
-    rerun(opts, evaluator, evaluator_ref, input)
-  end
-
-  defp handle_take_over(
+  defp handle_common(
          {:DOWN, evaluator_ref, :process, evaluator, reason},
          state,
          evaluator,
@@ -316,27 +276,23 @@ defmodule IEx.Server do
         io_error("** (IEx.Error) #{type} when printing EXIT message: #{inspect(detail)}")
     end
 
-    opts =
-      if state.previous_state,
-        do: state.previous_state.evaluator_options,
-        else: state.evaluator_options
-
-    rerun(opts, evaluator, evaluator_ref, input)
+    rerun(state, [], evaluator, evaluator_ref, input)
   end
 
-  defp handle_take_over(_, state, _evaluator, _evaluator_ref, _input, callback) do
+  defp handle_common(_, state, _evaluator, _evaluator_ref, _input, callback) do
     callback.(state)
   end
 
-  defp take_over?(take_pid, take_ref, take_location, take_whereami, take_opts) do
-    evaluator = take_opts[:evaluator]
+  defp take_over?(take_pid, take_ref, take_location, take_whereami, take_opts, counter) do
+    evaluator = take_opts[:evaluator] || self()
     message = "Request to pry #{inspect(evaluator)} at #{take_location}#{take_whereami}"
     interrupt = IEx.color(:eval_interrupt, "#{message}\nAllow? [Yn] ")
-    take_over?(take_pid, take_ref, yes?(IO.gets(:stdio, interrupt)))
+    answer = yes?(IO.gets(:stdio, interrupt))
+    take_over?(take_pid, take_ref, counter, answer)
   end
 
-  defp take_over?(take_pid, take_ref, take_response) when is_boolean(take_response) do
-    case IEx.Broker.respond(take_pid, take_ref, take_response) do
+  defp take_over?(take_pid, take_ref, counter, response) when is_boolean(response) do
+    case IEx.Broker.respond(take_pid, take_ref, counter, response) do
       :ok ->
         true
 
@@ -349,40 +305,70 @@ defmodule IEx.Server do
     end
   end
 
-  defp kill_input(nil), do: :ok
-  defp kill_input(input), do: Process.exit(input, :kill)
+  defp yes?(string) when is_binary(string),
+    do: String.trim(string) in ["", "y", "Y", "yes", "YES", "Yes"]
 
-  defp yes?(string) do
-    is_binary(string) and String.trim(string) in ["", "y", "Y", "yes", "YES", "Yes"]
-  end
+  defp yes?(charlist) when is_list(charlist),
+    do: yes?(List.to_string(charlist))
+
+  defp yes?(_), do: false
 
   ## State
 
-  defp iex_state(opts) do
+  defp init_state(opts) do
     prefix = Keyword.get(opts, :prefix, "iex")
     on_eof = Keyword.get(opts, :on_eof, :stop_evaluator)
+    gl = Process.group_leader()
 
-    %IEx.State{
+    expand_fun =
+      if node(gl) != node() do
+        IEx.Autocomplete.remsh(node())
+      else
+        &IEx.Autocomplete.expand/1
+      end
+
+    %IEx.Server{
       prefix: prefix,
       on_eof: on_eof,
-      previous_state: Keyword.get(opts, :previous_state),
-      evaluator_options: Keyword.take(opts, [:dot_iex_path])
+      expand_fun: expand_fun,
+      evaluator_options: Keyword.take(opts, [:dot_iex])
     }
   end
 
+  # For the state, reset only reset the parser state.
+  # The counter will continue going up as the input process is shared.
+  # The opts can also set "dot_iex" and the "evaluator" itself,
+  # but those are not stored: they are temporary to whatever is rerunning.
+  # Once the rerunning session restarts, we keep the same evaluator_options
+  # and rollback to a new evaluator.
+  defp reset_state(state) do
+    %{state | parser_state: ""}
+  end
+
+  defp bump_counter(state) do
+    update_in(state.counter, &(&1 + 1))
+  end
+
   ## IO
 
-  defp io_get(pid, prompt_type, prefix, counter) do
-    prompt = prompt(prompt_type, prefix, counter)
-    send(pid, {:input, self(), IO.gets(:stdio, prompt)})
+  defp io_get(prompt) do
+    gl = Process.group_leader()
+    ref = Process.monitor(gl)
+    command = {:get_until, :unicode, prompt, __MODULE__, :__parse__, []}
+    send(gl, {:io_request, self(), ref, command})
+    ref
   end
 
-  defp prompt(prompt_type, prefix, counter) do
+  @doc false
+  def __parse__([], :eof), do: {:done, :eof, []}
+  def __parse__([], chars), do: {:done, List.to_string(chars), []}
+
+  defp prompt(status, prefix, counter) do
     {mode, prefix} =
       if Node.alive?() do
-        {prompt_mode(prompt_type, :alive), prefix || remote_prefix()}
+        {prompt_mode(status, :alive), default_prefix(status, prefix)}
       else
-        {prompt_mode(prompt_type, :default), prefix || "iex"}
+        {prompt_mode(status, :default), default_prefix(status, prefix)}
       end
 
     prompt =
@@ -391,19 +377,18 @@ defmodule IEx.Server do
       |> String.replace("%prefix", to_string(prefix))
       |> String.replace("%node", to_string(node()))
 
-    prompt <> " "
+    [prompt, " "]
   end
 
-  defp prompt_mode(:prompt, :default), do: :default_prompt
-  defp prompt_mode(:prompt, :alive), do: :alive_prompt
-  defp prompt_mode(:continuation_prompt, :default), do: :continuation_prompt
-  defp prompt_mode(:continuation_prompt, :alive), do: :alive_continuation_prompt
+  defp default_prefix(:incomplete, _prefix), do: "..."
+  defp default_prefix(_ok_or_error, prefix), do: prefix
+
+  defp prompt_mode(:incomplete, :default), do: :continuation_prompt
+  defp prompt_mode(:incomplete, :alive), do: :alive_continuation_prompt
+  defp prompt_mode(_ok_or_error, :default), do: :default_prompt
+  defp prompt_mode(_ok_or_error, :alive), do: :alive_prompt
 
   defp io_error(result) do
     IO.puts(:stdio, IEx.color(:eval_error, result))
   end
-
-  defp remote_prefix do
-    if node() == node(Process.group_leader()), do: "iex", else: "rem"
-  end
 end
