@@ -19,7 +19,6 @@ defmodule IEx.InfoTest do
     test "loaded module (without docs)" do
       info = Info.info(Foo)
       assert get_key(info, "Data type") == "Atom"
-      assert get_key(info, "Source") == Path.relative_to_cwd(__ENV__.file)
 
       assert get_key(info, "Description") ==
                "Call IEx.InfoTest.Foo.module_info() to access metadata."
@@ -27,6 +26,12 @@ defmodule IEx.InfoTest do
       assert get_key(info, "Raw representation") == ~s(:"Elixir.IEx.InfoTest.Foo")
     end
 
+    @tag :requires_source
+    test "loaded module (with source)" do
+      info = Info.info(Foo)
+      assert get_key(info, "Source") == Path.relative_to_cwd(__ENV__.file)
+    end
+
     test "loaded module (with docs)" do
       info = Info.info(List)
 
@@ -62,7 +67,7 @@ defmodule IEx.InfoTest do
 
   describe "lists" do
     test "charlists" do
-      info = Info.info('foo')
+      info = Info.info(~c"foo")
       assert get_key(info, "Description") =~ "This is a list of integers that is printed"
       assert get_key(info, "Raw representation") == "[102, 111, 111]"
     end
@@ -190,6 +195,21 @@ defmodule IEx.InfoTest do
       assert get_key(info, "Description") =~ "`~T`"
     end
 
+    test "datetime" do
+      {:ok, date} = Date.new(2017, 1, 1)
+      {:ok, time} = Time.new(23, 59, 59)
+      {:ok, date_time} = DateTime.new(date, time)
+      info = Info.info(date_time)
+      assert get_key(info, "Data type") == "DateTime"
+
+      assert get_key(info, "Raw representation") ==
+               "%DateTime{year: 2017, month: 1, day: 1, hour: 23, minute: 59, second: 59, time_zone: \"Etc/UTC\", zone_abbr: \"UTC\", utc_offset: 0, std_offset: 0, microsecond: {0, 0}, calendar: Calendar.ISO}"
+
+      assert get_key(info, "Reference modules") == "DateTime, Calendar, Map"
+      assert get_key(info, "Description") =~ "a datetime"
+      assert get_key(info, "Description") =~ "`~U`"
+    end
+
     test "naive datetime" do
       {:ok, time} = NaiveDateTime.new(2017, 1, 1, 23, 59, 59)
       info = Info.info(time)
@@ -207,6 +227,22 @@ defmodule IEx.InfoTest do
     end
   end
 
+  test "Regex" do
+    info = Info.info(~r/(ab)+c/)
+    assert get_key(info, "Data type") == "Regex"
+    assert get_key(info, "Description")
+    assert get_key(info, "Raw representation") =~ "%Regex{re_pattern: {:re_pattern, "
+    assert get_key(info, "Reference modules") == "Regex, :re"
+  end
+
+  test "Range" do
+    info = Info.info(1..10//2)
+    assert get_key(info, "Data type") == "Range"
+    assert get_key(info, "Description")
+    assert get_key(info, "Raw representation") == "%Range{first: 1, last: 10, step: 2}"
+    assert get_key(info, "Reference modules") == "Range"
+  end
+
   test "structs" do
     info = Info.info(%Foo{})
     assert get_key(info, "Data type") == "IEx.InfoTest.Foo"
