@@ -5,7 +5,7 @@ defmodule IEx.InteractionTest do
 
   test "whole output" do
     assert capture_io("IO.puts \"Hello world\"", fn ->
-             IEx.Server.run(dot_iex_path: "")
+             IEx.Server.run(dot_iex: "")
            end) =~
              "Interactive Elixir (#{System.version()}) - press Ctrl+C to exit (type h() ENTER for help)" <>
                "\niex(1)> Hello world\n:ok\niex(2)>"
@@ -19,9 +19,38 @@ defmodule IEx.InteractionTest do
     assert capture_iex("1 + 2") == "3"
   end
 
+  test "omits error color if exception has ansi reset character" do
+    enabled? = IO.ANSI.enabled?()
+    Application.put_env(:elixir, :ansi_enabled, true)
+
+    expected = """
+    ** (SyntaxError) invalid syntax found on iex:1:4:
+        \e[31merror:\e[0m syntax error before: '='
+        │
+      1 │ a += 2
+        │ \e[31m   ^\e[0m
+        │
+        └─ iex:1:4
+    """
+
+    opts = [colors: [enabled: true]]
+    output = capture_iex("a += 2", opts)
+
+    refute String.starts_with?(output, IO.ANSI.red())
+    assert output =~ expected
+
+    Application.put_env(:elixir, :ansi_enabled, enabled?)
+  end
+
   test "invalid input" do
-    assert capture_iex("if true do ) false end") =~
-             "** (SyntaxError) iex:1:12: unexpected token: ). The \"do\" at line 1 is missing terminator \"end\""
+    output = capture_iex("if true do ) false end")
+
+    assert output =~ "** (MismatchedDelimiterError) mismatched delimiter found on iex:1:12:"
+    assert output =~ "unexpected token: )"
+    assert output =~ "iex:1:12"
+    assert output =~ "if true do ) false end"
+    assert output =~ "└ unclosed delimiter"
+    assert output =~ "└ mismatched closing delimiter"
   end
 
   test "multiple vars" do
@@ -61,7 +90,12 @@ defmodule IEx.InteractionTest do
     #iex:break
     """
 
-    assert capture_iex(input) =~ "** (TokenMissingError) iex:1: incomplete expression"
+    output = capture_iex(input)
+
+    assert output =~ "** (TokenMissingError) token missing on iex:1:"
+    assert output =~ "error:"
+    assert output =~ "incomplete expression\n"
+    assert output =~ "iex:1"
   end
 
   test "module definition" do
@@ -124,7 +158,15 @@ defmodule IEx.InteractionTest do
   end
 
   test "inspect opts" do
-    opts = [inspect: [binaries: :as_binaries, charlists: :as_lists, structs: false, limit: 4]]
+    opts = [
+      inspect: [
+        binaries: :as_binaries,
+        charlists: :as_lists,
+        structs: false,
+        limit: 4,
+        custom_options: [sort_maps: true]
+      ]
+    ]
 
     assert capture_iex("<<45, 46, 47>>\n[45, 46, 47]\n%IO.Stream{}", opts) ==
              "<<45, 46, 47>>\n[45, 46, 47]\n%{__struct__: IO.Stream, device: nil, line_or_bytes: :line, raw: true}"
@@ -142,7 +184,9 @@ defmodule IEx.InteractionTest do
   test "exception while invoking conflicting helpers" do
     import File, only: [open: 1], warn: false
 
-    assert capture_iex("open('README.md')", [], env: __ENV__) =~
+    assert capture_io(:stderr, fn ->
+             capture_iex("open('README.md')", [], env: __ENV__)
+           end) =~
              ~r"function open/1 imported from both File and IEx.Helpers"
   end
 
@@ -154,6 +198,16 @@ defmodule IEx.InteractionTest do
              ~r"\*\* \(EXIT from #PID<\d+\.\d+\.\d+>\) shell process exited with reason: {:bye, \[:world\]}"
   end
 
+  test "receive normal exits" do
+    assert capture_iex("spawn_link(fn -> exit(:normal) end); Process.sleep(1000)") =~ ":ok"
+
+    assert capture_iex("spawn_link(fn -> exit(:shutdown) end); Process.sleep(1000)") =~
+             ~r"\*\* \(EXIT from #PID<\d+\.\d+\.\d+>\) shell process exited with reason: shutdown"
+
+    assert capture_iex("spawn_link(fn -> exit({:shutdown, :bye}) end); Process.sleep(1000)") =~
+             ~r"\*\* \(EXIT from #PID<\d+\.\d+\.\d+>\) shell process exited with reason: shutdown: :bye"
+  end
+
   test "receive exit from exception" do
     # use exit/1 to fake an error so that an error message
     # is not sent to the error logger.
@@ -198,10 +252,8 @@ defmodule IEx.InteractionTest do
 
   describe ".iex" do
     test "no .iex" do
-      capture_io(:stderr, fn ->
-        assert "** (CompileError) iex:1: undefined function my_variable/0" <> _ =
-                 capture_iex("my_variable")
-      end)
+      assert capture_io(:stderr, fn -> capture_iex("my_variable") end) =~
+               "undefined variable \"my_variable\""
     end
 
     @tag :tmp_dir
@@ -215,7 +267,7 @@ defmodule IEx.InteractionTest do
         my_variable = 42
         """)
 
-      assert capture_iex("{my_fun_single(), my_variable}", [], dot_iex_path: path) ==
+      assert capture_iex("{my_fun_single(), my_variable}", [], dot_iex: path) ==
                "{:single, 42}"
     end
 
@@ -233,7 +285,22 @@ defmodule IEx.InteractionTest do
         write_dot_iex!(tmp_dir, "dot-iex", "import_file \"#{tmp_dir}/dot-iex-1\"\nmy_variable=13")
 
       input = "nested_var\nmy_variable\nmy_fun_nested()"
-      assert capture_iex(input, [], dot_iex_path: path) == "42\n13\n:nested"
+      assert capture_iex(input, [], dot_iex: path) == "42\n13\n:nested"
+    end
+
+    @tag :tmp_dir
+    test "configured .iex", %{tmp_dir: tmp_dir} do
+      path =
+        write_dot_iex!(tmp_dir, "configured-dot-iex", """
+        defmodule ConfiguredDotIEx do
+          def my_fun_single, do: :single
+        end
+        import ConfiguredDotIEx
+        my_variable = 42
+        """)
+
+      assert capture_iex("{my_fun_single(), my_variable}", [dot_iex: path], dot_iex: nil) ==
+               "{:single, 42}"
     end
   end
 
