@@ -1,10 +1,33 @@
 assert_timeout = String.to_integer(System.get_env("ELIXIR_ASSERT_TIMEOUT") || "500")
-
 System.put_env("ELIXIR_EDITOR", "echo")
 
 {:ok, _} = Application.ensure_all_started(:iex)
 IEx.configure(colors: [enabled: false])
-ExUnit.start(trace: "--trace" in System.argv(), assert_receive_timeout: assert_timeout)
+
+{line_exclude, line_include} =
+  if line = System.get_env("LINE"), do: {[:test], [line: line]}, else: {[], []}
+
+erlang_doc_exclude =
+  if match?({:docs_v1, _, _, _, %{}, _, _}, Code.fetch_docs(:array)) do
+    []
+  else
+    IO.puts("Erlang/OTP compiled without docs, some tests are excluded...")
+    [:erlang_doc]
+  end
+
+source_exclude =
+  if :deterministic in :compile.env_compiler_options() do
+    [:requires_source]
+  else
+    []
+  end
+
+ExUnit.start(
+  assert_receive_timeout: assert_timeout,
+  trace: !!System.get_env("TRACE"),
+  include: line_include,
+  exclude: line_exclude ++ erlang_doc_exclude ++ source_exclude
+)
 
 defmodule IEx.Case do
   use ExUnit.CaseTemplate
@@ -36,13 +59,13 @@ defmodule IEx.Case do
     end
   end
 
-  keys = [:default_prompt, :alive_prompt, :inspect, :colors, :history_size]
-  @iex_env Application.get_all_env(:iex) |> Keyword.take(keys)
+  @keys [:default_prompt, :alive_prompt, :inspect, :colors, :history_size, :dot_iex]
+  @iex_env Application.get_all_env(:iex) |> Keyword.take(@keys)
 
   setup do
     on_exit(fn ->
       env = @iex_env
-      Enum.each(env, fn {k, _} -> Application.delete_env(:iex, k) end)
+      Enum.each(@keys, &Application.delete_env(:iex, &1))
       IEx.configure(env)
     end)
 
@@ -63,7 +86,7 @@ defmodule IEx.Case do
     IEx.configure(options)
 
     ExUnit.CaptureIO.capture_io([input: input, capture_prompt: capture_prompt], fn ->
-      server_options = Keyword.put_new(server_options, :dot_iex_path, "")
+      server_options = Keyword.put_new(server_options, :dot_iex, "")
       IEx.Server.run(server_options)
     end)
     |> strip_iex()
