@@ -11,24 +11,23 @@ defmodule Logger.MixProject do
 
   def application do
     [
-      registered: [Logger, Logger.BackendSupervisor, Logger.Supervisor, Logger.Watcher],
+      registered: [Logger, Logger.Supervisor],
       mod: {Logger.App, []},
       env: [
         utc_log: false,
         truncate: 8096,
-        backends: [:console],
         translators: [{Logger.Translator, :translate}],
-        sync_threshold: 20,
-        discard_threshold: 500,
         handle_otp_reports: true,
         handle_sasl_reports: false,
-        discard_threshold_periodic_check: 30_000,
-        discard_threshold_for_error_logger: 500,
+        always_evaluate_messages: false,
         compile_time_purge_matching: [],
         compile_time_application: nil,
         translator_inspect_opts: [],
-        console: [],
-        start_options: []
+        start_options: [],
+        sync_threshold: 20,
+        discard_threshold: 500,
+        discard_threshold_periodic_check: 30_000,
+        metadata: []
       ]
     ]
   end
