@@ -4,22 +4,22 @@ defmodule Mix.Compilers.Erlang do
   @manifest_vsn 1
 
   @doc """
-  Compiles the files in `mappings` with given extensions into
-  the destination, automatically invoking the callback for each
-  stale input and output pair (or for all if `force` is `true`) and
-  removing files that no longer have a source, while keeping the
-  `manifest` up to date.
+  Compiles the given `mappings`.
 
-  `mappings` should be a list of tuples in the form of `{src, dest}` paths.
+  `mappings` is a list of `{src, dest}` pairs, where the source
+  extensions are compiled into the destination extension,
+  automatically invoking the callback for each stale pair (or for
+  all if `force` is `true`) and removing files that no longer have
+  a source, while keeping the `manifest` up to date.
 
   ## Options
 
     * `:force` - forces compilation regardless of modification times
 
-    * `:parallel` - if `true` all files will be compiled in parallel,
-      otherwise the given list of source file names will be compiled
-      in parallel, all other files are compiled serially before the
-      parallel files
+    * `:parallel` - a mapset of files to compile in parallel
+
+    * `:preload` - any code that must be preloaded if any pending
+      entry needs to be compiled
 
   ## Examples
 
@@ -54,66 +54,39 @@ defmodule Mix.Compilers.Erlang do
   `{:error, errors, warnings}` in case of error. This function returns
   `{status, diagnostics}` as specified in `Mix.Task.Compiler`.
   """
-  def compile(manifest, mappings, src_ext, dest_ext, opts, callback) when is_list(opts) do
+  def compile(manifest, mappings, src_ext, dest_ext, opts, callback)
+      when is_atom(src_ext) and is_atom(dest_ext) and is_list(opts) do
     force = opts[:force]
 
-    files =
+    entries =
       for {src, dest} <- mappings,
-          target <- extract_targets(src, src_ext, dest, dest_ext, force),
+          target <- extract_entries(src, src_ext, dest, dest_ext, force),
           do: target
 
-    compile(manifest, files, src_ext, opts, callback)
+    if preload = entries != [] && opts[:preload] do
+      preload.()
+    end
+
+    compile_entries(manifest, entries, src_ext, dest_ext, opts, callback)
   end
 
-  def compile(manifest, mappings, src_ext, dest_ext, force, callback)
-      when is_boolean(force) or is_nil(force) do
-    IO.warn(
-      "Mix.Compilers.Erlang.compile/6 with a boolean or nil as 5th argument is deprecated, " <>
-        "please pass [force: true] or [] instead"
-    )
-
-    compile(manifest, mappings, src_ext, dest_ext, [force: force], callback)
+  # TODO: remove me on v1.22
+  @deprecated "Use compile/6 or open an up an issue"
+  def compile(manifest, entries, opts \\ [], callback) do
+    compile_entries(manifest, entries, :erl, :beam, opts, callback)
   end
 
-  @doc """
-  Compiles the given `mappings`.
-
-  `mappings` should be a list of tuples in the form of `{src, dest}`.
-
-  A `manifest` file and a `callback` to be invoked for each src/dest pair
-  must be given. A src/dest pair where destination is `nil` is considered
-  to be up to date and won't be (re-)compiled.
-
-  ## Options
-
-    * `:force` - forces compilation regardless of modification times
-
-    * `:parallel` - if `true` all files will be compiled in parallel,
-      otherwise the given list of source file names will be compiled
-      in parallel, all other files are compiled serially before the
-      parallel files
-
-  """
-  def compile(manifest, mappings, opts \\ [], callback) do
-    compile(manifest, mappings, :erl, opts, callback)
-  end
-
-  defp compile(manifest, mappings, ext, opts, callback) do
+  @doc false
+  def compile_entries(manifest, mappings, src_ext, dest_ext, opts, callback) do
     stale = for {:stale, src, dest} <- mappings, do: {src, dest}
 
     # Get the previous entries from the manifest
     timestamp = System.os_time(:second)
-    entries = read_manifest(manifest)
+    old_entries = entries = read_manifest(manifest)
 
-    # Files to remove are the ones in the manifest
-    # but they no longer have a source
+    # Files to remove are the ones in the manifest but they no longer have a source
     removed =
-      Enum.filter(entries, fn {dest, _} ->
-        not Enum.any?(mappings, fn {_status, _mapping_src, mapping_dest} ->
-          mapping_dest == dest
-        end)
-      end)
-      |> Enum.map(&elem(&1, 0))
+      for {dest, _} <- entries, not List.keymember?(mappings, dest, 2), do: dest
 
     # Remove manifest entries with no source
     Enum.each(removed, &File.rm/1)
@@ -122,20 +95,21 @@ defmodule Mix.Compilers.Erlang do
     # Clear stale and removed files from manifest
     entries =
       Enum.reject(entries, fn {dest, _warnings} ->
-        dest in removed || Enum.any?(stale, fn {_, stale_dest} -> dest == stale_dest end)
+        dest in removed || List.keymember?(stale, dest, 1)
       end)
 
-    if opts[:all_warnings], do: show_warnings(entries)
+    if Keyword.get(opts, :all_warnings, true), do: show_warnings(entries)
 
-    if stale == [] && removed == [] do
+    if stale == [] and removed == [] do
       {:noop, manifest_warnings(entries)}
     else
-      Mix.Utils.compiling_n(length(stale), ext)
+      Mix.Utils.compiling_n(length(stale), src_ext)
       Mix.Project.ensure_structure()
 
       # Let's prepend the newly created path so compiled files
       # can be accessed still during compilation (for behaviours
-      # and what not).
+      # and what not). Note we don't want to cache this path as
+      # we will write to it.
       Code.prepend_path(Mix.Project.compile_path())
 
       {parallel, serial} =
@@ -164,6 +138,19 @@ defmodule Mix.Compilers.Erlang do
       # Return status and diagnostics
       warnings = manifest_warnings(entries) ++ to_diagnostics(warnings, :warning)
 
+      if dest_ext == :beam do
+        lazy_modules_diff = fn ->
+          {changed, added} =
+            stale
+            |> Enum.map(&elem(&1, 1))
+            |> Enum.split_with(fn dest -> List.keymember?(old_entries, dest, 0) end)
+
+          modules_diff(added, changed, removed, timestamp)
+        end
+
+        Mix.Task.Compiler.notify_modules_compiled(lazy_modules_diff)
+      end
+
       case status do
         :ok ->
           {:ok, warnings}
@@ -175,22 +162,11 @@ defmodule Mix.Compilers.Erlang do
     end
   end
 
-  @doc """
-  Ensures the native OTP application is available.
-  """
-  def ensure_application!(app, input) do
-    case Application.ensure_all_started(app) do
-      {:ok, _} ->
-        :ok
-
-      {:error, _} ->
-        Mix.raise(
-          "Could not compile #{inspect(Path.relative_to_cwd(input))} because " <>
-            "the application \"#{app}\" could not be found. This may happen if " <>
-            "your package manager broke Erlang into multiple packages and may " <>
-            "be fixed by installing the missing \"erlang-dev\" and \"erlang-#{app}\" packages"
-        )
-    end
+  # TODO: Deprecate this in favor of `Mix.ensure_application!/1` in Elixir v1.19.
+  @doc false
+  def ensure_application!(app, _input) do
+    Mix.ensure_application!(app)
+    {:ok, _} = Application.ensure_all_started(app)
   end
 
   @doc """
@@ -224,7 +200,22 @@ defmodule Mix.Compilers.Erlang do
     end
   end
 
-  defp extract_targets(src_dir, src_ext, dest_dir, dest_ext, force) do
+  @doc """
+  Returns the output paths in the manifest.
+  """
+  def outputs(manifest) do
+    manifest |> read_manifest() |> Enum.map(&elem(&1, 0))
+  end
+
+  @doc """
+  Retrieves all diagnostics from the given manifest.
+  """
+  def diagnostics(manifest) do
+    entries = read_manifest(manifest)
+    manifest_warnings(entries)
+  end
+
+  defp extract_entries(src_dir, src_ext, dest_dir, dest_ext, force) do
     files = Mix.Utils.extract_files(List.wrap(src_dir), List.wrap(src_ext))
 
     for file <- files do
@@ -304,12 +295,12 @@ defmodule Mix.Compilers.Erlang do
 
   defp to_diagnostics(warnings_or_errors, severity) do
     for {file, issues} <- warnings_or_errors,
-        {line, module, data} <- issues do
-      position = line(line)
-
+        file = if(?/ in file, do: Path.absname(file), else: List.to_string(file)),
+        {location, module, data} <- issues do
       %Mix.Task.Compiler.Diagnostic{
-        file: Path.absname(file),
-        position: position,
+        file: file,
+        source: file,
+        position: location_normalize(location),
         message: to_string(module.format_error(data)),
         severity: severity,
         compiler_name: to_string(module),
@@ -321,13 +312,36 @@ defmodule Mix.Compilers.Erlang do
   defp show_warnings(entries) do
     for {_, warnings} <- entries,
         {file, issues} <- warnings,
-        {line, module, message} <- issues do
-      IO.puts("#{file}:#{line(line)}: Warning: #{module.format_error(message)}")
+        {location, module, message} <- issues do
+      IO.puts("#{file}:#{location_to_string(location)} warning: #{module.format_error(message)}")
     end
   end
 
-  defp line({line, _column}) when is_integer(line) and line >= 1, do: line
-  # TODO: remove when we require OTP 24
-  defp line(line) when is_integer(line) and line >= 1, do: line
-  defp line(_), do: nil
+  defp location_normalize({line, column})
+       when is_integer(line) and line >= 1 and is_integer(column) and column >= 0,
+       do: {line, column}
+
+  defp location_normalize(line) when is_integer(line) and line >= 1, do: line
+  defp location_normalize(_), do: 0
+
+  defp location_to_string({line, column}), do: "#{line}:#{column}:"
+  defp location_to_string(0), do: ""
+  defp location_to_string(line), do: "#{line}:"
+
+  defp modules_diff(added, changed, removed, timestamp) do
+    %{
+      added: modules_from_paths(added),
+      changed: modules_from_paths(changed),
+      removed: modules_from_paths(removed),
+      timestamp: timestamp
+    }
+  end
+
+  defp modules_from_paths(paths) do
+    Enum.map(paths, &module_from_path/1)
+  end
+
+  defp module_from_path(path) do
+    path |> Path.basename() |> Path.rootname() |> String.to_atom()
+  end
 end
