@@ -67,15 +67,15 @@ defmodule Mix.Release do
   @default_apps [kernel: :permanent, stdlib: :permanent, elixir: :permanent, sasl: :permanent]
   @safe_modes [:permanent, :temporary, :transient]
   @unsafe_modes [:load, :none]
-  @significant_chunks ~w(Atom AtU8 Attr Code StrT ImpT ExpT FunT LitT Line)c
-  @copy_app_dirs ["priv"]
+  @additional_chunks ~w(Attr)c
+  @copy_app_dirs ["priv", "include"]
 
   @doc false
   @spec from_config!(atom, keyword, keyword) :: t
   def from_config!(name, config, overrides) do
     {name, apps, opts} = find_release(name, config)
 
-    unless Atom.to_string(name) =~ ~r/^[a-z][a-z0-9_]*$/ do
+    if not (Atom.to_string(name) =~ ~r/^[a-z][a-z0-9_]*$/) do
       Mix.raise(
         "Invalid release name. A release name must start with a lowercase ASCII letter, " <>
           "followed by lowercase ASCII letters, numbers, or underscores, got: #{inspect(name)}"
@@ -253,7 +253,7 @@ defmodule Mix.Release do
 
   defp erts_data(true) do
     version = :erlang.system_info(:version)
-    {:filename.join(:code.root_dir(), 'erts-#{version}'), :code.lib_dir(), version}
+    {:filename.join(:code.root_dir(), ~c"erts-#{version}"), :code.lib_dir(), version}
   end
 
   defp erts_data(erts_source) when is_binary(erts_source) do
@@ -285,7 +285,8 @@ defmodule Mix.Release do
               |> Enum.map(&{&1, new_mode})
 
             seen = put_in(seen[app][:mode], new_mode)
-            load_apps(apps, deps_apps, seen, otp_root, [], overrides)
+            optional = Keyword.get(properties, :optional_applications, [])
+            load_apps(apps, deps_apps, seen, otp_root, optional, overrides)
 
           true ->
             seen
@@ -391,7 +392,7 @@ defmodule Mix.Release do
   defp boot_mode(mode), do: mode
 
   defp build_start_clean_boot(boot) do
-    for({app, _mode} <- boot, do: {app, :none})
+    for({app, mode} <- boot, do: {app, if(mode == :none, do: :none, else: :load)})
     |> Keyword.put(:stdlib, :permanent)
     |> Keyword.put(:kernel, :permanent)
   end
@@ -432,7 +433,7 @@ defmodule Mix.Release do
   the `:elixir` application configuration in `sys_config` to be
   read during boot and trigger the providers.
 
-  It uses the following release options to customize its behaviour:
+  It uses the following release options to customize its behavior:
 
     * `:reboot_system_after_config`
     * `:start_distribution_during_config`
@@ -605,7 +606,7 @@ defmodule Mix.Release do
 
       case :systools.make_script(sys_path, sys_options) do
         {:ok, _module, _warnings} ->
-          script_path = sys_path ++ '.script'
+          script_path = sys_path ++ ~c".script"
           {:ok, [{:script, rel_info, instructions}]} = :file.consult(script_path)
 
           instructions =
@@ -628,7 +629,7 @@ defmodule Mix.Release do
     for {_, properties} <- release.applications,
         not Keyword.fetch!(properties, :otp_app?),
         uniq: true,
-        do: {'RELEASE_LIB', properties |> Keyword.fetch!(:path) |> :filename.dirname()}
+        do: {~c"RELEASE_LIB", properties |> Keyword.fetch!(:path) |> :filename.dirname()}
   end
 
   defp build_paths(release) do
@@ -655,7 +656,8 @@ defmodule Mix.Release do
       for {app, mode} <- modes do
         properties = Map.get(apps, app) || throw({:error, "Unknown application #{inspect(app)}"})
         children = Keyword.get(properties, :applications, [])
-        app in skip_mode_validation_for || validate_mode!(app, mode, modes, children)
+        optional = Keyword.get(properties, :optional_applications, [])
+        app in skip_mode_validation_for || validate_mode!(app, mode, modes, children, optional)
         build_app_for_release(app, mode, properties)
       end
 
@@ -664,7 +666,7 @@ defmodule Mix.Release do
     {:error, message} -> {:error, message}
   end
 
-  defp validate_mode!(app, mode, modes, children) do
+  defp validate_mode!(app, mode, modes, children, optional) do
     safe_mode? = mode in @safe_modes
 
     if not safe_mode? and mode not in @unsafe_modes do
@@ -679,7 +681,7 @@ defmodule Mix.Release do
       child_mode = Keyword.get(modes, child)
 
       cond do
-        is_nil(child_mode) ->
+        is_nil(child_mode) and child not in optional ->
           throw(
             {:error,
              "Application #{inspect(app)} is listed in the release boot, " <>
@@ -735,7 +737,7 @@ defmodule Mix.Release do
 
     Enum.map(instructions, fn
       {:path, paths} ->
-        if Enum.any?(paths, &List.starts_with?(&1, '$RELEASE_LIB')) do
+        if Enum.any?(paths, &List.starts_with?(&1, ~c"$RELEASE_LIB")) do
           {:path, prepend_paths ++ paths}
         else
           {:path, paths}
@@ -753,6 +755,7 @@ defmodule Mix.Release do
   @doc """
   Finds a template path for the release.
   """
+  @spec rel_templates_path(t, Path.t()) :: binary
   def rel_templates_path(release, path) do
     Path.join(release.options[:rel_templates_path] || "rel", path)
   end
@@ -760,7 +763,7 @@ defmodule Mix.Release do
   @doc """
   Copies ERTS if the release is configured to do so.
 
-  Returns true if the release was copied, false otherwise.
+  Returns `true` if the release was copied, `false` otherwise.
   """
   @spec copy_erts(t) :: boolean()
   def copy_erts(%{erts_source: nil}) do
@@ -848,10 +851,13 @@ defmodule Mix.Release do
         source_file = Path.join(source, file)
         target_file = Path.join(target, file)
 
-        with true <- is_list(strip_options) and String.ends_with?(file, ".beam"),
-             {:ok, binary} <- strip_beam(File.read!(source_file), strip_options) do
-          File.write!(target_file, binary)
-        else
+        case Path.extname(file) do
+          ".beam" ->
+            process_beam_file(source_file, target_file, strip_options)
+
+          ".app" ->
+            process_app_file(source_file, target_file)
+
           _ ->
             # Use File.cp!/3 to preserve file mode for any executables stored
             # in the ebin directory.
@@ -865,6 +871,23 @@ defmodule Mix.Release do
     end
   end
 
+  defp process_beam_file(source_file, target_file, strip_options) do
+    with true <- is_list(strip_options),
+         {:ok, binary} <- strip_beam(File.read!(source_file), strip_options) do
+      File.write!(target_file, binary)
+    else
+      _ -> File.cp!(source_file, target_file)
+    end
+  end
+
+  defp process_app_file(source_file, target_file) do
+    with {:ok, [{:application, app, info}]} <- :file.consult(source_file) do
+      File.write!(target_file, :io_lib.format("~tp.~n", [{:application, app, info}]))
+    else
+      _ -> File.cp!(source_file, target_file)
+    end
+  end
+
   @doc """
   Strips a beam file for a release.
 
@@ -877,7 +900,7 @@ defmodule Mix.Release do
   @spec strip_beam(binary(), keyword()) :: {:ok, binary()} | {:error, :beam_lib, term()}
   def strip_beam(binary, options \\ []) when is_list(options) do
     chunks_to_keep = options[:keep] |> List.wrap() |> Enum.map(&String.to_charlist/1)
-    all_chunks = Enum.uniq(@significant_chunks ++ chunks_to_keep)
+    all_chunks = Enum.uniq(@additional_chunks ++ :beam_lib.significant_chunks() ++ chunks_to_keep)
     compress? = Keyword.get(options, :compress, false)
 
     case :beam_lib.chunks(binary, all_chunks, [:allow_missing_chunks]) do
