@@ -128,21 +128,28 @@ defmodule Mix.SCM do
   until a matching one is found.
   """
   def available do
-    {:ok, scm} = Mix.State.fetch(:scm)
-    scm
+    Mix.State.get(:scm)
+  end
+
+  @doc """
+  Deletes the given SCM from the list of available SCMs.
+  """
+  @doc since: "1.16.2"
+  def delete(mod) when is_atom(mod) do
+    Mix.State.update(:scm, &List.delete(&1, mod))
   end
 
   @doc """
   Prepends the given SCM module to the list of available SCMs.
   """
   def prepend(mod) when is_atom(mod) do
-    Mix.State.prepend_scm(mod)
+    Mix.State.update(:scm, &[mod | List.delete(&1, mod)])
   end
 
   @doc """
   Appends the given SCM module to the list of available SCMs.
   """
   def append(mod) when is_atom(mod) do
-    Mix.State.append_scm(mod)
+    Mix.State.update(:scm, &(List.delete(&1, mod) ++ [mod]))
   end
 end
