@@ -2,10 +2,12 @@ defmodule Mix.SCM.Git do
   @behaviour Mix.SCM
   @moduledoc false
 
+  @impl true
   def fetchable? do
     true
   end
 
+  @impl true
   def format(opts) do
     if rev = get_opts_rev(opts) do
       "#{redact_uri(opts[:git])} - #{rev}"
@@ -14,12 +16,13 @@ defmodule Mix.SCM.Git do
     end
   end
 
+  @impl true
   def format_lock(opts) do
     case opts[:lock] do
       {:git, _, lock_rev, lock_opts} ->
         lock = String.slice(lock_rev, 0, 7)
 
-        case Enum.find_value([:branch, :ref, :tag], &List.keyfind(lock_opts, &1, 0)) do
+        case Enum.find_value([:ref, :branch, :tag], &List.keyfind(lock_opts, &1, 0)) do
           {:ref, _} -> lock <> " (ref)"
           {key, val} -> lock <> " (#{key}: #{val})"
           nil -> lock
@@ -30,28 +33,31 @@ defmodule Mix.SCM.Git do
     end
   end
 
+  @impl true
   def accepts_options(_app, opts) do
     opts =
       opts
       |> Keyword.put(:checkout, opts[:dest])
       |> sparse_opts()
+      |> subdir_opts()
 
     cond do
       gh = opts[:github] ->
         opts
         |> Keyword.delete(:github)
         |> Keyword.put(:git, "https://github.com/#{gh}.git")
-        |> validate_git_options
+        |> validate_git_options()
 
       opts[:git] ->
         opts
-        |> validate_git_options
+        |> validate_git_options()
 
       true ->
         nil
     end
   end
 
+  @impl true
   def checked_out?(opts) do
     # Are we inside a Git repository?
     opts[:checkout]
@@ -59,8 +65,8 @@ defmodule Mix.SCM.Git do
     |> File.regular?()
   end
 
+  @impl true
   def lock_status(opts) do
-    assert_git!()
     lock = opts[:lock]
 
     cond do
@@ -83,19 +89,21 @@ defmodule Mix.SCM.Git do
     end
   end
 
+  @impl true
   def equal?(opts1, opts2) do
     opts1[:git] == opts2[:git] and get_lock_opts(opts1) == get_lock_opts(opts2)
   end
 
+  @impl true
   def managers(_opts) do
     []
   end
 
+  @impl true
   def checkout(opts) do
-    assert_git!()
     path = opts[:checkout]
     File.rm_rf!(path)
-    File.mkdir_p!(path)
+    File.mkdir_p!(opts[:dest])
 
     File.cd!(path, fn ->
       git!(~w[-c core.hooksPath='' init --quiet])
@@ -104,8 +112,8 @@ defmodule Mix.SCM.Git do
     end)
   end
 
+  @impl true
   def update(opts) do
-    assert_git!()
     path = opts[:checkout]
     File.cd!(path, fn -> checkout(path, opts) end)
   end
@@ -118,13 +126,17 @@ defmodule Mix.SCM.Git do
     update_origin(opts[:git])
 
     # Fetch external data
+    lock_rev = get_lock_rev(opts[:lock], opts)
+
     ["--git-dir=.git", "fetch", "--force", "--quiet"]
     |> Kernel.++(progress_switch(git_version()))
     |> Kernel.++(tags_switch(opts[:tag]))
+    |> Kernel.++(depth_switch(opts[:depth]))
+    |> Kernel.++(refspec_switch(opts, lock_rev || get_opts_rev(opts)))
     |> git!()
 
     # Migrate the Git repo
-    rev = get_lock_rev(opts[:lock], opts) || get_opts_rev(opts) || default_branch()
+    rev = lock_rev || get_origin_opts_rev(opts) || default_branch()
     git!(["--git-dir=.git", "checkout", "--quiet", rev])
 
     if opts[:submodules] do
@@ -144,10 +156,19 @@ defmodule Mix.SCM.Git do
     end
   end
 
+  defp subdir_opts(opts) do
+    if opts[:subdir] do
+      dest = Path.join(opts[:dest], opts[:subdir])
+      Keyword.put(opts, :dest, dest)
+    else
+      opts
+    end
+  end
+
   defp sparse_toggle(opts) do
     cond do
       sparse = opts[:sparse] ->
-        sparse_check(git_version())
+        check_sparse_support(git_version())
         git!(["--git-dir=.git", "config", "core.sparsecheckout", "true"])
         File.mkdir_p!(".git/info")
         File.write!(".git/info/sparse-checkout", sparse)
@@ -163,39 +184,104 @@ defmodule Mix.SCM.Git do
     end
   end
 
-  defp sparse_check(version) do
-    unless {1, 7, 4} <= version do
-      version = version |> Tuple.to_list() |> Enum.join(".")
+  @min_git_version_sparse {1, 7, 4}
+  @min_git_version_depth {1, 5, 0}
+  @min_git_version_progress {1, 7, 1}
 
+  defp check_sparse_support(version) do
+    ensure_feature_compatibility(version, @min_git_version_sparse, "sparse checkout")
+  end
+
+  defp check_depth_support(version) do
+    ensure_feature_compatibility(version, @min_git_version_depth, "depth (shallow clone)")
+  end
+
+  defp ensure_feature_compatibility(version, required_version, feature) do
+    if not (required_version <= version) do
       Mix.raise(
-        "Git >= 1.7.4 is required to use sparse checkout. " <>
-          "You are running version #{version}"
+        "Git >= #{format_version(required_version)} is required to use #{feature}. " <>
+          "You are running version #{format_version(version)}"
       )
     end
   end
 
   defp progress_switch(version) do
-    if {1, 7, 1} <= version, do: ["--progress"], else: []
+    if @min_git_version_progress <= version, do: ["--progress"], else: []
   end
 
   defp tags_switch(nil), do: []
   defp tags_switch(_), do: ["--tags"]
 
+  defp depth_switch(nil), do: []
+
+  defp depth_switch(n) when is_integer(n) and n > 0 do
+    check_depth_support(git_version())
+    ["--depth=#{n}"]
+  end
+
+  defp refspec_switch(_opts, nil), do: []
+
+  defp refspec_switch(opts, rev) do
+    case Keyword.take(opts, [:depth, :branch, :tag]) do
+      [_ | _] -> ["origin", rev]
+      _ -> []
+    end
+  end
+
   ## Helpers
 
   defp validate_git_options(opts) do
-    err =
-      "You should specify only one of branch, ref or tag, and only once. " <>
-        "Error on Git dependency: #{redact_uri(opts[:git])}"
-
-    validate_single_uniq(opts, [:branch, :ref, :tag], err)
+    opts
+    |> validate_refspec()
+    |> validate_depth()
   end
 
-  defp validate_single_uniq(opts, take, error) do
-    case Keyword.take(opts, take) do
-      [] -> opts
-      [_] -> opts
-      _ -> Mix.raise(error)
+  defp validate_refspec(opts) do
+    case Keyword.take(opts, [:branch, :ref, :tag]) do
+      [] ->
+        opts
+
+      [{_refspec, value}] when is_binary(value) ->
+        opts
+
+      [{refspec, value}] ->
+        Mix.raise(
+          "A dependency's #{refspec} must be a string, got: #{inspect(value)}. " <>
+            "Error on Git dependency: #{redact_uri(opts[:git])}"
+        )
+
+      _ ->
+        Mix.raise(
+          "You should specify only one of branch, ref or tag, and only once. " <>
+            "Error on Git dependency: #{redact_uri(opts[:git])}"
+        )
+    end
+  end
+
+  @sha1_size 40
+
+  defp validate_depth(opts) do
+    case Keyword.take(opts, [:depth]) do
+      [] ->
+        opts
+
+      [{:depth, depth}] when is_integer(depth) and depth > 0 ->
+        ref = opts[:ref]
+
+        if ref && byte_size(ref) < @sha1_size do
+          Mix.raise(
+            "When :depth is used with :ref, a full commit hash is required. " <>
+              "Error on Git dependency: #{redact_uri(opts[:git])}"
+          )
+        end
+
+        opts
+
+      invalid_depth ->
+        Mix.raise(
+          "The depth must be a positive integer, and be specified only once, got: #{inspect(invalid_depth)}. " <>
+            "Error on Git dependency: #{redact_uri(opts[:git])}"
+        )
     end
   end
 
@@ -215,7 +301,7 @@ defmodule Mix.SCM.Git do
   defp get_lock_rev(_, _), do: nil
 
   defp get_lock_opts(opts) do
-    lock_opts = Keyword.take(opts, [:branch, :ref, :tag, :sparse])
+    lock_opts = Keyword.take(opts, [:branch, :ref, :tag, :sparse, :subdir, :depth])
 
     if opts[:submodules] do
       lock_opts ++ [submodules: true]
@@ -225,6 +311,10 @@ defmodule Mix.SCM.Git do
   end
 
   defp get_opts_rev(opts) do
+    opts[:branch] || opts[:ref] || opts[:tag]
+  end
+
+  defp get_origin_opts_rev(opts) do
     if branch = opts[:branch] do
       "origin/#{branch}"
     else
@@ -259,6 +349,8 @@ defmodule Mix.SCM.Git do
   end
 
   defp default_branch() do
+    # Note: the `set-head -a` command requires the remote reference to be
+    # fetched first.
     git!(["--git-dir=.git", "remote", "set-head", "origin", "-a"])
     "origin/HEAD"
   end
@@ -266,7 +358,17 @@ defmodule Mix.SCM.Git do
   defp git!(args, into \\ default_into()) do
     opts = cmd_opts(into: into, stderr_to_stdout: true)
 
-    case System.cmd("git", args, opts) do
+    try do
+      System.cmd("git", args, opts)
+    catch
+      :error, :enoent ->
+        Mix.raise(
+          "Error fetching/updating Git repository: the \"git\" " <>
+            "executable is not available in your PATH. Please install " <>
+            "Git on this machine or pass --no-deps-check if you want to " <>
+            "run a previously built application on a system without Git."
+        )
+    else
       {response, 0} ->
         response
 
@@ -285,26 +387,27 @@ defmodule Mix.SCM.Git do
     end
   end
 
-  defp assert_git! do
-    case Mix.State.fetch(:git_available) do
-      {:ok, true} ->
-        :ok
-
-      :error ->
-        if System.find_executable("git") do
-          Mix.State.put(:git_available, true)
-        else
-          Mix.raise(
-            "Error fetching/updating Git repository: the \"git\" " <>
-              "executable is not available in your PATH. Please install " <>
-              "Git on this machine or pass --no-deps-check if you want to " <>
-              "run a previously built application on a system without Git."
-          )
-        end
+  # Attempt to set the current working directory by default.
+  # This addresses an issue changing the working directory when executing from
+  # within a secondary node since file I/O is done through the main node.
+  defp cmd_opts(opts) do
+    case File.cwd() do
+      {:ok, cwd} -> Keyword.put(opts, :cd, cwd)
+      _ -> opts
     end
   end
 
-  def git_version do
+  # Invoked by lib/mix/test/test_helper.exs
+  @doc false
+  def unsupported_options do
+    git_version = git_version()
+
+    []
+    |> Kernel.++(if git_version < @min_git_version_sparse, do: [:sparse], else: [])
+    |> Kernel.++(if git_version < @min_git_version_depth, do: [:depth], else: [])
+  end
+
+  defp git_version do
     case Mix.State.fetch(:git_version) do
       {:ok, version} ->
         version
@@ -313,7 +416,7 @@ defmodule Mix.SCM.Git do
         version =
           ["--version"]
           |> git!("")
-          |> parse_version
+          |> parse_version()
 
         Mix.State.put(:git_version, version)
         version
@@ -328,18 +431,12 @@ defmodule Mix.SCM.Git do
     |> List.to_tuple()
   end
 
+  defp format_version(version) do
+    version |> Tuple.to_list() |> Enum.join(".")
+  end
+
   defp to_integer(string) do
     {int, _} = Integer.parse(string)
     int
   end
-
-  # Attempt to set the current working directory by default.
-  # This addresses an issue changing the working directory when executing from
-  # within a secondary node since file I/O is done through the main node.
-  defp cmd_opts(opts) do
-    case File.cwd() do
-      {:ok, cwd} -> Keyword.put(opts, :cd, cwd)
-      _ -> opts
-    end
-  end
 end
