@@ -2,11 +2,17 @@ defmodule Mix.Task do
   @moduledoc """
   Provides conveniences for creating, loading, and manipulating Mix tasks.
 
-  A Mix task can be defined by using `Mix.Task` in a module whose name
-  begins with `Mix.Tasks.` and which defines the `run/1` function.
+  To create a new Mix task, you'll need to:
+
+    1. Create a module whose name begins with `Mix.Tasks.` (for example,
+       `Mix.Tasks.MyTask`).
+    2. Call `use Mix.Task` in that module.
+    3. Implement the `Mix.Task` behaviour in that module (that is,
+       implement the `c:run/1` callback).
+
   Typically, task modules live inside the `lib/mix/tasks/` directory,
   and their file names use dot separators instead of underscores
-  (e.g. `deps.clean.ex`) - although ultimately the file name is not
+  (for example, `deps.clean.ex`) - although ultimately the file name is not
   relevant.
 
   For example:
@@ -34,52 +40,89 @@ defmodule Mix.Task do
   For example, if the `args` in the above `echo` task were
   inspected, you might see something like this:
 
-      mix echo 'A and B' C --test
+      $ mix echo 'A and B' C --test
       ["A and B", "C", "--test"]
 
+  > #### `use Mix.Task` {: .info}
+  >
+  > When you `use Mix.Task`, the `Mix.Task` module will
+  > set `@behaviour Mix.Task` and define default values
+  > for the module attributes documented in the section
+  > below.
+
+  ## Module attributes
+
+  You can control some behavior of your Mix task by setting module
+  attributes. This section documents the available attributes.
+
+  ### `@shortdoc`
+
   Define the `@shortdoc` attribute if you wish to make the task
   publicly visible on `mix help`. Omit this attribute if you do
   not want your task to be listed via `mix help`.
+
+  ### `@moduledoc`
+
   The `@moduledoc` attribute may override `@shortdoc`. The task
   will not appear in `mix help` if documentation for the entire
   module is hidden with `@moduledoc false`.
 
-  If a task has requirements, they can be listed using the
-  `@requirements` attribute. For example:
+  ### `@requirements`
+
+  If a task has *requirements*, they can be listed using the
+  `@requirements` attribute. Requirements are other Mix
+  tasks that this task requires to have run. For example:
 
       @requirements ["app.config"]
 
-  Tasks typically depend on the `"app.config"` task, when they
-  need to access code from the current project with all apps
-  already configured, or the "app.start" task, when they also
-  need those apps to be already started:
+  A task will typically depend on one of the following tasks:
 
-      @requirements ["app.start"]
+    * ["loadpaths"](`Mix.Tasks.Loadpaths`) - this ensures
+      dependencies are available and compiled. If you are publishing
+      a task as part of a library to be used by others, and your
+      task does not need to interact with the user code in any way,
+      this is the recommended requirement
 
-  You can also run tasks directly with `run/2`.
+    * ["app.config"](`Mix.Tasks.App.Config`) - additionally compiles
+      and load the runtime configuration for the current project. If
+      you are creating a task to be used within your application or
+      as part of a library, which must invoke or interact with the
+      user code, this is the minimum recommended requirement
 
-  ## Attributes
+    * ["app.start"](`Mix.Tasks.App.Start`) - additionally starts the
+      supervision tree of the current project and its dependencies
 
-  There are a few attributes available in Mix tasks to
-  configure them in Mix:
+  ### `@recursive`
 
-    * `@shortdoc`  - makes the task public with a short description that appears
-      on `mix help`
-    * `@recursive` - runs the task recursively in umbrella projects
-    * `@requirements` - list of required tasks to be run before the task
-    * `@preferred_cli_env` - recommends an environment in which to run the task.
-      It is used only if `MIX_ENV` is not yet set. Note `@preferred_cli_env` is
-      not loaded from dependencies as we need to know the environment in order to
-      load the dependencies themselves. In those cases, you can set the
-      `preferred_cli_env` configuration under `def project` in your `mix.exs`
+  Set `@recursive true` if you want the task to run
+  on each umbrella child in an umbrella project.
+
+  ### `@preferred_cli_env`
+
+  Sets the preferred Mix environment for this task. For example,
+  if your task is meant to be used for testing, you could set
+
+      @preferred_cli_env :test
 
   ## Documentation
 
-  Users can read the documentation for public Mix tasks by running `mix help my_task`.
-  The documentation that will be shown is the `@moduledoc` of the task's module.
+  Users can read the documentation for public Mix tasks by
+  running `mix help my_task`. The documentation that will be
+  shown is the `@moduledoc` of the task's module.
   """
 
+  @typedoc """
+  The name of a task.
+
+  For example, `"deps.clean"` or `:"deps.clean"`.
+  """
   @type task_name :: String.t() | atom
+
+  @typedoc """
+  The module that implements a Mix task.
+
+  For example, `Mix.Tasks.MyTask`.
+  """
   @type task_module :: atom
 
   @doc """
@@ -140,7 +183,7 @@ defmodule Mix.Task do
     part = byte_size(base) - @prefix_size - @suffix_size
 
     case base do
-      <<"Elixir.Mix.Tasks.", rest::binary-size(part), ".beam">> ->
+      <<"Elixir.Mix.Tasks.", rest::binary-size(^part), ".beam">> ->
         mod = :"Elixir.Mix.Tasks.#{rest}"
         ensure_task?(mod) && mod
 
@@ -204,7 +247,7 @@ defmodule Mix.Task do
   @doc """
   Indicates if the current task is recursing.
 
-  This returns true if a task is marked as recursive
+  This returns `true` if a task is marked as recursive
   and it is being executed inside an umbrella project.
   """
   @doc since: "1.8.0"
@@ -214,23 +257,10 @@ defmodule Mix.Task do
   end
 
   @doc """
-  Gets preferred CLI environment for the task.
-
-  Returns environment (for example, `:test`, or `:prod`), or `nil`.
+  Available for backwards compatibility.
   """
-  @spec preferred_cli_env(task_name) :: atom | nil
-  def preferred_cli_env(task) when is_atom(task) or is_binary(task) do
-    case get(task) do
-      nil ->
-        nil
-
-      module ->
-        case List.keyfind(module.__info__(:attributes), :preferred_cli_env, 0) do
-          {:preferred_cli_env, [setting]} -> setting
-          _ -> nil
-        end
-    end
-  end
+  @deprecated "Configure the environment in your mix.exs"
+  defdelegate preferred_cli_env(task), to: Mix.CLI
 
   @doc """
   Gets the list of requirements for the given task.
@@ -264,7 +294,7 @@ defmodule Mix.Task do
   @doc """
   Checks if the given `task` name is an alias.
 
-  Returns false if the given name is not an alias or if it is not a task.
+  Returns `false` if the given name is not an alias or if it is not a task.
 
   For more information about task aliasing, take a look at the
   ["Aliases"](https://hexdocs.pm/mix/Mix.html#module-aliases) section in the
@@ -332,7 +362,7 @@ defmodule Mix.Task do
   If there exists a task matching the given task name and it has not yet been
   invoked, this will run the task with the given `args` and return the result.
 
-  If there is an [alias](https://hexdocs.pm/mix/Mix.html#module-aliases) defined
+  If there is an [alias](Mix.html#module-aliases) defined
   for the given task name, the alias will be invoked instead of the original
   task.
 
@@ -345,6 +375,12 @@ defmodule Mix.Task do
 
   `run/2` raises an exception if an alias or a task cannot be found or if the
   task is invalid. See `get!/1` for more information.
+
+  ## Examples
+
+      iex> Mix.Task.run("format", ["mix.exs"])
+      :ok
+
   """
   @spec run(task_name, [any]) :: any
   def run(task, args \\ []) do
@@ -361,7 +397,7 @@ defmodule Mix.Task do
   @doc since: "1.14.0"
   @spec run_in_apps(task_name, [atom], [any]) :: any
   def run_in_apps(task, apps, args \\ []) do
-    unless Mix.Project.umbrella?() do
+    if !Mix.Project.umbrella?() do
       Mix.raise(
         "Could not run #{inspect(task)} with the --app option because this is not an umbrella project"
       )
@@ -382,23 +418,61 @@ defmodule Mix.Task do
       alias && Mix.TasksServer.run({:alias, task, proj}) ->
         run_alias(List.wrap(alias), args, proj, task, apps, :ok)
 
-      !Mix.TasksServer.get({:task, task, proj}) ->
-        run_task(proj, task, args, apps)
+      Mix.TasksServer.get({:task, task, proj}) ->
+        :noop
+
+      module = maybe_load_or_compile_task(proj, task) ->
+        run_task(proj, module, task, args, apps)
+
+      results = Mix.Project.umbrella?() && recur_umbrella_children_alias(task, args, apps) ->
+        results
 
       true ->
-        :noop
+        # We could not find the task, let's raise
+        get!(task)
     end
   end
 
-  defp run_task(proj, task, args, apps) do
+  defp recur_umbrella_children_alias(task, args, apps) do
+    alias_key = String.to_atom(task)
+
+    entries =
+      Mix.ProjectStack.recur(fn ->
+        recur(
+          fn proj ->
+            alias = Mix.Project.config()[:aliases][alias_key]
+
+            cond do
+              is_nil(alias) ->
+                :error
+
+              Mix.TasksServer.run({:alias, task, proj}) ->
+                {:ok, run_alias(List.wrap(alias), args, proj, task, nil, :ok)}
+
+              true ->
+                {:ok, :noop}
+            end
+          end,
+          apps
+        )
+      end)
+
+    case for({:ok, res} <- entries, do: res) do
+      [] -> nil
+      result -> result
+    end
+  end
+
+  defp maybe_load_or_compile_task(proj, task) do
     # 1. If the task is available, we run it.
     # 2. Otherwise we compile and load dependencies
     # 3. Finally, we compile the current project in hope it is available.
-    module =
-      get_task_or_run(proj, task, fn -> run("deps.loadpaths") end) ||
-        get_task_or_run(proj, task, fn -> run("compile", []) end) ||
-        get!(task)
+    get_task_or_run(proj, task, fn -> run("deps.loadpaths") end) ||
+      get_task_or_run(proj, task, fn -> run("compile", []) end) ||
+      get(task)
+  end
 
+  defp run_task(proj, module, task, args, apps) do
     recursive = recursive(module)
 
     cond do
@@ -438,14 +512,21 @@ defmodule Mix.Task do
   end
 
   defp with_debug(task, args, proj, fun) do
-    if Mix.debug?() do
-      shell = Mix.shell()
-      shell.info(["-> Running mix ", task_to_string(task, args), project_to_string(proj)])
-      {time, res} = :timer.tc(fun)
-      shell.info(["<- Ran mix ", task, " in ", Integer.to_string(div(time, 1000)), "ms"])
-      res
-    else
-      fun.()
+    cond do
+      Mix.debug?() ->
+        shell = Mix.shell()
+        shell.info(["-> Running mix ", task_to_string(task, args), project_to_string(proj)])
+        {time, res} = :timer.tc(fun)
+        shell.info(["<- Ran mix ", task, " in ", Integer.to_string(div(time, 1000)), "ms"])
+        res
+
+      task in Mix.State.get(:profile, []) ->
+        shell = Mix.shell()
+        shell.info(["-> Profiling mix ", task_to_string(task, args), project_to_string(proj)])
+        Mix.Tasks.Profile.Tprof.profile(fun, warmup: false, set_on_spawn: false)
+
+      true ->
+        fun.()
     end
   end
 
@@ -472,7 +553,8 @@ defmodule Mix.Task do
   defp run_alias([h | t], alias_args, proj, original_task, apps, _res) when is_binary(h) do
     case OptionParser.split(h) do
       [^original_task | args] ->
-        res = run_task(proj, original_task, args ++ alias_args, apps)
+        module = maybe_load_or_compile_task(proj, original_task) || get!(original_task)
+        res = run_task(proj, module, original_task, args ++ alias_args, apps)
         run_alias(t, [], proj, original_task, apps, res)
 
       [task | args] ->
@@ -595,7 +677,7 @@ defmodule Mix.Task do
   """
   @spec task?(task_module) :: boolean
   def task?(module) when is_atom(module) do
-    match?('Elixir.Mix.Tasks.' ++ _, Atom.to_charlist(module)) and ensure_task?(module)
+    match?(~c"Elixir.Mix.Tasks." ++ _, Atom.to_charlist(module)) and ensure_task?(module)
   end
 
   defp ensure_task?(module) do
