@@ -10,28 +10,28 @@ defmodule Mix.Tasks.Archive.Install do
   root directory (created with `mix archive.build`), then the archive
   will be installed locally. For example:
 
-      mix do archive.build + archive.install
+      $ mix do archive.build + archive.install
 
   If an argument is provided, it should be a local path to a
   prebuilt archive, a Git repository, a GitHub repository, or a Hex
   package.
 
-      mix archive.install archive.ez
-      mix archive.install path/to/archive.ez
-      mix archive.install git https://path/to/git/repo
-      mix archive.install git https://path/to/git/repo branch git_branch
-      mix archive.install git https://path/to/git/repo tag git_tag
-      mix archive.install git https://path/to/git/repo ref git_ref
-      mix archive.install github user/project
-      mix archive.install github user/project branch git_branch
-      mix archive.install github user/project tag git_tag
-      mix archive.install github user/project ref git_ref
-      mix archive.install hex hex_package
-      mix archive.install hex hex_package 1.2.3
+      $ mix archive.install archive.ez
+      $ mix archive.install path/to/archive.ez
+      $ mix archive.install git https://path/to/git/repo
+      $ mix archive.install git https://path/to/git/repo branch git_branch
+      $ mix archive.install git https://path/to/git/repo tag git_tag
+      $ mix archive.install git https://path/to/git/repo ref git_ref
+      $ mix archive.install github user/project
+      $ mix archive.install github user/project branch git_branch
+      $ mix archive.install github user/project tag git_tag
+      $ mix archive.install github user/project ref git_ref
+      $ mix archive.install hex hex_package
+      $ mix archive.install hex hex_package 1.2.3
 
   After installation, the tasks in the archive are available locally:
 
-      mix some_task
+      $ mix some_task
 
   Note that installing via Git, GitHub, or Hex fetches the source
   of the archive and builds it, while using local path uses a pre-built archive.
@@ -47,6 +47,9 @@ defmodule Mix.Tasks.Archive.Install do
     * `--submodules` - fetches repository submodules before building archive from
       Git or GitHub
 
+    * `--sparse` - checkout a single directory inside the Git repository and use
+      it as the archive root directory
+
     * `--app` - specifies a custom app name to be used for building the archive
       from Git, GitHub, or Hex
 
@@ -63,6 +66,7 @@ defmodule Mix.Tasks.Archive.Install do
     force: :boolean,
     sha512: :string,
     submodules: :boolean,
+    sparse: :string,
     app: :string,
     organization: :string,
     repo: :string,
@@ -121,7 +125,7 @@ defmodule Mix.Tasks.Archive.Install do
 
     ebin = Mix.Local.archive_ebin(dir_dest)
     Mix.Local.check_elixir_version_in_ebin(ebin)
-    true = Code.append_path(ebin)
+    true = Code.append_path(ebin, cache: true)
     :ok
   end
 
