@@ -13,6 +13,12 @@ defmodule Mix.Tasks.Clean do
 
   By default this task works across all environments, unless `--only`
   is given.
+
+  ## Command line options
+
+    * `--deps` - clean dependencies as well as the current project's files
+    * `--only` - only clean the given environment
+
   """
 
   @switches [deps: :boolean, only: :string]
@@ -24,11 +30,21 @@ defmodule Mix.Tasks.Clean do
 
     {opts, _, _} = OptionParser.parse(args, switches: @switches)
 
-    _ =
-      for compiler <- [:protocols] ++ Mix.Tasks.Compile.compilers(),
+    Mix.Project.with_build_lock(fn ->
+      do_run(opts)
+    end)
+  end
+
+  defp do_run(opts) do
+    # First, we get the tasks. After that, we clean them.
+    # This is to avoid a task cleaning a compiler module.
+    tasks =
+      for compiler <- [:protocols] ++ Mix.Task.Compiler.compilers(),
           module = Mix.Task.get("compile.#{compiler}"),
           function_exported?(module, :clean, 0),
-          do: module.clean
+          do: module
+
+    Enum.each(tasks, & &1.clean())
 
     build =
       Mix.Project.build_path()
