@@ -1,5 +1,5 @@
 defmodule Mix.Tasks.Compile do
-  use Mix.Task.Compiler
+  use Mix.Task
 
   @shortdoc "Compiles source files"
 
@@ -9,21 +9,24 @@ defmodule Mix.Tasks.Compile do
   It simply runs the compilers registered in your project and returns
   a tuple with the compilation status and a list of diagnostics.
 
-  Before compiling code, it loads the code in all dependencies and
-  perform a series of checks to ensure the project is up to date.
+  Before compiling code, it performs a series of checks to ensure all
+  dependencies are compiled and the project is up to date. Then the
+  code path of your Elixir system is pruned to only contain the dependencies
+  and applications that you have explicitly listed in your `mix.exs`.
 
   ## Configuration
 
+    * `:build_embedded` - this option was used to copy all code and
+      priv content to the `_build` directory. However, this option no
+      longer has an effect as Elixir will now copy those at release time
+
     * `:compilers` - compilers to run, defaults to `Mix.compilers/0`,
-      which are `[:yecc, :leex, :erlang, :elixir, :app]`.
+      which are `[:erlang, :elixir, :app]`.
 
     * `:consolidate_protocols` - when `true`, runs protocol
       consolidation via the `mix compile.protocols` task. The default
       value is `true`.
 
-    * `:build_embedded` - when `true`, embeds all code and priv
-      content in the `_build` directory instead of using symlinks.
-
     * `:build_path` - the directory where build artifacts
       should be written to. This option is intended only for
       child apps within a larger umbrella application so that
@@ -32,20 +35,30 @@ defmodule Mix.Tasks.Compile do
       this has undesirable side-effects (such as skipping some
       compiler checks) and should be avoided.
 
+    * `:prune_code_paths` - prune code paths before compilation. When true
+      (default), this prunes code paths of applications that are not listed
+      in the project file with dependencies.  When false, this keeps the
+      entirety of Erlang/OTP available when the project starts, including
+      the paths set by the code loader from the `ERL_LIBS` environment as
+      well as explicitly listed by providing `-pa` and `-pz` options
+      to Erlang.
+
   ## Compilers
 
   To see documentation for each specific compiler, you must
   invoke `help` directly for the compiler command:
 
-      mix help compile.elixir
-      mix help compile.erlang
+      $ mix help compile.elixir
+      $ mix help compile.erlang
 
   You can get a list of all compilers by running:
 
-      mix compile --list
+      $ mix compile --list
 
   ## Command line options
 
+    * `--all-warnings` (`--no-all-warnings`) - prints all warnings, including previous compilations
+      (default is true except on errors)
     * `--erl-config` - path to an Erlang term file that will be loaded as Mix config
     * `--force` - forces compilation
     * `--list` - lists all enabled compilers
@@ -57,29 +70,18 @@ defmodule Mix.Tasks.Compile do
     * `--no-optional-deps` - does not compile or load optional deps. Useful for testing
       if a library still successfully compiles without optional dependencies (which is the
       default case with dependencies)
+    * `--no-prune-code-paths` - do not prune code paths before compilation, this keeps
+      the entirety of Erlang/OTP available when the project starts
     * `--no-protocol-consolidation` - skips protocol consolidation
     * `--no-validate-compile-env` - does not validate the application compile environment
     * `--return-errors` - returns error status and diagnostics instead of exiting on error
-    * `--warnings-as-errors` - exit with non-zero status if compilation has one or more warnings
+    * `--warnings-as-errors` - exit with non-zero status if compilation has one or more
+      warnings. Only concerns compilation warnings from the project, not its dependencies.
 
   """
 
-  @doc """
-  Returns all compilers.
-  """
-  def compilers(config \\ Mix.Project.config()) do
-    compilers = config[:compilers] || Mix.compilers()
-
-    if :xref in compilers do
-      IO.warn(
-        "the :xref compiler is deprecated, please remove it from your mix.exs :compilers options"
-      )
-
-      List.delete(compilers, :xref)
-    else
-      compilers
-    end
-  end
+  @deprecated "Use Mix.Task.Compiler.compilers/1 instead"
+  defdelegate compilers(config \\ Mix.Project.config()), to: Mix.Task.Compiler
 
   @impl true
   def run(["--list"]) do
@@ -111,7 +113,7 @@ defmodule Mix.Tasks.Compile do
     sorted = Enum.sort(docs)
 
     Enum.each(sorted, fn {task, doc} ->
-      shell.info(format('mix ~-#{max}s # ~ts', [task, doc]))
+      shell.info(format(~c"mix ~-#{max}s # ~ts", [task, doc]))
     end)
 
     consolidate_protocols? = Mix.Project.config()[:consolidate_protocols]
@@ -123,13 +125,12 @@ defmodule Mix.Tasks.Compile do
   @impl true
   def run(args) do
     Mix.Project.get!()
+
     Mix.Task.run("loadpaths", args)
 
     {opts, _, _} = OptionParser.parse(args, switches: [erl_config: :string])
     load_erl_config(opts)
 
-    # We invoke compile.all even on --no-compile because compile.all
-    # loads the apps, and we still want to do that.
     {res, diagnostics} =
       Mix.Task.run("compile.all", args)
       |> List.wrap()
@@ -138,6 +139,18 @@ defmodule Mix.Tasks.Compile do
 
     config = Mix.Project.config()
 
+    # If we are in an umbrella project, now load paths from all children.
+    if apps_paths = Mix.Project.apps_paths(config) do
+      loaded_paths =
+        (Mix.Tasks.Compile.All.project_apps(config) ++ Map.keys(apps_paths))
+        |> Mix.AppLoader.load_apps(Mix.Dep.cached(), config, [], fn
+          {_app, path}, acc -> if path, do: [path | acc], else: acc
+        end)
+
+      # We don't cache umbrella paths as we may write to them
+      Code.prepend_paths(loaded_paths -- :code.get_path())
+    end
+
     consolidate_protocols? =
       config[:consolidate_protocols] and "--no-protocol-consolidation" not in args
 
@@ -158,6 +171,7 @@ defmodule Mix.Tasks.Compile do
     with true <- consolidate_protocols?,
          path = Mix.Project.consolidation_path(config),
          {:ok, protocols} <- File.ls(path) do
+      # We don't cache consolidation path as we may write to it
       Code.prepend_path(path)
       Enum.each(protocols, &load_protocol/1)
     end
@@ -191,18 +205,8 @@ defmodule Mix.Tasks.Compile do
     end
   end
 
-  @impl true
-  def manifests do
-    Enum.flat_map(compilers(), fn compiler ->
-      module = Mix.Task.get("compile.#{compiler}")
-
-      if module && function_exported?(module, :manifests, 0) do
-        module.manifests
-      else
-        []
-      end
-    end)
-  end
+  @deprecated "Use Mix.Task.Compiler.manifests/0 instead"
+  defdelegate manifests, to: Mix.Task.Compiler
 
   ## Consolidation handling
 
