@@ -9,22 +9,31 @@ defmodule Mix.Tasks.Deps.Get do
 
   ## Command line options
 
-    * `--only` - only fetches dependencies for given environment
+    * `--check-locked` - raises if there are pending changes to the lockfile
     * `--no-archives-check` - does not check archives before fetching deps
+    * `--only` - only fetches dependencies for given environment
 
   """
 
   @impl true
   def run(args) do
-    unless "--no-archives-check" in args do
+    if "--no-archives-check" not in args do
       Mix.Task.run("archive.check", args)
     end
 
     Mix.Project.get!()
-    {opts, _, _} = OptionParser.parse(args, switches: [only: :string, target: :string])
 
+    {opts, _, _} =
+      OptionParser.parse(args, switches: [only: :string, target: :string, check_locked: :boolean])
+
+    Mix.Project.with_deps_lock(fn ->
+      do_run(opts)
+    end)
+  end
+
+  defp do_run(opts) do
     fetch_opts =
-      for {switch, key} <- [only: :env, target: :target],
+      for {switch, key} <- [only: :env, target: :target, check_locked: :check_locked],
           value = opts[switch],
           do: {key, :"#{value}"}
 
