@@ -38,12 +38,13 @@ defmodule Mix.Tasks.Release do
 
   If the given name does not exist, an error is raised.
 
-  If `mix release`, without a name, is invoked and there are multiple names,
-  an error will be raised unless you set `default_release: NAME` at the root
-  of your project configuration.
+  If `mix release` is invoked, without specifying a release name, and
+  there are multiple releases configured, an error will be raised
+  unless you set `default_release: NAME` at the root of your project
+  configuration.
 
-  If `mix release` is invoked and there are no names, a release using the
-  application name and default values is assembled.
+  If `mix release` is invoked and there are no releases configured, a
+  release is assembled using the application name and default values.
 
   ## Why releases?
 
@@ -57,8 +58,7 @@ defmodule Mix.Tasks.Release do
       the VM will find the `Enum` module and load it. There's a downside:
       when you start a new server in production, it may need to load
       many other modules, causing the first requests to have an unusual
-      spike in response time. With releases, the system is configured in
-      interactive mode and then it swaps to embedded mode, which preloads
+      spike in response time. With releases, the system preloads
       all modules and guarantees your system is ready to handle requests
       after booting.
 
@@ -115,15 +115,12 @@ defmodule Mix.Tasks.Release do
   distribution. For example, if you need to do some prep work before
   running the actual system, like migrating your database, `eval` can
   be a good fit. Just keep in mind any application you may use during
-  eval has to be explicitly loaded and/or started.
+  eval has to be explicitly started.
 
   You can start an application by calling `Application.ensure_all_started/1`.
-  However, if for some reason you cannot start an application, maybe
-  because it will run other services you do not want, you must at least
-  load the application by calling `Application.load/1`. If you don't
-  load the application, any attempt at reading its environment or
-  configuration may fail. Note that if you start an application,
-  it is automatically loaded before started.
+  From Elixir v1.16, it is guaranteed the applications have been
+  at least loaded. In earlier versions, if you needed to load applications
+  but not start them, you also needed to call `Application.load/1`.
 
   Another way to run commands is with `rpc`, which will connect to the
   system currently running and instruct it to execute the given
@@ -170,7 +167,7 @@ defmodule Mix.Tasks.Release do
       $ bin/RELEASE_NAME daemon
 
   In daemon mode, the system is started on the background via
-  [`run_erl`](https://www.erlang.org/doc/man/run_erl.html). You may also
+  [`run_erl`](https://www.erlang.org/doc/apps/erts/run_erl_cmd.html). You may also
   want to enable [`heart`](https://www.erlang.org/doc/man/heart.html)
   in daemon mode so it automatically restarts the system in case
   of crashes. See the generated `releases/RELEASE_VSN/env.sh` file.
@@ -194,8 +191,8 @@ defmodule Mix.Tasks.Release do
 
   While daemons are not available on Windows, it is possible to install a
   released system as a service on Windows with the help of
-  [`erlsrv`](https://www.erlang.org/doc/man/erlsrv.html). This can be done by
-  running:
+  [`erlsrv`](https://www.erlang.org/doc/apps/erts/erlsrv_cmd.html).
+  This can be done by running:
 
       $ bin/RELEASE_NAME install
 
@@ -245,8 +242,8 @@ defmodule Mix.Tasks.Release do
   but usually separate, and often there are many targets (either multiple
   instances, or the release is deployed to heterogeneous environments).
 
-  To deploy straight from a host to a separate target without cross-compilation,
-  the following must be the same between the host and the target:
+  To deploy straight from a host to a separate target, the following must be
+  the same between the host and the target:
 
     * Target architecture (for example, x86_64 or ARM)
     * Target vendor + operating system (for example, Windows, Linux, or Darwin/macOS)
@@ -254,23 +251,24 @@ defmodule Mix.Tasks.Release do
 
   This is often represented in the form of target triples, for example,
   `x86_64-unknown-linux-gnu`, `x86_64-unknown-linux-musl`, `x86_64-apple-darwin`.
+  If you are building on a MacBook (`x86_64-apple-darwin`) and trying to deploy
+  to a typical Ubuntu machine (`x86_64-unknown-linux-gnu`), the release will not
+  work. Instead you should build the release on a `x86_64-unknown-linux-gnu` host.
 
-  So to be more precise, to deploy straight from a host to a separate target,
-  the Erlang Runtime System (ERTS), and any native dependencies (NIFs), must
-  be compiled for the same target triple. If you are building on a MacBook
-  (`x86_64-apple-darwin`) and trying to deploy to a typical Ubuntu machine
-  (`x86_64-unknown-linux-gnu`), the release will not work. Instead you should
-  build the release on a `x86_64-unknown-linux-gnu` host. As we will see, this
-  can be done in multiple ways, such as releasing on the target itself, or by
-  using virtual machines or containers, usually as part of your release pipeline.
+  Typically, different versions of Erlang VM and Elixir are available for
+  different targets via package managers, precompiled artifacts, and similar.
+  However, to deploy from a host to a separate target, you must also guarantee
+  that any dependency with NIFs (Natively-Implemented Functions) are compiled
+  for the same triplet. As we will see, this can be done in different ways,
+  such as releasing on the target itself, or by using virtual machines or
+  containers, usually as part of your release pipeline.
 
   In addition to matching the target triple, it is also important that the
   target has all of the system packages that your application will need at
   runtime. A common one is the need for OpenSSL when building an application
-  that uses `:crypto` or `:ssl`, which is dynamically linked to ERTS. The other
-  common source for native dependencies like this comes from dependencies
-  containing NIFs (natively-implemented functions) which may expect to
-  dynamically link to libraries they use.
+  that uses `:crypto` or `:ssl`, which is dynamically linked to the Erlang VM.
+  Project dependencies containing NIFs (natively-implemented functions) may
+  also dynamically link to system libraries, so check those accordingly.
 
   Of course, some operating systems and package managers can differ between
   versions, so if your goal is to have full compatibility between host and
@@ -279,26 +277,7 @@ defmodule Mix.Tasks.Release do
   some systems, especially so with package managers that try to create fully
   reproducible environments (Nix, Guix).
 
-  Similarly, when creating a stand-alone package and release for Windows, note
-  the Erlang Runtime System has a dependency to some Microsoft libraries
-  (Visual C++ Redistributable Packages for Visual Studio 2013). These libraries
-  are installed (if not present before) when Erlang is installed but it is not
-  part of the standard Windows environment. Deploying a stand-alone release on
-  a computer without these libraries will result in a failure when trying to
-  run the release. One way to solve this is to download and install these
-  Microsoft libraries the first time a release is deployed (the Erlang installer
-  version 10.6 ships with “Microsoft Visual C++ 2013 Redistributable - 12.0.30501”).
-
-  Alternatively, you can also bundle the compiled object files in the release,
-  as long as they were compiled for the same target. If doing so, you need to
-  update `LD_LIBRARY_PATH` environment variable with the paths containing the
-  bundled objects on Unix-like systems or the `PATH` environment variable on
-  Windows systems.
-
-  Currently, there is no official way to cross-compile a release from one
-  target triple to another, due to the complexities involved in the process.
-
-  ### Techniques
+  ### Using matching host and target
 
   There are a couple of ways to guarantee that a release is built on a host with
   the same properties as the target. A simple option is to fetch the source,
@@ -337,6 +316,8 @@ defmodule Mix.Tasks.Release do
   production machines can fetch the deployment from the network storage
   and run `bin/my_app start`.
 
+  ### Using images
+
   Another mechanism to automate deployments is to use images, such as
   Amazon Machine Images, or container platforms, such as Docker.
   For instance, you can use Docker to run locally a system with the
@@ -345,9 +326,14 @@ defmodule Mix.Tasks.Release do
   a complete image and/or container with the operating system, all
   dependencies as well as the releases.
 
-  In other words, there are multiple ways systems can be deployed and
-  releases can be automated and incorporated into all of them as long
-  as you remember to build the system in the same target triple.
+  However, when building such images on your machine, those technologies
+  use emulation which may not interplay well with Erlang VM's JIT
+  (just-in time) compiler. To address this, you can set this environment
+  variable on your build stage:
+
+      ENV ERL_AFLAGS "+JMsingle true"
+
+  ## Shutting down
 
   Once a system is deployed, shutting down the system can be done by
   sending SIGINT/SIGTERM to the system, which is what most containers,
@@ -392,10 +378,10 @@ defmodule Mix.Tasks.Release do
     * `:strip_beams` - controls if BEAM files should have their debug information,
       documentation chunks, and other non-essential metadata removed. Defaults to
       `true`. May be set to `false` to disable stripping. Also accepts
-     `[keep: ["Docs", "Dbgi"]]` to keep certain chunks that are usually stripped.
-     You can also set the `:compress` option to true to enable individual
-     compression of BEAM files, although it is typically preferred to compress
-     the whole release instead.
+      `[keep: ["Docs", "Dbgi"]]` to keep certain chunks that are usually stripped.
+      You can also set the `:compress` option to true to enable individual
+      compression of BEAM files, although it is typically preferred to compress
+      the whole release instead.
 
     * `:cookie` - a string representing the Erlang Distribution cookie. If this
       option is not set, a random cookie is written to the `releases/COOKIE` file
@@ -405,8 +391,8 @@ defmodule Mix.Tasks.Release do
 
       If you are setting this option manually, we recommend the cookie option
       to be a long and randomly generated string, such as:
-      `Base.url_encode64(:crypto.strong_rand_bytes(40))`. We also recommend to restrict
-      the characters in the cookie to the subset returned by `Base.url_encode64/1`.
+      `Base.encode32(:crypto.strong_rand_bytes(40))`. We also recommend restricting
+      the characters in the cookie to only alphanumeric characters and underscore.
 
     * `:validate_compile_env` - by default a release will match all runtime
       configuration against any configuration that was marked at compile time
@@ -578,6 +564,36 @@ defmodule Mix.Tasks.Release do
   )
   ```
 
+  Inside `env.sh` and `env.bat` files you can access command-line arguments given to release commands.
+  For example, given this `env.sh.eex`:
+
+  ```bash
+  echo $@
+  ```
+
+  or this `env.bat.eex`:
+
+  ```bash
+  echo %*
+  ```
+
+  starting the release with `bin/myapp start --foo bar baz` will print `start --foo bar baz`.
+
+  ### `epmd`-less deployment
+
+  When a distributed Erlang/Elixir node starts, it runs a separate daemon called EPMD
+  (Erlang Port Mapper Daemon) and registers the node name within EPMD. It is possible
+  to skip this additional Operating System process by setting the following flags in
+  your vm.args files:
+
+      # In vm.args.eex
+      -start_epmd false -erl_epmd_port 6789
+
+      # In remote.vm.args.eex
+      -start_epmd false -erl_epmd_port 6789 -dist_listen false
+
+  You can pick any port of your choice.
+
   ## Application configuration
 
   Mix provides two mechanisms for configuring the application environment
@@ -624,9 +640,7 @@ defmodule Mix.Tasks.Release do
 
   If a `config/runtime.exs` exists, it will be copied to your release
   and executed early in the boot process, when only Elixir and Erlang's
-  main applications have been started. Once the configuration is loaded,
-  the Erlang system will be restarted (within the same Operating System
-  process) and the new configuration will take place.
+  main applications have been started.
 
   You can change the path to the runtime configuration file by setting
   `:runtime_config_path` inside each release configuration. This path is
@@ -642,14 +656,6 @@ defmodule Mix.Tasks.Release do
   By setting `:runtime_config_path` to `false` it can be used to prevent
   a runtime configuration file to be included in the release.
 
-  Finally, in order for runtime configuration to work properly (as well
-  as any other "Config provider" as defined next), it needs to be able
-  to persist the newly computed configuration to disk. The computed config
-  file will be written to "tmp" directory inside the release every time
-  the system boots. You can configure the "tmp" directory by setting the
-  `RELEASE_TMP` environment variable, either explicitly or inside your
-  `releases/RELEASE_VSN/env.sh` (or `env.bat` on Windows).
-
   ### Config providers
 
   Releases also supports custom mechanisms, called config providers, to load
@@ -662,13 +668,17 @@ defmodule Mix.Tasks.Release do
   The following options can be set inside your releases key in your `mix.exs`
   to control how config providers work:
 
-    * `:reboot_system_after_config` - every time your release is configured,
-      the system is rebooted to allow the new configuration to take place.
-      You can set this option to `false` to disable the rebooting for applications
-      that are sensitive to boot time but, in doing so, note you won't be able
-      to configure system applications, such as `:kernel` and `:stdlib`.
-      Defaults to `true` if using the deprecated `config/releases.exs`,
-      `false` otherwise.
+    * `:reboot_system_after_config` - reboot the system after configuration
+      so you can configure system applications, such as `:kernel` and `:stdlib`,
+      in your `config/runtime.exs`. Generally speaking, it is best to configure
+      `:kernel` and `:stdlib` using the `vm.args` file but this option is available
+      for those who need more complex configuration. When set to `true`, the
+      release will first boot in interactive mode to compute a config file and
+      write it to the "tmp" directory. Then it reboots in the configured `RELEASE_MODE`.
+      You can configure the "tmp" directory by setting the `RELEASE_TMP` environment
+      variable, either explicitly or inside your `releases/RELEASE_VSN/env.sh`
+      (or `env.bat` on Windows). Defaults to `true` if using the deprecated
+      `config/releases.exs`, `false` otherwise.
 
     * `:prune_runtime_sys_config_after_boot` - if `:reboot_system_after_config`
       is set, every time your system boots, the release will write a config file
@@ -751,8 +761,7 @@ defmodule Mix.Tasks.Release do
   We document this structure for completeness. In practice, developers
   should not modify any of those files after the release is assembled.
   Instead use env scripts, custom config provider, overlays, and all
-  other mechanisms described in this guide to configure how your release
-  works.
+  other mechanisms described here to configure how your release works.
 
   ## Environment variables
 
@@ -813,9 +822,9 @@ defmodule Mix.Tasks.Release do
 
     * `RELEASE_DISTRIBUTION` - how do we want to run the distribution.
       May be `name` (long names), `sname` (short names) or `none`
-      (distribution is not started automatically). Defaults to
-      `sname` which allows access only within the current system.
-      `name` allows external connections
+      (distribution is not started automatically). Defaults to `sname`.
+      When connecting nodes across hosts, you typically want to set
+      this to `name` (required to use IPs as host names)
 
     * `RELEASE_BOOT_SCRIPT` - the name of the boot script to use when starting
       the release. This script is used when running commands such as `start` and
@@ -1015,7 +1024,7 @@ defmodule Mix.Tasks.Release do
     * `--no-deps-check` - does not check dependencies
     * `--no-elixir-version-check` - does not check Elixir version
     * `--no-compile` - does not compile before assembling the release
-    * `--overwrite` - if there is an existing release version, overwrite it
+    * `--overwrite` - overwrite existing files instead of prompting the user for action
     * `--path` - the path of the release
     * `--quiet` - does not write progress to the standard output
     * `--version` - the version of the release
@@ -1044,6 +1053,8 @@ defmodule Mix.Tasks.Release do
   def run(args) do
     Mix.Project.get!()
     Mix.Task.run("compile", args)
+    Mix.ensure_application!(:sasl)
+    Mix.ensure_application!(:crypto)
 
     config = Mix.Project.config()
 
@@ -1105,7 +1116,7 @@ defmodule Mix.Tasks.Release do
     # releases/
     #   COOKIE
     #   start_erl.data
-    consolidation_path = build_rel(release, config)
+    {consolidation_path, release} = build_rel(release, config)
 
     [
       # erts-VSN/
@@ -1210,7 +1221,7 @@ defmodule Mix.Tasks.Release do
          :ok <- Mix.Release.make_sys_config(release, sys_config, config_provider_path),
          :ok <- Mix.Release.make_cookie(release, cookie_path),
          :ok <- Mix.Release.make_start_erl(release, start_erl_path) do
-      consolidation_path
+      {consolidation_path, release}
     else
       {:error, message} ->
         File.rm_rf!(version_path)
@@ -1335,7 +1346,7 @@ defmodule Mix.Tasks.Release do
   end
 
   defp info(release, message) do
-    unless release.options[:quiet] do
+    if !release.options[:quiet] do
       Mix.shell().info(message)
     end
   end
@@ -1437,7 +1448,10 @@ defmodule Mix.Tasks.Release do
       {"elixir",
        &(&1
          |> File.read!()
-         |> String.replace(~s[ -pa "$SCRIPT_PATH"/../lib/*/ebin], "")
+         |> String.replace(
+           ~s[ -elixir_root "$SCRIPT_PATH"/../lib -pa "$SCRIPT_PATH"/../lib/elixir/ebin ],
+           " "
+         )
          |> replace_erts_bin(release, ~s["$SCRIPT_PATH"/../../erts-#{release.erts_version}/bin/]))},
       {"iex", &File.read!/1}
     ]
@@ -1448,7 +1462,10 @@ defmodule Mix.Tasks.Release do
       {"elixir.bat",
        &(&1
          |> File.read!()
-         |> String.replace(~s[goto expand_erl_libs], ~s[goto run])
+         |> String.replace(
+           ~s[ -elixir_root !SCRIPT_PATH!..\\lib -pa !SCRIPT_PATH!..\\lib\\elixir\\ebin ],
+           " "
+         )
          |> replace_erts_bin(release, ~s[%~dp0\\..\\..\\erts-#{release.erts_version}\\bin\\]))},
       {"iex.bat", &File.read!/1}
     ]
@@ -1472,13 +1489,13 @@ defmodule Mix.Tasks.Release do
     reboot? = Keyword.get(release.options, :reboot_system_after_config, false)
 
     if reboot? and release.config_providers != [] do
-      "-elixir -config_provider_reboot_mode #{env_var}"
+      "-elixir config_provider_reboot_mode #{env_var}"
     else
       "-mode #{env_var}"
     end
   end
 
-  embed_template(:vm_args, Mix.Tasks.Release.Init.vm_args_text())
+  embed_template(:vm_args, Mix.Tasks.Release.Init.vm_args_text(false))
   embed_template(:env, Mix.Tasks.Release.Init.env_text())
   embed_template(:cli, Mix.Tasks.Release.Init.cli_text())
   embed_template(:env_bat, Mix.Tasks.Release.Init.env_bat_text())
