@@ -32,16 +32,16 @@ defmodule Mix.Tasks.Release.Init do
       Mix.raise("Expected \"mix release.init\" without arguments, got: #{inspect(args)}")
     end
 
-    create_file("rel/vm.args.eex", vm_args_text(), opts)
-    create_file("rel/remote.vm.args.eex", vm_args_text(), opts)
+    create_file("rel/vm.args.eex", vm_args_text(false), opts)
+    create_file("rel/remote.vm.args.eex", vm_args_text(true), opts)
     create_file("rel/env.sh.eex", env_text(), opts)
     create_file("rel/env.bat.eex", env_bat_text(), opts)
     :ok
   end
 
   @doc false
-  def vm_args_text,
-    do: ~S"""
+  def vm_args_text(remote?),
+    do: """
     ## Customize flags given to the VM: https://www.erlang.org/doc/man/erl.html
     ## -mode/-name/-sname/-setcookie are configured via env vars, do not set them here
 
@@ -50,6 +50,10 @@ defmodule Mix.Tasks.Release.Init do
 
     ## Tweak GC to run more often
     ##-env ERL_FULLSWEEP_AFTER 10
+
+    ## Enable deployment without epmd
+    ## (requires changing both vm.args and remote.vm.args)
+    ##-start_epmd false -erl_epmd_port 6789#{if(remote?, do: " -dist_listen false")}
     """
 
   @doc false
@@ -83,8 +87,17 @@ defmodule Mix.Tasks.Release.Init do
     #!/bin/sh
     set -e
 
-    SELF=$(readlink "$0" || true)
-    if [ -z "$SELF" ]; then SELF="$0"; fi
+    readlink_f () {
+      cd "$(dirname "$1")" > /dev/null
+      filename="$(basename "$1")"
+      if [ -h "$filename" ]; then
+        readlink_f "$(readlink "$filename")"
+      else
+        echo "$(pwd -P)/$filename"
+      fi
+    }
+
+    SELF=$(readlink_f "$0")
     RELEASE_ROOT="$(CDPATH='' cd "$(dirname "$SELF")/.." && pwd -P)"
     export RELEASE_ROOT
     RELEASE_NAME="${RELEASE_NAME:-"<%= @release.name %>"}"
@@ -132,7 +145,7 @@ defmodule Mix.Tasks.Release.Init do
           ;;
 
         *)
-          echo "ERROR: Expected sname, name, or none in RELEASE_DISTRIBUTION, got: $RELEASE_DISTRIBUTION" >&2
+          echo "ERROR: Expected RELEASE_DISTRIBUTION to be sname, name, or none, got: $RELEASE_DISTRIBUTION" >&2
           exit 1
           ;;
       esac
@@ -185,7 +198,7 @@ defmodule Mix.Tasks.Release.Init do
         ;;
 
       start_iex)
-        start "iex" --werl
+        start "iex"
         ;;
 
       daemon)
@@ -201,19 +214,20 @@ defmodule Mix.Tasks.Release.Init do
           echo "ERROR: EVAL expects an expression as argument" >&2
           exit 1
         fi
-
+        script="$2"
+        shift 2
         export_release_sys_config
         exec "$REL_VSN_DIR/elixir" \
            --cookie "$RELEASE_COOKIE" \
            --erl-config "$RELEASE_SYS_CONFIG" \
            --boot "$REL_VSN_DIR/$RELEASE_BOOT_SCRIPT_CLEAN" \
            --boot-var RELEASE_LIB "$RELEASE_ROOT/lib" \
-           --vm-args "$RELEASE_VM_ARGS" --eval "$2"
+           --vm-args "$RELEASE_VM_ARGS" --eval "$script" -- "$@"
         ;;
 
       remote)
         exec "$REL_VSN_DIR/iex" \
-             --werl --hidden --cookie "$RELEASE_COOKIE" \
+             --hidden --cookie "$RELEASE_COOKIE" \
              $(release_distribution "rem-$(rand)-$RELEASE_NODE") \
              --boot "$REL_VSN_DIR/$RELEASE_BOOT_SCRIPT_CLEAN" \
              --boot-var RELEASE_LIB "$RELEASE_ROOT/lib" \
@@ -275,7 +289,7 @@ defmodule Mix.Tasks.Release.Init do
     rem set RELEASE_MODE=interactive
 
     rem Set the release to work across nodes.
-    rem RELEASE_DISTRIBUTION must be "sname" (local), "name" (distributed) or "none".
+    rem RELEASE_DISTRIBUTION must be sname (local), name (distributed) or none.
     rem set RELEASE_DISTRIBUTION=name
     rem set RELEASE_NODE=<%= @release.name %>
     """
@@ -287,16 +301,17 @@ defmodule Mix.Tasks.Release.Init do
     setlocal enabledelayedexpansion
 
     pushd .
-    cd "%~dp0\.."
+    pushd "%~dp0\.."
     set RELEASE_ROOT=%cd%
     popd
+    popd
 
     if not defined RELEASE_NAME (set RELEASE_NAME=<%= @release.name %>)
     if not defined RELEASE_VSN (for /f "tokens=1,2" %%K in ('type "!RELEASE_ROOT!\releases\start_erl.data"') do (set ERTS_VSN=%%K) && (set RELEASE_VSN=%%L))
     if not defined RELEASE_PROG (set RELEASE_PROG=%~nx0)
     set RELEASE_COMMAND=%~1
     set REL_VSN_DIR=!RELEASE_ROOT!\releases\!RELEASE_VSN!
-    call "!REL_VSN_DIR!\env.bat"
+    call "!REL_VSN_DIR!\env.bat" %*
 
     if not defined RELEASE_COOKIE (set /p RELEASE_COOKIE=<!RELEASE_ROOT!\releases\COOKIE)
     if not defined RELEASE_MODE (set RELEASE_MODE=embedded)
@@ -309,13 +324,33 @@ defmodule Mix.Tasks.Release.Init do
     if not defined RELEASE_BOOT_SCRIPT_CLEAN (set RELEASE_BOOT_SCRIPT_CLEAN=start_clean)
     if not defined RELEASE_SYS_CONFIG (set RELEASE_SYS_CONFIG=!REL_VSN_DIR!\sys)
 
+    if "!RELEASE_DISTRIBUTION!" == "none" (
+      rem
+    ) else if "!RELEASE_DISTRIBUTION!" == "name" (
+      rem
+    ) else if "!RELEASE_DISTRIBUTION!" == "sname" (
+      rem
+    ) else (
+      echo ERROR: Expected RELEASE_DISTRIBUTION to be sname, name, or none, got: !RELEASE_DISTRIBUTION!
+      exit /B 1
+    )
+
+    if "!RELEASE_MODE!" == "embedded" (
+      rem
+    ) else if "!RELEASE_MODE!" == "interactive" (
+      rem
+    ) else (
+      echo ERROR: Expected RELEASE_MODE to be embedded or interactive, got: !RELEASE_MODE!
+      exit /B 1
+    )
+
     if "%~1" == "start" (set "REL_EXEC=elixir" && set "REL_EXTRA=--no-halt" && set "REL_GOTO=start")
-    if "%~1" == "start_iex" (set "REL_EXEC=iex" && set "REL_EXTRA=--werl" && set "REL_GOTO=start")
+    if "%~1" == "start_iex" (set "REL_EXEC=iex" && set "REL_GOTO=start")
     if "%~1" == "install" (set "REL_GOTO=install")
     if "%~1" == "eval" (
       if "%~2" == "" (
         echo ERROR: EVAL expects an expression as argument
-        goto end
+        exit /B 1
       )
       set "REL_GOTO=eval"
     )
@@ -343,7 +378,7 @@ defmodule Mix.Tasks.Release.Init do
     if "%~1" == "rpc" (
       if "%~2" == "" (
         echo ERROR: RPC expects an expression as argument
-        goto end
+        exit /B 1
       )
       set "REL_RPC=%~2"
       goto rpc
@@ -364,7 +399,10 @@ defmodule Mix.Tasks.Release.Init do
     echo    pid          Prints the operating system PID of the running system via a remote command
     echo    version      Prints the release name and version to be booted
     echo.
-    if not "%~1" == "" (echo ERROR: Unknown command %~1)
+    if not "%~1" == "" (
+      echo ERROR: Unknown command %~1
+      exit /B 1
+    )
     goto end
 
     :start
@@ -385,13 +423,21 @@ defmodule Mix.Tasks.Release.Init do
     goto end
 
     :eval
+    set EVAL=%~2
+    shift
+    :loop
+    shift
+    if not "%1"=="" (
+      set args=%args% %1
+      goto :loop
+    )
     "!REL_VSN_DIR!\elixir.bat" ^
-      --eval "%~2" ^
+      --eval "!EVAL!" ^
       --cookie "!RELEASE_COOKIE!" ^
       --erl-config "!RELEASE_SYS_CONFIG!" ^
       --boot "!REL_VSN_DIR!\!RELEASE_BOOT_SCRIPT_CLEAN!" ^
       --boot-var RELEASE_LIB "!RELEASE_ROOT!\lib" ^
-      --vm-args "!RELEASE_VM_ARGS!"
+      --vm-args "!RELEASE_VM_ARGS!" -- %args%
     goto end
 
     :remote
@@ -402,7 +448,7 @@ defmodule Mix.Tasks.Release.Init do
     )
 
     "!REL_VSN_DIR!\iex.bat" ^
-      --werl --hidden --cookie "!RELEASE_COOKIE!" ^
+      --hidden --cookie "!RELEASE_COOKIE!" ^
       !RELEASE_DISTRIBUTION_FLAG! ^
       --boot "!REL_VSN_DIR!\!RELEASE_BOOT_SCRIPT_CLEAN!" ^
       --boot-var RELEASE_LIB "!RELEASE_ROOT!\lib" ^
@@ -439,14 +485,13 @@ defmodule Mix.Tasks.Release.Init do
 
     if "!RELEASE_DISTRIBUTION!" == "none" (
       echo ERROR: RELEASE_DISTRIBUTION is required in install command
-      goto end
+      exit /B 1
     )
 
     "!ERLSRV!" add "!RELEASE_NAME!_!RELEASE_NAME!" ^
       -!RELEASE_DISTRIBUTION! "!RELEASE_NODE!" ^
       -env RELEASE_ROOT="!RELEASE_ROOT!" -env RELEASE_NAME="!RELEASE_NAME!" -env RELEASE_VSN="!RELEASE_VSN!" -env RELEASE_MODE="!RELEASE_MODE!" -env RELEASE_COOKIE="!RELEASE_COOKIE!" -env RELEASE_NODE="!RELEASE_NODE!" -env RELEASE_VM_ARGS="!RELEASE_VM_ARGS!" -env RELEASE_TMP="!RELEASE_TMP!" -env RELEASE_SYS_CONFIG="!RELEASE_SYS_CONFIG!" ^
-      -args "-setcookie !RELEASE_COOKIE! -config !RELEASE_SYS_CONFIG! <%= release_mode(@release, "!RELEASE_MODE!") %> -boot !REL_VSN_DIR!\start -boot_var RELEASE_LIB !RELEASE_ROOT!\lib -args_file !REL_VSN_DIR!\vm.args"
-
+      -args "-setcookie !RELEASE_COOKIE! -config ""!RELEASE_SYS_CONFIG!"" <%= release_mode(@release, "!RELEASE_MODE!") %> -boot ""!REL_VSN_DIR!\start"" -boot_var RELEASE_LIB ""!RELEASE_ROOT!\lib"" -args_file ""!REL_VSN_DIR!\vm.args\"""
     if %ERRORLEVEL% EQU 0 (
       echo Service installed but not started. From now on, it must be started and stopped by erlsrv:
       echo.
