@@ -1,3 +1,3 @@
-use Mix.Config
+import Config
 
 config :escript_test, erl_val: "Erlang value"
