@@ -7,7 +7,8 @@ defmodule Foo.MixProject do
       version: "0.1.0",
       # Choose something besides *_test.exs so that these test files don't
       # get accidentally swept up into the actual Mix test suite.
-      test_pattern: "*_tests.exs"
+      test_pattern: "*_tests.exs",
+      aliases: [mytask: fn _ -> Mix.shell().info("foo_running") end]
     ]
   end
 end
