@@ -6,8 +6,8 @@ defmodule Mix.ReleaseTest do
   import Mix.Release
   doctest Mix.Release
 
-  _ = Application.ensure_started(:eex)
-  _ = Application.ensure_started(:runtime_tools)
+  Application.ensure_loaded(:eex)
+  Application.ensure_loaded(:runtime_tools)
 
   @erts_version :erlang.system_info(:version)
   @erts_source Path.join(:code.root_dir(), "erts-#{@erts_version}")
@@ -16,6 +16,12 @@ defmodule Mix.ReleaseTest do
   @runtime_tools_version Application.spec(:runtime_tools, :vsn)
   @eex_ebin Application.app_dir(:eex, "ebin")
 
+  setup_all do
+    Mix.ensure_application!(:sasl)
+    Mix.ensure_application!(:crypto)
+    :ok
+  end
+
   setup do
     File.rm_rf!(tmp_path("mix_release"))
     File.mkdir_p!(tmp_path("mix_release"))
@@ -168,7 +174,7 @@ defmodule Mix.ReleaseTest do
         Mix.Project.in_project(:mix, project_path, app_config, fn _ ->
           Code.prepend_path(ebin_dir)
           release = from_config!(nil, app_config, [])
-          assert release.applications.cowboy[:vsn] == '1.1.2'
+          assert release.applications.cowboy[:vsn] == ~c"1.1.2"
         end)
       end)
     end
@@ -331,17 +337,14 @@ defmodule Mix.ReleaseTest do
 
     test "configures other applications in cascade", context do
       in_tmp(context.test, fn ->
-        app =
+        write_app!(
+          "my_sample_mode/ebin/my_sample_mode.app",
           {:application, :my_sample_mode,
            applications: [:kernel, :stdlib, :elixir, :runtime_tools, :compiler],
-           description: 'my_sample_mode',
+           description: ~c"my_sample_mode",
            modules: [],
-           vsn: '1.0.0'}
-
-        File.mkdir_p!("my_sample_mode/ebin")
-        Code.prepend_path("my_sample_mode/ebin")
-        format = :io_lib.format("%% coding: utf-8~n~p.~n", [app])
-        File.write!("my_sample_mode/ebin/my_sample_mode.app", format)
+           vsn: ~c"1.0.0"}
+        )
 
         apps = [my_sample_mode: :temporary]
         release = release(applications: apps)
@@ -375,11 +378,11 @@ defmodule Mix.ReleaseTest do
       assert release.boot_scripts.start_clean == [
                kernel: :permanent,
                stdlib: :permanent,
-               elixir: :none,
-               sasl: :none,
-               mix: :none,
+               elixir: :load,
+               sasl: :load,
+               mix: :load,
                iex: :none,
-               compiler: :none
+               compiler: :load
              ]
     end
   end
@@ -429,7 +432,7 @@ defmodule Mix.ReleaseTest do
 
       assert {:ok,
               [
-                {:release, {'demo', '0.1.0'}, {:erts, @erts_version},
+                {:release, {~c"demo", ~c"0.1.0"}, {:erts, @erts_version},
                  [
                    {:kernel, _, :permanent},
                    {:stdlib, _, :permanent},
@@ -441,14 +444,14 @@ defmodule Mix.ReleaseTest do
                  ]}
               ]} = :file.consult(@boot_script_path <> ".rel")
 
-      assert {:ok, [{:script, {'demo', '0.1.0'}, instructions}]} =
+      assert {:ok, [{:script, {~c"demo", ~c"0.1.0"}, instructions}]} =
                :file.consult(@boot_script_path <> ".script")
 
-      assert {:path, ['$ROOT/lib/kernel-#{@kernel_version}/ebin']} in instructions
-      assert {:path, ['$RELEASE_LIB/elixir-#{@elixir_version}/ebin']} in instructions
+      assert {:path, [~c"$ROOT/lib/kernel-#{@kernel_version}/ebin"]} in instructions
+      assert {:path, [~c"$RELEASE_LIB/elixir-#{@elixir_version}/ebin"]} in instructions
 
       assert File.read!(@boot_script_path <> ".boot") |> :erlang.binary_to_term() ==
-               {:script, {'demo', '0.1.0'}, instructions}
+               {:script, {~c"demo", ~c"0.1.0"}, instructions}
     end
 
     test "prepends relevant paths" do
@@ -461,16 +464,16 @@ defmodule Mix.ReleaseTest do
                ["$RELEASE_LIB/sample"]
              ) == :ok
 
-      assert {:ok, [{:script, {'demo', '0.1.0'}, instructions}]} =
+      assert {:ok, [{:script, {~c"demo", ~c"0.1.0"}, instructions}]} =
                :file.consult(@boot_script_path <> ".script")
 
-      assert {:path, ['$ROOT/lib/kernel-#{@kernel_version}/ebin']} in instructions
-      refute {:path, ['$RELEASE_LIB/elixir-#{@elixir_version}/ebin']} in instructions
+      assert {:path, [~c"$ROOT/lib/kernel-#{@kernel_version}/ebin"]} in instructions
+      refute {:path, [~c"$RELEASE_LIB/elixir-#{@elixir_version}/ebin"]} in instructions
 
-      assert {:path, ['$RELEASE_LIB/sample', '$RELEASE_LIB/elixir-#{@elixir_version}/ebin']} in instructions
+      assert {:path, [~c"$RELEASE_LIB/sample", ~c"$RELEASE_LIB/elixir-#{@elixir_version}/ebin"]} in instructions
 
       assert File.read!(@boot_script_path <> ".boot") |> :erlang.binary_to_term() ==
-               {:script, {'demo', '0.1.0'}, instructions}
+               {:script, {~c"demo", ~c"0.1.0"}, instructions}
     end
 
     test "works when :load/:none is set at the leaf" do
@@ -586,13 +589,13 @@ defmodule Mix.ReleaseTest do
     test "writes sys_config with encoding" do
       assert make_sys_config(
                release([]),
-               [encoding: {:_μ, :"£", "£", '£'}],
+               [encoding: {:_μ, :"£", "£", ~c"£"}],
                "unused/runtime/path"
              ) ==
                :ok
 
       {:ok, contents} = :file.consult(@sys_config)
-      assert contents == [[encoding: {:_μ, :"£", "£", '£'}]]
+      assert contents == [[encoding: {:_μ, :"£", "£", ~c"£"}]]
     end
 
     test "writes the given sys_config with config providers" do
@@ -656,7 +659,7 @@ defmodule Mix.ReleaseTest do
       assert File.read!(Path.join(destination, "bin/erl")) =~
                ~s|ROOTDIR="${ERL_ROOTDIR:-"$(dirname "$(dirname "$BINDIR")")"}|
 
-      unless match?({:win32, _}, :os.type()) do
+      if not match?({:win32, _}, :os.type()) do
         assert File.lstat!(Path.join(destination, "bin/erl")).mode |> rem(0o1000) == 0o755
       end
 
@@ -683,9 +686,6 @@ defmodule Mix.ReleaseTest do
     test "copies and strips beams" do
       assert copy_ebin(release([]), @eex_ebin, tmp_path("eex_ebin"))
 
-      assert size!(Path.join(@eex_ebin, "eex.app")) ==
-               size!(tmp_path("eex_ebin/eex.app"))
-
       assert size!(Path.join(@eex_ebin, "Elixir.EEx.beam")) >
                size!(tmp_path("eex_ebin/Elixir.EEx.beam"))
     end
@@ -693,9 +693,6 @@ defmodule Mix.ReleaseTest do
     test "copies without stripping beams" do
       assert copy_ebin(release(strip_beams: false), @eex_ebin, tmp_path("eex_ebin"))
 
-      assert size!(Path.join(@eex_ebin, "eex.app")) ==
-               size!(tmp_path("eex_ebin/eex.app"))
-
       assert size!(Path.join(@eex_ebin, "Elixir.EEx.beam")) ==
                size!(tmp_path("eex_ebin/Elixir.EEx.beam"))
     end
@@ -727,9 +724,6 @@ defmodule Mix.ReleaseTest do
     test "copies and strips beams" do
       assert copy_app(release(applications: [eex: :permanent]), :eex)
 
-      assert size!(Path.join(@eex_ebin, "eex.app")) ==
-               size!(Path.join(@release_lib, "eex-#{@elixir_version}/ebin/eex.app"))
-
       assert size!(Path.join(@eex_ebin, "Elixir.EEx.beam")) >
                size!(Path.join(@release_lib, "eex-#{@elixir_version}/ebin/Elixir.EEx.beam"))
     end
@@ -737,9 +731,6 @@ defmodule Mix.ReleaseTest do
     test "copies without stripping beams" do
       assert copy_app(release(strip_beams: false, applications: [eex: :permanent]), :eex)
 
-      assert size!(Path.join(@eex_ebin, "eex.app")) ==
-               size!(Path.join(@release_lib, "eex-#{@elixir_version}/ebin/eex.app"))
-
       assert size!(Path.join(@eex_ebin, "Elixir.EEx.beam")) ==
                size!(Path.join(@release_lib, "eex-#{@elixir_version}/ebin/Elixir.EEx.beam"))
     end
@@ -766,8 +757,11 @@ defmodule Mix.ReleaseTest do
         |> File.read!()
         |> strip_beam()
 
-      assert {:error, :beam_lib, {:missing_chunk, _, 'Dbgi'}} = :beam_lib.chunks(beam, ['Dbgi'])
-      assert {:error, :beam_lib, {:missing_chunk, _, 'Docs'}} = :beam_lib.chunks(beam, ['Docs'])
+      assert {:error, :beam_lib, {:missing_chunk, _, ~c"Dbgi"}} =
+               :beam_lib.chunks(beam, [~c"Dbgi"])
+
+      assert {:error, :beam_lib, {:missing_chunk, _, ~c"Docs"}} =
+               :beam_lib.chunks(beam, [~c"Docs"])
     end
 
     test "can keep docs and debug info, if requested" do
@@ -776,8 +770,8 @@ defmodule Mix.ReleaseTest do
         |> File.read!()
         |> strip_beam(keep: ["Docs", "Dbgi"])
 
-      assert {:ok, {EEx, [{'Dbgi', _}]}} = :beam_lib.chunks(beam, ['Dbgi'])
-      assert {:ok, {EEx, [{'Docs', _}]}} = :beam_lib.chunks(beam, ['Docs'])
+      assert {:ok, {EEx, [{~c"Dbgi", _}]}} = :beam_lib.chunks(beam, [~c"Dbgi"])
+      assert {:ok, {EEx, [{~c"Docs", _}]}} = :beam_lib.chunks(beam, [~c"Docs"])
     end
 
     test "strip beams without compression" do
@@ -793,18 +787,15 @@ defmodule Mix.ReleaseTest do
   describe "included applications" do
     test "are included in the release", context do
       in_tmp(context.test, fn ->
-        app =
+        write_app!(
+          "my_sample1/ebin/my_sample1.app",
           {:application, :my_sample1,
            applications: [:kernel, :stdlib, :elixir],
-           description: 'my_sample1',
+           description: ~c"my_sample1",
            modules: [],
-           vsn: '1.0.0',
+           vsn: ~c"1.0.0",
            included_applications: [:runtime_tools]}
-
-        File.mkdir_p!("my_sample1/ebin")
-        Code.prepend_path("my_sample1/ebin")
-        format = :io_lib.format("%% coding: utf-8~n~p.~n", [app])
-        File.write!("my_sample1/ebin/my_sample1.app", format)
+        )
 
         release = release(applications: [my_sample1: :permanent])
         assert release.boot_scripts.start[:runtime_tools] == :load
@@ -816,18 +807,15 @@ defmodule Mix.ReleaseTest do
 
     test "raise on conflict", context do
       in_tmp(context.test, fn ->
-        app =
+        write_app!(
+          "my_sample2/ebin/my_sample2.app",
           {:application, :my_sample2,
            applications: [:kernel, :stdlib, :elixir, :runtime_tools],
-           description: 'my_sample',
+           description: ~c"my_sample",
            modules: [],
-           vsn: '1.0.0',
+           vsn: ~c"1.0.0",
            included_applications: [:runtime_tools]}
-
-        File.mkdir_p!("my_sample2/ebin")
-        Code.prepend_path("my_sample2/ebin")
-        format = :io_lib.format("%% coding: utf-8~n~p.~n", [app])
-        File.write!("my_sample2/ebin/my_sample2.app", format)
+        )
 
         assert_raise Mix.Error,
                      ":runtime_tools is listed both as a regular application and as an included application",
@@ -839,23 +827,70 @@ defmodule Mix.ReleaseTest do
   describe "optional applications" do
     test "are ignored if not available", context do
       in_tmp(context.test, fn ->
-        app =
-          {:application, :my_sample1,
+        write_app!(
+          "my_sample3/ebin/my_sample3.app",
+          {:application, :my_sample3,
            applications: [:kernel, :stdlib, :elixir, :unknown],
            optional_applications: [:unknown],
-           description: 'my_sample1',
+           description: ~c"my_sample3",
            modules: [],
-           vsn: '1.0.0'}
+           vsn: ~c"1.0.0"}
+        )
 
-        File.mkdir_p!("my_sample1/ebin")
-        Code.prepend_path("my_sample1/ebin")
-        format = :io_lib.format("%% coding: utf-8~n~p.~n", [app])
-        File.write!("my_sample1/ebin/my_sample1.app", format)
-
-        release = release(applications: [my_sample1: :permanent])
+        release = release(applications: [my_sample3: :permanent])
         assert release.boot_scripts.start[:unknown] == nil
       end)
     end
+
+    test "are ignored even if mode changes", context do
+      in_tmp(context.test, fn ->
+        write_app!(
+          "has_optional/ebin/has_optional.app",
+          {:application, :has_optional,
+           applications: [:kernel, :stdlib, :elixir, :unknown],
+           optional_applications: [:unknown],
+           description: ~c"has_optional",
+           modules: [],
+           vsn: ~c"1.0.0"}
+        )
+
+        write_app!(
+          "points_as_permanent/ebin/points_as_permanent.app",
+          {:application, :points_as_permanent,
+           applications: [:kernel, :stdlib, :elixir, :has_optional],
+           optional_applications: [:unknown],
+           description: ~c"points_as_permanent",
+           modules: [],
+           vsn: ~c"1.0.0"}
+        )
+
+        write_app!(
+          "points_as_temporary/ebin/points_as_temporary.app",
+          {:application, :points_as_temporary,
+           applications: [:kernel, :stdlib, :elixir, :has_optional],
+           optional_applications: [:unknown],
+           description: ~c"points_as_temporary",
+           modules: [],
+           vsn: ~c"1.0.0"}
+        )
+
+        release =
+          release(
+            applications: [points_as_permanent: :permanent, points_as_temporary: :temporary]
+          )
+
+        assert release.boot_scripts.start[:has_optional] == :permanent
+        assert release.boot_scripts.start[:unknown] == nil
+      end)
+    end
+  end
+
+  defp write_app!(path, app) do
+    dir = Path.dirname(path)
+    File.mkdir_p!(dir)
+    Code.prepend_path(dir)
+    format = :io_lib.format("%% coding: utf-8~n~p.~n", [app])
+    File.write!(path, format)
   end
 
   defp size!(path) do
