@@ -160,6 +160,58 @@ defmodule Mix.TaskTest do
     end)
   end
 
+  test "aliases considered for umbrella apps" do
+    in_fixture("umbrella_test", fn ->
+      Mix.Project.in_project(:umbrella, ".", fn _ ->
+        File.mkdir_p!("apps/baz/lib/")
+
+        File.write!("apps/baz/mix.exs", """
+        defmodule Baz.MixProject do
+          use Mix.Project
+
+          def project do
+            [
+              app: :baz,
+              version: "0.1.0",
+              aliases: [help: fn _ -> raise "oops" end]
+            ]
+          end
+        end
+        """)
+
+        File.write!("apps/baz/lib/baz.ex", """
+        defmodule Baz do
+          def hello do
+            :world
+          end
+        end
+        """)
+
+        Mix.Task.run("compile")
+        assert_received {:mix_shell, :info, ["==> foo"]}
+        assert_received {:mix_shell, :info, ["==> bar"]}
+        assert_received {:mix_shell, :info, ["==> baz"]}
+        Mix.shell().flush()
+
+        # A child alias does not overlap an umbrella task
+        assert Mix.Task.run("help") == :ok
+
+        # A missing umbrella alias can be found in children
+        assert [:ok, :ok] = Mix.Task.run("mytask")
+        assert_received {:mix_shell, :info, ["==> foo"]}
+        assert_received {:mix_shell, :info, ["foo_running"]}
+        assert_received {:mix_shell, :info, ["==> bar"]}
+        assert_received {:mix_shell, :info, ["bar_running"]}
+        refute_received {:mix_shell, :info, ["==> baz"]}
+
+        # An unknown task or alias anywhere fails
+        assert_raise Mix.NoTaskError, fn ->
+          Mix.Task.run("completely_unknown")
+        end
+      end)
+    end)
+  end
+
   test "reenable/1 for recursive inside umbrella child" do
     in_fixture("umbrella_dep/deps/umbrella", fn ->
       Mix.Project.in_project(:umbrella, ".", fn _ ->
@@ -236,18 +288,6 @@ defmodule Mix.TaskTest do
     assert Mix.Task.moduledoc(Mix.Tasks.Hello) == "A test task.\n"
   end
 
-  test "preferred_cli_env/1 returns nil for missing task" do
-    assert Mix.Task.preferred_cli_env(:no_task) == nil
-  end
-
-  test "preferred_cli_env/1 returns nil when task does not have @preferred_cli_env attribute" do
-    assert Mix.Task.preferred_cli_env(:deps) == nil
-  end
-
-  test "preferred_cli_env/1 returns specified @preferred_cli_env attribute" do
-    assert Mix.Task.preferred_cli_env(:test) == :test
-  end
-
   test "shortdoc/1" do
     assert Mix.Task.shortdoc(Mix.Tasks.Hello) == "This is short documentation, see"
   end
