@@ -13,6 +13,8 @@ defmodule Mix.Tasks.App.StartTest do
     end
   end
 
+  @moduletag :capture_log
+
   test "compiles and starts the project" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(AppStartSample)
@@ -30,7 +32,7 @@ defmodule Mix.Tasks.App.StartTest do
       assert File.regular?("_build/dev/lib/app_start_sample/ebin/Elixir.A.beam")
       assert File.regular?("_build/dev/lib/app_start_sample/ebin/app_start_sample.app")
 
-      assert :code.is_loaded(A)
+      assert Code.loaded?(A)
       refute List.keyfind(Application.started_applications(), :app_start_sample, 0)
       assert List.keyfind(Application.started_applications(), :logger, 0)
       purge([A])
@@ -38,15 +40,17 @@ defmodule Mix.Tasks.App.StartTest do
       Mix.Task.reenable("app.config")
       Mix.Task.reenable("app.start")
       Mix.Task.run("app.start", [])
-      refute :code.is_loaded(A)
+      refute Code.loaded?(A)
       assert List.keyfind(Application.started_applications(), :app_start_sample, 0)
       assert List.keyfind(Application.started_applications(), :logger, 0)
     end)
+  after
+    Application.stop(:app_start_sample)
   end
 
   describe "unit tests" do
     test "start does nothing if no apps are given" do
-      assert Mix.Tasks.App.Start.start([], :temporary) == :ok
+      assert Mix.Tasks.App.Start.start([], :temporary, :serial) == :ok
     end
 
     test "allows type to be configured" do
@@ -85,7 +89,7 @@ defmodule Mix.Tasks.App.StartTest do
             "returned an error: :bye"
 
         assert_raise Mix.Error, message, fn ->
-          Mix.Tasks.App.Start.start([:return_sample], :temporary)
+          Mix.Tasks.App.Start.start([:return_sample], :temporary, :serial)
         end
       end)
     end
@@ -106,7 +110,7 @@ defmodule Mix.Tasks.App.StartTest do
             "        Mix.Tasks.App.StartTest.ReturnApp.start/2"
 
         assert_raise Mix.Error, message, fn ->
-          Mix.Tasks.App.Start.start([:return_sample], :temporary)
+          Mix.Tasks.App.Start.start([:return_sample], :temporary, :serial)
         end
       end)
     end
@@ -124,7 +128,7 @@ defmodule Mix.Tasks.App.StartTest do
             "returned a bad value: :bad"
 
         assert_raise Mix.Error, message, fn ->
-          Mix.Tasks.App.Start.start([:return_sample], :temporary)
+          Mix.Tasks.App.Start.start([:return_sample], :temporary, :serial)
         end
       end)
     end
@@ -157,7 +161,7 @@ defmodule Mix.Tasks.App.StartTest do
             "Mix.Tasks.App.StartTest.ExitApp.start(:normal, :bye)\n" <> "    ** (EXIT) :bye"
 
         assert_raise Mix.Error, message, fn ->
-          Mix.Tasks.App.Start.start([:exit_sample], :temporary)
+          Mix.Tasks.App.Start.start([:exit_sample], :temporary, :serial)
         end
       end)
     end
@@ -174,7 +178,7 @@ defmodule Mix.Tasks.App.StartTest do
             "Mix.Tasks.App.StartTest.ExitApp.start(:normal, :normal)\n" <> "    ** (EXIT) normal"
 
         assert_raise Mix.Error, message, fn ->
-          Mix.Tasks.App.Start.start([:exit_sample], :temporary)
+          Mix.Tasks.App.Start.start([:exit_sample], :temporary, :serial)
         end
       end)
     end
