@@ -31,10 +31,11 @@ defmodule Mix.Tasks.ArchiveTest do
       Mix.Tasks.Archive.Build.run(["--no-elixir-version-check"])
       message = "Generated archive \"archive-0.1.0.ez\" with MIX_ENV=dev"
       assert_received {:mix_shell, :info, [^message]}
-      assert File.regular?('archive-0.1.0.ez')
+      assert File.regular?(~c"archive-0.1.0.ez")
+      assert to_charlist(Mix.Project.consolidation_path()) not in :code.get_path()
 
       assert_archive_content_default()
-      refute has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/priv/.dot_file')
+      refute has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/priv/.dot_file")
     end)
   end
 
@@ -43,24 +44,24 @@ defmodule Mix.Tasks.ArchiveTest do
       Mix.Tasks.Archive.Build.run(["--no-elixir-version-check", "--include-dot-files"])
       message = "Generated archive \"archive-0.1.0.ez\" with MIX_ENV=dev"
       assert_received {:mix_shell, :info, [^message]}
-      assert File.regular?('archive-0.1.0.ez')
+      assert File.regular?(~c"archive-0.1.0.ez")
 
       assert_archive_content_default()
-      assert has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/priv/.dot_file')
+      assert has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/priv/.dot_file")
     end)
   end
 
   def assert_archive_content_default() do
-    assert File.regular?('archive-0.1.0.ez')
-    assert has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/.elixir')
-    assert has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/priv/not_really_an.so')
+    assert File.regular?(~c"archive-0.1.0.ez")
+    assert has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/.elixir")
+    assert has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/priv/not_really_an.so")
 
     assert has_in_zip_file?(
-             'archive-0.1.0.ez',
-             'archive-0.1.0/ebin/Elixir.Mix.Tasks.Local.Sample.beam'
+             ~c"archive-0.1.0.ez",
+             ~c"archive-0.1.0/ebin/Elixir.Mix.Tasks.Local.Sample.beam"
            )
 
-    assert has_in_zip_file?('archive-0.1.0.ez', 'archive-0.1.0/ebin/archive.app')
+    assert has_in_zip_file?(~c"archive-0.1.0.ez", ~c"archive-0.1.0/ebin/archive.app")
   end
 
   test "archive install" do
@@ -171,10 +172,10 @@ defmodule Mix.Tasks.ArchiveTest do
       assert File.dir?(tmp_path("userhome/.mix/archives/git_repo-0.1.0/git_repo-0.1.0/ebin"))
     end)
   after
-    purge([GitRepo.Archive, GitRepo.MixProject])
+    purge([GitRepo.Archive, GitRepo.MixProject, Mix.Local.Installer.MixProject])
   end
 
-  test "archive install, update, and uninstall life-cycle" do
+  test "archive install, update, and uninstall life cycle" do
     in_fixture("archive", fn ->
       # Install previous version
       Mix.Tasks.Archive.Build.run(["--no-elixir-version-check"])
