@@ -71,36 +71,47 @@ defmodule Mix.Tasks.Compile.AppTest do
       assert Mix.Tasks.Compile.App.run([]) == {:ok, []}
 
       properties = parse_resource_file(:sample)
-      assert properties[:vsn] == '0.1.0'
+      assert Application.spec(:sample, :vsn) == ~c"0.1.0"
+      assert properties[:vsn] == ~c"0.1.0"
       assert properties[:modules] == [A, B]
       assert properties[:applications] == [:kernel, :stdlib, :elixir]
       refute Keyword.has_key?(properties, :compile_env)
 
+      Application.unload(:sample)
       assert Mix.Tasks.Compile.App.run([]) == {:noop, []}
+      assert Application.spec(:sample, :vsn) == ~c"0.1.0"
     end)
   end
 
   test "generates .app file with compile_env" do
     in_fixture("no_mixfile", fn ->
       Mix.Project.push(MixTest.Case.Sample)
+      File.mkdir_p!("config")
+      File.write!("config/config.exs", "[]")
+      Mix.Task.run("loadconfig")
+
+      reset_config = fn ->
+        Mix.ProjectStack.reset_config_mtime()
+        ensure_touched("config/config.exs", "_build/dev/lib/sample/ebin/sample.app")
+      end
 
       Mix.ProjectStack.compile_env([{:app, :key, :error}])
       assert Mix.Tasks.Compile.App.run([]) == {:ok, []}
       assert parse_resource_file(:sample)[:compile_env] == [{:app, :key, :error}]
 
       # No-op with untouched unset compile_env
-      assert Mix.Tasks.Compile.App.run([]) == {:noop, []}
-
-      # No-op with same compile_env
-      Mix.ProjectStack.compile_env([{:app, :key, :error}])
-      assert Mix.Tasks.Compile.App.run([]) == {:noop, []}
+      reset_config.()
+      assert Mix.Tasks.Compile.App.run([]) == {:ok, []}
+      assert parse_resource_file(:sample)[:compile_env] == [{:app, :key, :error}]
 
       # Recompiles with new compile_env
+      reset_config.()
       Mix.ProjectStack.compile_env([{:app, :another, :error}])
       assert Mix.Tasks.Compile.App.run([]) == {:ok, []}
       assert parse_resource_file(:sample)[:compile_env] == [{:app, :another, :error}]
 
       # Keeps compile_env if forcing
+      reset_config.()
       assert Mix.Tasks.Compile.App.run(["--force"]) == {:ok, []}
       assert parse_resource_file(:sample)[:compile_env] == [{:app, :another, :error}]
     end)
@@ -114,10 +125,15 @@ defmodule Mix.Tasks.Compile.AppTest do
       Mix.Tasks.Compile.App.run([])
 
       properties = parse_resource_file(:custom_project)
-      assert properties[:vsn] == '0.2.0'
+      assert Application.spec(:custom_project, :vsn) == ~c"0.2.0"
+      assert properties[:vsn] == ~c"0.2.0"
       assert properties[:maxT] == :infinity
-      assert properties[:applications] == [:kernel, :stdlib, :elixir, :logger, :example_app]
-      assert properties[:description] == 'Some UTF-8 dëscriptión'
+      assert properties[:optional_applications] == [:ex_unit, :mix]
+      assert properties[:description] == ~c"Some UTF-8 dëscriptión"
+
+      assert properties[:applications] ==
+               [:kernel, :stdlib, :elixir, :logger, :ex_unit, :example_app, :mix]
+
       refute Keyword.has_key?(properties, :extra_applications)
     end)
   end
@@ -132,7 +148,9 @@ defmodule Mix.Tasks.Compile.AppTest do
       properties = parse_resource_file(:custom_deps)
 
       assert properties[:applications] ==
-               [:kernel, :stdlib, :elixir, :logger, :ok1, :ok3, :ok4, :ok7, :ok11]
+               [:kernel, :stdlib, :elixir, :logger, :ok1, :ok3, :ok4, :ok6, :ok7, :ok11]
+
+      assert properties[:optional_applications] == [:ok6]
     end)
   end
 
@@ -242,7 +260,7 @@ defmodule Mix.Tasks.Compile.AppTest do
 
       properties = parse_resource_file(:sample)
       assert properties[:registered] == []
-      assert properties[:description] == 'sample'
+      assert properties[:description] == ~c"sample"
       assert properties[:applications] == [:kernel, :stdlib, :elixir]
 
       assert Mix.Tasks.Compile.App.run([]) == {:noop, []}
