@@ -4,6 +4,8 @@ defmodule Mix.Tasks.Compile.ErlangTest do
   use MixTest.Case
   import ExUnit.CaptureIO
 
+  defmacro position(line, column), do: {line, column}
+
   setup config do
     erlc_options = Map.get(config, :erlc_options, [])
     Mix.ProjectStack.post_config(erlc_options: erlc_options)
@@ -11,10 +13,10 @@ defmodule Mix.Tasks.Compile.ErlangTest do
     :ok
   end
 
-  @tag erlc_options: [{:d, 'foo', 'bar'}]
+  @tag erlc_options: [{:d, ~c"foo", ~c"bar"}]
   test "raises on invalid erlc_options" do
     in_fixture("compile_erlang", fn ->
-      assert_raise Mix.Error, ~r"Compiling Erlang file '.*' failed", fn ->
+      assert_raise Mix.Error, ~r/Compiling Erlang file ".*" failed/, fn ->
         capture_io(fn ->
           Mix.Tasks.Compile.Erlang.run([])
         end)
@@ -81,6 +83,7 @@ defmodule Mix.Tasks.Compile.ErlangTest do
   test "continues even if one file fails to compile" do
     in_fixture("compile_erlang", fn ->
       file = Path.absname("src/zzz.erl")
+      source = deterministic_source(file)
 
       File.write!(file, """
       -module(zzz).
@@ -92,9 +95,10 @@ defmodule Mix.Tasks.Compile.ErlangTest do
 
         assert %Mix.Task.Compiler.Diagnostic{
                  compiler_name: "erl_parse",
-                 file: ^file,
+                 file: ^source,
+                 source: ^source,
                  message: "syntax error before: zzz",
-                 position: 2,
+                 position: position(2, 5),
                  severity: :error
                } = diagnostic
       end)
@@ -107,6 +111,7 @@ defmodule Mix.Tasks.Compile.ErlangTest do
   test "saves warnings between builds" do
     in_fixture("compile_erlang", fn ->
       file = Path.absname("src/has_warning.erl")
+      source = deterministic_source(file)
 
       File.write!(file, """
       -module(has_warning).
@@ -117,10 +122,11 @@ defmodule Mix.Tasks.Compile.ErlangTest do
         assert {:ok, [diagnostic]} = Mix.Tasks.Compile.Erlang.run([])
 
         assert %Mix.Task.Compiler.Diagnostic{
-                 file: ^file,
+                 file: ^source,
+                 source: ^source,
                  compiler_name: "erl_lint",
                  message: "function my_fn/0 is unused",
-                 position: 2,
+                 position: position(2, 1),
                  severity: :warning
                } = diagnostic
 
@@ -128,6 +134,9 @@ defmodule Mix.Tasks.Compile.ErlangTest do
         assert {:noop, [^diagnostic]} = Mix.Tasks.Compile.Erlang.run(["--verbose"])
         refute_received {:mix_shell, :info, ["Compiled src/has_warning.erl"]}
 
+        assert [^diagnostic] = Mix.Tasks.Compile.Erlang.diagnostics()
+        assert [^diagnostic] = Mix.Task.Compiler.diagnostics()
+
         # Should not return warning after changing file
         File.write!(file, """
         -module(has_warning).
@@ -152,12 +161,13 @@ defmodule Mix.Tasks.Compile.ErlangTest do
 
       capture_io(fn -> Mix.Tasks.Compile.Erlang.run([]) end)
 
-      output =
-        capture_io(fn ->
-          assert {:noop, _} = Mix.Tasks.Compile.Erlang.run(["--all-warnings"])
-        end)
+      assert capture_io(fn ->
+               assert {:noop, _} = Mix.Tasks.Compile.Erlang.run([])
+             end) =~ ~r"has_warning.erl:2:(1:)? warning: function my_fn/0 is unused\n"
 
-      assert output == "src/has_warning.erl:2: Warning: function my_fn/0 is unused\n"
+      assert capture_io(fn ->
+               assert {:noop, _} = Mix.Tasks.Compile.Erlang.run([])
+             end) =~ ~r"has_warning.erl:2:(1:)? warning: function my_fn/0 is unused\n"
 
       # Should not print old warnings after fixing
       File.write!(file, """
@@ -175,6 +185,39 @@ defmodule Mix.Tasks.Compile.ErlangTest do
     end)
   end
 
+  test "returns syntax error from an Erlang file when --return-errors is set" do
+    in_fixture("no_mixfile", fn ->
+      import ExUnit.CaptureIO
+
+      file = Path.absname("src/a.erl")
+      source = deterministic_source(file)
+
+      File.mkdir!("src")
+
+      File.write!(file, """
+      -module(b).
+      def b(), do: b
+      """)
+
+      capture_io(fn ->
+        assert {:error, [diagnostic]} =
+                 Mix.Tasks.Compile.Erlang.run(["--force", "--return-errors"])
+
+        assert %Mix.Task.Compiler.Diagnostic{
+                 compiler_name: "erl_parse",
+                 file: ^source,
+                 source: ^source,
+                 message: "syntax error before: b",
+                 position: position(2, 5),
+                 severity: :error
+               } = diagnostic
+      end)
+
+      refute File.regular?("ebin/Elixir.A.beam")
+      refute File.regular?("ebin/Elixir.B.beam")
+    end)
+  end
+
   @tag erlc_options: [{:warnings_as_errors, true}]
   test "adds :debug_info to erlc_options by default" do
     in_fixture("compile_erlang", fn ->
@@ -188,4 +231,10 @@ defmodule Mix.Tasks.Compile.ErlangTest do
       assert debug_info != :none
     end)
   end
+
+  if :deterministic in :compile.env_compiler_options() do
+    defp deterministic_source(file), do: Path.basename(file)
+  else
+    defp deterministic_source(file), do: file
+  end
 end
