@@ -9,7 +9,7 @@ defmodule Mix.Tasks.EvalTest do
 
   test "does not start applications", context do
     in_tmp(context.test, fn ->
-      expr = "send self(), {:apps, Application.loaded_applications()}"
+      expr = "send self(), {:apps, Application.started_applications()}"
       Mix.Tasks.Eval.run([expr])
       assert_received {:apps, apps}
       refute List.keyfind(apps, :sample, 0)
@@ -26,6 +26,30 @@ defmodule Mix.Tasks.EvalTest do
     end)
   end
 
+  test "accepts arguments", context do
+    in_tmp(context.test, fn ->
+      Mix.Tasks.Eval.run(["send self(), {:argv, System.argv()}", "bar", "baz"])
+      assert_received {:argv, ~w[bar baz]}
+    end)
+  end
+
+  test "resets argv to previous value", context do
+    argv = System.argv()
+
+    try do
+      in_tmp(context.test, fn ->
+        System.argv(["foo"])
+
+        Mix.Tasks.Eval.run(["send self(), {:argv, System.argv()}", "bar", "baz"])
+        assert_received {:argv, ~w[bar baz]}
+
+        assert ["foo"] == System.argv()
+      end)
+    after
+      System.argv(argv)
+    end
+  end
+
   test "runs without mix.exs" do
     Mix.Project.pop()
 
