@@ -33,15 +33,14 @@ defmodule Mix.Tasks.TestTest do
     end
 
     test "accepts custom :exit_status" do
-      assert {:exit_status, 5} in ex_unit_opts(exit_status: 5)
+      assert {:exit_status, 5} in ex_unit_opts(exit_status: 5, failures_manifest_path: "foo.bar")
     end
 
     test "includes some default options" do
-      assert ex_unit_opts([]) == [
+      assert ex_unit_opts(failures_manifest_path: "foo.bar") == [
                autorun: false,
                exit_status: 2,
-               failures_manifest_file:
-                 Path.join(Mix.Project.manifest_path(), ".mix_test_failures")
+               failures_manifest_path: "foo.bar"
              ]
     end
 
@@ -52,8 +51,9 @@ defmodule Mix.Tasks.TestTest do
 
     defp ex_unit_opts_from_given(passed) do
       passed
+      |> Keyword.put(:failures_manifest_path, "foo.bar")
       |> ex_unit_opts()
-      |> Keyword.drop([:failures_manifest_file, :autorun, :exit_status])
+      |> Keyword.drop([:failures_manifest_path, :autorun, :exit_status])
     end
   end
 
@@ -134,6 +134,7 @@ defmodule Mix.Tasks.TestTest do
   end
 
   describe "--cover" do
+    @describetag :cover
     test "reports the coverage of each app's modules in an umbrella" do
       in_fixture("umbrella_test", fn ->
         # This fixture by default results in coverage above the default threshold
@@ -223,6 +224,15 @@ defmodule Mix.Tasks.TestTest do
         output = mix(["test", "test/passing_and_failing_test_failed.exs", "--failed"])
         assert output =~ "1 test, 1 failure"
 
+        # Plus line
+        output = mix(["test", "test/passing_and_failing_test_failed.exs:5", "--failed"])
+        assert output =~ "1 test, 1 failure"
+
+        if windows?() do
+          output = mix(["test", "test\\passing_and_failing_test_failed.exs:5", "--failed"])
+          assert output =~ "1 test, 1 failure"
+        end
+
         # `--failed` composes with an `--only` filter by running the intersection.
         # Of the failing tests, 1 is tagged with `@tag :foo`.
         # Of the passing tests, 1 is tagged with `@tag :foo`.
@@ -237,6 +247,11 @@ defmodule Mix.Tasks.TestTest do
         # Nothing should get run if we try it again since everything is passing.
         assert mix(["test", "--failed"]) =~ "There are no tests to run"
 
+        # When everything is passing and a file is passed, we return the proper message
+        output = mix(["test", "test/passing_and_failing_test_failed.exs", "--failed"])
+        assert output =~ "There are no tests to run"
+        refute output =~ "does not match"
+
         # `--failed` and `--stale` cannot be combined
         output = mix(["test", "--failed", "--stale"])
         assert output =~ "Combining --failed and --stale is not supported"
@@ -244,6 +259,18 @@ defmodule Mix.Tasks.TestTest do
     after
       System.delete_env("PASS_FAILING_TESTS")
     end
+
+    test "marks the whole suite as failed on compilation error" do
+      in_fixture("test_failed", fn ->
+        File.write!("test/passing_and_failing_test_failed.exs", "raise ~s(oops)")
+
+        output = mix(["test"])
+        assert output =~ "** (RuntimeError) oops"
+
+        output = mix(["test", "--failed"])
+        assert output =~ "** (RuntimeError) oops"
+      end)
+    end
   end
 
   describe "--listen-on-stdin" do
@@ -251,7 +278,7 @@ defmodule Mix.Tasks.TestTest do
       in_fixture("test_stale", fn ->
         port = mix_port(~w[test --stale --listen-on-stdin])
 
-        assert receive_until_match(port, "seed", "") =~ "2 tests"
+        assert receive_until_match(port, "0 failures", "") =~ "2 tests"
 
         Port.command(port, "\n")
 
@@ -259,6 +286,7 @@ defmodule Mix.Tasks.TestTest do
       end)
     end
 
+    @tag :unix
     test "does not exit on compilation failure" do
       in_fixture("test_stale", fn ->
         File.write!("lib/b.ex", """
@@ -279,7 +307,7 @@ defmodule Mix.Tasks.TestTest do
 
         Port.command(port, "\n")
 
-        assert receive_until_match(port, "seed", "") =~ "2 tests"
+        assert receive_until_match(port, "0 failures", "") =~ "2 tests"
 
         File.write!("test/b_test_stale.exs", """
         defmodule BTest do
@@ -293,7 +321,7 @@ defmodule Mix.Tasks.TestTest do
 
         Port.command(port, "\n")
 
-        message = "undefined function error_not_a_var"
+        message = "undefined variable \"error_not_a_var\""
         assert receive_until_match(port, message, "") =~ "test/b_test_stale.exs"
 
         File.write!("test/b_test_stale.exs", """
@@ -308,13 +336,14 @@ defmodule Mix.Tasks.TestTest do
 
         Port.command(port, "\n")
 
-        assert receive_until_match(port, "seed", "") =~ "2 tests"
+        assert receive_until_match(port, "0 failures", "") =~ "2 tests"
       end)
     end
   end
 
   describe "--partitions" do
-    test "splits tests into partitions" do
+    @tag :cover
+    test "splits tests into partitions (with coverage)" do
       in_fixture("test_stale", fn ->
         assert mix(["test", "--partitions", "3", "--cover"], [{"MIX_TEST_PARTITION", "1"}]) =~
                  "1 test, 0 failures"
@@ -410,9 +439,10 @@ defmodule Mix.Tasks.TestTest do
     test "logs test absence for a project with no test paths" do
       in_fixture("test_stale", fn ->
         File.rm_rf!("test")
+        assert_run_output("There are no tests to run")
+
         File.mkdir_p!("test")
         File.write!("test/test_helper.exs", "ExUnit.start()")
-
         assert_run_output("There are no tests to run")
       end)
     end
@@ -431,14 +461,14 @@ defmodule Mix.Tasks.TestTest do
       end)
     end
 
-    test "raises an exception if line numbers are given with multiple files" do
+    test "runs multiple test files if line numbers are given" do
       in_fixture("test_stale", fn ->
         assert_run_output(
-          [
-            "test/a_test_stale.exs",
-            "test/b_test_stale.exs:4"
-          ],
-          "Line numbers can only be used when running a single test file"
+          ["test/a_test_stale.exs:2", "test/b_test_stale.exs:4"],
+          """
+          Excluding tags: [:test]
+          Including tags: [location: {"test/a_test_stale.exs", 2}, location: {"test/b_test_stale.exs", 4}]
+          """
         )
       end)
     end
@@ -461,36 +491,64 @@ defmodule Mix.Tasks.TestTest do
 
         output = mix(["test", "apps/bar/test/bar_tests.exs"])
 
-        assert output =~ """
-               ==> bar
-               ....
-               """
+        assert output =~ "==> bar"
+        assert output =~ "...."
 
         refute output =~ "==> foo"
         refute output =~ "Paths given to \"mix test\" did not match any directory/file"
 
         output = mix(["test", "./apps/bar/test/bar_tests.exs"])
 
-        assert output =~ """
-               ==> bar
-               ....
-               """
+        assert output =~ "==> bar"
+        assert output =~ "...."
+
+        refute output =~ "==> foo"
+        refute output =~ "Paths given to \"mix test\" did not match any directory/file"
+
+        output = mix(["test", Path.expand("apps/bar/test/bar_tests.exs")])
+
+        assert output =~ "==> bar"
+        assert output =~ "...."
 
         refute output =~ "==> foo"
         refute output =~ "Paths given to \"mix test\" did not match any directory/file"
 
         output = mix(["test", "apps/bar/test/bar_tests.exs:10"])
 
+        assert output =~ "==> bar"
+
         assert output =~ """
-               ==> bar
                Excluding tags: [:test]
-               Including tags: [line: \"10\"]
+               Including tags: [location: {"test/bar_tests.exs", 10}]
 
                .
                """
 
         refute output =~ "==> foo"
         refute output =~ "Paths given to \"mix test\" did not match any directory/file"
+
+        casing =
+          if windows?() do
+            "apps\\bar\\test\\bar_tests.exs:5"
+          else
+            "apps/bar/test/bar_tests.exs:5"
+          end
+
+        output = mix(["test", "apps/foo/test/foo_tests.exs:9", casing])
+
+        assert output =~ """
+               Excluding tags: [:test]
+               Including tags: [location: {"test/foo_tests.exs", 9}]
+               """
+
+        assert output =~ "1 test, 0 failures\n"
+
+        assert output =~ """
+               Excluding tags: [:test]
+               Including tags: [location: {"test/bar_tests.exs", 5}]
+               """
+
+        assert output =~ "4 tests, 0 failures, 3 excluded\n"
       end)
     end
   end
@@ -555,6 +613,17 @@ defmodule Mix.Tasks.TestTest do
     end
   end
 
+  describe "--max-requires" do
+    test "runs tests with --max-requires 1" do
+      # this is only a smoke test to ensure that tests run with --max-requires 1
+      # it does not test the concurrency behavior
+      in_fixture("test_stale", fn ->
+        output = mix(["test", "--max-requires", "1"])
+        assert output =~ "0 failures"
+      end)
+    end
+  end
+
   defp receive_until_match(port, expected, acc) do
     receive do
       {^port, {:data, output}} ->
@@ -565,6 +634,13 @@ defmodule Mix.Tasks.TestTest do
         else
           receive_until_match(port, expected, acc)
         end
+    after
+      15_000 ->
+        raise """
+        nothing received from port after 15s.
+        Expected: #{inspect(expected)}
+        Got: #{inspect(acc)}
+        """
     end
   end
 
